package org.jblooming.transaction.hibernate;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.transaction.Transaction;

import java.sql.SQLException;


public class HibernateTransaction implements Transaction {

  protected static ThreadLocal threadLocalTransaction = new ThreadLocal() {
    protected Object initialValue() {
      return null;
    }
  };

  boolean failed = false;
  protected org.hibernate.Transaction hibTrans;


  public HibernateTransaction() {

    if (threadLocalTransaction.get() != null)
      throw new PlatformRuntimeException("more than a transaction per thread detected!");

    try {
      hibTrans = HibernateFactory.newSession().beginTransaction();
    } catch (Throwable ex) {
      throw new PlatformRuntimeException("HibernateTransaction constuctor. Unable to start transaction.", ex);
    }

    threadLocalTransaction.set(this);
  }

  public boolean isFailed() {
    return failed;
  }

  public void setFailed() {
    this.failed = true;
  }

  public boolean isActive() {
    boolean result = false;
    if (hibTrans != null)
      result = hibTrans.isActive();

    //check also if connection is still open
    if (result) {
      Session session = HibernateFactory.getSessionNoInit();
      if (session != null) {
        try {
          result = result && !session.connection().isClosed();
        } catch (SQLException e) {
          result = false;
        }
      }
    }

    return result;
  }

  public void commit() {

    if (threadLocalTransaction.get() == null)
      throw new PlatformRuntimeException("HibernateTransaction inconsistency. Thread transactions is null!");

    threadLocalTransaction.set(null);

    PlatformRuntimeException exception = null;

    try {
      if (isFailed())
        throw new PlatformRuntimeException("Trying to commit a failed transaction");
      //if (hibTrans.isActive())
      hibTrans.commit();
    } catch (Throwable ex) {
      exception = new PlatformRuntimeException("HibernateTransaction commit exception.", ex);
      try {
        if (hibTrans.isActive())
          hibTrans.rollback();
      } catch (Throwable e) {
        exception = new PlatformRuntimeException("HibernateTransaction failed commit and abort exception.", exception);
      }
    } finally {
      HibernateFactory.closeSession();
    }

    if (exception != null)
      throw exception;

  }

  public static HibernateTransaction getCurrentTransaction() {
    return (HibernateTransaction) threadLocalTransaction.get();
  }

  public void abort() {

    threadLocalTransaction.set(null);

    try {
      if (hibTrans.isActive())
        hibTrans.rollback();
    } catch (Throwable ex) {
      throw new PlatformRuntimeException("HibernateFactory abort exception.", ex);
    } finally {
      HibernateFactory.closeSession();
    }
  }

  public void rollback() throws HibernateException {
    if (hibTrans != null && hibTrans.isActive())
      hibTrans.rollback();
  }


  public void checkPoint() {
    //if (hibTrans.isActive()) {
    hibTrans.commit();

    // 24/01/2007 changed in Munich: it was hibTrans.begin(), but for unknows reasons this spaccated the session read in same trans
    // in particular setting progress by worklog failed; notice also that now it is the same code used above in transaction instantiation
    hibTrans = HibernateFactory.getSession().beginTransaction();
    //
    //}
  }


}

