package org.jblooming.agenda;

import org.jblooming.persistence.PersistenceHome;
import org.jblooming.utilities.DateUtilities;

import java.util.Date;


public class Period extends ScheduleSupport implements Schedule, Comparable {

  /**
   * @deprecated use Period(long start, long end)
   */
  protected Period() {
  }

  public Period(long start, long end) {
    this(new Date(start), new Date(end));
  }

  public Period(Date start, long duration) {
    CompanyCalendar st = new CompanyCalendar();
    st.setTime(start);
    st.add(CompanyCalendar.MILLISECOND, (int) duration);
    buildInstance(start, st.getTime());
  }

  public Period(Date start, Date end) {
    buildInstance(start, end);
  }


  private void buildInstance(Date start, Date end) {
    this.setStart(start);
    this.setEnd(end);
    recomputeDuration();
  }

  public void recomputeDuration() {
    CompanyCalendar st = new CompanyCalendar();
    if (this.getStart() != null) {
      st.setTime(this.getStart());
      this.setStartTime(st.getMillisFromMidnight());
    } else {
      this.setStartTime(0);
    }
    this.setDuration(getValidityEndTime() - getValidityStartTime());
  }

  /**
   * @param aDate
   * @return a day period for the passed date
   */
  public static Period getDayPeriodInstance(Date aDate) {
    CompanyCalendar cc = new CompanyCalendar();
    cc.setTime(aDate);
    return new Period(cc.setAndGetTimeToDayStart(), cc.setAndGetTimeToDayEnd());
  }

  /**
   * create a new Period as intersection with the passed one
   *
   * @param p the intersecting Period
   * @return null if no intersection
   */
  public Period intersection(Period p) {
    Period result = null;
    if (!((p.getValidityEndTime() < this.getValidityStartTime()) || (p.getValidityStartTime() > this.getValidityEndTime())))
    {
      result = new Period(Math.max(this.getValidityStartTime(), p.getValidityStartTime()), Math.min(this.getValidityEndTime(), p.getValidityEndTime()));
    }
    return result;
  }

  public Period union(Period p) {
    return new Period(Math.min(this.getValidityStartTime(), p.getValidityStartTime()), Math.max(this.getValidityEndTime(), p.getValidityEndTime()));
  }

  public long getNextFireTimeAfter(long afterTime) {
    if (getValidityStartTime() > afterTime)
      return getValidityStartTime();
    else
      return Long.MAX_VALUE;
  }

  public long getPreviousFireTimeBefore(long beforeTime) {
    if (getValidityStartTime() < beforeTime)
      return getValidityStartTime();
    else
      return Long.MIN_VALUE;
  }

  //todo: this is not valid in daylight saving transitions
  public int lenghtInDays() {
    return (new Long(getDurationInMillis() / (CompanyCalendar.MILLIS_IN_DAY))).intValue();
  }

  public String toString() {
    return DateUtilities.dateToString(getStart(), "yyyy MM dd HH:mm:ss") + " - " + DateUtilities.dateToString(getEnd(), "yyyy MM dd HH:mm:ss");
  }


  public int getFrequency() {
    return 0;
  }


  public int getRepetitions() {
    // a Period is repeated always once
    return 1;
  }

  protected void recalculateFields() {
    //To change body of implemented methods use File | Settings | File Templates.
  }

  public String getName() {
    return "period";
  }

  public void setEndDate(Date date) {
    setEnd(date);
  }


  public int hashCode() {
    int result = 0;
    if (PersistenceHome.NEW_EMPTY_ID.equals(id) || id == null)
      result = (this.getValidityStartTime() + "").hashCode();
    else
      result = (id + "").hashCode();
    return result;
  }

}
