package org.jblooming.cache;

import org.hibernate.HibernateException;
import org.hibernate.proxy.HibernateProxyHelper;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.operator.Operator;
import org.jblooming.operator.User;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.tracer.Tracer;

/**
 * Holds a cache for a http session.
 * The cache gets initialized on need, by calling any of its public methods;
 * the objects cache are:
 * - operator (fully initialized)
 * - a map of (permission, collection of areas for which the operator has the permission)
 */
public class SessionCache {

  public Class operatorClass = Operator.class;
  public org.jblooming.operator.Operator cachedOperator;

  /**
   * @param opId the operator that logged in the session
   */
  public User getCachedUser(int opId) {
    if (cachedOperator == null)
      initialize(opId, this);
    return cachedOperator;
  }

  public void resetCachedUser(int opId) throws PersistenceException {
    if (cachedOperator != null) {
      operatorClass = HibernateProxyHelper.getClassWithoutInitializingProxy(cachedOperator);
      cachedOperator = null;
    }
    getCachedUser(opId);
  }

  /**
   * @param opId the operator that logged in the session
   */
  private void initialize(int opId, SessionCache sc) {

    if (opId != -1) {
      Operator op = null;
      try {
        op = (Operator) PersistenceHome.findByPrimaryKey(operatorClass, opId);
        try {
          op.operatorInitializer();
          if (sc.cachedOperator != null)
            Tracer.platformLogger.debug("operator reloaded in http session - may cause non fatal hibernate transaction-scoped narrowing hit");
          sc.cachedOperator = op;

        } catch (HibernateException e) {
          throw new PlatformRuntimeException(e);
        }
      } catch (FindByPrimaryKeyException e) {
        throw new PlatformRuntimeException(e);
      }

    }
  }
}
