package org.jblooming.cursor;

import org.jblooming.cursor.exceptions.CursorException;

import java.util.*;

/**
 * @deprecated use org.jblooming.cursor.Page
 *             Page number starts at 0 ends at (getPageCount() - 1)
 */

public class LazyFetchCursor extends AbstractCursor {

  protected List objectReferences;
  protected CursorServer cursorServer;

  public LazyFetchCursor(List or, CursorServer cs) {
    this(or.iterator(), cs);
  }

  public LazyFetchCursor(Iterator ite, CursorServer cs) {
    super();

    objectReferences = new LinkedList();
    while (ite.hasNext())
      objectReferences.add(ite.next());

    cursorServer = cs;
  }

  protected CursorServer getCursorServer() {
    return cursorServer;
  }

  public List getObjectReferences() {
    return objectReferences;
  }

  public void setObjectReferences(List objectReferences) {
    this.objectReferences = objectReferences;
  }

  /**
   * @throws java.lang.IndexOutOfBoundsException
   *          if the element number is greater than getPageCount().
   */
  public Iterator getPage(int i) throws CursorException {

    if (i >= getPageCount() || i < 0) {
      if (getElementCount() > 0)
        throw new IndexOutOfBoundsException();
      else
        return Collections.EMPTY_LIST.iterator();
    }

    currentPage = i;
    int fromIndex = getPageElementCount() * currentPage;
    int toIndex = fromIndex + getPageElementCount();
    if (toIndex > (objectReferences.size() - 1)) toIndex = objectReferences.size();
    List page = cursorServer.loadPage(objectReferences, fromIndex, toIndex);
    currentElement = 0;
    return page.iterator();
  }


  public void invalidateBuffer() {
  }

  public Object getElement(int i) throws CursorException {
    if (i >= getCurrentPageElementCount() || i < 0) throw new IndexOutOfBoundsException();
    Iterator ite = getPage();
    currentElement = i;
    for (; i > 0; --i) ite.next();
    return ite.next();
  }

  public int getElementCount() throws CursorException {
    if (objectReferences == null)
      return 0;
    else
      return objectReferences.size();
  }

  //**
  public boolean contains(Object o) {
    return objectReferences.contains(o);
  }

  protected int seek(Object o) {
    ListIterator iterator = objectReferences.listIterator();
    while (iterator.hasNext()) {
      int index = iterator.nextIndex();
      if (iterator.next().equals(o))
        return index;
    }
    throw new NoSuchElementException();
  }

  public Cursor union(Cursor c) throws CursorException {
    if (c instanceof LazyFetchCursor) {
      LazyFetchCursor jc = (LazyFetchCursor) c;
      if (jc.getCursorServer().getClass().equals(getCursorServer().getClass())) {
        List ref = new LinkedList(getObjectReferences());
        ref.addAll(jc.getObjectReferences());
        return new LazyFetchCursor(ref, getCursorServer());
      }
    }
    return listUnion(CursorTransformer.toList(this), c);
  }

  public Cursor intersection(Cursor c) throws CursorException {
    if (c instanceof LazyFetchCursor) {
      LazyFetchCursor jc = (LazyFetchCursor) c;
      if (jc.getCursorServer().getClass().equals(getCursorServer().getClass())) {
        List ref = new LinkedList(getObjectReferences());
        ref.retainAll(jc.getObjectReferences());
        return new LazyFetchCursor(ref, getCursorServer());
      }
    }
    return listIntersection(CursorTransformer.toList(this), c);
  }

  public Cursor difference(Cursor c) throws CursorException {
    if (c instanceof LazyFetchCursor) {
      LazyFetchCursor jc = (LazyFetchCursor) c;
      if (jc.getCursorServer().getClass().equals(getCursorServer().getClass())) {
        List ref = new LinkedList(getObjectReferences());
        ref.removeAll(jc.getObjectReferences());
        return new LazyFetchCursor(ref, getCursorServer());
      }
    }
    return listDifference(CursorTransformer.toList(this), c);
  }

  public ListCursor toListCursor(int i) throws CursorException {
    return null;
  }


}
