package org.jblooming.ldap;

import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.settings.ApplicationState;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.*;
import java.util.*;

public class LdapUtilities {
  public static final String LDAP = "LDAP_";
  public static final String INITIAL_CONTEXT_FACTORY = LDAP + "INITIAL_CONTEXT_FACTORY";
  public static final String PROVIDER_URL = LDAP + "PROVIDER_URL";
  public static final String SECURITY_AUTHENTICATION = LDAP + "SECURITY_AUTHENTICATION";
  public static final String SECURITY_PRINCIPAL = LDAP + "SECURITY_PRINCIPAL";
  public static final String SECURITY_CREDENTIALS = LDAP + "SECURITY_CREDENTIALS";
  public static final String BASE_DN = LDAP + "BASE_DN";

  public static List<String> getMemberOf(String baseDN ,String cn, DirContext ctx, List<String> groups) throws NamingException {

    SearchControls ctls = new SearchControls();
    ctls.setCountLimit(1);
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String []{"memberof"});

    NamingEnumeration nm = ctx.search(baseDN, cn, ctls);
    if (nm != null && nm.hasMoreElements()) {
      SearchResult sr = (SearchResult) nm.nextElement();
      Attributes atts = sr.getAttributes();
      NamingEnumeration allAttr = atts.getAll();
      if (allAttr != null && allAttr.hasMore()) {
        if (groups == null)
          groups = new ArrayList<String>();
        Attribute att = (Attribute) allAttr.next();
        NamingEnumeration attEn = att.getAll();
        while (attEn.hasMore()) {
          String groupSer = (String) attEn.next();
          String group = StringUtilities.splitToArray(groupSer, ",")[0];
          if (!groups.contains(group))
            groups.add(group);
          getMemberOf(baseDN, group, ctx, groups);
        }
      }
    }
    return groups;
  }


  public static Map<String, Set<String>> getPropertyOf(String baseDN ,String cn, DirContext ctx) throws NamingException {
    SearchControls ctls = new SearchControls();
    ctls.setCountLimit(1);
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);

    NamingEnumeration nm = ctx.search(baseDN, cn, ctls);

    Map<String, Set<String>> props = new HashMap<String, Set<String>>();

    if (nm.hasMoreElements()) {
      SearchResult sr = (SearchResult) nm.nextElement();
      Attributes atts = sr.getAttributes();
      NamingEnumeration allAttr = atts.getAll();
      while (allAttr.hasMore()) {
        Attribute att = (Attribute) allAttr.next();
        String propId = att.getID();
        Set<String>propValSet = new HashSet<String>();
        NamingEnumeration attEn = att.getAll();
        if (attEn != null && attEn.hasMore()) {
          while (attEn.hasMore()) {
            propValSet.add((String) attEn.next());
          }
        } else {
          propValSet.add(att.get().toString());
        }
        props.put(propId,propValSet);
      }
    }
    return props;
  }


  public static DirContext getDefaultContext() throws NamingException {
    Hashtable env = new Hashtable();
    if (ApplicationState.getApplicationSetting(INITIAL_CONTEXT_FACTORY) != null)
      env.put(Context.INITIAL_CONTEXT_FACTORY, ApplicationState.getApplicationSetting(INITIAL_CONTEXT_FACTORY));
    else
      env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");

    env.put(Context.PROVIDER_URL, ApplicationState.getApplicationSetting(PROVIDER_URL));
    env.put(Context.SECURITY_AUTHENTICATION, ApplicationState.getApplicationSetting(SECURITY_AUTHENTICATION));
    env.put(Context.SECURITY_PRINCIPAL, ApplicationState.getApplicationSetting(SECURITY_PRINCIPAL));
    env.put(Context.SECURITY_CREDENTIALS, ApplicationState.getApplicationSetting(SECURITY_CREDENTIALS));
    DirContext ctx = new InitialDirContext(env);

    return ctx;
  }

}
