package org.jblooming.messaging;

import org.jblooming.operator.Operator;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.scheduler.ExecutableSupport;
import org.jblooming.scheduler.JobLogData;
import org.jblooming.system.SystemConstants;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.waf.settings.ApplicationState;

import java.util.Date;
import java.util.List;

public class EmailMessageDispatcher extends ExecutableSupport {

  public JobLogData run(JobLogData jobLogData) throws Exception {
    boolean exception = true;
    PersistenceContext pc = null;


    try {

      pc = HibernateFactory.newFreeSession();

      String hql = "from " + Message.class.getName() + " as mess where mess.media = :media and (mess.lastTry is null or mess.lastTry < :nowMinusDelay)";
      OqlQuery query = new OqlQuery(hql,pc);
      query.getQuery().setString("media", MessagingSystem.Media.EMAIL.toString());
      Date nmd = new Date(System.currentTimeMillis()-MessagingSystem.DELAY_BETWEEN_ATTEMPTS);
      query.getQuery().setTimestamp("nowMinusDelay", nmd);

      List<Message> messages = query.list();

      for (Message message : messages) {

        if (message.getNumberOfTries()>MessagingSystem.NUMBER_OF_ATTEMPTS) {
          PersistenceHome.remove(message,pc);
          Tracer.platformLogger.error("EmailMessageDispatcher: could not send mail message with parameters: to operator of id "+message.getToOperator());
        } else
          try {

          String fromEmail = null;
           if (message.getFromOperator()!=null)
             fromEmail = message.getFromOperator().getDefaultEmail();
          if (fromEmail==null)
            fromEmail = ApplicationState.getApplicationSetting(SystemConstants.FLD_MAIL_FROM);
          if (fromEmail==null)
            fromEmail = "";

          if (message.getToOperator()!=null) {
            Operator toOp = message.getToOperator();
            String toEmail = toOp.getDefaultEmail();
            if (toEmail!=null) {
              String body = message.getMessageBody();
              if (message.getLink()!=null)
                body = body + "<hr>"+message.getLink();
              MailHelper.sendHtmlMail(fromEmail,toEmail,message.getSubject(),body);
            }
          }
          PersistenceHome.remove(message,pc);
        } catch (Exception e) {
          Tracer.platformLogger.error("Problem sending mail",e);
          message.setNumberOfTries(message.getNumberOfTries()+1);
          message.setLastTry(new Date());
            message.store(pc);
        }

        PersistenceHome.remove(message,pc);
      }

      pc.commitAndClose();

      jobLogData.notes = jobLogData.notes + "EmailMessageDispatcher executed on " + DateUtilities.dateToTimestampString(DateUtilities.getAbsoluteNow());
      exception = false;
    } catch (Exception e) {
      Tracer.platformLogger.error("EmailMessageDispatcher error",e);
    } finally {
      if (exception) {
        if (pc != null) {
          pc.rollback();
        }
        jobLogData.successfull = false;
      }
    }

    return jobLogData;
  }
}
