package org.jblooming.messaging;

import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.operator.Operator;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.scheduler.PlatformExecutionService;
import org.jblooming.system.SystemConstants;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.HttpUtilities;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.PageState;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.mail.Message;
import javax.mail.*;
import javax.mail.internet.*;
import java.io.File;
import java.util.Date;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.Future;

//import oracle.sql.DATE;

/**
 * @author Pietro Polsinelli : ppolsinelli@open-lab.com
 * @author Ezio Manetti : emanetti@open-lab.com
 */
public class MailHelper {

  static MailDateFormat mailDateFormat = new MailDateFormat();

  // See Section 3.6.1 of RFC 2822

  /**
   * The name of the RFC 2822 header that stores the mail date.
   */
  protected static final String RFC_2822_DATE = "Date";

  /*static {
    MailcapCommandMap mailcapCommandMap = (MailcapCommandMap) MailcapCommandMap.getDefaultCommandMap();
    mailcapCommandMap.addMailcap("application/zip; ; x-java-content-handler=" + ApplicationZipHandler.class.getName());
    MailcapCommandMap.setDefaultCommandMap(mailcapCommandMap);
  }*/

  /**
   * The following application settings keys are required: SystemConstants.FLD_MAIL_SMTP
   *
   * @param receiver
   * @param psw
   * @param pageState
   * @throws ActionException
   * @throws ApplicationException
   * @throws PersistenceException
   */
  public static void sendPwdMail(Operator receiver, String psw, PageState pageState) throws ActionException, ApplicationException, PersistenceException {

    if (receiver != null) {

      String date = DateUtilities.dateToString(DateUtilities.getAbsoluteNow(), "EEEE, dd MMMM yyyy HH:mm:ss", pageState.getSessionState().getCurrentLocale());

      String message = "As you requested, this is your new password: " + psw;

      message += "\n\n[sent with platform messaging system - " + date + ']';

      sendPlainTextMail(receiver.getAnagraphicalData().getEmail(), receiver.getAnagraphicalData().getEmail(), "message - " + date, message);
    }
  }

  public static void sendPlainTextMail(String from, String to, String subject, String body) throws ApplicationException {
    javax.mail.Session mailSession = getEmailSession();
    if (mailSession != null)
      sendPlainTextMail(mailSession, from, to, subject, body);
  }


  public static void sendPlainTextMail(javax.mail.Session mailSession, String from, String to, String subject, String body) throws ApplicationException {
    Set<String> tos = new HashSet();
    tos.add(to);
    sendPlainTextMail(mailSession, from, tos, subject, body);
  }


  public static void sendPlainTextMail(String from, Set<String> to, String subject, String body) throws ApplicationException {
    javax.mail.Session mailSession = getEmailSession();
    if (mailSession != null)
      sendPlainTextMail(mailSession, from, to, subject, body);
  }


  public static void sendPlainTextMail(javax.mail.Session mailSession, String from, Set<String> to, String subject, String body) throws ApplicationException {
    try {
      MimeMessage message = new MimeMessage(mailSession);
      message.setFrom(new InternetAddress(from));
      for (String rec : to) {
        message.addRecipient(MimeMessage.RecipientType.TO, new InternetAddress(rec));
      }
      message.setSubject(subject);
      message.setText(body);

      String date = mailHeaderDate();
      if (message.getHeader(RFC_2822_DATE) == null) {
        message.addHeader(RFC_2822_DATE, date);
      }
      //this seems to be buggy
      //message.setSentDate(new Date());

      send(message, "smtp", mailSession);
    } catch (MessagingException e) {
      throw new ApplicationException(e);
    }

  }


  public static void sendHtmlMail(String from, String to, String subject, String body) throws ApplicationException {
    //javax.mail.Session mailSession = getMailSession(null);
    Set<String> tos = new HashSet();
    tos.add(to);
    sendHtmlMail(from, tos, subject, body);
//    if (mailSession!=null)
//      sendHtmlMail(mailSession, from, tos, subject, body);
  }

  public static void sendHtmlMail(String from, Set<String> to, String subject, String body) throws ApplicationException {
    javax.mail.Session mailSession = getEmailSession();
    if (mailSession != null)
      sendHtmlMail(mailSession, from, to, subject, body);
    else
      Tracer.platformLogger.warn("MailHelper.sendHtmlMail cannot create session");
  }


  public static Future sendHtmlMailInSeparateThread(final String from, final Set<String> to, final String subject, final String body) throws ApplicationException {
    return PlatformExecutionService.executorService.submit(
            new Thread() {
              public void run() {
                try {
                  sendHtmlMail(from, to, subject, body);
                } catch (ApplicationException e) {
                  Tracer.platformLogger.error(e);
                }
              }


            });

  }


  public static void sendHtmlMail(javax.mail.Session mailSession, String from, Set<String> to, String subject, String body) throws ApplicationException {
    try {

      MimeMessage message = new MimeMessage(mailSession);

      message.setFrom(new InternetAddress(from));
      for (String rec : to) {
        message.addRecipient(MimeMessage.RecipientType.TO, new InternetAddress(rec));
      }
      message.setSubject(subject, "UTF-8");
      message.setContent(body, "text/html; charset=\"UTF-8\"");

      String date = mailHeaderDate();
      if (message.getHeader(RFC_2822_DATE) == null) {
        message.addHeader(RFC_2822_DATE, date);
      }
      //this seems to be buggy
      //message.setSentDate(new Date());
      send(message, "smtp", mailSession);

    } catch (MessagingException e) {
      throw new ApplicationException(e);
    }

  }

  public static String mailHeaderDate() {
    return mailDateFormat.format(new Date());
  }

  public static void sendAuthenticatedMail(String from, String to, String subject, String body) throws ApplicationException {
    javax.mail.Session mailSession = getAuthenticatedSession();
    sendPlainTextMail(mailSession, from, to, subject, body);
  }

  /**
   * The following application settings keys are required: SystemConstants.FLD_MAIL_SMTP, SystemConstants.FLD_MAIL_USER (1),
   * SystemConstants.FLD_MAIL_PWD (1).
   * (1): Only if the smtp server requires authentication.
   *
   * @return A mail session with commonly used properties
   */
  public static Session getAuthenticatedSession() {

    //MailAutenticator autenticator = getAuthenticator();

    return getEmailSession();
  }

  private static Session getEmailSession() {

    Properties mailProps = new Properties();

    boolean useAuth = Fields.TRUE.equalsIgnoreCase(ApplicationState.getApplicationSetting(SystemConstants.FLD_MAIL_USE_AUTHENTICATED));
    String smtp = ApplicationState.getApplicationSetting(SystemConstants.FLD_MAIL_SMTP);
    if (smtp != null && smtp.trim().length() > 0) {
      mailProps.put("mail.smtp.host", smtp);
      if (useAuth)
        mailProps.put("mail.smtp.auth", "true");
    } else
      Tracer.platformLogger.warn("Tried to send mail without smtp configured in settings:" + SystemConstants.FLD_MAIL_SMTP);

    String useAuthenticatedMail = ApplicationState.getApplicationSetting(SystemConstants.FLD_MAIL_USE_AUTHENTICATED);
    MailAutenticator autenticator = null;
    if (Fields.TRUE.equalsIgnoreCase(useAuthenticatedMail)) {
      String smtpUser = ApplicationState.getApplicationSetting(SystemConstants.FLD_MAIL_USER);
      String smtpPwd = ApplicationState.getApplicationSetting(SystemConstants.FLD_MAIL_PWD);
      autenticator = new MailAutenticator(smtpUser, smtpPwd);
    }

    Session mailSession = Session.getInstance(mailProps, autenticator);

    return mailSession;
  }


  public static void attach(MimeMultipart multipart, String text, String charset, String disposition, String nameToBeGivenToAttach) throws MessagingException {

    int xid = multipart.getCount() + 1;
    //String xname = "TEXT" + xid + ".TXT";

    MimeBodyPart xbody = new MimeBodyPart();
    xbody.setText(text, charset);

    // UNDONE
    //xbody.setContentLanguage( String ); // this could be language from Locale
    //xbody.setContentMD5( String md5 ); // don't know about this yet
    xbody.setDescription("Text Attachment: " + nameToBeGivenToAttach, charset);
    xbody.setDisposition(disposition);
    setFileName(xbody, nameToBeGivenToAttach, charset);

    multipart.addBodyPart(xbody);
  }

  public static void attachFile(MimeMultipart multipart, String filePathAndName, String charset) throws MessagingException {

    int xid = multipart.getCount() + 1;
    //String xname = "TEXT" + xid + ".TXT";

    MimeBodyPart xbody = new MimeBodyPart();
    xbody.setFileName(filePathAndName);

    String nameToBeGivenToAttach = new File(filePathAndName).getName();
    // UNDONE
    //xbody.setContentLanguage( String ); // this could be language from Locale
    //xbody.setContentMD5( String md5 ); // don't know about this yet
    //xbody.setDescription("Text Attachment: " + xname, charset);
    xbody.setDisposition(Part.ATTACHMENT);
    setFileName(xbody, nameToBeGivenToAttach, charset);

    multipart.addBodyPart(xbody);
  }


  /**
   * A better version of setFileName() which will encode the name if necessary.
   * Why doesn't JAVA Mail do this?
   *
   * @param part    the part to manipulate
   * @param name    the give file name encoded in UTF (JAVA)
   * @param charset the encoding character set
   */
  public static void setFileName(Part part, String name, String charset) throws MessagingException {

    try {
      name = MimeUtility.encodeWord(name, charset, null);
    } catch (Exception xex) {
    }
    part.setFileName(name);
  }

  public static Future sendMultipartTextMail(final String from, final Set<String> tos, final String subject, final String fileName, final String textAttached) {

    return PlatformExecutionService.executorService.submit(
            new Thread() {
              public void run() {
                try {

                  // Get session
                  Session session = getAuthenticatedSession();

                  // Define message
                  MimeMessage message = new MimeMessage(session);

                  message.setFrom(new InternetAddress(from));
                  for (String rec : tos) {
                    message.addRecipient(MimeMessage.RecipientType.TO, new InternetAddress(rec));
                  }
                  message.setSubject(subject);

                  MimeMultipart multipart = new MimeMultipart();

                  attach(multipart, textAttached, "UTF-8", Part.ATTACHMENT, fileName);

                  // Put parts in message
                  message.setContent(multipart);

                  // Send message
                  send(message, "smtp", session);

                } catch (Exception e) {
                  Tracer.platformLogger.error(e);
                }
              }


            });
  }

  private static void send(Message message, String protocol, Session session) throws MessagingException {
    Transport t = session.getTransport(protocol);
    t.send(message);
    t.close();
  }


  public static Future sendPlainTextMailWithAttach(final String from, final String to, final Set<String> toBCC, final String subject, final String body, final String filePathAndName) {

    return PlatformExecutionService.executorService.submit(
            new Thread() {
              public void run() {
                try {
                  // Get session
                  Session session = MailHelper.getAuthenticatedSession();

                  // create a message
                  MimeMessage msg = new MimeMessage(session);
                  msg.setFrom(new InternetAddress(from));
                  msg.addRecipient(MimeMessage.RecipientType.TO, new InternetAddress(to));

                  for (String rec : toBCC) {
                    msg.addRecipient(MimeMessage.RecipientType.BCC, new InternetAddress(rec));
                  }

                  msg.setSubject(subject);
                  msg.setSentDate(new Date());

                  // create and fill the first message part
                  MimeBodyPart mbp1 = new MimeBodyPart();
                  mbp1.setText(body);


                  File file = new File(filePathAndName);
                  DataSource ds = new FileDataSource(file) {
                    public String getContentType() {
                      return HttpUtilities.getContentType(filePathAndName);
                    }
                  };
                  MimeBodyPart mbp2 = new MimeBodyPart();
                  mbp2.setDataHandler(new DataHandler(ds));
                  mbp2.setFileName(file.getName());
                  mbp2.setDisposition(Part.ATTACHMENT);

                  // create the Multipart and its parts to it
                  Multipart mp = new MimeMultipart();
                  mp.addBodyPart(mbp1);
                  mp.addBodyPart(mbp2);

                  // add the Multipart to the message
                  msg.setContent(mp);

                  // Send message
                  send(msg, "smtp", session);

                } catch (Exception e) {
                  //Tracer.platformLogger.error(e);
                  throw new PlatformRuntimeException(e);
                }
              }
            });
  }


  public static Future sendMailWithHeader(final InternetAddress from, final Set<InternetAddress> tos, final String subject, final String header, final String body) { //final Map<String,String> nameAndMail

    return PlatformExecutionService.executorService.submit(
            new Thread() {
              public void run() {
                try {

                  Session session = getAuthenticatedSession();
                  MimeMessage message = new MimeMessage(session);

                  //StringDataSource source = new StringDataSource(body, header, "aDataHandler");
                  //message.setDataHandler(new DataHandler(source));

                  // first set the body as text/plain
                  message.setText(body,"UTF-8");
                  // then force the content-type
                  message.addHeader("Content-Type",header); 

                  message.setFrom(from);

                  for (InternetAddress to : tos) {
                    message.addRecipient(MimeMessage.RecipientType.TO, to);
                  }

                  message.setSubject(subject, "UTF-8");

                  String date = mailHeaderDate();
                  if (message.getHeader(RFC_2822_DATE) == null) {
                    message.addHeader(RFC_2822_DATE, date);
                  }
                  send(message, "smtp", session);

                } catch (Exception e) {
                  //Tracer.platformLogger.error(e);
                  throw new PlatformRuntimeException(e);
                }
              }
            });
  }

  public static Store getPop3AndConnect() throws MessagingException {

    Session session = getAuthenticatedSession();
    Store store = session.getStore("pop3");
    String FLD_POP3_HOST = ApplicationState.getApplicationSetting(SystemConstants.FLD_POP3_HOST);
    String FLD_POP3_USER = ApplicationState.getApplicationSetting(SystemConstants.FLD_POP3_USER);
    String FLD_POP3_PSW = ApplicationState.getApplicationSetting(SystemConstants.FLD_POP3_PSW);

    if (FLD_POP3_HOST != null && FLD_POP3_USER != null && FLD_POP3_HOST.trim().length() > 0 && FLD_POP3_USER.trim().length() > 0) {
      store.connect(FLD_POP3_HOST, FLD_POP3_USER, FLD_POP3_PSW);
    }
    return store;
  }

  /*// The read() methods use the relative ByteBuffer get() methods. 
  public static InputStream newInputStream(final ByteBuffer buf) {
    return new InputStream() {
      public synchronized int read() throws IOException {
        if (!buf.hasRemaining()) {
          return -1;
        }
        return buf.get();
      }

      public synchronized int read(byte[] bytes, int off, int len) throws IOException {
        if (!buf.hasRemaining()) {
          return -1;
        }
        // Read only what's left
        len = Math.min(len, buf.remaining());
        buf.get(bytes, off, len);
        return len;
      }
    };
  }


  private static class StringDataSource implements DataSource {
    private String contents;
    private String mimetype;
    private String name;


    public StringDataSource(String contents, String mimetype, String name) {
      this.contents = contents;
      this.mimetype = mimetype;
      this.name = name;
    }

    public String getContentType() {
      return (mimetype);
    }

    public String getName() {
      return (name);
    }

    public InputStream getInputStream() {
      return (new StringBufferInputStream(contents));
    }

    public OutputStream getOutputStream() {
      throw new IllegalAccessError("This datasource cannot be written to");
    }
  }*/


  public static InternetAddress getSystemInternetAddress() {
    InternetAddress fromIA = null;
    try {
      String compFrom = ApplicationState.getApplicationSetting(SystemConstants.FLD_MAIL_FROM);
      if (compFrom != null && compFrom.trim().length() > 0) {
        fromIA = new InternetAddress(compFrom);
        fromIA.setPersonal("Teamwork Mail Service", "UTF-8");
      } else
        Tracer.platformLogger.warn(SystemConstants.FLD_MAIL_FROM + " is not configured in global settings");
    } catch (Exception e) {
      throw new PlatformRuntimeException(e);
    }
    return fromIA;
  }

}