package org.jblooming.messaging;

import org.jblooming.ontology.IdentifiableSupport;
import org.jblooming.ontology.SerializedMap;
import org.jblooming.operator.Operator;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.settings.Application;
import org.jblooming.waf.settings.ApplicationState;

import java.util.Date;
import java.util.List;

public class Message extends IdentifiableSupport {

  private String subject;
  private String messageBody;
  private Operator fromOperator;
  private Operator toOperator;
  private String media;
  private Date lastTry;
  private String status;
  private int numberOfTries;
  private Date expires;
  private Date received;
  private String lastError;
  private String link;

  public Message() {
  }

  public Message(String subject, String messageBody, Operator toOperator, String media) {

    this.subject = subject;
    this.messageBody = messageBody;
    this.toOperator = toOperator;
    this.media = media;
  }

  public String getSubject() {
    return subject;
  }

  public void setSubject(String subject) {
    this.subject = subject;
  }

  public String getMessageBody() {
    return messageBody;
  }

  public void setMessageBody(String messageBody) {
    this.messageBody = messageBody;
  }

  public Operator getFromOperator() {
    return fromOperator;
  }

  public void setFromOperator(Operator fromOperator) {
    this.fromOperator = fromOperator;
  }

  public Operator getToOperator() {
    return toOperator;
  }

  public void setToOperator(Operator toOperator) {
    this.toOperator = toOperator;
  }


  public String getMedia() {
    return media;
  }

  public void setMedia(String media) {
    this.media = media;
  }

  public Date getLastTry() {
    return lastTry;
  }

  public void setLastTry(Date lastTry) {
    this.lastTry = lastTry;
  }

  public String getStatus() {
    return status;
  }

  public void setStatus(String status) {
    this.status = status;
  }

  public int getNumberOfTries() {
    return numberOfTries;
  }

  public void setNumberOfTries(int numberOfTries) {
    this.numberOfTries = numberOfTries;
  }

  public Date getExpires() {
    return expires;
  }

  public void setExpires(Date expires) {
    this.expires = expires;
  }

  public Date getReceived() {
    return received;
  }

  public void setReceived(Date received) {
    this.received = received;
  }

  public String getLastError() {
    return lastError;
  }

  public void setLastError(String lastError) {
    this.lastError = lastError;
  }

  public String getLink() {
    return link;
  }

  public void setLink(String link) {
    this.link = link;
  }


  public static void generateAndPersistMessage(Listener l, SomethingHappened s, PersistenceContext pc) throws StoreException {

    Operator op = l.getOwner();
    Application app = ApplicationState.platformConfiguration.defaultApplication;
    String language = SessionState.getOperatorOption(op, OperatorConstants.FLD_SELECT_LANG);

    String template = ApplicationState.i18n.getLabelNoPageState(s.getMessageTemplate(), app.getName(), language);

    SerializedMap<String, String> mps = s.getMessageParams();
    if (mps != null) {

      for (String key : mps.keySet()) {
        String value = mps.get(key);
        if (value == null || value.trim().length()==0)
          value = "-";
        else
          value = ApplicationState.i18n.getLabelNoPageState(value, app.getName(), language);        
        template = StringUtilities.replaceAllNoRegex(template, "$" + key + "$", value);
      }
    }

    List<String> medias = StringUtilities.splitToList(l.getMedia(), ",");
    for (String media : medias) {
      Message message = new Message();
      if (l.getOwner() != null)
        message.setToOperator(l.getOwner());

      //31/03/2006 commented out by Pietro when trying to get TLog to talk to task -> after reset as it was
      else if (s.getWhoCausedTheEvent()!=null)
        message.setToOperator(s.getWhoCausedTheEvent());

      //else
      //  message.setToEntityId(s.getIdentifiableId());
      message.setMedia(media);
      message.setSubject(ApplicationState.i18n.getLabelNoPageState(s.getEventType(), app.getName(), language));
      if (MessagingSystem.Media.RSS.toString().equals(media)) {
        String link = s.getLink();
        link = link.substring(link.indexOf("\"")+1);
        link = link.substring(0, link.indexOf("\""));
        message.setLink(link);
      } else
        message.setLink(s.getLink());
      message.setMessageBody(template);
      message.setReceived(new Date());
      message.store(pc);
    }
  }


}
