package org.jblooming.ontology;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.RemoveException;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.PersistenceContext;

import java.io.Serializable;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */

public abstract class IdentifiableSupport<K> implements Identifiable, Comparable {


  protected Serializable id;

  public IdentifiableSupport() {
  }

  /**
   * @deprecated use  getId()
   */
  public int getIntId() {
    if (getId() != null && getId() instanceof Integer)
      return (Integer) getId();
    else if (getId() != null && getId() instanceof String) {
      String stringId = (String) getId();
      if ("newEmptyId".equals(getId()))
        throw new PlatformRuntimeException("Using getIntId with wrong argument: id is 'newEmptyId'");
      else
        return Integer.parseInt(stringId);
    } else
      throw new PlatformRuntimeException("Using getIntId with wrong arguments: id is " + getId());
  }

  public Serializable getId() {
    return id;
  }

  public void setId(Serializable id) {
    this.id = id;
  }

  public void setIdAsNew() {
/*
    try {
      ReflectionUtilities.getDeclaredInheritedMethods(this.getClass());
      Class clazz = this.getClass().getDeclaredMethod("getId").getReturnType();
      if (clazz.equals(Integer.class))
        this.id=-1;
      else if (clazz.equals(Long.class))
        this.id=-1L;
      else
*/
        this.id = PersistenceHome.NEW_EMPTY_ID;
/*
    } catch (NoSuchMethodException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
*/
  }

  public void setIdAsNewSer() {
    this.id = PersistenceHome.NEW_EMPTY_ID.toString();
  }

  public boolean isNew() {
    return id==null || PersistenceHome.NEW_EMPTY_ID.equals(id);
  }


  public boolean equals(Object o) {
    return this.compareTo(o) == 0;
  }

  public int hashCode() {
    int result = 0;
    if (PersistenceHome.NEW_EMPTY_ID.equals(id))
      result = System.identityHashCode(this);
    else
      result = (id + "").hashCode();

    return result;
  }

  public int compareTo(Object o) {
    if (this == o)
      return 0;
    if (o == null || this.getId() == null)
      return -1;

    else {
      if (this.getId() != null && this.getId() instanceof Integer && o instanceof Identifiable) {
        Integer integer = (Integer) (this.getId());
        Identifiable identifiable = ((Identifiable) o);
        Integer anotherInteger = (Integer) identifiable.getId();
        return integer.compareTo(anotherInteger);

      } else if (this.getId() != null && this.getId() instanceof Long && o instanceof Identifiable)
        return ((Long) (this.getId())).compareTo((Long) ((Identifiable) o).getId());
      else if (o instanceof Identifiable)
        return this.getId().toString().compareTo(((Identifiable) o).getId().toString());
      else
        return -1;
    }
  }

  public String getName() {
    return "" + id;
  }

  public void store() throws StoreException {
    store(HibernateFactory.getDefaultPersistenceContext());
  }

  public void store(PersistenceContext pc) throws StoreException {
    PersistenceHome.store(this,pc);
  }

  public void remove() throws RemoveException {
    remove(HibernateFactory.getDefaultPersistenceContext());
  }

  public void remove(PersistenceContext pc) throws RemoveException {
    PersistenceHome.remove(this, pc);
  }

   public void saveACopy() throws StoreException {
     saveACopy(HibernateFactory.getDefaultPersistenceContext());
  }

  public void saveACopy(PersistenceContext pc) throws StoreException {
    pc.session.evict(this);
    setId(null);
    store(pc);
  }
  
}