package org.jblooming.ontology;

import org.jblooming.utilities.StringUtilities;

import java.io.File;
import java.io.Serializable;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 */
public class PersistentFile implements Serializable {

  public static final String TYPE_DB = "DB";
  public static final String TYPE_FILESTORAGE = "FR";           // this is relative to the REPOSITORY_URL parameter 
  public static final String TYPE_FILESTORAGE_ABSOLUTE = "FS";  // this is absolute e.g. c:\tmp
  public static final String TYPE_WEBAPP_FILESTORAGE = "WF";    // relative to root

  private int UID;
  private String type = TYPE_DB;
  private String originalFileName;

  /**
   * this is the complete path to the file, file inclusive
   */
  private String fileLocation;

  /**
 * this is the complete path to the file, file exclusive; not persisted
 */
  public String fileDir;

  public static final String PERSISTENTFILE_ID = "PERSISTENTFILE_ID";

  /**
   *
   * @deprecated use constructor with type
   */
  public PersistentFile(int UID, String originalFileName) {
    this.setOriginalFileName(originalFileName);
    this.setUID(UID);
  }

  public PersistentFile(int UID, String originalFileName, String type) {
    this.setOriginalFileName(originalFileName);
    this.setUID(UID);
    this.type=type;
  }


  public int getUID() {
    return UID;
  }

  public void setUID(int UID) {
    this.UID = UID;
  }


  public boolean equals(Object o) {
    return this.compareTo(o) == 0;
  }

  public int hashCode() {
    return serialize().hashCode();
  }

  public int compareTo(Object o) {
    if (this == o)
      return 0;
    if (o == null)
      return -1;
    else {
      return serialize().compareTo(((PersistentFile) o).serialize());
    }
  }

  public String getType() {
    return type;
  }

  /**
   *
   * @deprecated use constructor
   */
  public void setType(String type) {
    this.type = type;
  }

  public String getOriginalFileName() {
    return originalFileName;
  }

  public void setOriginalFileName(String originalFileName) {
    this.originalFileName = originalFileName;
  }

  /**
   *
   * @return the file location relative to PersistentFile.type
  */
  public String getFileLocation() {
    return fileLocation;
  }

  public void setFileLocation(String fileLocation) {
    this.fileLocation = fileLocation;
  }

  public String serialize() {
    String ser = type + "_" + UID + "_" + originalFileName;
    if (TYPE_FILESTORAGE_ABSOLUTE.equals(type) || TYPE_FILESTORAGE.equals(type) || TYPE_WEBAPP_FILESTORAGE.equals(type))
      ser = ser + "+++" + fileLocation;
    return ser;
  }

  public String getName() {
    String name = "";
    if (TYPE_DB.equals(type)) {
      name = originalFileName;
    } else {
      String fileLocNorm = StringUtilities.replaceAllNoRegex(fileLocation,"\\","/");
      name = fileLocNorm.substring(Math.max(0,fileLocNorm.lastIndexOf("/")+1));
    }
    return name;
  }

 /*
  public void makedirs(){
   new File( ApplicationState.getApplicationSetting(SystemConstants.FLD_REPOSITORY_URL) + File.separator + "photos").mkdirs();
  }
  */

  public static PersistentFile deserialize(String serObj) {
    PersistentFile pf = null;
    if (serObj != null) {
      int first_ = serObj.indexOf("_");
      if (first_ > 0) {
        String type = serObj.substring(0, first_);

        int second_ = serObj.indexOf("_", first_ + 1);
        int third = serObj.length();
        if (TYPE_FILESTORAGE.equals(type) || TYPE_WEBAPP_FILESTORAGE.equals(type) || TYPE_FILESTORAGE_ABSOLUTE.equals(type))
          third = serObj.indexOf("+++");
        if (second_ > 0) {
          int UID = Integer.parseInt(serObj.substring(first_ + 1, second_));
          String originalFileName = serObj.substring(second_ + 1,third);
          pf = new PersistentFile(UID, originalFileName);
          pf.type = type;
        }
        if (TYPE_FILESTORAGE.equals(type) || TYPE_WEBAPP_FILESTORAGE.equals(type) || TYPE_FILESTORAGE_ABSOLUTE.equals(type)) {
          String fileLoc = serObj.substring(serObj.indexOf("+++")+3);
          pf.fileLocation = fileLoc;
          File file = new File(fileLoc);
          pf.fileDir = file.getParent();
        }
      }
    }
    return pf;
  }

}
