package org.jblooming.ontology.businessLogic;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.ontology.*;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.hibernate.HibernateUtilities;
import org.jblooming.utilities.ReflectionUtilities;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.view.ClientEntries;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import java.text.ParseException;
import java.util.Iterator;
import java.util.List;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 */
public class LookupAction {
  public void cmdSave(LookupConfiguration lookupConfiguration, PageState pageState) throws ActionException, PersistenceException {

    List<Class> inherited = ReflectionUtilities.getInheritedClasses(lookupConfiguration.lookupClass);
    boolean isLookupString = inherited.contains(LookupStringSupport.class);
    boolean isLookupInt = inherited.contains(LookupIntSupport.class);
    boolean isLookupWithAssignedId = HibernateUtilities.isIdAssigned(lookupConfiguration.lookupClass);

    ClientEntries ces = pageState.getClientEntries();
    if (ces != null && ces.size() > 0) {
      for (Iterator iterator = ces.getEntryKeys().iterator(); iterator.hasNext();) {
        ClientEntry clientEntry = ces.getEntry((String) iterator.next());

        if (clientEntry.name.startsWith("ID")) {
          int id = Integer.parseInt(clientEntry.name.substring("ID".length()));

          LookupSupport l = (LookupSupport) PersistenceHome.findByPrimaryKey(lookupConfiguration.lookupClass, id);
          if (isLookupString) {
            ClientEntry code = ces.getEntry("STRINGVALUE" + id);
            if (code != null && code.stringValue() != null)
              ((LookupString) l).setStringValue(code.stringValue());
          }
          if (isLookupInt) {
            ClientEntry ceValue = ces.getEntry("INTVALUE" + id);
            if (ceValue != null && ceValue.stringValueNullIfEmpty() != null) {
              int value = 0;
              try {
                value = ceValue.intValue();
              } catch (Exception e) {
                throw new ActionException(e);
              }
              ((LookupInt) l).setIntValue(value);
            }
          }
          l.setDescription(clientEntry.stringValueNullIfEmpty());
          /*if (isLookupWithAssignedId) {
            ClientEntry idNew = ces.getEntry("ASSIDNEW" + id);
            if (idNew != null)
              l.setId(idNew.intValue());
          }*/

          l.store();

        } else if (clientEntry.name.startsWith("NEW")) {
          Class aClass = lookupConfiguration.lookupClass;

          try {
            LookupSupport lk = (LookupSupport) aClass.newInstance();
            lk.setDescription(clientEntry.stringValueNullIfEmpty());
            if (isLookupString) {
              ClientEntry code = ces.getEntry("STRINGVALUEN" + clientEntry.name.substring("NEW".length()));
              if (code != null && code.stringValue() != null)
                ((LookupString) lk).setStringValue(code.stringValue());
            }

            if (isLookupWithAssignedId) {
              ClientEntry id = ces.getEntry("I18N_NEW_ENTRY" + clientEntry.name.substring("NEW".length()));
              if (id != null)
                try {
                  lk.setId(id.intValue());
                } catch (ParseException e) {
                  throw new PlatformRuntimeException(e);
                }
            }
            if (isLookupInt) {
              ClientEntry ceValue = ces.getEntry("INTVALUEN" + clientEntry.name.substring("NEW".length()));
              if (ceValue != null && ceValue.stringValueNullIfEmpty() != null) {
                int value = 0;
                try {
                  value = ceValue.intValue();
                } catch (Exception e) {
                  pageState.addClientEntry("FEEDBACK", ceValue.errorCode);
                  throw new ActionException(e);
                }
                ((LookupInt) lk).setIntValue(value);
              }
            }


            lk.store();
          } catch (InstantiationException e) {
            throw new PlatformRuntimeException(e);
          } catch (IllegalAccessException e) {
            throw new PlatformRuntimeException(e);
          }
        }
        //break;
      }
    }
  }


  public void cmdEdit(LookupConfiguration lookupConfiguration, PageState pageState) throws FindByPrimaryKeyException {
    Identifiable lk = PersistenceHome.findByPrimaryKey(lookupConfiguration.lookupClass, pageState.mainObjectId);
    pageState.setMainObject(lk);
  }

  public void cmdDelete(LookupConfiguration lookupConfiguration, PageState pageState) throws PersistenceException {
    LookupSupport course = (LookupSupport) PersistenceHome.findByPrimaryKey(lookupConfiguration.lookupClass, pageState.mainObjectId);
    pageState.setMainObject(course);
    DeleteHelper.cmdDelete(course, pageState);
  }


}
