package org.jblooming.operator;


import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.anagraphicalData.AnagraphicalData;
import org.jblooming.company.Department;
import org.jblooming.company.Location;
import org.jblooming.logging.Auditable;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.*;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.resource.Affiliate;
import org.jblooming.security.*;
import org.jblooming.security.SecurityException;
import org.jblooming.system.SystemConstants;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.HashTable;
import org.jblooming.utilities.JSP;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.constants.SecurityConstants;
import org.jblooming.waf.settings.ApplicationState;

import java.security.NoSuchAlgorithmException;
import java.util.*;

public class Operator extends User implements OperatorAggregator, Auditable {

  private Date lastLoggedOn;
  private AnagraphicalData anagraphicalData;
  private Map<String, String> options = new TreeMap<String, String>();
  public Map<String, String> filters = new HashTable<String, String>();
  private Map<String, String> favoriteUrls = new HashTable();

  private Operator owner;

  private Location location;

  //read only coll
  private Set<Affiliate> affiliations = new HashSet<Affiliate>();

  public static final String OPERATOR = "OP";


  // not persistent properties  BEGIN
  private Date lastRequestOn;
  // not persistent properties  END



  public Operator() {
  }

  public void putFilter(String key, String value) {
    filters.put(key, value);
  }

  public void removeFilterByKey(String key) {
    filters.remove(key);
  }

  public void removeFilterByValue(String value) {
    Iterator keySet = new HashSet(filters.keySet()).iterator();
    while (keySet.hasNext()) {
      final String key = (String) keySet.next();
      String loopValue = getFilter(key);
      if (loopValue.equals(value)) {
        removeFilterByKey(key);
      }
    }
  }

  public String getFilter(String key) {
    return (String) filters.get(key);
  }

  public Iterator getFiltersKeysetIterator() {
    return filters.keySet().iterator();
  }

  public int FilterKeysetSize() {
    return filters.keySet().size();
  }

  public void setAnagraphicalData(AnagraphicalData anagraphicalData) {
    this.anagraphicalData = anagraphicalData;
  }

  public AnagraphicalData getAnagraphicalData() {
    return anagraphicalData;
  }

  public Map<String, String> getOptions() {
    return options;
  }

  public void putOption(String key, String value) {
    getOptions().put(key, value);
  }


  public void removeOptionByKey(String key) {
    getOptions().remove(key);
  }

  public void removeOptionByValue(String value) {
    Iterator keySet = new HashSet(getOptions().keySet()).iterator();
    while (keySet.hasNext()) {
      final String key = (String) keySet.next();
      String loopValue = getOption(key);
      if (loopValue.equals(value)) {
        removeOptionByKey(key);
      }
    }
  }

  public String getOption(String key) {
    return (String) getOptions().get(key);
  }

  public Iterator getOptionsKeysetIterator() {
    return getOptions().keySet().iterator();
  }

  private void setOptions(Map options) {
    this.options = options;
  }

  public void setInherit(boolean inherit) {
  }

  public void setPropagate(boolean propagate) {
  }

  public boolean isPropagate() {
    return false;
  }

  public boolean isInherit() {
    return false;
  }

  public Collection getOperators() {
    Collection coll = new HashSet();
    coll.add(this);
    return coll;
  }

  public boolean isOperatorIn(Operator o) {
    return (this.equals(o));
  }

  /**
   * Is used to get the operator start working time
   *
   * @return The number of millis elapsed from midnight.
   *         If the operator do not have specific setting the general one is returned.
   */
  public int getWorkingTimeStart() {
    try {
      return (int)DateUtilities.millisFromHourMinuteSmart((String) options.get(OperatorConstants.FLD_WORKING_HOUR_BEGIN));
    } catch (Exception e) {
      return (int)DateUtilities.millisFromHourMinuteSmart(ApplicationState.getApplicationSetting(OperatorConstants.FLD_WORKING_HOUR_BEGIN));
    } finally {
      return CompanyCalendar.WORKING_HOUR_BEGIN;
    }
  }

  /**
   * Is used to get the operator end working time
   *
   * @return The number of millis elapsed from midnight.
   *         If the operator do not have specific setting the general one is returned.
   */
  public int getWorkingTimeEnd() {
    try {
      return (int) DateUtilities.millisFromHourMinuteSmart((String) options.get(OperatorConstants.FLD_WORKING_HOUR_END));
    } catch (Exception e) {
      return (int)DateUtilities.millisFromHourMinuteSmart(ApplicationState.getApplicationSetting(OperatorConstants.FLD_WORKING_HOUR_END));
    } finally {
      return CompanyCalendar.WORKING_HOUR_END;
    }
  }

  public int millisInWorkingDay() {
    //return ((Integer.valueOf(SessionState.getOperatorOption(this, OperatorConstants.FLD_WORKING_HOUR_TOTAL))).intValue() * (int) CompanyCalendar.MILLIS_IN_HOUR);
    return (int)DateUtilities.millisFromHourMinuteSmart(SessionState.getOperatorOption(this, OperatorConstants.FLD_WORKING_HOUR_TOTAL));
  }

  public String getShortenedName() {
    return getName().substring(0, 1) + '.' + getSurname();
  }

  public void operatorInitializer() {
    super.userInitializer();   
    HibernateFactory.initialize(getAnagraphicalData());
    HibernateFactory.initialize(favoriteUrls);
    HibernateFactory.initialize(getLastPasswordList());
    HibernateFactory.initialize(getOptions());
    HibernateFactory.initialize(affiliations);
    HibernateFactory.initialize(filters);
    HibernateFactory.initialize(filters);
  }

  public Operator getOwner() {
    return owner;
  }

  public void setOwner(Operator operator) {
    this.owner = operator;
  }

  public Iterator getAffiliationsIterator() {
    return affiliations.iterator();
  }

  public boolean affiliationsContains(Affiliate affiliation) {
    return affiliations.contains(affiliation);
  }

  public int affiliationsSize() {
    return affiliations.size();
  }

  public void removeAffiliateFrommemory(Affiliate affiliation) {
    affiliations.remove(affiliation);
  }

  public void addRoleAndPersist(Role role) throws StoreException {
    //check whether existing
    Iterator<OperatorRole> i = getOperatorRolesIterator();
    boolean exists = false;
    while (i.hasNext()) {
      OperatorRole or = i.next();
      if (role.equals(or.getRole())) {
        exists = true;
        break;
      }
    }
    if (!exists) {
      OperatorRole or = new OperatorRole();
      or.setOperator(this);
      or.setRole(role);
      or.store();
    }
    //getOperatorRoles().add(or); //todo non deve essere fatto. le collezioni inverse sono buone nella prossima request
  }

  public void addGroupAndPersist(Group group) throws StoreException {
    //check whether existing
    Iterator<OperatorGroup> i = getOperatorGroupsIterator();
    boolean exists = false;
    while (i.hasNext()) {
      OperatorGroup og = i.next();
      if (group.equals(og.getGroup())) {
        exists = true;
        break;
      }
    }
    if (!exists) {
      OperatorGroup or = new OperatorGroup();
      or.setOperator(this);
      or.setGroup(group);
      or.store();
    }
    //getOperatorGroups().add(or); //todo non deve essere fatto. le collezioni inverse sono buone nella prossima request
  }

  //todo  verificare caso in cui un permesso sia attribuito da un gruppo o ruolo
  public boolean hasPermissionForDepartment(Permission p, Department dept) {
    boolean perm = false;
//    if (getOwner() != null && getOwner().equals(op))
//      perm= true;
//    else
//      perm= super.hasPermissionFor(p);

    Set<Affiliate> affiliations = this.getAffiliations();
    if (affiliations != null && affiliations.size() > 0) {
      for (Affiliate affiliate : affiliations) {
        Department d = affiliate.getDepartment();
        if (d.equals(dept)) {
          Role role = affiliate.getRole();
          if (role != null && role.hasPermissionFor(p)) {
            //todo inserire anche startingDate?
            Date end = affiliate.getEndingDate();
            if (end != null && end.before(new Date())) {
              perm = true;
              break;
            }
          }
        }
      }
    }
    return perm;
  }

  public boolean hasRoleForDepartment(Role role, Department dept) {
    boolean perm = false;
    Set<Affiliate> affiliations = this.getAffiliations();
    if (affiliations != null && affiliations.size() > 0) {
      for (Affiliate affiliate : affiliations) {
        Department d = affiliate.getDepartment();
        if (d.equals(dept)) {
          Role r = affiliate.getRole();
          if (r != null && r.equals(role)) {
            //todo inserire anche startingDate?
            Date end = affiliate.getEndingDate();
            if (end == null) {
              perm = true;
              break;
            } else {
              if (end.after(new Date())) {
                perm = true;
                break;
              }
            }
          }
        }
      }
    }
    return perm;
  }

  /**
   * if u is owner, is always true, else calls super
   */
  public boolean hasPermissionFor(User u, Permission p) {
    if (getOwner() != null && getOwner().equals(u))
      return true;
    else
      return u.hasPermissionFor(p);
  }

  public void testPermission(User u, Permission p) throws SecurityException {
    if (!hasPermissionFor(u, p))
      throw new SecurityException(SecurityConstants.I18N_PERMISSION_LACKING, p);
  }

  public void removeRoleAndPersist(OperatorRole or) throws RemoveException {
    // getOperatorRoles().remove(or); //todo non deve essere fatto. le collezioni inverse sono buone nella prossima request
    or.remove();
  }

  public void removeRoleAndPersist(Role role) throws RemoveException {
    for (Iterator<OperatorRole> i = getOperatorRolesIterator(); i.hasNext();) {
      OperatorRole or = i.next();
      if (or.getRole().equals(role)) {
        removeRoleAndPersist(or);
        for (Iterator<OperatorRole> it = or.getRole().getOperatorsIterator(); it.hasNext();) {
          OperatorRole operRol = it.next();
          if (operRol.equals(or)) {
            it.remove();
            break;
          }
        }
        break;
      }
    }
  }

  public void removeGroupAndPersist(OperatorGroup or) throws RemoveException {
    or.remove();
    //getOperatorGroups().remove(or);//todo non deve essere fatto. le collezioni inverse sono buone nella prossima request
  }

  public Location getLocation() {
    return location;
  }

  public void setLocation(Location location) {
    this.location = location;
  }

  public Map<String,String> getFilters() {
    return filters;
  }

  private void setFilters(Map filters) {
    this.filters = filters;
  }

  public Map<String, String> getFavoriteUrls() {
    return favoriteUrls;
  }

  public void setFavoriteUrls(Map<String, String> favoriteUrls) {
    this.favoriteUrls = favoriteUrls;
  }

  private Set<Affiliate> getAffiliations() {
    return affiliations;
  }

  private void setAffiliations(Set<Affiliate> affiliations) {
    this.affiliations = affiliations;
  }

  public Date getLastLoggedOn() {
    return lastLoggedOn;
  }

  public void setLastLoggedOn(Date lastLoggedOn) {
    this.lastLoggedOn = lastLoggedOn;
  }

  public String getDisplayName() {
    return JSP.w(getName()) + " " + JSP.w(getSurname());
  }

  public String getDefaultEmail() {
    if (getAnagraphicalData() != null)
      return getAnagraphicalData().getEmail();
    else
      return null;
  }


  /**
   * only place where case insensitiveness always holds
   */
  public static Operator findByLoginName(String name) throws PersistenceException {
    Operator gr = null;

    OqlQuery oql = new OqlQuery("from " + Operator.class.getName() + " as op " +
            "where lower(op.loginName) = lower(:name)");
    oql.getQuery().setString("name", name);
    gr = (Operator) oql.uniqueResult();

    return gr;
  }

  public static Operator createSystemOperator() throws StoreException, ApplicationException {

    Operator operator = new Operator();
    operator.setId(OperatorConstants.SYSTEM_OPERATOR_ID);
    operator.setLoginName(OperatorConstants.SYSTEM_OPERATOR);
    operator.changePassword(OperatorConstants.SYSTEM_OPERATOR, "");
    operator.setAdministrator(true);
    operator.setEnabled(true);
    operator.setSurname(OperatorConstants.SYSTEM_OPERATOR);
    operator.store();
    return operator;
  }

  public static Operator getSystemOperator() {
    Operator op = null;
    try {
      op = (Operator) PersistenceHome.findByPrimaryKey(Operator.class, OperatorConstants.SYSTEM_OPERATOR_ID);
    } catch (Exception e) {
      try {
        op = createSystemOperator();
      } catch (StoreException e1) {
        throw new PlatformRuntimeException(e);
      } catch (ApplicationException ae) {
        throw new PlatformRuntimeException(e);
      }
    }
    return op;
  }

  public static Operator authenticateUser(String password, String username, boolean enabledCookies) throws PersistenceException, ApplicationException, org.jblooming.security.SecurityException {

    Operator user = null;
    try {
      user = findByLoginName(username);
      if (user == null)
        throw new FindException();
      if (!user.isEnabled()) {
        throw new org.jblooming.security.SecurityException(OperatorConstants.ERR_OPERATOR_DISABLED);
      } else if (user.getEnabledOnlyOn() != null && (!user.getEnabledOnlyOn().contains(new Date())))
        throw new SecurityException(OperatorConstants.ERR_OPERATOR_OUTSIDE_TIME_WINDOW);
    } catch (FindByPrimaryKeyException e) {
      throw new SecurityException(OperatorConstants.ERR_INVALID_LOGIN);
    } catch (FindException e) {
      throw new SecurityException(OperatorConstants.ERR_INVALID_LOGIN);
    }

    try {
      boolean pwdNull = user.getPassword() != null;
      boolean allowEmpty = !Fields.FALSE.equals(ApplicationState.getApplicationSetting(SystemConstants.ALLOW_EMPTY_STRING_PSW));
      boolean userPwdEmpty = user.getPassword().trim().equals("") ;
      boolean pwdEmpty = password==null || password.trim().equals("") ;
      boolean md5 =  user.getPassword().equalsIgnoreCase( StringUtilities.md5Encode(user.getLoginName() + password) );

      if (  pwdNull &&
          //authentication methods:
          //allowing empty string psw
           ! ( 
               ( allowEmpty && userPwdEmpty && pwdEmpty ) ||
                   ( md5 )
           )
         )
        throw new SecurityException(OperatorConstants.ERR_INVALID_LOGIN);

    } catch (NoSuchAlgorithmException e) {
      throw new ApplicationException(e);
    }
    return user;
  }

  public Date getLastRequestOn() {
    return lastRequestOn;
  }

  public void setLastRequestOn(Date lastRequestOn) {
    this.lastRequestOn = lastRequestOn;
  }

}