package org.jblooming.operator.businessLogic;

import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.messaging.Listener;
import org.jblooming.messaging.MessagingSystem;
import org.jblooming.operator.Operator;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.view.PageState;


/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class OptionAction {
  public void cmdEdit(PageState pageState) throws PersistenceException {

    make(pageState);
  }

  protected void make(PageState pageState) throws PersistenceException {
    SessionState sessionState = pageState.sessionState;
    pageState.addClientEntry(OperatorConstants.FLD_SELECT_LANG, sessionState.getCurrentLocale().getLanguage());
    //CompanyCalendar cal = new CompanyCalendar();
    String whb = sessionState.getOperatorOption(OperatorConstants.FLD_WORKING_HOUR_BEGIN);
    if (whb != null) {
      pageState.addClientEntry(OperatorConstants.FLD_WORKING_HOUR_BEGIN, whb);
    }

    String whe = sessionState.getOperatorOption(OperatorConstants.FLD_WORKING_HOUR_END);
    if (whe != null) {
      pageState.addClientEntry(OperatorConstants.FLD_WORKING_HOUR_END, whe);
    }

    whe = sessionState.getOperatorOption(OperatorConstants.FLD_WORKING_HOUR_TOTAL);
    if (whe != null) {
      pageState.addClientEntry(OperatorConstants.FLD_WORKING_HOUR_TOTAL, whe);
    }

    pageState.addClientEntry(OperatorConstants.OP_PAGE_SIZE, sessionState.getOperatorOption(OperatorConstants.OP_PAGE_SIZE));
    pageState.addClientEntry(OperatorConstants.FLD_CURRENT_SKIN, sessionState.getSkin().name);
    pageState.addClientEntry(OperatorConstants.INHIBIT_TOOLTIPS, sessionState.getOperatorOption(OperatorConstants.INHIBIT_TOOLTIPS));
    String operatorOption = sessionState.getOperatorOption(OperatorConstants.SEND_EVENT_BY_ICAL);
    pageState.addClientEntry(OperatorConstants.SEND_EVENT_BY_ICAL, operatorOption);
    MessagingSystem.makeMedias(sessionState.getOperatorOption(OperatorConstants.MEDIA_PREFERRED_CHANNEL),OperatorConstants.MEDIA_PREFERRED_CHANNEL+"_",pageState);

  }

  public void cmdSave(PageState pageState, String contextPath) throws PersistenceException {

    SessionState sessionState = pageState.sessionState;
    Operator logged = sessionState.getLoggedOperator();
    logged = (Operator) PersistenceHome.findByPrimaryKey(Operator.class, logged.getId());
    pageState.mainObject = logged;

    try {
      logged.putOption(OperatorConstants.FLD_SELECT_LANG, pageState.getEntry(OperatorConstants.FLD_SELECT_LANG).stringValue());
    } catch (ActionException e) {
    }
    try {
      logged.putOption(OperatorConstants.OP_PAGE_SIZE, pageState.getEntry(OperatorConstants.OP_PAGE_SIZE).stringValue());
    } catch (ActionException e) {
    }

    logged.putOption(OperatorConstants.INHIBIT_TOOLTIPS, pageState.getEntry(OperatorConstants.INHIBIT_TOOLTIPS).checkFieldHtmlValue());
    logged.putOption(OperatorConstants.SEND_EVENT_BY_ICAL, pageState.getEntry(OperatorConstants.SEND_EVENT_BY_ICAL).checkFieldHtmlValue());

    String value = pageState.getEntry(OperatorConstants.FLD_WORKING_HOUR_BEGIN).stringValueNullIfEmpty();
    if (value != null) {
      logged.putOption(OperatorConstants.FLD_WORKING_HOUR_BEGIN, value);
    }

    value = pageState.getEntry(OperatorConstants.FLD_WORKING_HOUR_END).stringValueNullIfEmpty();
    if (value != null) {
      logged.putOption(OperatorConstants.FLD_WORKING_HOUR_END, value);
    }

    value = pageState.getEntry(OperatorConstants.FLD_WORKING_HOUR_TOTAL).stringValueNullIfEmpty();
    if (value != null) {
      logged.putOption(OperatorConstants.FLD_WORKING_HOUR_TOTAL, value);
    }    

    try {
      String newSkin = pageState.getEntry(OperatorConstants.FLD_CURRENT_SKIN).stringValue();
      logged.putOption(OperatorConstants.FLD_CURRENT_SKIN, newSkin);
      sessionState.setSkin(SessionState.createSkin(contextPath, newSkin, pageState.getApplication().getRootFolder()));
    } catch (ActionException e) {
    }

    String mediaSubscribed = MessagingSystem.mediaSubscribed(OperatorConstants.MEDIA_PREFERRED_CHANNEL+"_", pageState);
    logged.putOption(OperatorConstants.MEDIA_PREFERRED_CHANNEL, mediaSubscribed);

    logged.store();
    try {
      sessionState.refreshLoggedOperator();
    } catch (ApplicationException e) {
      throw new PlatformRuntimeException(e);
    }
  }

  public static void cmdUpdateLoggedOption(PageState pageState, String optionName, String optionValue) throws PersistenceException {
    SessionState sessionState = pageState.sessionState;
    Operator logged = sessionState.getLoggedOperator();
    if (logged != null) {
      logged = (Operator) PersistenceHome.findByPrimaryKey(Operator.class, logged.getId());
      logged.putOption(optionName, optionValue);
      logged.store();

      try {
        sessionState.refreshLoggedOperator();
      } catch (ApplicationException e) {
        throw new PlatformRuntimeException(e);
      }
    }
  }

  public void cmdDelete(PageState pageState) throws PersistenceException {
    String id = pageState.getMainObjectId().toString();
    Listener l = (Listener) PersistenceHome.findByPrimaryKey(Listener.class, id);
    if (l != null)
      PersistenceHome.remove(l);
    make(pageState);
  }

  public void cmdRemoveOption(PageState pageState) throws PersistenceException {
    String key = (String) pageState.mainObjectId;
    SessionState sessionState = pageState.sessionState;
     Operator logged = sessionState.getLoggedOperator();
    if (logged != null) {
      logged = (Operator) PersistenceHome.findByPrimaryKey(Operator.class, logged.getId());
      logged.removeOptionByKey(key);
      logged.store();

      try {
        sessionState.refreshLoggedOperator();
      } catch (ApplicationException e) {
        throw new PlatformRuntimeException(e);
      }
    }

  }
}
