package org.jblooming.persistence;

import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.jblooming.ontology.Identifiable;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.exceptions.RemoveException;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.PersistenceContext;

import java.io.Serializable;

/**
 * Handles the persistence of all identifiable objects
 */
public class PersistenceHome {

  public static final NewEmptyId NEW_EMPTY_ID = new NewEmptyId();

  /**
   * @param clazz
   * @param id
   * @return
   * @throws FindByPrimaryKeyException if not found
   */
  public static Identifiable findByPrimaryKey(Class<? extends Identifiable> clazz, int id) throws FindByPrimaryKeyException {
    return HibernateFactory.findByPrimaryKey(clazz, id);
  }

  public static Identifiable findByPrimaryKey(Class<? extends Identifiable> clazz, Serializable id) throws FindByPrimaryKeyException {
    return HibernateFactory.findByPrimaryKey(clazz, id);
  }


  public static Identifiable findUnique(Class<? extends Identifiable> clazz, String field, Object value) throws PersistenceException {
    final OqlQuery oqlQuery = new OqlQuery("from " + clazz.getName() + " as obj where obj." + field + " = :aparam");
    try {
      return (Identifiable) oqlQuery.getQuery().setParameter("aparam", value).uniqueResult();
    } catch (HibernateException e) {
      throw new PersistenceException(oqlQuery.doDebug(new Object[]{value}),e);
    }
  }

  public static Identifiable findUnique(Class<? extends Identifiable> clazz, String field, Object value, PersistenceContext pc) throws PersistenceException {
    final OqlQuery oqlQuery = new OqlQuery("from " + clazz.getName() + " as obj where obj." + field + " = :aparam", pc);
    try {
      return (Identifiable) oqlQuery.getQuery().setParameter("aparam", value).uniqueResult();
    } catch (HibernateException e) {
      throw new PersistenceException(oqlQuery.doDebug(new Object[]{value}),e);
    }
  }

  public static Object findUniqueObject(Class clazz, String field, Object value, PersistenceContext pc) throws PersistenceException {
    final OqlQuery oqlQuery = new OqlQuery("from " + clazz.getName() + " as obj where obj." + field + " = :aparam", pc);
    try {
      return oqlQuery.getQuery().setParameter("aparam", value).uniqueResult();
    } catch (HibernateException e) {
      throw new PersistenceException(oqlQuery.doDebug(new Object[]{value}),e);
    }
  }


  public static Identifiable findUniqueNullIfEmpty(Class<? extends Identifiable> clazz, String field, Object value) {
    Identifiable res = null;
    try {
      res = findUnique(clazz, field, value);
    } catch (PersistenceException e) {
    }
    return res;
  }

  public static Identifiable findUniqueNullIfEmpty(Class<? extends Identifiable> clazz, String field, Object value, PersistenceContext pc) {
    Identifiable res = null;
    try {
      res = findUnique(clazz, field, value, pc);
    } catch (PersistenceException e) {
    }
    return res;
  }

  public static Identifiable findByPrimaryKey(Class<? extends Identifiable> clazz, Serializable id, PersistenceContext pc) throws FindByPrimaryKeyException {
    return HibernateFactory.findByPrimaryKey(clazz, id, pc);
  }

  /**
   * @param i
   * @throws StoreException if anything goes wrong, say the database is unreachable, the exception is wrapped in this
   * @deprecated use i.store()
   */
  public static void store(Identifiable i) throws StoreException {
    HibernateFactory.store(i);
  }

  public static void store(Identifiable i, PersistenceContext pc) throws StoreException {
    HibernateFactory.store(i, pc);
  }

  /**
   * This methods updates the persisted copy of i with the current i's properties.
   * The usage is correct only if i is already a persisted object, say in a previous transaction.
   * If i is only lazily
   *
   * @param i
   * @throws PersistenceException
   */
  public static void update(Identifiable i) throws PersistenceException {
    HibernateFactory.update(i);
  }

  public static void refresh(Identifiable i) {
    HibernateFactory.getSession().refresh(i);
  }

  public static void reAssociateUnmodified(Identifiable i) throws PersistenceException {
    HibernateFactory.lock(i, LockMode.NONE);
  }

  /**
   * @deprecated use i.remove()
   */
  public static void remove(Identifiable i) throws RemoveException {
    HibernateFactory.remove(i);
  }

  public static void remove(Identifiable i, PersistenceContext pc) throws RemoveException {
    HibernateFactory.remove(i, pc.session);
  }

  /**
   * deprecated use Hibernate.getClass on the object
   */
  public static String deProxy(String className) {

    String result = className;
    int cglibbed = className.indexOf("$$");
    if (cglibbed > -1)
      result = className.substring(0, cglibbed);
    return result;

  }

  public static boolean isInitialized(Object o) {
    return HibernateFactory.isInitialized(o);
  }

  public static void initialize(Object o) throws HibernateException {
    HibernateFactory.initialize(o);
  }

  public static String dePackage(String className) {
    return PersistenceHome.deProxy(className).substring(PersistenceHome.deProxy(className).lastIndexOf('.') + 1);
  }


  public static class NewEmptyId implements Serializable {

    static final String newEmptyId = "newEmptyId";

    public boolean equals(Object o) {
      return newEmptyId.equals(o + "");
    }

    public String toString() {
      return newEmptyId;
    }

  }

}
