package org.jblooming.persistence.hibernate;

import org.hibernate.type.IntegerType;
import org.hibernate.type.Type;
import org.jblooming.cursor.CursorServer;
import org.jblooming.cursor.exceptions.CursorException;
import org.jblooming.ontology.Identifiable;
import org.jblooming.tracer.Tracer;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * @author Andrea Costantinis (costantinis@remsoftware.it) modified by Pietro Polsinelli ppolsinelli@open-lab.com
 */
public class HibernateCursorServer implements CursorServer {
  /**
   * WARNING!
   * This class instances are referenced by CursorHelper instances;
   * these, in turns, are stored in HttpSession;
   * to be able to run this system  in clustered environments we do not reference the actual class but
   * its FQN only; this way if the session is serialized to another node
   * of the cluster we avoid serialization of the actual class (which might be not serializable) .
   */
  protected String theClassFQN;
  protected String idAttributeName = "id";

  private static final boolean debug = false;

  public HibernateCursorServer(Class cls) {
    this.theClassFQN = cls.getName();
  }

  public HibernateCursorServer(Class cls, String idAttributeName) {
    this(cls);
    this.idAttributeName = idAttributeName;
  }

  public List loadPage(List objectReferencesList, int fromIndex, int toIndex) throws CursorException {

    List orderedResult = new ArrayList();

    List chunk = objectReferencesList.subList(fromIndex, toIndex);
    int chunkSize = chunk.size();

    long timeToLoadPage = 0;
    if (debug) {
      timeToLoadPage = System.currentTimeMillis();
      Tracer.addTrace("HibernateCursorServer. Chunk start: " + fromIndex + " Chunk end: " + toIndex);
      Tracer.addTrace("HibernateCursorServer. Chunk size: " + chunkSize + " Extent size: " + objectReferencesList.size());
    }

    if (chunkSize <= 0)
      return Collections.EMPTY_LIST;

    Object params[] = new Object[chunkSize];
    Type types[] = new Type[chunkSize];
    List result = null;

    try {
      String queryString = "from tob in class " + theClassFQN + " where tob." + idAttributeName + " in (";
      for (int i = 0; i < chunkSize; ++i) {
        Object o = chunk.get(i);
        if (o instanceof Object[])
          params[i] = ((Object[]) o)[0];
        else
          params[i] = o;

        types[i] = new IntegerType();
        queryString += "?";
        if (i < (chunkSize - 1)) queryString += ",";
      }
      queryString += ")";

      result = HibernateFactory.bulkLoad(queryString, params, types);

      for (Iterator iterator = objectReferencesList.iterator(); iterator.hasNext();) {
        Object obj = iterator.next();
        Integer s = null;
        if (obj instanceof Object[])
          s = (Integer) ((Object[]) obj)[0];
        else
          s = (Integer) obj;

        for (Iterator iterator2 = result.iterator(); iterator2.hasNext();) {
          Identifiable o = (Identifiable) iterator2.next();
          if (o.getIntId() == s.intValue()) {
            orderedResult.add(o);
            break;
          }
        }
      }


    } catch (Throwable t) {
      throw new CursorException(t);
    }

    if (debug) {
      timeToLoadPage = System.currentTimeMillis() - timeToLoadPage;
      Tracer.addTrace("HibernateCursorServer. time load page (secs): " + timeToLoadPage / 1000.0);
    }

    return orderedResult;
  }


}
