package org.jblooming.persistence.hibernate;

import org.hibernate.mapping.Column;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.scheduler.Executable;
import org.jblooming.scheduler.JobLogData;
import org.jblooming.tracer.Tracer;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Open Lab
 * info@open-lab.com
 * Date: Sep 7, 2006
 * Time: 3:57:17 PM
 */
public class HibernatePropertySchemaUpdater {

  public boolean updateNeeded = false;

  List<PropertyWithDefault> pwds = new ArrayList();

  public PropertyWithDefault addPropertyToCheck(Class clazz, String propertyName, Object defaultValue) {
    PropertyWithDefault pwd = new PropertyWithDefault();
    pwd.clazz = clazz;
    pwd.propertyName = propertyName;
    pwd.defaultValue = defaultValue;
    pwds.add(pwd);
    return pwd;
  }


  public void verifyUpdatesNeeded() {

    Connection conn = null;
    try {
      conn = PersistenceContext.getNewConnection();
      conn.setAutoCommit(false);
      ResultSet ps = null;
      for (PropertyWithDefault propertyWithDefault : pwds) {
        try {

          String tableName = HibernateUtilities.getTableName(propertyWithDefault.clazz);
          propertyWithDefault.table = tableName;

          //get column name
          Column c = HibernateUtilities.getColumn(propertyWithDefault.clazz, propertyWithDefault.propertyName);
          if (c != null) {
            propertyWithDefault.column = c.getName();
            String sql = "select * from " + tableName;

            ps = conn.createStatement().executeQuery(sql);
            ResultSetMetaData metaData = ps.getMetaData();
            int cc = metaData.getColumnCount();
            boolean existsColumn = false;
            for (int i = 1; i <= cc; i++) {
              String columnName = metaData.getColumnName(i);
              if (c.getName().equalsIgnoreCase(columnName)) {
                existsColumn = true;
                break;
              }
            }
            propertyWithDefault.needsToBeUpdated = !existsColumn;
            if (!updateNeeded)
              updateNeeded = !existsColumn;
            ps.close();
            conn.commit();
          }
        } catch (Exception e) {

          if (ps != null)
            try {
              ps.close();
            } catch (SQLException e1) {
            }

          //may fail as schema does not exist
          //throw new PlatformRuntimeException(e);
          Tracer.platformLogger.error(e);
        }
      }

      conn.close();

    } catch (Exception e) {

      try {
        if (conn != null && !conn.isClosed())
          conn.close();
      } catch (SQLException e1) {
      }
      Tracer.platformLogger.error(e);
    }

  }

  public void schemaRefinementBeforeHibernateFactory() {
    Connection conn = null;
    try {

      conn = PersistenceContext.getNewConnection();
      conn.setAutoCommit(false);

      for (PropertyWithDefault propertyWithDefault : pwds) {

        if (propertyWithDefault.needsToBeUpdated) {

          for (String sql : propertyWithDefault.beforeHibSql) {
            PreparedStatement ps = null;
            try {
              ps = conn.prepareStatement(sql);
              ps.execute();
              ps.close();
            } catch (SQLException e) {
              if (ps != null)
                try {
                  ps.close();
                } catch (SQLException e1) {
                }
              Tracer.platformLogger.error(e);
            }
          }

        }
      }

      conn.commit();
      conn.close();

    } catch (Exception e) {

      try {
        if (conn != null && !conn.isClosed())
          conn.close();
      } catch (SQLException e1) {
      }

      throw new PlatformRuntimeException(e);
    }
  }

  public void schemaRefinementAfterHibernateFactory() {
    Connection conn = null;
    try {


      for (PropertyWithDefault propertyWithDefault : pwds) {

        conn = PersistenceContext.getNewConnection();
        conn.setAutoCommit(false);

        if (propertyWithDefault.needsToBeUpdated) {

          for (String sql : propertyWithDefault.preSql) {
            PreparedStatement ps = null;
            try {
              ps = conn.prepareStatement(sql);
              ps.execute();
              ps.close();
            } catch (SQLException e) {
              if (ps != null)
                try {
                  ps.close();
                } catch (Exception f) {
                  Tracer.platformLogger.error(f);
                }
              Tracer.platformLogger.error(e);
            }
          }

          {
            String sql = "update " + propertyWithDefault.table + " set " + propertyWithDefault.column + " = ?";
            PreparedStatement ps = conn.prepareStatement(sql);
            ps.setObject(1, propertyWithDefault.defaultValue);
            ps.execute();
            ps.close();

          }

          conn.commit();
          conn.close();

          for (Executable exe : propertyWithDefault.execs) {

            try {
              JobLogData log = new JobLogData();
              exe.run(log);
              if (log.notes != null && log.notes.length() > 0)
                Tracer.platformLogger.info(log.notes);
            } catch (Exception e) {
              Tracer.platformLogger.error(e);
            }
          }

          conn = PersistenceContext.getNewConnection();
          conn.setAutoCommit(false);

          for (String sql : propertyWithDefault.postSql) {
            try {
              PreparedStatement ps = conn.prepareStatement(sql);
              ps.execute();
              ps.close();
            } catch (SQLException e) {
              Tracer.platformLogger.error(e);
            }
          }

          conn.commit();
          conn.close();
        }
      }


    } catch (Exception e) {

      try {
        if (conn != null && !conn.isClosed())
          conn.close();
      } catch (SQLException e1) {
      }
      throw new PlatformRuntimeException(e);
    }
  }


  public class PropertyWithDefault {

    public Class clazz;
    public String propertyName;
    public Object defaultValue;

    public List<String> beforeHibSql = new ArrayList();

    public List<String> preSql = new ArrayList();
    public List<String> postSql = new ArrayList();
    public List<Executable> execs = new ArrayList();

    //computed:
    public boolean needsToBeUpdated = false;
    public String table;
    public String column;


  }

  public PropertyWithDefault getProperty(Class clazz, String propertyName) {
    for (PropertyWithDefault propertyWithDefault : pwds) {
      if (propertyName.equalsIgnoreCase(propertyWithDefault.propertyName) && clazz.equals(propertyWithDefault.clazz))
        return propertyWithDefault;
    }
    return null;
  }

}
