package org.jblooming.persistence.hibernate;


import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.waf.settings.ApplicationState;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 */
public class PersistenceContext {
  public Session session;
  public Transaction tx;
  public boolean committedOrRollbacked = false;

  public static PersistenceContext getNewPersistenceContext(Session hibSession) throws PersistenceException {
    PersistenceContext pc = new PersistenceContext();
    try {
      pc.session = hibSession;
      pc.tx = pc.session.beginTransaction();

    } catch (HibernateException e) {
      throw new PersistenceException(e);
    }
    return pc;
  }

  public static Connection getNewConnection() throws ClassNotFoundException, SQLException {
    Class.forName(ApplicationState.platformConfiguration.driver_class);
    return DriverManager.getConnection(ApplicationState.platformConfiguration.driver_url, ApplicationState.platformConfiguration.db_user_name, ApplicationState.platformConfiguration.db_user_psw);
  }


  public void commitAndClose() throws PersistenceException {
    try {
      tx.commit();
      session.close();
      committedOrRollbacked = true;
    } catch (Exception e) {
      if (tx != null)
        try {
          tx.rollback();
        } catch (HibernateException e1) {
          throw new PersistenceException(e1);
        }
      if (session != null && session.isOpen())
        try {
          session.close();
        } catch (HibernateException e1) {
          throw new PersistenceException(e);
        }
      throw new PersistenceException(e);
    }

  }


  public void rollback() {

    if (tx != null)
      tx.rollback();
    if (session != null && session.isOpen())
      try {
        session.close();
      } catch (HibernateException e1) {
      }
    committedOrRollbacked = true;
  }
}