package org.jblooming.persistence.transfer.db;

import org.jblooming.ApplicationException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.utilities.DbUtilities;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

public abstract class DbTransfererAccess implements DbTransferer {
  protected Properties properties;

  protected abstract void send(Connection connection) throws SQLException, PersistenceException, ApplicationException;

  /**
   * Must be called after initializing properties. The following keys are required:
   * - declared in DbHome
   * dbdriverClassName, dbUser, dbPwd, dbUrl
   */
  public final void send() throws PersistenceException {
    Connection connection = DbUtilities.getConnection(properties);
    try {
      send(connection);
      connection.commit();
      connection.close();
    } catch (Exception e) {
      try {
        connection.rollback();
        connection.close();
      } catch (Exception re) {
        throw new PersistenceException(re);
      }
      throw new PersistenceException(e);
    }
  }

  protected abstract void receive(Connection connection) throws SQLException, PersistenceException;

  /**
   * Must be called after initializing properties. The following keys are required:
   * - declared in DbHome
   * dbdriverClassName, dbUser, dbPwd, dbUrl
   */
  public final void receive() throws PersistenceException {
    try {
      Connection connection = DbUtilities.getConnection(properties);
      receive(connection);
      connection.close();
    } catch (SQLException sqle) {
      throw new PersistenceException(sqle);
    }
  }
}
