package org.jblooming.remoteFile.businessLogic;

import org.jblooming.ApplicationException;
import org.jblooming.messaging.Listener;
import org.jblooming.ontology.PerformantNodeSupport;
import org.jblooming.ontology.PersistentFile;
import org.jblooming.ontology.VersionHome;
import org.jblooming.ontology.businessLogic.DeleteHelper;
import org.jblooming.operator.Operator;
import org.jblooming.oql.QueryHelper;
import org.jblooming.page.HibernatePage;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.remoteFile.BasicDocument;
import org.jblooming.security.Area;
import org.jblooming.security.PlatformPermissions;
import org.jblooming.system.SystemConstants;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.html.display.Paginator;
import org.jblooming.waf.html.input.Uploader;
import org.jblooming.waf.html.table.ListHeader;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.state.PersistentSearch;
import org.jblooming.waf.view.PageState;

import java.io.File;
import java.text.ParseException;
import java.util.List;

public class DocumentAction {

  public boolean passedSecurity = false;

  public void cmdAdd(PageState pageState) throws FindByPrimaryKeyException {

    BasicDocument mainObject = new BasicDocument();
    mainObject.setIdAsNew();
    mainObject.setVersion(VersionHome.VERSION_ROOT);
    pageState.addClientEntry("DOCUMENT_VERSION", mainObject.getVersion());
    pageState.setMainObject(mainObject);

  }

  public void cmdAddVersion(PageState pageState) throws FindByPrimaryKeyException {
    BasicDocument oldVersion = (BasicDocument) PersistenceHome.findByPrimaryKey(BasicDocument.class, pageState.getMainObjectId());
    pageState.addClientEntry("DOC_ROOT_ID",pageState.mainObjectId);
    BasicDocument mainObject = new BasicDocument();
    mainObject.setIdAsNew();

    mainObject.setVersion(oldVersion.nextVersion());
    pageState.addClientEntry("DOCUMENT_VERSION", mainObject.getVersion());

    mainObject.setType(oldVersion.getType());
    pageState.addClientEntry("DOCUMENT_TYPE", mainObject.getType());

    pageState.setMainObject(mainObject);
  }

  public void cmdEdit(PageState pageState) throws PersistenceException {

    BasicDocument document = (BasicDocument) PersistenceHome.findByPrimaryKey(BasicDocument.class, pageState.getMainObjectId());
    pageState.setMainObject(document);
    make(document, pageState);
  }

  public void cmdChangeState(PageState pageState) throws FindByPrimaryKeyException {
    BasicDocument document = (BasicDocument) PersistenceHome.findByPrimaryKey(BasicDocument.class, pageState.getMainObjectId());
    pageState.setMainObject(document);
  }

  public void cmdSave(PageState pageState) throws PersistenceException, ActionException, ApplicationException {
    BasicDocument document;
    if (PersistenceHome.NEW_EMPTY_ID.equals(pageState.mainObjectId)) {
      document = new BasicDocument();
      document.setIdAsNew();
    } else {
      document = (BasicDocument) PersistenceHome.findByPrimaryKey(BasicDocument.class, pageState.getMainObjectId());
    }
    pageState.setMainObject(document);
    document.setCode(pageState.getEntry("DOCUMENT_CODE").stringValueNullIfEmpty());
    try {
      String name = pageState.getEntryAndSetRequired("DOCUMENT_NAME").stringValue();
      document.setName(name);
    } catch (ActionException e) {
      e.printStackTrace();
    }
    try {
      document.setAuthored(pageState.getEntry("DOCUMENT_AUTHORED").dateValue());
    } catch (ParseException e) {
      e.printStackTrace();
    }
    document.setAuthor(pageState.getEntry("DOCUMENT_AUTHOR").stringValue());
    String idArea = pageState.getEntry("DOCUMENT_AREA").stringValueNullIfEmpty();
    if (idArea != null) {
      Area area = (Area) PersistenceHome.findByPrimaryKey(Area.class, idArea);
      document.setArea(area);
    }
    document.setKeywords(pageState.getEntry("DOCUMENT_KEYWORDS").stringValue());
    document.setVersion(pageState.getEntry("DOCUMENT_VERSION").stringValue());
    document.setVersionLabel(pageState.getEntry("DOCUMENT_VERSION_LABEL").stringValue());
    int typeId = 0;
    try {
      typeId = pageState.getEntry("DOCUMENT_TYPE").intValue();
    } catch (ParseException e) {
      e.printStackTrace();
    }
    document.setType(typeId);
    String summa = pageState.getEntry("SUMMA").stringValueNullIfEmpty();
    if (document.getType() == BasicDocument.IS_CONTENT) {
      document.setContent(summa);
    } else {
      document.setSumma(summa);
      if (document.getType() == BasicDocument.IS_UPLOAD) {
        PersistentFile persistentFile = document.getFile();
        if (persistentFile == null) {
          persistentFile = new PersistentFile(0, null, PersistentFile.TYPE_FILESTORAGE);
          persistentFile.fileDir = File.separator + "documents";
          new File(ApplicationState.getApplicationSetting(SystemConstants.FLD_REPOSITORY_URL) + persistentFile.fileDir).mkdirs();
        }
        document.setFile(Uploader.save(document, persistentFile, "DOCUMENT_UPLOAD", pageState));
      } else if (document.getType() == BasicDocument.IS_URL) {
        String value = pageState.getEntry("DOCUMENT_URL_TO_CONTENT").stringValueNullIfEmpty();
        document.setContent(value);
      }
    }
//todo
//    boolean isLocked = pageState.getEntry("IS_LOCKED").checkFieldValue();
//    if (isLocked && document.getLockedBy() == null)
//      document.setLockedBy(Person.getLoggedPerson(pageState.getSessionState()));
//    else
//      document.setLockedBy(null);

    if (pageState.validEntries()) {

      if (PersistenceHome.NEW_EMPTY_ID.equals(pageState.mainObjectId)) {
        document.setOwner(pageState.sessionState.getLoggedOperator());
      }

      document.store();

      if (pageState.getEntry("DOC_ROOT_ID").stringValueNullIfEmpty()!=null) {
        BasicDocument oldRoot = (BasicDocument) PersistenceHome.findByPrimaryKey(BasicDocument.class,pageState.getEntry("DOC_ROOT_ID").stringValue());
        oldRoot.setParentAndStore(document);

        //set references
        //todo oldRoot.addNewVersionToReferralAndStore(document);


      }

      document.store();
    }
  }


  public void cmdDelete(PageState pageState) throws org.jblooming.security.SecurityException, PersistenceException {
    SessionState sm = pageState.getSessionState();
    Operator user = sm.getLoggedOperator();
    BasicDocument delenda = (BasicDocument) PersistenceHome.findByPrimaryKey(BasicDocument.class, pageState.getMainObjectId());
    delenda.testPermission(user,PlatformPermissions.document_canWrite);
    DeleteHelper.cmdDelete(delenda, pageState);
  }


  public void cmdFind(PageState pageState) throws ActionException, PersistenceException {

    Operator logged = (Operator) pageState.getSessionState().getLoggedOperator();

    String hql = "select distinct document from " + BasicDocument.class.getName() + " as document";

    QueryHelper qhelp = new QueryHelper(hql);
    qhelp.addQBEClause("document.type", "excludeFileStorage", "!" + BasicDocument.IS_FILE_STORAGE + "", QueryHelper.TYPE_NUM);
    boolean recoveredFromSavedFilter = PersistentSearch.feedFromSavedSearch(pageState, logged);
    boolean somethingSearched = recoveredFromSavedFilter;

    String filter = pageState.getEntry("ID_CODE").stringValueNullIfEmpty();
    if (filter != null) {
      qhelp.addQueryClause(qhelp.getQbeClause("document.id", "idx", filter, QueryHelper.TYPE_CHAR)
        + " OR " +
        qhelp.getQbeClause("document.code", "code", filter, QueryHelper.TYPE_CHAR));
      somethingSearched = true;
    }

    filter = pageState.getEntry("NAME_DESCRIPTION").stringValueNullIfEmpty();
    if (filter != null) {
      qhelp.addQueryClause(qhelp.getQbeClause("document.name", "name", filter, QueryHelper.TYPE_CHAR)
        + " OR " +
        qhelp.getQbeClause("document.summa", "summa", filter, QueryHelper.TYPE_CHAR));

      somethingSearched = true;
    }

    filter = pageState.getEntry("DOCUMENT_KEYWORDS").stringValueNullIfEmpty();
    if (filter != null) {
      qhelp.addQBEClause("document.keywords", "keywords", filter, QueryHelper.TYPE_CHAR);
      somethingSearched = true;
    }

    String value = pageState.getEntry("TYPE").stringValueNullIfEmpty();
    if (value != null) {
      qhelp.addQBEClause("document.type", "intValue", value, QueryHelper.TYPE_NUM);
      somethingSearched = true;
    }
    PersistentSearch.saveSearch(BasicDocument.DOCUMENT, recoveredFromSavedFilter, pageState, logged);

    if (!somethingSearched && Commands.FIND.equals(pageState.getCommand())) {
      qhelp.addQBEClause("document.name", "document", "*", QueryHelper.TYPE_CHAR);
      somethingSearched = true;
    }

    if (!logged.hasPermissionAsAdmin()) {
/*
________________________________________________________________________________________________________________________________________________________________________


  begin security

________________________________________________________________________________________________________________________________________________________________________

*/

    //take care that this alias is used also out of the method e.g. in search
    qhelp.addJoinAlias(" left outer join task.assignments as assignment");


    qhelp.addJoinAlias(" left outer join document.task  as task");
    qhelp.addJoinAlias(" left outer join document.resource  as resource");
    //qhelp.addJoinAlias(" left outer join document.resource  as resource");

    // TASK CLAUSES

    //open unique clause and set owner; qhelp puts the "and" before; all following clauses are in or
    qhelp.addOQLClause("( ( task.owner = :logged", "logged", logged);

/*//todo
    //areas
    Set<Area> areas = logged.getAreasForPermission(PlatformPermissions.document_canRead);
    if (areas.size() > 0) {
      qhelp.addOrQueryClause("task.area in (:areas) or task.area is null");
      qhelp.addParameter("areas", areas);
    }
*/

 /*   //assignments
    Person myPerson = logged.getMyPersonFromPersistence();
    if (myPerson != null) {

      OqlQuery oqlQuery = new OqlQuery(" from " + Role.class.getName() + " as role where role.permissionIds like :docRead");
      oqlQuery.getQuery().setString("docRead", "%" + PlatformPermissions.document_canRead.toString() + "%");
      List<Role> roles = oqlQuery.list();
      if (roles.size() > 0) {
        qhelp.addOrQueryClause("assignment.role in (:assigRoles) and assignment.resource = :myself");
        qhelp.addParameter("myself", myPerson);
        qhelp.addParameter("assigRoles", roles);
      }
    }*/

    //in order to keep all security conditions in a unique and clause
    qhelp.addToHqlString(" ) or task is null ");

     // RESOURCE CLAUSES
    //open unique clause and set owner; qhelp puts the "and" before; all following clauses are in or
    qhelp.addOQLClause("( resource.owner = :logged", "logged", logged);

/*//todo
    //areas
    if (areas.size() > 0) {
      qhelp.addOrQueryClause("resource.area in (:areas) or resource.area is null");
    }
*/

    //in order to keep all security conditions in a unique and clause
    qhelp.addToHqlString(")");

    //end security big clause
    qhelp.addToHqlString(")");

/*
________________________________________________________________________________________________________________________________________________________________________


  end security

________________________________________________________________________________________________________________________________________________________________________

*/
    }

    if (somethingSearched) {

      ListHeader.orderAction(qhelp, "DOCUMENTLH", pageState, "document.name");

      pageState.setPage(HibernatePage.getHibernatePageInstance(qhelp.toHql().getQuery(),
              Paginator.getWantedPageNumber(pageState),
              Paginator.getWantedPageSize("DOCUMENTLH", pageState)));
    }

  }



  public void make(BasicDocument document, PageState pageState) throws PersistenceException {
    pageState.addClientEntry("DOCUMENT_CODE", document.getCode());
    pageState.addClientEntry("DOCUMENT_NAME", document.getName());
    pageState.addClientEntry("DOCUMENT_AUTHORED", DateUtilities.dateToString(document.getAuthored(), pageState.getSessionState()));
    pageState.addClientEntry("DOCUMENT_AUTHOR", document.getAuthor());
    pageState.addClientEntry("DOCUMENT_AREA", (document.getArea() != null ? document.getArea().getId() : ""));
    pageState.addClientEntry("DOCUMENT_KEYWORDS", document.getKeywords());
    pageState.addClientEntry("DOCUMENT_VERSION", document.getVersion());
    pageState.addClientEntry("DOCUMENT_VERSION_LABEL", document.getVersionLabel());
    pageState.addClientEntry("DOCUMENT_TYPE", document.getType());
//   todo  if (document.getLockedBy() != null)
//      pageState.addClientEntry("IS_LOCKED", Fields.TRUE);
    if (document.getType() == BasicDocument.IS_UPLOAD && document.getFile() != null) {
      pageState.addClientEntry("DOCUMENT_UPLOAD", document.getFile().serialize());
    } else if (document.getType() == BasicDocument.IS_URL) {
      pageState.addClientEntry("DOCUMENT_URL_TO_CONTENT", document.getContent());
    } else if (document.getType() == BasicDocument.IS_CONTENT) {
      pageState.addClientEntry("SUMMA", document.getContent());
    } else if (document.getType() == BasicDocument.IS_FILE_STORAGE) {
      pageState.addClientEntry("DOCUMENT_URL_TO_CONTENT", document.getContent());
      pageState.addClientEntry("connectionType", document.getConnType());
      pageState.addClientEntry("connectionHost", document.getConnectionHost());
      pageState.addClientEntry("connectionUser", document.getConnectionUser());
      pageState.addClientEntry("connectionPwd", document.getConnectionPwd());
      pageState.addClientEntry("connectionNotes", document.getConnectionNotes());
    }
    if (document.getType() != BasicDocument.IS_CONTENT)
      pageState.addClientEntry("SUMMA", document.getSumma());

    Operator w = pageState.getSessionState().getLoggedOperator();
    String hql = "select listener from " + Listener.class.getName() + " as listener";
    QueryHelper listenerQH = new QueryHelper(hql);
    listenerQH.addQBEClause("ownerx", "owner", w.getId().toString(), QueryHelper.TYPE_CHAR);
    listenerQH.addQBEClause("identifiableId", "identifiableId", document.getId().toString(), QueryHelper.TYPE_CHAR);
    listenerQH.addQBEClause("theClass", "theClass", BasicDocument.class.getName(), QueryHelper.TYPE_CHAR);
//    listenerQH.addQBEClause();
    List myListeners = listenerQH.toHql().list();
    if (myListeners.size() > 0) {
      pageState.addClientEntry("ENABLE_FOLDER_AUTODISCOVERY", Fields.TRUE);
    }
  }

  public void cmdCut(PageState pageState) {
    pageState.sessionState.clipboardPush(pageState.mainObject);
  }


  public void cmdSaveSecurity(PageState pageState) throws PersistenceException {

    cmdEdit(pageState);
    BasicDocument task = (BasicDocument) pageState.getMainObject();

    String areaId = pageState.getEntry("DOCUMENT_AREA").stringValueNullIfEmpty();
    if (areaId != null) {
      Area area = (Area) PersistenceHome.findByPrimaryKey(Area.class, areaId);
      task.setArea(area);
      task.store();
      if (pageState.getEntry("DOCUMENT_AREA_PROPAGATE").checkFieldValue()) {
        List<PerformantNodeSupport> descs = task.getDescendants(BasicDocument.class);
        for (PerformantNodeSupport desc : descs) {
          ((BasicDocument) desc).setArea(area);
          desc.store();
        }
      }
    }
  }

  public void cmdTakeOwnership(PageState pageState) throws PersistenceException {
    cmdEdit(pageState);
    BasicDocument task = (BasicDocument) pageState.getMainObject();
    Operator teamworkOperator = (Operator) pageState.sessionState.getLoggedOperator();

    task.setOwner(teamworkOperator);
    task.store();
    if (pageState.getEntry("TAKE_OWNERSHIP_PROPAGATE").checkFieldValue()) {
      List<PerformantNodeSupport> descs = task.getDescendants(BasicDocument.class);
      for (PerformantNodeSupport desc : descs) {
        ((BasicDocument) desc).setOwner(teamworkOperator);
        desc.store();
      }
    }
  }


}