package org.jblooming.remoteFile.businessLogic;

import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.remoteFile.Document;
import org.jblooming.remoteFile.RemoteFile;
import org.jblooming.utilities.JSP;
import org.jblooming.waf.UploadHelper;
import org.jblooming.waf.constants.FieldErrorConstants;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;


/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class ExplorerAction {

  public void cmdZip(Class docClass, HttpServletRequest request, HttpServletResponse response, PageState pageState) throws PersistenceException, IOException {

    Document document = (Document) PersistenceHome.findByPrimaryKey(docClass, pageState.mainObjectId);
    response.setContentType("application/zip");

    RemoteFile rfs = RemoteFile.getInstance(document,pageState.sessionState.getLoggedOperator());
    String path = JSP.w(pageState.getEntry("PATH").stringValueNullIfEmpty());
    rfs.setTarget(path);

    //get selected files
    Set<String> selFiles = pageState.getClientEntries().getEntriesStartingWithStripped("FILE_", Fields.TRUE).keySet();
    String zipName = document.getName();
    if (selFiles.size()==1)
      zipName = selFiles.iterator().next();

    response.setHeader("Content-Disposition", "attachment; filename=\"" + zipName + ".zip\"");
    ZipOutputStream zipout = new ZipOutputStream(response.getOutputStream());
    zipout.setComment("File Storage Service");

    for (String fileName : selFiles) {

      rfs.setTarget(path + File.separator + fileName);
      if (rfs.isDirectory()) {
        List<RemoteFile> lrf = rfs.expandFileList();
        for (RemoteFile foundRF : lrf) {
          if (!foundRF.isDirectory())
            zipRemoteFile(foundRF, path, zipout);
        }
      } else
        zipRemoteFile(rfs, path, zipout);
    }

    try {
      zipout.finish();
      response.getWriter().flush();
    } catch (java.util.zip.ZipException e) {
    }

  }

   public void cmdDelete(Class docClass, PageState pageState) throws PersistenceException, IOException {

    Document document = (Document) PersistenceHome.findByPrimaryKey(docClass, pageState.mainObjectId);

    RemoteFile rfs = RemoteFile.getInstance(document,pageState.sessionState.getLoggedOperator());
    String path = JSP.w(pageState.getEntry("PATH").stringValueNullIfEmpty());
    rfs.setTarget(path);

    //get selected files
    Set<String> selFiles = pageState.getClientEntries().getEntriesStartingWithStripped("FILE_", Fields.TRUE).keySet();

    for (String fileName : selFiles) {
      rfs.setTarget(path + File.separator + fileName);     
      rfs.delete();
    }

  }


  private void zipRemoteFile(RemoteFile foundRF, String currentPath, ZipOutputStream zipout) throws IOException {
    String fileDir = foundRF.getPathFromDocument().substring(currentPath.length() + File.separator.length());
    ZipEntry zipEntry = new ZipEntry(fileDir);
    zipout.putNextEntry(zipEntry);
    InputStream remoteInputStream = foundRF.getRemoteInputStream();
    BufferedInputStream fr = new BufferedInputStream(remoteInputStream);
    int b;
    byte[] buf = new byte[1024];
    int len;
    while ((len = fr.read(buf)) > 0) {
      zipout.write(buf, 0, len);
    }
    fr.close();
    zipout.closeEntry();
    remoteInputStream.close();
  }

  public void mkdir(Class docClass, PageState pageState) throws FindByPrimaryKeyException {
    Document document = (Document) PersistenceHome.findByPrimaryKey(docClass, pageState.mainObjectId);
    try {
      String dirName = pageState.getEntryAndSetRequired("DIR_NAME").stringValueNullIfEmpty();
      if (dirName != null) {
        RemoteFile rfs = RemoteFile.getInstance(document,pageState.sessionState.getLoggedOperator());
        String path = JSP.w(pageState.getEntry("PATH").stringValueNullIfEmpty());
        rfs.setTarget(path + File.separator + dirName);
        rfs.mkdirs();
      }
    } catch( Exception e){
      pageState.getEntry("DIR_NAME").errorCode = e.getMessage();
    }
  }

  public void upload(Class docClass, PageState pageState) throws PersistenceException, IOException {
    Document document = (Document) PersistenceHome.findByPrimaryKey(docClass, pageState.mainObjectId);
    UploadHelper uh= UploadHelper.getInstance("UPLOAD_HERE",pageState);
    RemoteFile rfs = RemoteFile.getInstance(document,pageState.sessionState.getLoggedOperator());
    String path = pageState.getEntry("PATH").stringValueNullIfEmpty();
    String fileName = pageState.getEntryAndSetRequired("UPLOAD_HERE").stringValueNullIfEmpty();
    if (fileName!=null) {
      rfs.setTarget((path!=null ? path + File.separator : "") + uh.originalFileName);
      //rfs.setTarget(path + uh.originalFileName);
      rfs.upload(uh);
    } else {
      pageState.getEntry("UPLOAD_HERE").errorCode = FieldErrorConstants.ERR_FIELD_CANT_BE_EMPTY;
    }
  }

}
