package org.jblooming.scheduler;

import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.HibernateUtilities;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.DateUtilities;

import java.sql.PreparedStatement;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class SchedulerLogKiller extends ExecutableSupport {
 // @Parameter("[how may days of log do you want to preserve]")
  public String daysToSave = "2";

  public JobLogData run(JobLogData jobLogData) throws Exception {
    boolean exception = true;
    PersistenceContext pc = null;

    try {

      CompanyCalendar cc = new CompanyCalendar();
      cc.set(CompanyCalendar.DAY_OF_YEAR,-Integer.parseInt(daysToSave));
      pc = HibernateFactory.newFreeSession();

      String jobLogTableName = HibernateUtilities.getTableName(JobLog.class);
      String delSQL = "DELETE FROM " + jobLogTableName + " WHERE "+jobLogTableName+".DATEX < ?;";

      PreparedStatement s = pc.session.connection().prepareStatement(delSQL);
      s.setDate(1, new java.sql.Date(cc.getTimeInMillis()));
      s.execute();
      s.close();

      pc.commitAndClose();

      jobLogData.notes = jobLogData.notes + "SchedulerLogKiller executed on " + DateUtilities.dateToTimestampString(DateUtilities.getAbsoluteNow());
      exception = false;
    } catch (Exception e) {
      Tracer.platformLogger.error("SchedulerLogKiller error", e);

    } finally {
      if (exception) {
        if (pc != null) {
          pc.rollback();
        }
        jobLogData.successfull = false;
      }
    }

    return jobLogData;
  }

}
