package org.jblooming.scheduler.businessLogic;

import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.agenda.ScheduleSupport;
import org.jblooming.ontology.businessLogic.DeleteHelper;
import org.jblooming.oql.QueryHelper;
import org.jblooming.page.HibernatePage;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.scheduler.Executable;
import org.jblooming.scheduler.Job;
import org.jblooming.scheduler.Parameter;
import org.jblooming.scheduler.Scheduler;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.ReflectionUtilities;
import org.jblooming.waf.ActionUtilities;
import org.jblooming.waf.constants.AgendaConstants;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.html.display.Paginator;
import org.jblooming.waf.html.input.ScheduleComposer;
import org.jblooming.waf.html.table.ListHeader;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import java.lang.reflect.Field;
import java.text.ParseException;
import java.util.*;

public class JobAction {
  private SortedMap<Long, Long> zoominmap;
  private SortedMap<Long, Long> zoomoutmap;
  private SortedMap<Long, Integer> defaultDeltaColumnsRay;

  public JobAction() {

    zoomoutmap = new TreeMap<Long, Long>();
    zoomoutmap.put(CompanyCalendar.MILLIS_IN_MINUTE, (CompanyCalendar.MILLIS_IN_MINUTE * 5));
    zoomoutmap.put((CompanyCalendar.MILLIS_IN_MINUTE * 5), (CompanyCalendar.MILLIS_IN_MINUTE * 30));
    zoomoutmap.put((CompanyCalendar.MILLIS_IN_MINUTE * 30), CompanyCalendar.MILLIS_IN_HOUR);
    zoomoutmap.put(CompanyCalendar.MILLIS_IN_HOUR, CompanyCalendar.MILLIS_IN_DAY);
    zoomoutmap.put(CompanyCalendar.MILLIS_IN_DAY, CompanyCalendar.MILLIS_IN_MONTH);
    zoomoutmap.put(CompanyCalendar.MILLIS_IN_MONTH, CompanyCalendar.MILLIS_IN_MONTH);

    zoominmap = new TreeMap<Long, Long>();
    zoominmap.put(CompanyCalendar.MILLIS_IN_MINUTE, CompanyCalendar.MILLIS_IN_MINUTE);
    zoominmap.put((CompanyCalendar.MILLIS_IN_MINUTE * 5), CompanyCalendar.MILLIS_IN_MINUTE);
    zoominmap.put((CompanyCalendar.MILLIS_IN_MINUTE * 30), (CompanyCalendar.MILLIS_IN_MINUTE * 5));
    zoominmap.put(CompanyCalendar.MILLIS_IN_HOUR, (CompanyCalendar.MILLIS_IN_MINUTE * 30));
    zoominmap.put(CompanyCalendar.MILLIS_IN_DAY, CompanyCalendar.MILLIS_IN_HOUR);
    zoominmap.put(CompanyCalendar.MILLIS_IN_MONTH, CompanyCalendar.MILLIS_IN_DAY);

    defaultDeltaColumnsRay = new TreeMap<Long, Integer>();
    defaultDeltaColumnsRay.put(CompanyCalendar.MILLIS_IN_MINUTE, 7);
    defaultDeltaColumnsRay.put((CompanyCalendar.MILLIS_IN_MINUTE * 5), 6);
    defaultDeltaColumnsRay.put((CompanyCalendar.MILLIS_IN_MINUTE * 30), 10);
    defaultDeltaColumnsRay.put(CompanyCalendar.MILLIS_IN_HOUR, 12);
    defaultDeltaColumnsRay.put(CompanyCalendar.MILLIS_IN_DAY, 7);
    defaultDeltaColumnsRay.put(CompanyCalendar.MILLIS_IN_MONTH, 6);
  }


  public void cmdSave(PageState pageState) throws FindByPrimaryKeyException, ActionException, StoreException {

    boolean exOccurred = false;
//______________________________________________________________________________________________________________________________________________________________________


// Job


//______________________________________________________________________________________________________________________________________________________________________

    Job job = null;
    if (PersistenceHome.NEW_EMPTY_ID.equals(pageState.getMainObjectId())) {
      job = new Job();
      job.setIdAsNew();
    } else {
      job = (Job) PersistenceHome.findByPrimaryKey(Job.class, pageState.getMainObjectId());
    }
    pageState.setMainObject(job);

    try {
      job.setName(pageState.getEntryAndSetRequired(AgendaConstants.FLD_NAME).stringValue());
    } catch (ActionException e) {
      exOccurred = true;
    }

    job.setDescription(pageState.getEntry(AgendaConstants.FLD_OBJECT).stringValue());

    try {
      job.setExecutable(pageState.getEntryAndSetRequired("LAUNCHER_CLASS").stringValue());
    } catch (ActionException e) {
      exOccurred = true;
    }
    try {
      job.setEstimatedDuration(pageState.getEntry(AgendaConstants.FLD_ESTIMATED_DURATION).intValue());
    } catch (ParseException e) {
    }

    ActionUtilities.setLong(pageState.getEntry("timeoutTime"),job,"timeoutTime");

//______________________________________________________________________________________________________________________________________________________________________


// ScheduleSupport


//______________________________________________________________________________________________________________________________________________________________________

    ScheduleSupport schedule = ScheduleComposer.getSchedule("ScheduleComposer", pageState);

//______________________________________________________________________________________________________________________________________________________________________


// Class


//______________________________________________________________________________________________________________________________________________________________________


    ClientEntry clazz = pageState.getEntry("LAUNCHER_CLASS");

    if (clazz.stringValueNullIfEmpty() != null) {

      try {
        Class theClass = Class.forName(clazz.stringValue());
        if (ReflectionUtilities.getInheritedClasses(theClass).contains(Executable.class)) {
          Map<String, Field> flds = ReflectionUtilities.getDeclaredInheritedParameterFields(theClass, Parameter.class);
          if (flds.values() != null && flds.values().size() > 0) {

            for (Field field : flds.values()) {

              ClientEntry entry = pageState.getEntry(field.getName());
              if (entry.stringValue() != null) {
                String value = entry.stringValue();
                if (value != null && value.trim().length() > 0) {

                  job.getParameters().put(field.getName(), value);
                } else  {
                  job.getParameters().remove(field.getName());
                }
              }
            }
          }
        }
      } catch (ClassNotFoundException e) {
      }
    }

//______________________________________________________________________________________________________________________________________________________________________


// store


//______________________________________________________________________________________________________________________________________________________________________

    if (exOccurred)
      throw new ActionException();

    schedule.store();
    job.setSchedule(schedule);
    job.store();

//______________________________________________________________________________________________________________________________________________________________________


// addJob


//______________________________________________________________________________________________________________________________________________________________________

    Scheduler instance = Scheduler.getInstance();
    if (instance != null)
      instance.addJob(job);

  }

  public void cmdAdd(PageState pageState) {
    Job job = new Job();
    job.setIdAsNew();
    pageState.setMainObject(job);
    make(pageState, job);
  }

  public void cmdEdit(PageState pageState) throws FindByPrimaryKeyException {
    Job job = (Job) PersistenceHome.findByPrimaryKey(Job.class, pageState.getMainObjectIntId());
    pageState.setMainObject(job);
    make(pageState, job);

    if (job.getSchedule() != null)
      ScheduleComposer.make("ScheduleComposer", job.getSchedule(), pageState);
  }

  private void make(PageState pageState, Job job) {
    pageState.addClientEntry(AgendaConstants.FLD_NAME, job.getName());
    pageState.addClientEntry(AgendaConstants.FLD_OBJECT, job.getDescription());
    pageState.addClientEntry("LAUNCHER_CLASS", job.getExecutable());

    pageState.addClientEntry(AgendaConstants.FLD_ESTIMATED_DURATION, job.getEstimatedDuration());
    pageState.addClientEntry("timeoutTime", job.getTimeoutTime() + "");
    if (job.isEnabled())
      pageState.addClientEntry("enabled", "on");
    if (job.isOnErrorSuspendScheduling())
      pageState.addClientEntry("onErrorSuspendScheduling", "on");
    if (job.isOnErrorRetryNow())
      pageState.addClientEntry("onErrorRetryNow", "on");

    if (job.getParameters() != null && job.getParameters().size() > 0) {
      for (String key : job.getParameters().keySet()) {
        String value = job.getParameters().get(key);
        pageState.addClientEntry(key, value);
      }
    }

  }

  public void cmdFind(PageState pageState) throws PersistenceException, ActionException {
    String hql = "from " + Job.class.getName() + " as job";
    QueryHelper qhelp = new QueryHelper(hql);
    String filter = pageState.getEntryValueNullIfEmpty(Fields.FORM_PREFIX + "search");
    if (filter != null && filter.trim().length() > 0) {
      qhelp.addQBEClause("job.name", "name", filter, QueryHelper.TYPE_CHAR);
    }
    ListHeader.orderAction(qhelp, "LH", pageState);
    pageState.setPage(HibernatePage.getHibernatePageInstance(qhelp.toHql().getQuery(),
            Paginator.getWantedPageNumber(pageState),
            Paginator.getWantedPageSize("JOBPS", pageState)));
  }

  /* public void cmdNext(PageState pageState) throws ActionException {
       Date date =  null;
       try {
         date = DateUtilities.dateFromString(pageState.getEntry(AgendaConstants.FLD_START).stringValue(),"dd/MM/yyyy-HH:mm");
       } catch (ParseException e) {}
       long intTemp = 0;
       try {
         intTemp = pageState.getEntry(AgendaConstants.FLD_TYPE).longValue();
       } catch (ParseException e) {}
       CompanyCalendar cal = new CompanyCalendar();
       cal.setTime(date);
       if(intTemp == CompanyCalendar.MILLIS_IN_MONTH) {
         cal.add(Calendar.MONTH,1);
       } else {
         cal.add(Calendar.MILLISECOND,(int)intTemp);
       }
       pageState.addClientEntry(AgendaConstants.FLD_START,DateUtilities.dateToString(cal.getTime(), "dd/MM/yyyy-HH:mm"));
       try {
         date = DateUtilities.dateFromString(pageState.getEntry(AgendaConstants.FLD_END).stringValue(),"dd/MM/yyyy-HH:mm");
       } catch (ParseException e) {}
       cal = new CompanyCalendar();
       cal.setTime(date);
       if(intTemp == CompanyCalendar.MILLIS_IN_MONTH) {
         cal.add(Calendar.MONTH,1);
       } else {
         cal.add(Calendar.MILLISECOND,(int)intTemp);
       }
       pageState.addClientEntry(AgendaConstants.FLD_END,DateUtilities.dateToString(cal.getTime(), "dd/MM/yyyy-HH:mm"));
       pageState.addClientEntry(AgendaConstants.FLD_TYPE,String.valueOf(intTemp));
     }

     public void cmdPrevious(PageState pageState) throws ActionException {
       Date date =  null;
       try {
         date = DateUtilities.dateFromString(pageState.getEntry(AgendaConstants.FLD_START).stringValue(),"dd/MM/yyyy-HH:mm");
       } catch (ParseException e) {}
       long intTemp = 0;
       try {
         intTemp = pageState.getEntry(AgendaConstants.FLD_TYPE).longValue();
       } catch (ParseException e) {}
       CompanyCalendar cal = new CompanyCalendar();
       cal.setTime(date);
       if(intTemp == CompanyCalendar.MILLIS_IN_MONTH) {
         int mese = cal.get(Calendar.MONTH);
         cal.set(Calendar.MONTH,mese-1);
       } else {
         long millis = cal.getTimeInMillis();
         cal.setTimeInMillis(millis-intTemp);
       }
       pageState.addClientEntry(AgendaConstants.FLD_START,DateUtilities.dateToString(cal.getTime(), "dd/MM/yyyy-HH:mm"));
       try {
         date = DateUtilities.dateFromString(pageState.getEntry(AgendaConstants.FLD_END).stringValue(),"dd/MM/yyyy-HH:mm");
       } catch (ParseException e) {}
       cal = new CompanyCalendar();
       cal.setTime(date);
       if(intTemp == CompanyCalendar.MILLIS_IN_MONTH) {
         int mese = cal.get(Calendar.MONTH);
         cal.set(Calendar.MONTH,mese-1);
       } else {
         long millis = cal.getTimeInMillis();
         cal.setTimeInMillis(millis-intTemp);
       }
       pageState.addClientEntry(AgendaConstants.FLD_END,DateUtilities.dateToString(cal.getTime(), "dd/MM/yyyy-HH:mm"));
       pageState.addClientEntry(AgendaConstants.FLD_TYPE,String.valueOf(intTemp));
     }   */

  public void cmdZoomOut(PageState pageState) throws ActionException {

    Date start = null;
    Date end = null;
    Date center = null;

    try {
      center = DateUtilities.dateFromString(pageState.getEntry(AgendaConstants.FLD_FOCUSED_DATE).stringValue(), DateUtilities.getTimestampFormat());
    } catch (ParseException e) {
    }
    long intTemp = 0;
    try {
      intTemp = pageState.getEntry(AgendaConstants.FLD_TYPE).longValue();
    } catch (ParseException e) {
    }
    if (intTemp < CompanyCalendar.MILLIS_IN_MONTH) {
      if (intTemp == CompanyCalendar.MILLIS_IN_DAY) {
        CompanyCalendar cal = new CompanyCalendar();
        cal.setTime(center);
        cal.add(Calendar.MONTH, -defaultDeltaColumnsRay.get(intTemp));
        start = cal. getTime();
        cal.setTime(center);
        cal.add(Calendar.MONTH, defaultDeltaColumnsRay.get(intTemp));
        end = cal.getTime();
      } else {
        long millis = zoomoutmap.get(intTemp) * defaultDeltaColumnsRay.get(zoomoutmap.get(intTemp));
        start = new Date(center.getTime() - millis);
        end = new Date(center.getTime() + millis);
      }
    }
    pageState.addClientEntry(AgendaConstants.FLD_TYPE, String.valueOf(zoomoutmap.get(intTemp)));
    pageState.addClientEntry(AgendaConstants.FLD_FOCUSED_DATE, DateUtilities.dateToString(center, DateUtilities.getTimestampFormat()));
    pageState.addClientEntry(AgendaConstants.FLD_START, DateUtilities.dateToString(start, DateUtilities.getTimestampFormat()));
    pageState.addClientEntry(AgendaConstants.FLD_END, DateUtilities.dateToString(end, DateUtilities.getTimestampFormat()));
  }

  public void cmdZoomIn(PageState pageState) throws ActionException {
    Date center = null;
    Date start = null;
    Date end = null;
    try {
      center = DateUtilities.dateFromString(pageState.getEntry(AgendaConstants.FLD_FOCUSED_DATE).stringValue(), DateUtilities.getTimestampFormat());
    } catch (ParseException e) {
    }
    long intTemp = 0;
    try {
      intTemp = pageState.getEntry(AgendaConstants.FLD_TYPE).longValue();
    } catch (ParseException e) {
    }
    if (intTemp > CompanyCalendar.MILLIS_IN_MINUTE) {
      long millis = zoominmap.get(intTemp) * defaultDeltaColumnsRay.get(zoominmap.get(intTemp));
      start = new Date(center.getTime() - millis);
      end = new Date(center.getTime() + millis);
    }
    pageState.addClientEntry(AgendaConstants.FLD_TYPE, String.valueOf(zoominmap.get(intTemp)));
    pageState.addClientEntry(AgendaConstants.FLD_FOCUSED_DATE, DateUtilities.dateToString(center, DateUtilities.getTimestampFormat()));
    pageState.addClientEntry(AgendaConstants.FLD_START, DateUtilities.dateToString(start, DateUtilities.getTimestampFormat()));
    pageState.addClientEntry(AgendaConstants.FLD_END, DateUtilities.dateToString(end, DateUtilities.getTimestampFormat()));
  }

  public void cmdFocus(PageState pageState) throws ActionException {
    Date start = null;
    Date end = null;
    Date center = null;
    try {
      center = DateUtilities.dateFromString(pageState.getEntry(AgendaConstants.FLD_FOCUSED_DATE).stringValue(), DateUtilities.getTimestampFormat());
      if (center == null)
        center = new Date();
    } catch (ParseException e) {
      throw new ActionException();
    }
    long intTemp = 0;
    try {
      intTemp = pageState.getEntry(AgendaConstants.FLD_TYPE).longValue();
    } catch (ParseException e) {
      intTemp = CompanyCalendar.MILLIS_IN_DAY;
    }
    if (intTemp == CompanyCalendar.MILLIS_IN_MONTH) {
      CompanyCalendar cal = new CompanyCalendar();
      cal.setTime(center);
      cal.add(Calendar.MONTH, -defaultDeltaColumnsRay.get(intTemp));
      start = cal.getTime();
      cal.setTime(center);
      cal.add(Calendar.MONTH, defaultDeltaColumnsRay.get(intTemp));
      end = cal.getTime();
    } else {
      long millis = intTemp * defaultDeltaColumnsRay.get(intTemp);
      start = new Date(center.getTime() - millis);
      end = new Date(center.getTime() + millis);
    }
    pageState .addClientEntry(AgendaConstants.FLD_TYPE, String.valueOf(intTemp));
    pageState.addClientEntry(AgendaConstants. FLD_FOCUSED_DATE, DateUtilities.dateToString(center, DateUtilities.getTimestampFormat()));
    pageState.addClientEntry(AgendaConstants.FLD_START, DateUtilities.dateToString(start, DateUtilities.getTimestampFormat()));
    pageState.addClientEntry(AgendaConstants.FLD_END, DateUtilities.dateToString(end, DateUtilities.getTimestampFormat()));
  }

  public void cmdDelete(PageState pageState) throws org.jblooming.security.SecurityException, PersistenceException {
    Job delenda = (Job) PersistenceHome.findByPrimaryKey(Job.class, pageState.getMainObjectId());
    DeleteHelper.cmdDelete(delenda, pageState);
    Scheduler instance = Scheduler.getInstance();
    if (instance!=null)
      instance.removeJob(delenda);
  }

  public void cmdRunNow(PageState pageState) throws PersistenceException {
    cmdEdit(pageState);
    Job job = (Job)pageState.mainObject;

    Scheduler instance = Scheduler.getInstance();
    if (instance==null) {
      Scheduler.instantiate(5000, pageState.sessionState.getLoggedOperator().getDisplayName());
      instance.run();
    }
    job.setLastExecutionTime(System.currentTimeMillis());
    job.store();
    instance.runAJob(job);
  }
}
