package org.jblooming.security.businessLogic;

import org.jblooming.company.Department;
import org.jblooming.ontology.PerformantNode;
import org.jblooming.ontology.businessLogic.DeleteHelper;
import org.jblooming.operator.Operator;
import org.jblooming.oql.QueryHelper;
import org.jblooming.page.HibernatePage;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.resource.Affiliate;
import org.jblooming.security.PlatformPermissions;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.waf.ActionUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.html.display.Paginator;
import org.jblooming.waf.html.table.ListHeader;
import org.jblooming.waf.state.PersistentSearch;
import org.jblooming.waf.view.ClientEntries;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * Created by IntelliJ IDEA.
 * User: angantcha
 * Date: 18-ott-2005
 * Time: 16.53.58
 * To change this template use File | Settings | File Templates.
 */
public class AffiliateAction {

  public boolean hasRoles;
  SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy", Locale.ITALIAN);


  public void cmdDelete(PageState pageState) throws org.jblooming.security.SecurityException, PersistenceException {
    SessionState sm = pageState.getSessionState();
    Operator user = sm.getLoggedOperator();
    user.testPermission(PlatformPermissions.affiliate_canCreate);
    Affiliate delenda = (Affiliate) PersistenceHome.findByPrimaryKey(Affiliate.class, pageState.getMainObjectId());
    DeleteHelper.cmdDelete(delenda, pageState);
    // doubled PersistenceHome.remove(delenda);
  }

  public void cmdSave(PageState pageState) throws org.jblooming.security.SecurityException, PersistenceException, ActionException {
    boolean invalidEntries = false;
    SessionState sm = pageState.getSessionState();
    Operator user = sm.getLoggedOperator();
    user.testPermission(PlatformPermissions.affiliate_canCreate);

    Affiliate aff = null;

    if (PersistenceHome.NEW_EMPTY_ID.equals(pageState.mainObjectId)) {
      aff = new Affiliate();
      aff.setIdAsNew();
    } else
      aff = (Affiliate) PersistenceHome.findByPrimaryKey(Affiliate.class, pageState.getMainObjectId());

    ActionUtilities.setIdentifiable(pageState.getEntryAndSetRequired("ROLE"),aff,"role");
    ActionUtilities.setIdentifiable(pageState.getEntryAndSetRequired("DEPARTMENT"),aff,"department");
    ActionUtilities.setIdentifiable(pageState.getEntryAndSetRequired("OPERATOR"),aff,"operator");

    pageState.setMainObject(aff);

    try {
      String data1 = pageState.getEntry("ENDINGDATE").stringValueNullIfEmpty();
      if (data1 != null) {
        data1 = data1.replaceAll("-", "/");
        Date data = DateUtilities.dateFromString(data1);
        if (data != null) {
          aff.setEndingDate(data);
        } else {
          aff.setEndingDate(null);
        }
      } else {
        aff.setEndingDate(null);
      }
    } catch (ParseException e) {
      invalidEntries = true;
    }
    if (invalidEntries || !pageState.validEntries()) {
      throw new ActionException();
    }

    aff.store();
  }

  public void cmdAdd(PageState pageState) throws PersistenceException, org.jblooming.security.SecurityException {

    SessionState sm = pageState.getSessionState();
    Operator user = sm.getLoggedOperator();
    user.testPermission(PlatformPermissions.affiliate_canCreate);
    Affiliate aff = new Affiliate();
    aff.setIdAsNew();
    pageState.setMainObject(aff);

  }


  public void cmdEdit(PageState pageState) throws PersistenceException, org.jblooming.security.SecurityException {

    SessionState sm = pageState.getSessionState();
    Operator user = sm.getLoggedOperator();
    user.testPermission(PlatformPermissions.affiliate_canCreate);
    Affiliate aff = (Affiliate) PersistenceHome.findByPrimaryKey(Affiliate.class, pageState.getMainObjectId());
    pageState.setMainObject(aff);

    ClientEntries ces = new ClientEntries();
    ces.addEntry(new ClientEntry("ENDINGDATE", aff.getEndingDate() != null ? formatDate.format(aff.getEndingDate()) : ""));

    pageState.addClientEntries(ces);
    //pageState.getClientEntries().addEntries(make(aff, pageState));
  }

  public void cmdFind(PageState pageState) throws PersistenceException, ActionException {

    final Operator logged = pageState.sessionState.getLoggedOperator();
    String hql = "from " + Affiliate.class.getName() + " as af";
    QueryHelper qhelp = new QueryHelper(hql);

    boolean recoveredFromSavedFilter = PersistentSearch.feedFromSavedSearch(pageState, logged);

    String anc = "";
    String company = pageState.getEntry("SEDE_0").stringValueNullIfEmpty();
    String dir = pageState.getEntry("SEDE_1").stringValueNullIfEmpty();
    String rep = pageState.getEntry("SEDE_2").stringValueNullIfEmpty();


    if (company != null || dir != null || rep != null) {
      rep = (rep != null ? rep : (dir != null ? dir : (company)));
      int repid = Integer.parseInt(rep);
      Department dpt = (Department) PersistenceHome.findByPrimaryKey(Department.class, repid);
      anc = dpt.getAncestorIds() == null ? dpt + PerformantNode.SEPARATOR : dpt.getAncestorIds() + rep + PerformantNode.SEPARATOR;
      qhelp.addQueryClause(qhelp.getQbeClause("af.department.ancestorIds", "ancestorIds", anc + "%", QueryHelper.TYPE_CHAR)
        + " OR " +
        qhelp.getQbeClause("af.department.id", "idUguale", "=" + repid, QueryHelper.TYPE_CHAR));
    }

    String fld_role = pageState.getEntry("ROLE").stringValueNullIfEmpty();
    if (fld_role != null) {
      qhelp.addQBEClause("af.role.id", "roleid", fld_role, QueryHelper.TYPE_CHAR);
    }

    String fld_role_name = pageState.getEntry("ROLE_NAME").stringValueNullIfEmpty();
    if (fld_role_name != null) {
      qhelp.addQBEClause("af.role.name", "rolename", fld_role_name, QueryHelper.TYPE_CHAR);
    }

    String fld_usr_name = pageState.getEntry(OperatorConstants.FLD_NAME + "filter").stringValueNullIfEmpty();
    if (fld_usr_name != null) {
      qhelp.addQueryClause(qhelp.getQbeClause("af.operator.name", "name", fld_usr_name, QueryHelper.TYPE_CHAR)
        + " OR " +
        qhelp.getQbeClause("af.operator.surname", "surname", fld_usr_name, QueryHelper.TYPE_CHAR));
    }

    PersistentSearch.saveSearch("AFFILIATE", recoveredFromSavedFilter, pageState, logged);

    ListHeader.orderAction(qhelp, "AFFILIATELH", pageState);
    pageState.setPage(HibernatePage.getHibernatePageInstance(qhelp.toHql().getQuery(), Paginator.getWantedPageNumber(pageState), Paginator.getWantedPageSize(pageState)));

  }
}
