package org.jblooming.security.businessLogic;

import org.jblooming.ApplicationException;
import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.operator.Operator;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.scheduler.Scheduler;
import org.jblooming.system.SystemConstants;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.html.state.ScreenElementStatus;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.security.NoSuchAlgorithmException;
import java.util.Iterator;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 */
public class LoginAction {

  public void login(PageState pageState, HttpServletRequest request, HttpServletResponse response) throws PersistenceException, ApplicationException, FindByPrimaryKeyException {

    if (pageState.getCommand() != null && pageState.getCommand().equals(Commands.LOGOUT)) {
      pageState.setClientEntries(null);
      return;
    }

    Operator user = null;
    String newPassword = null;

    boolean invalidEntries = false;
    String password = null;
/*
________________________________________________________________________________________________________________________________________________________________________


        HTTP_AUTHENTICATION

________________________________________________________________________________________________________________________________________________________________________

*/
    if (Fields.TRUE.equals(ApplicationState.getApplicationSetting(SystemConstants.ENABLE_HTTP_AUTHENTICATION))) {
       try {
      if (request.getRemoteUser()!=null)
        user = Operator.findByLoginName(request.getRemoteUser());
      } catch (Exception e) {
          invalidEntries = true;
        }
/*
________________________________________________________________________________________________________________________________________________________________________


        INNER LOGIN

________________________________________________________________________________________________________________________________________________________________________

*/
    } else {

      try {
        ClientEntry ceName = pageState.getEntryAndSetRequired(OperatorConstants.FLD_LOGIN_NAME);
        ClientEntry cePassword = pageState.getEntry(OperatorConstants.FLD_PWD);

        password = cePassword.stringValue();
        String username = ceName.stringValue();

        try {
          user = Operator.authenticateUser(password, username, pageState.getApplication().isLoginCookieEnabled());
        } catch (org.jblooming.security.SecurityException e) {
          ceName.errorCode = e.getMessage();
          invalidEntries = true;
        }

        if (!invalidEntries) {

          final String pass_exp = ApplicationState.getApplicationSetting(SystemConstants.FLD_PASSWORD_EXPIRY);
          int maxDaysPassed = pass_exp != null ? Integer.parseInt(pass_exp) : 0;
          if (
                  (maxDaysPassed > 0 &&
                          user.getLastPasswordChangeDate() != null &&
                          ((DateUtilities.getAbsoluteNow().getTime() - user.getLastPasswordChangeDate().getTime()) / (CompanyCalendar.MILLIS_IN_HOUR * 24))
                                  >
                                  maxDaysPassed)
                  ) {
            try {
              ClientEntry ceNewPassword = pageState.getEntryAndSetRequired(OperatorConstants.FLD_LOGIN_NEW_PWD);

              ClientEntry ceNewPasswordConfirm = pageState.getEntryAndSetRequired(OperatorConstants.FLD_LOGIN_NEW_PWD_RETYPE);

              if (!ceNewPassword.stringValue().equals(ceNewPasswordConfirm.stringValue()))
                throw new ActionException();
              final Iterator lastPasswordIterator = user.getLastPasswordIterator();
              while (lastPasswordIterator.hasNext()) {
                String s = (String) lastPasswordIterator.next();
                try {
                  if (s.equals(StringUtilities.md5Encode(user.getLoginName() + ceNewPassword.stringValue())))
                    throw new ActionException();
                } catch (NoSuchAlgorithmException e) {
                  throw new ApplicationException(e);
                }
              }

              //passed all obstacles
              newPassword = ceNewPassword.stringValue();


            } catch (ActionException e) {
              cePassword.errorCode = OperatorConstants.ERR_EXPIRED;
              invalidEntries = true;
            }
          }
        }

      } catch (ActionException e) {
        invalidEntries = true;
      }
    }
/*
________________________________________________________________________________________________________________________________________________________________________


        ACT

________________________________________________________________________________________________________________________________________________________________________

*/

    if (!invalidEntries) {

      SessionState sm = pageState.getSessionState();
      Operator op = (Operator) PersistenceHome.findByPrimaryKey(Operator.class, Integer.parseInt(user.getId().toString()));

      if (newPassword != null) {
        op.changePassword(op.getLoginName(), newPassword);
      }
      doLog(op, sm);

      /*if (pageState.getEntry("REMEMBER_ME").checkFieldValue())
        pageState.sessionState.setAttribute("SAVE_PSW_IN_COOKIE", password);
      */
      pageState.setCommand(Commands.FIND);
    }
  }

  public static void doLog(Operator op, SessionState sm) throws PersistenceException, ApplicationException {
    op.setLastLoggedOn(DateUtilities.getAbsoluteNow());
    op.store();
    sm.setLoggedOperator(op);
    sm.screenElementsStatus = ScreenElementStatus.getInstanceFromOptions(op);
    String newSkin = op.getOption(OperatorConstants.FLD_CURRENT_SKIN);

    /**
     * test that skin exists!
     */
    if (!new File(ApplicationState.webAppFileSystemRootPath+"commons"+File.separator+"skin"+File.separator+newSkin).exists()) {
      Tracer.platformLogger.error("Skin "+newSkin+" in option of operator "+op.getDisplayName()+ " (id="+op.getId()+") does not exist");
      newSkin = ApplicationState.getApplicationSetting(OperatorConstants.FLD_CURRENT_SKIN);
      op.getOptions().put(OperatorConstants.FLD_CURRENT_SKIN,newSkin);
    }

    if (newSkin != null && newSkin.trim().length() > 0)
      sm.setSkin(SessionState.createSkin(ApplicationState.contextPath, newSkin, ApplicationState.platformConfiguration.defaultApplication.getRootFolder()));

    if (ApplicationState.platformConfiguration.schedulerRunsByDefault && !Scheduler.isRunning()) {
      ApplicationState.applicationSettings.put(SystemConstants.ADMIN_MESSAGE,"Scheduler is NOT running. Start it in tools -> administration -> scheduler manager -> monitor");
    }


  }
}
