package org.jblooming.uidgen;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.persister.entity.SingleTableEntityPersister;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.persistence.hibernate.HibernateFactory;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Map;


public class CounterHome {

  private static String uniqueIdTable;

  public static String nextSer(Class theClass) throws StoreException {
    return getNextId(theClass.getName(), true, HibernateFactory.getSessionFactory()) + "";
  }

  public static String nextSer(String name) throws StoreException {
    return getNextId(name, true, HibernateFactory.getSessionFactory()) + "";
  }

  public static int next(Class theClass) throws StoreException {
    return getNextId(theClass.getName(), true, HibernateFactory.getSessionFactory());
  }


  public static int next(String counterName) throws StoreException {
    return getNextId(counterName, true, HibernateFactory.getSessionFactory());
  }

  public static int next(String counterName, SessionFactory sf) throws StoreException {
    return getNextId(counterName, true, sf);
  }

  public static int getUniqueIdInCurrentTransaction(String counterName) throws StoreException {
    return getNextId(counterName, false, HibernateFactory.getSessionFactory());
  }

  public static int getUniqueIdInCurrentTransaction(String counterName, SessionFactory sf) throws StoreException {
    return getNextId(counterName, false, sf);
  }

  private static int getNextId(String counterName, boolean newTransaction, SessionFactory sf) throws StoreException {

    if (uniqueIdTable == null) {

      Map acm = null;
      try {
        acm = sf.getAllClassMetadata();
      } catch (HibernateException e) {
        throw new StoreException(e);
      }
      SingleTableEntityPersister ep = (SingleTableEntityPersister) acm.get(Counter.class.getName());
      uniqueIdTable = ep.getTableName();
    }
    int nextValue = -1;
    PreparedStatement select = null;
    PreparedStatement insert = null;
    PreparedStatement update = null;
    Connection connection = null;
    ResultSet result = null;
    Throwable exception = null;
    Session session = null;
    int originalAutoCommitSetting = -1;

    try {
      if (newTransaction) {
        session = sf.openSession();
        connection = session.connection();
        originalAutoCommitSetting = connection.getAutoCommit() ? 1 : 0;
        connection.setAutoCommit(false);
      } else {
        session = HibernateFactory.getSession();
        connection = session.connection();
      }

      try {
        insert = connection.prepareStatement("insert into " + uniqueIdTable + " (id,valuex) values(?,?)");
        insert.setString(1, counterName);
        insert.setInt(2, 1);
        insert.executeUpdate();
      } catch (Throwable t) {
        connection.rollback();
      }

      update = connection.prepareStatement("update " + uniqueIdTable + " set valuex=valuex+1 where id = ?");
      update.setString(1, counterName);
      int updated = update.executeUpdate();
      if (updated <= 0)
        throw new Throwable("It was not possibile to update valuex for counter = " + counterName);

      select = connection.prepareStatement("select valuex from " + uniqueIdTable + " where id = ?");
      select.setString(1, counterName);
      result = select.executeQuery();
      result.next();
      nextValue = result.getInt(1);

      if (newTransaction)
        connection.commit();

    } catch (Throwable ex) {
      exception = ex;
    }

    try {
      result.close();
    } catch (Throwable t) {
    }
    try {
      select.close();
    } catch (Throwable t) {
    }
    try {
      insert.close();
    } catch (Throwable t) {
    }
    try {
      update.close();
    } catch (Throwable t) {
    }

    try {
      if (exception != null && newTransaction) connection.rollback();
    } catch (Throwable t) {
    }

    try {
      if (originalAutoCommitSetting >= 0 && newTransaction) {
        if (originalAutoCommitSetting == 1)
          connection.setAutoCommit(true);
        else
          connection.setAutoCommit(false);
      }
    } catch (Throwable t) {
    }

    if (newTransaction) {
      try {
        session.close();
      } catch (Throwable t) {
      }
    }

    if (exception != null)
      throw new StoreException(exception);
    else
      return nextValue;
  }


  public static void setCounterSeed(String counterName, int newSeed) throws StoreException {
    SessionFactory sf = HibernateFactory.getSessionFactory();

    if (counterName != null) {
      if (uniqueIdTable == null) {

        Map acm = null;
        try {
          acm = sf.getAllClassMetadata();
        } catch (HibernateException e) {
          throw new StoreException(e);
        }
        SingleTableEntityPersister ep = (SingleTableEntityPersister) acm.get(Counter.class.getName());
        uniqueIdTable = ep.getTableName();
      }
      PreparedStatement select = null;
      PreparedStatement insert = null;
      PreparedStatement update = null;
      Connection connection = null;
      ResultSet result = null;
      Throwable exception = null;
      Session session = null;
      int originalAutoCommitSetting = -1;

      try {
        session = sf.openSession();
        connection = session.connection();
        originalAutoCommitSetting = connection.getAutoCommit() ? 1 : 0;
        connection.setAutoCommit(false);

        try {
          insert = connection.prepareStatement("insert into " + uniqueIdTable + " (id,valuex) values(?,?)");
          insert.setString(1, counterName);
          insert.setInt(2, newSeed);
          insert.executeUpdate();
        } catch (Throwable t) {
          connection.rollback();
        }

        update = connection.prepareStatement("update " + uniqueIdTable + " set valuex=" + newSeed + " where id = ?");
        update.setString(1, counterName);
        int updated = update.executeUpdate();
        if (updated <= 0)
          throw new Throwable("It was not possibile to update valuex for counter = " + counterName);

        connection.commit();

      } catch (Throwable ex) {
        exception = ex;
      }

      try {
        result.close();
      } catch (Throwable t) {
      }
      try {
        select.close();
      } catch (Throwable t) {
      }
      try {
        insert.close();
      } catch (Throwable t) {
      }
      try {
        update.close();
      } catch (Throwable t) {
      }

      try {
        if (exception != null) connection.rollback();
      } catch (Throwable t) {
      }

      try {
        if (originalAutoCommitSetting >= 0) {
          if (originalAutoCommitSetting == 1)
            connection.setAutoCommit(true);
          else
            connection.setAutoCommit(false);
        }
      } catch (Throwable t) {
      }

      try {
        session.close();
      } catch (Throwable t) {
      }

      if (exception != null)
        throw new StoreException(exception);
    }
  }


  public Counter findByPrimaryKey(String id) throws FindByPrimaryKeyException {
    //Counter c = (Counter) PersistenceHome.findByPrimaryKey(Counter.class, id);
    //return c;

    if (id == null)
      throw new FindByPrimaryKeyException("HibernateFactory::findByPrimaryKey: passed null id.");

    Counter o = null;
    try {
      o = (Counter) HibernateFactory.getSession().load(Counter.class, id);
    } catch (HibernateException e) {
      throw new FindByPrimaryKeyException(e);
    }
    return o;
  }

}