package org.jblooming.utilities;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.system.SystemConstants;
import org.jblooming.tracer.Tracer;
import org.jblooming.waf.settings.ApplicationState;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class HttpUtilities {


  public static String getCompleteFileSystemPath(String webPath, ServletContext pc) {
    return StringUtilities.replaceAllNoRegex(StringUtilities.replaceAllNoRegex(pc.getRealPath(webPath), "/", File.separator), "\\", File.separator);
  }

  /**
   * //issue_WW2_1
   *
   * @param request
   * @return the result is in lower case. This is the file on which this method is called
   */
  public static String getCanonicalFileSystemPathOfPartFromURI(HttpServletRequest request) {
    String key = realURI(request);
    key = request.getSession(true).getServletContext().getRealPath(key);
    try {
      File file = new File(key);
      key = file.getCanonicalPath();
      if (file.isDirectory() && !key.endsWith(File.separator))
        key = key + File.separator;
    } catch (IOException e) {
      throw new PlatformRuntimeException(e);
    }
    return key.toLowerCase();
  }

  /**
   * @param request
   * @return This is the file of request.getRequestURI()
   */
  public static String getCanonicalFileSystemPathFromURI(HttpServletRequest request) {
    String key = request.getRequestURI().substring(request.getContextPath().length());
    key = request.getSession(true).getServletContext().getRealPath(key);
    try {
      key = new File(key).getCanonicalPath();
    } catch (IOException e) {
      throw new PlatformRuntimeException(e);
    }
    return key;
  }

  /**
   * //issue_WW2_1
   *
   * @param request
   * @return the result is in lower case
   */
  public static String getFileSystemRootPathForRequest(HttpServletRequest request) {
    String key = request.getSession(true).getServletContext().getRealPath("/");
    try {
      key = new File(key).getCanonicalPath();
    } catch (IOException e) {
      throw new PlatformRuntimeException(e);
    }
    return key+File.separator;
  }

  public static String getFileNameFromUri(HttpServletRequest request) {
    String uri = request.getRequestURI();
    return uri.substring(uri.lastIndexOf("/") + 1, uri.length());
  }

  public static String getContentType(String fileExtension) {
    fileExtension = fileExtension.toLowerCase().trim().replaceAll("\\.", "");

// images
    if (fileExtension.endsWith("jpg") || fileExtension.endsWith("jpeg") || fileExtension.endsWith("jpe"))
      return "image/jpeg";
    else if (fileExtension.endsWith("gif"))
      return "image/gif";
    else if (fileExtension.endsWith("png"))
      return "image/png";
    else if (fileExtension.endsWith("bmp"))
      return "image/bmp";
    else if (fileExtension.endsWith("tiff") || fileExtension.endsWith("tif"))
      return "image/tiff";
    else if (fileExtension.endsWith("svg"))
      return "image/svg+xml";

// multimedia
    else if (fileExtension.endsWith("mp3") || fileExtension.endsWith("mp4"))
      return "audio/mpeg";
    else if (fileExtension.endsWith("avi"))
      return "video/x-msvideo";
    else if (fileExtension.endsWith("mov") || fileExtension.endsWith("qt"))
      return "video/quicktime";
    else if (fileExtension.endsWith("mpg") || fileExtension.endsWith("mpeg") || fileExtension.endsWith("mpe"))
      return "video/mpeg";
    else if (fileExtension.endsWith("ra") || fileExtension.endsWith("ram"))
      return "application/x-pn-realaudio ";
    else if (fileExtension.endsWith("rm") || fileExtension.endsWith("rpm"))
      return "application/x-pn-realaudio-plugin";
    else if (fileExtension.endsWith("mid") || fileExtension.endsWith("midi"))
      return "audio/x-midi";

// office
    else if (fileExtension.endsWith("pdf"))
      return "application/pdf";
    else if (fileExtension.endsWith("rtf"))
      return "text/richtext";
    else if (fileExtension.endsWith("xl") || fileExtension.endsWith("xls") || fileExtension.endsWith("xlv") || fileExtension.endsWith("xla")
            || fileExtension.endsWith("xlb") || fileExtension.endsWith("xlt") || fileExtension.endsWith("xlm") || fileExtension.endsWith("xlk"))
      return "application/excel";
    else if (fileExtension.endsWith("doc") || fileExtension.endsWith("dot"))
      return "application/msword";
    else if (fileExtension.endsWith("mdb"))
      return "application/msaccess";
    else if (fileExtension.endsWith("ppt"))
      return "application/mspowerpoint";
    else if (fileExtension.endsWith("mpx"))
      return "application/msproject";

// apps
    else if (fileExtension.endsWith("zip"))
      return "application/zip";
    else if (fileExtension.endsWith("fh10") || fileExtension.endsWith("fh9"))
      return "image/x-freehand";

// html xml
    else if (fileExtension.endsWith("htm") || fileExtension.endsWith("html") || fileExtension.endsWith("shtml"))
      return "text/html";
    else if (fileExtension.endsWith("xml"))
      return "text/xml";

// plain text
    else if (fileExtension.endsWith("txt"))
      return "text/plain";

    else
      return "unknown";

  }

  public static String realURI(HttpServletRequest request) {
    String realUri = null;
    final String includeURI = ((String) request.getAttribute("javax.servlet.include.request_uri"));
    if (includeURI != null) {
      realUri = includeURI.substring(request.getContextPath().length());
    } else
      realUri = request.getRequestURI().substring(request.getContextPath().length());

    return realUri;
  }

  public static String serverURL(HttpServletRequest request) {

    String psn = ApplicationState.getApplicationSetting(SystemConstants.PUBLIC_SERVER_NAME);
    if (psn==null || psn.trim().length()==0)
      psn = request.getServerName();

    int serverPort = request.getServerPort();
    
    String psp = ApplicationState.getApplicationSetting(SystemConstants.PUBLIC_SERVER_PORT);
    if (psp!=null && psp.trim().length()>0)
      try {
        serverPort = Integer.parseInt(psp);
      } catch (NumberFormatException e) {
        Tracer.platformLogger.error("PUBLIC_SERVER_PORT global config is not a number: "+psp);
      }

    return request.getRequestURL().substring(0, request.getRequestURL().indexOf("//") + 2) + psn + ":" + serverPort + request.getContextPath();    
  }

  /**
   *
   * @param email
   * @return true if the param indicates a valid email address
   */
  public static boolean checkValidEmail(String email) {
    Pattern pattern = Pattern.compile("^[\\w-\\.]{1,}\\@([\\da-zA-Z-]{1,}\\.){1,}[\\da-zA-Z-]{2,4}$");
    Matcher matcher = pattern.matcher(email);
    return matcher.matches();
  }
}
