package org.jblooming.utilities;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.agenda.Period;
import org.jblooming.ontology.Identifiable;
import org.jblooming.ontology.Lookup;
import org.jblooming.ontology.PersistentFile;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.system.SystemConstants;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.constants.I18nConstants;
import org.jblooming.waf.html.core.JspIncluder;
import org.jblooming.waf.html.display.FeedbackError;
import org.jblooming.waf.html.layout.Css;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.NumberFormat;
import java.util.Date;

//import org.jblooming.utilities.SmileyUtilities;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class JSP {

  public static String w(String s) {
    if (s == null)
      return "";
    else
      return s;
  }

  public static String wNice(String s, PageContext pageContext) {
    //todo: recognise html links
    return SmileyUtilities.getTextWithSmileys(JSP.w(s), pageContext);
  }


  public static String w(Period p) {
    if (p == null)
      return "";
    else
      return DateUtilities.dateToHourMinutes(p.getStartDate()) + "-" + DateUtilities.dateToHourMinutes(p.getEndDate());
  }


  public static String w(String s, PageState pageState) {
    return pageState.getI18n(w(s));
  }

  public static String w(Date d) {
    if (d != null)
      return DateUtilities.dateToString(d);
    else
      return "";
  }

  public static String wDisplayDate(Date d) {
     if (d != null)
       return DateUtilities.dateToString(d, ApplicationState.getApplicationSetting(SystemConstants.DEFAULT_DATE_DISPLAY_FORMAT));
     else
       return "";
   }


  public static String w(Date d, String format) {
    if (d != null)
      return DateUtilities.dateToString(d, format);
    else
      return "";
  }

  public static String w(Date d, PageState pageState) {
    if (d != null)
      return DateUtilities.dateToString(d, pageState.sessionState);
    else
      return "";
  }

  public static String perc(double d) {
    if (Double.isNaN(d))
      d = 0;
    NumberFormat nf = NumberFormat.getNumberInstance();
    nf.setMaximumFractionDigits(2);
    return nf.format(d);
  }

  public static String perc(double d, int fractionDigits) {
    if (Double.isNaN(d))
      d = 0;
    NumberFormat nf = NumberFormat.getNumberInstance();
    nf.setMaximumFractionDigits(fractionDigits);
    return nf.format(d);
  }


  public static String noNull(String s) {
    if (s.indexOf("null") == -1)
      return s;
    else
      return "";
  }


  public static String w(Serializable s) {

    if (s != null) {
      return w(s.toString());
    } else
      return "";
  }

  public static String w(boolean t, PageState pageState) {
    if (t)
      return pageState.getI18n(I18nConstants.YES);
    else
      return pageState.getI18n(I18nConstants.NO);
  }

  public static String w(PersistentFile pf) {
    if (pf != null)
      return pf.getFileLocation();
    else
      return "";
  }

  public static String wId(Identifiable obj) {
    if (PersistenceHome.NEW_EMPTY_ID.equals(obj.getId()))
      return "NEW";
    else
      return w(obj.getId());
  }

  public static void feedbackError(ClientEntry statefulCE, PageContext pageContext) {
    feedbackError(statefulCE, true, pageContext);
  }

  public static void feedbackError(ClientEntry statefulCE, boolean translateError, PageContext pageContext) {
    FeedbackError fe = new FeedbackError();
    if (statefulCE != null) {
      fe.suggestedValue = statefulCE.suggestedValue;
      fe.errorCode = statefulCE.errorCode;
    }
    fe.translateError = translateError;
    fe.toHtml(pageContext);
  }

  /**
   * Metodo per mettere qualcosa all'interno di un tag html
   *
   * @param tagName  nome del tag (es. "font")
   * @param tagParam parametri da inserire nel tag (es. color="#ff0000" size="3" )
   * @param text     testo
   */
  public static String makeTag(String tagName, String tagParam, String text) {
    StringBuffer sb = new StringBuffer(512);

    sb.append(tagName == null ? "" : "<" + tagName).append(' ');
    sb.append(tagParam == null ? "" : tagParam);
    sb.append(tagName == null ? "" : '>');
    sb.append(text == null ? "&nbsp;" : text);
    sb.append(tagName == null ? "" : "</" + tagName + ">");
    return sb.toString();
  }

  /**
   * Metodo per mettere qualcosa all'interno di un <td></td>
   * String content the reqired content
   */
  public static String cellContent(Object object) {
    if (object != null) {
      String content = object.toString().trim();
      if (content.length() > 0)
        return content;
    }
    return "&nbsp;";
  }

  public static String limWr(String content, int maxLength) {
    if (content != null) {
      if (content.length() > maxLength)
        content = content.substring(0, maxLength) + "...";
      return content;
    } else
      return "";
  }

  public static String addObjectId(String url, String objectId) {
    return url + '&' + Fields.OBJECT_ID + '=' + objectId;
  }

  public static String addCommand(String url, String commandValue) {
    return url + '&' + Commands.COMMAND + '=' + commandValue;
  }

  public static String encode(String s) {
    return htmlEncodeTag(htmlEncodeApexes(s));
  }

  public static String urlEncode(String s) {
    try {
      return URLEncoder.encode(s,"UTF8");
    } catch (UnsupportedEncodingException e) {
      throw new PlatformRuntimeException(e);
    }
  }

  public static final String htmlEncodeApexesAndTags(String source) {
    return htmlEncodeTag(htmlEncodeApexes(source));
  }

  public static final String htmlEncodeApexes(String source) {
    if (source != null) {
      String result = StringUtilities.replaceAllNoRegex(source, new String[]{"\"", "'"}, new String[]{"&quot;", "&#39;"});
      return result;
    } else
      return null;
  }

  public static final String htmlEncodeTag(String source) {
    if (source != null) {
      String result = StringUtilities.replaceAllNoRegex(source, new String[]{"<", ">"}, new String[]{"&lt;", "&gt;"});
      return result;
    } else
      return null;
  }

  public static String removeLineFeed(String text) {

    if (text != null)
      return StringUtilities.replaceAllNoRegex(text, new String[]{"\n", "\f"}, new String[]{" ", " "});
    else
      return null;
  }

  /**
   * use to display in html a text area content
   */
  public static String convertLineFeedToBR(String text) {
    if (text != null)
      return StringUtilities.replaceAllNoRegex(text, new String[]{"\n", "\f"}, new String[]{"<br>", "<br>"});
    else
      return null;
  }

  /**
   * codifica il contenuto di str secondo la codifica per le costanti stringa di Javascript
   * da usare per passare valori a script javascript es: onclick="alert('< % =javascriptEncoder(variabile) % >')"
   */
  public static final String javascriptEncode(String source) {
    if (source != null)
      return StringUtilities.replaceAllNoRegex(source, new String[]{"\"", "'", "’"}, new String[]{"\\\"", "\\'", "\\'"});
    else
      return null;
  }

  public static String w(Identifiable o) {
    if (o == null)
      return w("");
    else
      return w(o.getName());
  }

  public static String w(Object o) {
    if (o == null)
      return w("");
    else
      return w(o + "");
  }

  public static String writeId(Identifiable i) {
    if (i != null)
      return i.getId().toString();
    else
      return "";
  }

  public static String writeName(Identifiable i) {
    if (i != null) {
      return i.getName();
    } else
      return "&nbsp;";
  }

  public static String writeDescription(Lookup i) {
    if (i != null) {
      return i.getDescription();
    } else
      return "&nbsp;";
  }

  public static String brVerify(String text) {
    if (text != null && text.trim().length() > 0 && !"<br>".equals(text.toLowerCase())) {
      return text;
    } else {
      return null;
    }
  }

  public static String boolToHtml(boolean b) {

    if (b)
      return Fields.TRUE;
    else
      return Fields.FALSE;
  }

  public static String b(String s) {
    return makeTag("b", "", w(s));
  }

  public static String i(String s) {
    return makeTag("i", "", w(s));
  }


  /**
   * @param what          string to indent
   * @param level         is the level
   * @param pixelForLevel how many pixel for each level
   * @return the string wrapped in a table with the
   */
  public static String indent(String what, int level, int pixelForLevel) {
    return "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\"><tr><td width=\"" + level * pixelForLevel + "\">&nbsp;</td><td>" + what + "</td></tr></table>";
  }

  public static void indent(JspIncluder what, int level, int pixelForLevel, PageContext pageContext) throws IOException, ServletException {
    String indent = indent("||||", level, pixelForLevel);
    pageContext.getOut().write(indent.substring(0, indent.indexOf("||||")));
    what.toHtml(pageContext);
    pageContext.getOut().write(indent.substring(indent.indexOf("||||") + 4));

  }

  public static String currency(Number amount) {
    return NumberUtilities.currency(amount);
  }


  public static String timeStamp(Date when) {
    return DateUtilities.dateToTimestampString(when);
  }

  public static String cleanHTML(String s) {
    if (s != null) {
      StringBuffer result = new StringBuffer();
      int pos = 0;
      String sLower = s.toLowerCase();
      while (pos < s.length()) {

        char c = s.charAt(pos);
        String nextTen = sLower.substring(pos, Math.min(s.length(), pos + 10));
        if (nextTen.startsWith("<script"))
          pos = moveToMarkertEnd(pos, "</script>", sLower);
        else if (nextTen.startsWith("<!--"))
          pos = moveToMarkertEnd(pos, "-->", sLower);
        else if (nextTen.startsWith("<"))
          pos = moveToMarkertEnd(pos, ">", sLower);
        else {
          result.append(c);
          pos++;
        }
      }

      return result.toString();
    } else
      return "";
  }

  private static int moveToMarkertEnd(int pos, String marker, String s) {
    int i = s.indexOf(marker, pos);
    if (i > -1)
      pos = i + marker.length();
    else
      pos = s.length();
    return pos;
  }

  public static final String r(String source, String search, String replace) {
    return StringUtilities.replaceAllNoRegex(source,search,replace);
  }


  public static String wHelp(String text) {
    return "<span class=\""+ Css.descriptiveElement+"\">"+text+"</span>";
  }

  public static boolean ex(String contents) {
    return contents!=null && contents.trim().length()>0;
  }
}
