package org.jblooming.waf;

import org.jblooming.ApplicationException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.system.SystemConstants;
import org.jblooming.tracer.Tracer;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.html.container.Container;
import org.jblooming.waf.html.core.JspIncluder;
import org.jblooming.waf.html.state.ScreenElementStatus;
import org.jblooming.waf.settings.I18n;
import org.jblooming.waf.settings.businessLogic.I18nController;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 *         Date: 14-feb-2006 : 18.01.30
 */
public class DefaultCommandController implements ActionController{


  public PageState perform(HttpServletRequest request, HttpServletResponse response) throws PersistenceException, ActionException, org.jblooming.security.SecurityException, ApplicationException, IOException {

    SessionState sm = SessionState.getSessionState(request);
    boolean debug = false;
    PageState pageState = PageState.getCurrentPageState(request);

    try {

      if (pageState != null && pageState.getCommand() != null) {
        String command = pageState.getCommand();
        if (debug) {
          Tracer.platformLogger.info(" System.out commands.jsp: " + command + "\n" + pageState.getClientEntries().toString());
        }

        String id = pageState.getEntry(SystemConstants.FLD_DOM_ID).stringValueNullIfEmpty();

        if (command.equals(Commands.UPDATE_WIN_SIZE)) {

          ClientEntry ceW = pageState.getEntry(OperatorConstants.FLD_PAGE_WIDTH);
          ClientEntry ceH = pageState.getEntry(OperatorConstants.FLD_PAGE_HEIGHT);
          if (ceW.stringValue() != null && ceH.stringValue() != null) {
            String width = ceW.stringValue();
            String height = ceH.stringValue();
            try {
            pageState.getSessionState().setPageHeight(Integer.parseInt(height));
            pageState.getSessionState().setPageWidth(Integer.parseInt(width));
            } catch (Exception e) {
              Tracer.platformLogger.warn(e);
            }
          }
        } else if (command.equals(Commands.CMD_MOVE)) {

          if (!id.startsWith(JspIncluder.DOM_ID)) {
            String x = pageState.getEntry(SystemConstants.X).stringValueNullIfEmpty();
            String y = pageState.getEntry(SystemConstants.Y).stringValueNullIfEmpty();
            ScreenElementStatus cs = sm.screenElementsStatus.get(id);
            if (cs == null)
              cs = new ScreenElementStatus(id);
            cs.x = x;
            cs.y = y;
            sm.screenElementsStatus.put(id, cs);
          }
        } else if (command.equals(Commands.CMD_COLLAPSE)) {

          //we save only containers whose id is semantical, i.e. specified by hand
          if (!id.startsWith(JspIncluder.DOM_ID)) {
            ScreenElementStatus cs = sm.screenElementsStatus.get(id);
            if (cs == null)
              cs = new ScreenElementStatus(id);
            cs.status = Container.COLLAPSED;
            sm.screenElementsStatus.put(id, cs);
          }

        } else if (command.equals(Commands.CMD_RESTORE)) {

          if (!id.startsWith(JspIncluder.DOM_ID)) {
            ScreenElementStatus cs = sm.screenElementsStatus.get(id);
            if (cs == null)
              cs = new ScreenElementStatus(id);
            cs.status = Container.DEFAULT;
            sm.screenElementsStatus.put(id, cs);
          }
        } else if (command.equals(Commands.CMD_HIDE)) {

          if (!id.startsWith(JspIncluder.DOM_ID)) {
            ScreenElementStatus cs = sm.screenElementsStatus.get(id);
            if (cs == null)
              cs = new ScreenElementStatus(id);
            cs.status = Container.HIDDEN;
            sm.screenElementsStatus.put(id, cs);
          }

        } else if (command.equals(Commands.CMD_RESIZE)) {

          if (!id.startsWith(JspIncluder.DOM_ID)) {
            ScreenElementStatus cs = sm.screenElementsStatus.get(id);
            if (cs == null)
              cs = new ScreenElementStatus(id);
            String w = pageState.getEntry(SystemConstants.W).stringValueNullIfEmpty();
            String h = pageState.getEntry(SystemConstants.H).stringValueNullIfEmpty();
            cs.w = w;
            cs.h = h;

            sm.screenElementsStatus.put(id, cs);
          }

        } else if (command.equals(I18n.CMD_STORE_LABEL)) {
          new I18nController().perform(request, response);
        }
      }
    } catch (Exception e) {
      Tracer.platformLogger.error(e);
    }
    return pageState;
  }
}
