package org.jblooming.waf;

import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.operator.Operator;
import org.jblooming.persistence.PersistenceSession;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.tracer.Tracer;
import org.jblooming.transaction.Transaction;
import org.jblooming.transaction.hibernate.HibernateTransaction;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.PageState;
import org.jblooming.waf.view.ViewHlp;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.ParseException;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;


public class FrontControllerFilter implements Filter {

  public static Set<String> notMonitoredPaths = new HashSet();

  //issue_ELE_3
  private static String SETTINGS_PAGE_PATH_FROM_ROOT = "/commons/settings/settings.jsp";
  public static String ERROR_PAGE_PATH_FROM_ROOT = "/commons/administration/error.jsp";
  public static final String page = ".page";

  public void init(FilterConfig config) throws ServletException {

    //linux no x-windows compatibility attempt
    System.setProperty("java.awt.headless", "true");

    /*String value = config.getInitParameter("SETTINGS_PAGE_PATH_FROM_ROOT");
    if (value != null && value.trim().length() > 0)
      FrontControllerFilter.SETTINGS_PAGE_PATH_FROM_ROOT = value;

    value = config.getInitParameter("ERROR_PAGE_PATH_FROM_ROOT");
    if (value != null && value.trim().length() > 0)
      FrontControllerFilter.ERROR_PAGE_PATH_FROM_ROOT = value;
*/
  }

  public void destroy() {
  }

  void synchronizedLoadSettings(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    if (!ApplicationState.loaded)
      synchronized (this) {
        if (!ApplicationState.loaded) {
          request.getRequestDispatcher(SETTINGS_PAGE_PATH_FROM_ROOT + "?id=" + (new Date()).getTime()).include(request, response);
        }
      }
  }

  /**
   * In some cases this filter is skipped:
   * <ol>
   * <li><i>.gif, .js, .css</i>: loading settings and building page state has no useful goal, it can only lead to malfunctions.
   * <li><i>schemaanddata.jsp, register.jsp, persistence.jsp</i>: introduced to guarantee right behavior of the installation
   * procedure.
   * <li><i>index.jsp</i>: it's always called without parameters and simply redirects to operator/home.jsp: the actions of this
   * filter would simply be a waste of time.
   * <li><i>request.getServletPath().trim().length()>0</i>: when the user enters the application without asking for particular
   * pages, this filter would try to build a page state (=> NullPointerException because no jsp would be found in request)
   * </ol>
   *
   * @param req
   * @param res
   * @param chain
   * @throws IOException
   * @throws ServletException
   */
  public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {

    HttpServletRequest request = (HttpServletRequest) req;
    HttpServletResponse response = (HttpServletResponse) res;

    String charset = "utf-8";
    request.setCharacterEncoding(charset);
    // in case of use with tomcat 4.1
    // response.setCharacterEncoding non supported, it must be replaced with response.setContentType
    //response.setContentType("charset=\"utf-8\"");
    response.setCharacterEncoding(charset);

    final String resource = request.getServletPath().toLowerCase();

    if (resource.equalsIgnoreCase("") || resource.equalsIgnoreCase("/") || resource.toUpperCase().endsWith(".JSP") || resource.toUpperCase().endsWith(page.toUpperCase())) {

      if (ApplicationState.platformConfiguration != null && ApplicationState.platformConfiguration.development) {
        request.setAttribute("time", System.currentTimeMillis());
      }

      String pageStateName = "pageState name unknown";
      String pageStateNameHolder[] = new String[]{pageStateName};
      boolean tCommitted = false;

      try {

        boolean doChain = true;
        synchronizedLoadSettings(request, response);

        //loggable, audit trail and subscribe enaction
        SessionState sessionState = SessionState.getSessionState(request);

        if (sessionState != null && ApplicationState.platformConfiguration.loadedBasicPersistentSettings) {
          HibernateFactory.getSession();
          PersistenceSession persistenceSession = HibernateFactory.getPersistenceSession();
          if (persistenceSession != null) {
            Operator loggedOperator = sessionState.getLoggedOperator();
            persistenceSession.operator = loggedOperator;
            if (loggedOperator!=null){
              loggedOperator.setLastRequestOn(new Date());
            }
          }
        }

        if (request.getRequestURI().toUpperCase().endsWith(".JSP") || request.getRequestURI().toUpperCase().endsWith(page.toUpperCase())) {
          try {
            doChain = perform(request, response, pageStateNameHolder);
          } catch (ApplicationException e) {
            throw new RuntimeException(e);
          }
        }

        if (doChain) {
          chain.doFilter(req, response);

          // todo remove the comment or the code. No way! the Query is associated to the closed session
          /*
          // copy LAST_SEARCH_PAGE in session
          PageState pageState=PageState.getCurrentPageState(request);
          if (pageState.getPage()!=null)
            pageState.sessionState.setAttribute("LAST_SEARCH_PAGE",pageState.getPage());
          */

        }

        Transaction t = HibernateTransaction.getCurrentTransaction();
        if (t != null) {
          if (t.isFailed())
            t.abort();
          else
            t.commit();
        }
        tCommitted = true;

      } catch (Throwable e) {

        Tracer.platformLogger.error(e);
        Transaction t = HibernateTransaction.getCurrentTransaction();
        if (t != null)
          t.abort();
        tCommitted = true;

        if (e instanceof PlatformRuntimeException)
          throw (PlatformRuntimeException) e;
        else
          throw new PlatformRuntimeException(e);

      } finally {
        if (!tCommitted) {
          Transaction t = HibernateTransaction.getCurrentTransaction();
          if (t != null)
            t.abort();
        }
      }
    } else
      chain.doFilter(request, res);
  }

  protected boolean perform(HttpServletRequest request, HttpServletResponse response, String[] pageStateNameHolder)
    throws ServletException, IOException, ApplicationException {

    SessionState sessionState = SessionState.getSessionState(request);
    PageState pageState = sessionState.getPageState(request, response);
    /**
     *   This is a workaround for the immutability of Strings: this page name is returned in case of error.
     */
    pageStateNameHolder[0] = pageState.getName();

    final String href = pageState.getHref();
    if (Tracer.viewDebug) {
      if (notMonitoredPaths.size() > 0) {
        boolean doNotMonitor = false;
        for (String notMonitoredPath : notMonitoredPaths) {
          String s = (String) notMonitoredPath;
          doNotMonitor = href.toLowerCase().indexOf(s.toLowerCase()) > -1;
          if (doNotMonitor)
            break;
        }
        if (!doNotMonitor) {

          if (pageState.getHref().indexOf("part") == -1 && Tracer.clearDebugRootView)
            Tracer.resetTrace(request);
          Tracer.addTrace(ViewHlp.viewBeforeDispatchDebug(pageState, request), request);
        }
      }
    }

    if (Commands.BACK_TO.equals(pageState.getCommand())) {
      try {
        return sessionState.pageHistory.goBackTo(request, response, pageState);
      } catch (ParseException e) {
        throw new PlatformRuntimeException(e);
      } catch (ActionException e) {
        throw new PlatformRuntimeException(e);
      }
    } else if (Commands.BACK.equals(pageState.getCommand())) {
      return sessionState.pageHistory.goBack(request, response);

/*    } else if (

        !pageState.isPopup() &&
        (Commands.EDIT.equals(pageState.getCommand()) ||
        Commands.FIND.equals(pageState.getCommand())) //||
        //remmed as it may save an old image of the object
        //(Commands.SAVE.equals(pageState.getCommand()) && pageState.validEntries())
      ) {
      pageState.saveInHistory();

      return true;
*/
    } else {
      return true;
    }

  }

  /*protected void errorDisplay(Throwable e, String errorKind, String pageStateName, HttpServletRequest request, HttpServletResponse response)
          throws IOException, ServletException {

    if (ApplicationState.getApplicationSetting(SystemConstants.FLD_FRIENDLY_ERRORS) != null &&
            ApplicationState.getApplicationSetting(SystemConstants.FLD_FRIENDLY_ERRORS).equals(Fields.TRUE)) {
      Tracer.printTrace(e);
      request.setAttribute("errorMessage", pageStateName + '-' + errorKind);
      request.setAttribute("platformError", e);
      request.getRequestDispatcher(ERROR_PAGE_PATH_FROM_ROOT).forward(request, response);
      return;
    } else {
      e.printStackTrace();
      if (e instanceof PlatformRuntimeException)
        throw (PlatformRuntimeException) e;
      else
        throw new PlatformRuntimeException(e);
    }
  }*/
}
