package org.jblooming.waf;

import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.cache.SessionCache;
import org.jblooming.cursor.Cursor;
import org.jblooming.http.multipartfilter.UploadListener;
import org.jblooming.ontology.Identifiable;
import org.jblooming.operator.Operator;
import org.jblooming.operator.User;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.security.Permission;
import org.jblooming.tracer.Tracer;
import org.jblooming.transaction.hibernate.HibernateTransaction;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.HashTable;
import org.jblooming.utilities.ReflectionUtilities;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.html.layout.Skin;
import org.jblooming.waf.html.state.ScreenElementStatus;
import org.jblooming.waf.settings.Application;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.state.PageHistory;
import org.jblooming.waf.view.ClientEntries;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionBindingEvent;
import java.util.*;


public class SessionState implements UploadListener, javax.servlet.http.HttpSessionBindingListener {
  
  private Map attributes = new HashTable();

  private PageSeed loginPendingUrl;

  public Stack<ClipProxy> clipboard = new Stack<ClipProxy>();
  private long dataTransferred;
  private long dataToTransfer;
  private long dataTransferTotalTime;

  public PageHistory pageHistory = new PageHistory();

  /**
   * @deprecated use pageHistory
   */
  public Map<String, ClientEntries> recentViews = new HashTable();

  /**
   * Used to aggregate client entries from several form submissions
   * Use in wizards, pages with large tabsets,... .
   * Key is URI.
   */
  public Map<String, ClientEntries> formsUnion = new HashTable();

  private Object[] performerData;

  private int opId = -1;
  private Locale currentLocale;
  private int pageWidth;
  private int pageHeight;
  private String browser = null;

  public String mainActionToken;
  public String iframeActionToken;
  public String monitorToken;

  private String currentCounterAction;
  public boolean resizeingTodo = true;

  private SessionCache sc = new SessionCache();

  public Set noResubmitUrls = new HashSet();

  private Skin skin;

  public Map<String, ScreenElementStatus> screenElementsStatus = new HashMap<String, ScreenElementStatus>();

  public static int totalSessionStates = 0;
  public static Map<Integer, Operator> totalOpIds = new HashMap<Integer, Operator>();

  public static int DEFAULT_PAGE_SIZE = 8;

  private Application application;

  public SessionState() {

  }

  public int getDefaultPageSize() throws FindByPrimaryKeyException {
    int height = getPageHeight();
    if (height > 250)
      return ((height - 200) / 35);
    else
      return 2;
  }

  public PageSeed getLoginPendingUrl() {
    return loginPendingUrl;
  }

  public void setLoginPendingUrl(PageSeed loginPendingUrl) {
    this.loginPendingUrl = loginPendingUrl;
  }

  public void setAttribute(Object key, Object object) {
    if (getAttributes() == null) {
      setAttributes(new HashTable());
    }
    if (object == null)
      getAttributes().remove(key);
    else
      getAttributes().put(key, object);
  }

  public Object getAttribute(Object key) {
    if (getAttributes() == null) {
      setAttributes(new HashTable());
    }
    return getAttributes().get(key);
  }

  public Locale getCurrentLocale() {
    if (currentLocale == null)
      try {
        String operatorOption = getOperatorOption(OperatorConstants.FLD_SELECT_LANG);
        if (operatorOption != null)
          currentLocale = new Locale(operatorOption);
        else
          currentLocale = DateUtilities.DEFAULT_LOCALE;
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      }
    return currentLocale;
  }

  public boolean isOperatorLogged() {
    return (opId != -1);
  }

  /**
   * @deprecated use getLoggedOperator
   */
  public Operator getLogged() throws PersistenceException {
    return getLoggedOperator();
  }

  /**
   * @deprecated use getLoggedOperator
   */
  public User getLogged(HttpServletRequest request) throws PersistenceException {
    return getLoggedOperator(request);
  }

  /**
   * @deprecated
   */
  public void setCursor(String pageName, Cursor c) {
    if (attributes == null) {
      attributes = new HashMap();
    }
    if (c == null)
      attributes.remove(Fields.PAGER_PREFIX + pageName);
    else
      this.setAttribute(Fields.PAGER_PREFIX + pageName, c);
  }

  /**
   * @deprecated
   */
  public Cursor getCursor(String cursorName) {
    try {
      Cursor cursor = (Cursor) this.getAttribute(Fields.PAGER_PREFIX + cursorName);

      return cursor;
    } catch (Exception e) {
      return null;
    }
  }

  public Map getAttributes() {
    return attributes;
  }

  public void setAttributes(Map attributes) {
    this.attributes = attributes;
  }

  /**
   * @deprecated use skin
   */
  public String getPathToImages() {
    return getSkin().imgPath;
  }

  public void clipboardPush(Identifiable o) {
    clipboard.push(new ClipProxy(o.getId(), ReflectionUtilities.getUnderlyingObjectClass(o)));
  }

  public Identifiable clipboardPop() throws FindByPrimaryKeyException {
    ClipProxy cp = clipboard.pop();
    return PersistenceHome.findByPrimaryKey(cp.clazz, cp.id);
  }

  public Identifiable clipboardPeek() throws FindByPrimaryKeyException {
    ClipProxy cp = clipboard.peek();
    return PersistenceHome.findByPrimaryKey(cp.clazz, cp.id);
  }

  public synchronized void setDataTransferred(long dataTransferred) {
    this.dataTransferred = dataTransferred;
  }

  public long getDataToTransfer() {
    return dataToTransfer;
  }

  public synchronized void setDataToTransfer(long dataToTransfer) {
    this.dataToTransfer = dataToTransfer;
  }

  public long getDataTransferTotalTime() {
    return dataTransferTotalTime;
  }

  public void setDataTransferTotalTime(long l) {
    dataTransferTotalTime = l;
  }

  /*
  public void addRecentView(String title, PageSeed view) throws PersistenceException {
    addRecentView(title, view, false);
  }*/

  /**
   * The objects are pairs (CodeValue) name, url.
   */
  /*public void addRecentView(String title, PageSeed view, boolean replaceOccurring) throws PersistenceException {

    mostRecentTitle = title;
    mostRecentFullUrl = view.getNewInstance();

    if (getRecentViews() == null)
      setRecentViews(new LinkedList());
    boolean occurring = false;
    for (int i = 0; i < getRecentViews().size(); i++) {
      ArrayList l = (ArrayList) getRecentViews().get(i);
      if (l.get(0).equals(title)) {
        if (replaceOccurring)
          getRecentViews().remove(i);
        else
          occurring = true;
        break;
      }
    }

    if (!occurring) {
      List l = new ArrayList();
      l.add(title);
      l.add(view);
      getRecentViews().addFirst(l);
      if (getRecentViews().size() > getRecentViewsSize())
        getRecentViews().removeLast();
    }
  }

  private int getRecentViewsSize() throws PersistenceException {
    return Integer.parseInt(getOperatorOption(OperatorConstants.FLD_OPERATOR_RECENT_VIEWS_SIZE));
  }

  public PageSeed getMostRecentView() {
    if (recentViews != null)
      return (PageSeed) ((List) recentViews.getFirst()).get(1);
    return null;
  }*/
  public Object[] getPerformerData() {
    return performerData;
  }

  public void setPerformerData(Object[] os) {
    this.performerData = os;
  }

  public Operator getLoggedOperator() throws PersistenceException {
    if (opId != -1)
      return (org.jblooming.operator.Operator) sc.getCachedUser(opId);
    else
      return null;
  }

  /**
   * @deprecated use getLoggedOperator
   */
  public org.jblooming.operator.Operator getLoggedOperator(HttpServletRequest request) throws PersistenceException {
    return getLoggedOperator();
  }

  public void setLoggedOperator(Operator op) throws PersistenceException, ApplicationException {
    this.opId = (Integer) op.getId();
    sc.operatorClass = ReflectionUtilities.getUnderlyingObjectClass(op);
    refreshLoggedOperator();
    totalOpIds.put(opId, getLoggedOperator());
  }

  public void resetLoggedOperator() throws PersistenceException {
    opId = -1;
    sc.resetCachedUser(opId);
  }

  public String getOperatorOption(String name) throws PersistenceException {
    Operator op = null;
    try {
      op = getLoggedOperator();
    } catch (FindByPrimaryKeyException e) {
    }

    return getOperatorOption(op, name);
  }

  public static String getOperatorOption(org.jblooming.operator.Operator op, String name) {

    boolean found = false;
    String result = null;
    if (op != null) {
      Object o = op.getOption(name);
      if (o != null) {
        result = (String) o;
        found = true;
      }
    }
    if (!found)
      result = ApplicationState.getApplicationSetting(name);

    return result;
  }

  public void setCurrentCounterAction(String action) {
    this.currentCounterAction = action;
  }

  public String getCurrentCounterAction() {
    if (this.currentCounterAction == null)
      return null;
    String r = new String(this.currentCounterAction);
    return r;
  }

  public void setCurrentLocale(Locale currentLocale) {
    this.currentLocale = currentLocale;
  }

  public int getPageWidth() {
    return pageWidth;
  }

  public void setPageWidth(int pageWidth) {
    this.pageWidth = pageWidth;
  }

  public int getPageHeight() {
    return pageHeight;
  }

  public void setPageHeight(int pageHeight) {
    this.pageHeight = pageHeight;
  }

  public Collection getLoggedOperatorAreas(Permission permission) throws PersistenceException {
    return null;
  }

  /**
   * reloads operator FROM db
   */
  public void refreshLoggedOperator() throws PersistenceException, ApplicationException {
    sc.resetCachedUser(opId);
    String langAndCountry = getOperatorOption(OperatorConstants.FLD_SELECT_LANG);
    if (langAndCountry==null)
      langAndCountry = "EN";
    int sep = langAndCountry.indexOf("_");
    Locale loc = null;
    if (sep > 0) {
      loc = new Locale(langAndCountry.substring(0, sep), langAndCountry.substring(sep + 1));
    } else
      loc = new Locale(langAndCountry);

    setCurrentLocale(loc);
  }

  public static SessionState getSessionState(HttpServletRequest request) {

    HttpSession session = request.getSession(true);
    SessionState stateMachine = (SessionState) session.getAttribute(Fields.SESSION);

    if (stateMachine == null) {
      stateMachine = new SessionState();
      if (session != null)
        session.setAttribute(Fields.SESSION, stateMachine);
      // fill the browser type
      stateMachine.setBrowser(request.getHeader("User-Agent").toUpperCase());
      try {
        stateMachine.setSkinForApplicationIfNull();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);

      }
    }

    if (stateMachine.getCurrentLocale() == null) {
      stateMachine.setCurrentLocale(request.getLocale());
    }

    return stateMachine;
  }

  public static Skin createSkin(String contextPath, String skinDefaultName, String applicationRootFolder) {
    Skin skin = new Skin();
    skin.name = skinDefaultName;
    skin.partsPath = "/commons/layout/";
    skin.imgPath = contextPath+ "/commons/skin/" + skinDefaultName + "/images/";
    skin.css = contextPath+ "/commons/skin/" + skinDefaultName + "/css/";
    skin.setAppImgPath(applicationRootFolder, contextPath);
    return skin;
  }

  public void setSkinForApplicationIfNull() throws PersistenceException {
    String currSkin = ApplicationState.getApplicationSetting(OperatorConstants.FLD_CURRENT_SKIN);
    setSkinForApplicationIfNull(
            ApplicationState.contextPath,
            currSkin,
            getApplication().getRootFolder());
  }

  public Skin setSkinForApplicationIfNull(String contextPath, String skinDefaultName, String applicationRootFolder) throws PersistenceException {
    
    String skinNameFromOptions = getLoggedOperator() != null ? getOperatorOption(OperatorConstants.FLD_CURRENT_SKIN) : null;

    if (skin == null || skin.imgPathPlus.indexOf(applicationRootFolder) == -1 || !skin.name.equals(skinNameFromOptions))
    {
      if (skinNameFromOptions != null)
        setSkin(SessionState.createSkin(contextPath, skinNameFromOptions, applicationRootFolder));
      else
        setSkin(SessionState.createSkin(contextPath, skinDefaultName, applicationRootFolder));
    }
    return skin;
  }

  public Skin getSkin() {
    return skin;
  }

  public void setSkin(Skin skin) {
    this.skin = skin;
  }

  public PageState getPageState(HttpServletRequest request, HttpServletResponse response) {

    String url = request.getServletPath();
    PageState pageState = (PageState) request.getAttribute(Fields.VIEW);
    if (pageState == null) {
      pageState = new PageState(url, this);

/*
      String withoutContextPath = request.getRequestURI().substring(request.getContextPath().length());
      int dots = StringUtilities.count(withoutContextPath.substring(1),'/');
      StringBuffer pisser = new StringBuffer();
      for (int i = 0; i < dots; i++) {
        pisser.append("../");
      }
      pageState.relativeUrlToRoot = pisser.toString();
*/

/*
      //put application in which it runs
      Collection apps = ApplicationState.platformConfiguration.applications.values();

      if (apps != null && apps.size() > 0) {
        for (Iterator iterator = apps.iterator(); iterator.hasNext();) {
          Application application = (Application) iterator.next();

          String normalized = StringUtilities.replaceAllNoRegex(HttpUtilities.getCanonicalFileSystemPathOfPartFromURI(request), "\\", "/");
          final String approot = StringUtilities.replaceAllNoRegex(request.getContextPath() + File.separator + application.getRootFolder(), "\\", "/").toLowerCase();
          if (normalized.indexOf(approot) > -1) {
            pageState.setApplication(application);
            break;
          }
        }
      }
      if (pageState.getApplication() == null)
        pageState.setApplication(getApplication());
*/
      PageState.buildPartsAndClientEntries(request, pageState);
      request.setAttribute(Fields.VIEW, pageState);
    }
    return pageState;
  }

  public Application getApplication() {
    if (application == null)
      application = ApplicationState.platformConfiguration.getDefaultApplication();
    return application;
  }

  public void setApplication(Application application) {
    this.application = application;
  }

  public void valueBound(HttpSessionBindingEvent event) {
    totalSessionStates++;
  }

  public void valueUnbound(HttpSessionBindingEvent event) {
    totalSessionStates--;
    try {

      Operator loggedOperator = this.getLoggedOperator();
      if (loggedOperator != null) {

        PersistenceContext pc = HibernateFactory.newFreeSession();

        Operator op = (Operator) PersistenceHome.findByPrimaryKey(Operator.class, loggedOperator.getId(),pc);
        final Map screenElementsStatuses = this.screenElementsStatus;
        if (screenElementsStatuses != null && screenElementsStatuses.size() > 0) {
          for (Iterator iterator = screenElementsStatuses.keySet().iterator(); iterator.hasNext();) {
            final String key = (String) iterator.next();
            ScreenElementStatus screenElementStatus = (ScreenElementStatus) screenElementsStatuses.get(key);
            op.putOption(ScreenElementStatus.SES_QUALIFIER + key, screenElementStatus.toPersistentString(key));
          }
        }
        
        op.store(pc);
        pc.commitAndClose();

        totalOpIds.remove(op.getId());
      }
    } catch (Throwable e) {
      Tracer.platformLogger.warn("Attempting to find operator in session and save it in valueUnbound - failed", e);
    }
  }

  public String getBrowser() {
    return browser;
  }

  public void setBrowser(String browser) {
    this.browser = browser;
  }

  public boolean isFirefox() {
    return getBrowser().contains("FIREFOX");
  }

  public boolean isExplorer() {
    return getBrowser().contains("MSIE");
  }

  public boolean isWindows() {
    return getBrowser().contains("WINDOWS");
  }

  public boolean isMac() {
    return getBrowser().contains("MAC");
  }

  public boolean isLinux() {
    return getBrowser().contains("LINUX");
  }

  public void setApplication(String applicationName) {
    setApplication(ApplicationState.platformConfiguration.applications.get(applicationName));
  }

  public void putOptionAndPersist(String key, String value) throws PersistenceException {
    Operator op = (Operator) PersistenceHome.findByPrimaryKey(sc.operatorClass, getLoggedOperator().getId());
    op.putOption(key, value);
    op.store();
    HibernateTransaction.getCurrentTransaction().checkPoint();
    try {
      refreshLoggedOperator();
    } catch (ApplicationException e) {
      throw new PlatformRuntimeException(e);
    }
  }

  public Operator getLoggedOperatorFromPersistence() throws PersistenceException {
    return (Operator) PersistenceHome.findByPrimaryKey(sc.operatorClass, getLoggedOperator().getId());
  }

}