package org.jblooming.waf.html.button;

import org.jblooming.ontology.LoggableIdentifiableSupport;
import org.jblooming.security.Area;
import org.jblooming.security.SecurableWithArea;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.JSP;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.html.core.UrlComposer;
import org.jblooming.waf.html.display.Img;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;

public class ButtonLink extends ButtonSupport {

  public String popup_location;
  public String popup_menubar;
  public String popup_resizable;
  public String popup_scrollbars;
  public String popup_status;
  public String popup_titlebar;
  public String popup_toolbar;
  public String popup_height;
  public String popup_left;
  public String popup_top;
  public String popup_width;


  public PageSeed pageSeed = new PageSeed();

  //in order to generate a link without any parameter
  public boolean inhibitParams = false;

  //these useful params are for preserving from the incumbent danger of html syntax changes :-D
  public static String TARGET_BLANK = "_blank";
  public static String TARGET_TOP = "_top";

  // used to ajax Link style
  private boolean ajaxEnabled = false;
  private String ajaxDomIdToReload = null;


  public ButtonLink(PageSeed ps) {
    this(null, ps);
  }

  public ButtonLink(String label, PageSeed ps) {
    super("partButton.jsp");
    this.pageSeed = ps;
    this.label = label;
  }

  public String generateLaunchJs() {

    StringBuffer sb = new StringBuffer();
    if (enabled) {
      sb.append(" onClick=\"stopBubble(event);");
      sb.append(generateJs());
      sb.append("\" ");// close onclick string
      if (additionalScript != null && additionalScript.trim().length() > 0)
        sb.append(' ').append(additionalScript);
    }
    return sb.toString();
  }

  public boolean isEnabled() {
    return enabled;
  }

  public String getCEHref() {
    Link fake = new Link(pageSeed);
    fake.outputModality = UrlComposer.OUTPUT_AS_JS_LAUNCH_CES;
    return fake.getHref();
  }

  public StringBuffer generateJs() {
    StringBuffer sb = new StringBuffer();
    if (confirmRequire) {
      sb.append("if (confirm('");
      if (confirmQuestion != null && confirmQuestion.length() > 0)
        sb.append(JSP.javascriptEncode(confirmQuestion));
      else
        sb.append("proceed?");
      sb.append("')");
      sb.append(')'); // close if parenthesis
      sb.append("{\n"); // open if block
    }

    //added by Pietro on 23/03/2006; why was it missing ?
    if (additionalOnClickScript != null)
      sb.append(additionalOnClickScript);

    // when ajax is enabled change the call
    if (!ajaxEnabled) {
      // TODO: added 
      sb.append("try{ ");
      //if (!pageSeed.isPopup())
      if (target==null)
        sb.append("window.open('");
      else
        sb.append("openCenteredWindow('");

      sb.append(pageSeed.getHref());
      if (!inhibitParams)
        sb.append('?' + getCEHref());

      sb.append("',")
              .append(target == null ? "'_self'" : '\'' + target + '\'')
              .append(",'" + getWindowFeatures() + '\'')
              .append(") } catch(e){};"); // close location string
    } else {
      sb.append("loadAsyncroContent('" + pageSeed.getHref());
      if (!inhibitParams)
        sb.append('?' + getCEHref());
      sb.append("'" + (ajaxDomIdToReload == null ? ");" : ",'" + ajaxDomIdToReload + "');"));
    }

    if (confirmRequire) {
      sb.append("}\n"); // close if block
    }


    return sb;
  }

  public String toLink() {
    return "<a href=\"#\"" + this.generateLaunchJs() + ">" + this.label + "</a>";
  }

  public String toPlainLink() {
    StringBuffer sb = new StringBuffer();

    sb.append("<a href=\"");

    sb.append(toUrlWithParams());
    sb.append("\" ");
    if (target != null)
      sb.append(" target=\"").append(target).append("\"");
    sb.append(">");
    sb.append(label);
    sb.append("</a>");
    return sb.toString();
  }

  public String toUrlWithParams() {
    StringBuffer sb = new StringBuffer();
    sb.append(pageSeed.getHref());
    if (!inhibitParams)
      sb.append('?' + getCEHref());
    return sb.toString();
  }

  private String getWindowFeatures() {

    if (target == null)
      return "";
    StringBuffer sb = new StringBuffer();
    sb.append(popup_location != null ? "location=" + popup_location + ',' : "");
    sb.append(popup_menubar != null ? "menubar=" + popup_menubar + ',' : "");
    sb.append(popup_resizable != null ? "resizable=" + popup_resizable + ',' : "");
    sb.append(popup_scrollbars != null ? "scrollbars=" + popup_scrollbars + ',' : "");
    sb.append(popup_status != null ? "status=" + popup_status + ',' : "");
    sb.append(popup_titlebar != null ? "titlebar=" + popup_titlebar + ',' : "");
    sb.append(popup_toolbar != null ? "toolbar=" + popup_toolbar + ',' : "");
    sb.append(popup_height != null ? "height=" + popup_height + ',' : "");
    sb.append(popup_left != null ? "left=" + popup_left + ',' : "");
    sb.append(popup_top != null ? "top=" + popup_top + ',' : "");
    sb.append(popup_width != null ? "width=" + popup_width + ',' : "");

    return sb.substring(0, Math.max(sb.length() - 1, 0));
  }

  public String getLabel() {
    return label;
  }


  public static void toHtmlDelete(String deletePreviewHref, int objectId, String label, PageContext pageContext) throws IOException, ServletException {
    ButtonLink bs = getDeleteInstance(deletePreviewHref, objectId, label);
    bs.toHtml(pageContext);
  }

  public static ButtonLink getDeleteInstance(String deletePreviewHref, int objectId, String label) {
    PageSeed deletePreview = new PageSeed(deletePreviewHref);
    deletePreview.setCommand(Commands.EDIT);
    deletePreview.setMainObjectId(objectId);
    ButtonLink bs = new ButtonLink(deletePreview);
    bs.label = label;
    return bs;
  }

  public static ButtonLink getTextualInstance(String label, PageSeed ps) {
    ButtonLink bl = new ButtonLink(label, ps);
    bl.outputModality = ButtonSupport.TEXT_ONLY;
    return bl;
  }

  public static ButtonLink getDescriptiveLinkInstance(String label, String href) {
    ButtonLink bl = new ButtonLink(JSP.wHelp(label), new PageSeed(href));
    bl.outputModality = ButtonSupport.TEXT_ONLY;
    bl.inhibitParams=true;
    bl.target= TARGET_BLANK;
    return bl;
  }

  public static ButtonLink getPopupTextualInstance(String label, int popup_height, int popup_width, PageSeed ps) {
    ButtonLink bl = getPopupInstance(label,popup_height,popup_width,ps);
    bl.outputModality = ButtonSupport.TEXT_ONLY;
    return bl;
  }

   public static ButtonLink getPopupInstance(String label, int popup_height, int popup_width, PageSeed ps) {
    ps.setPopup(true);
    ButtonLink bl = new ButtonLink(label, ps);
    bl.target = ButtonLink.TARGET_BLANK;
    bl.popup_height = popup_height+"";
    bl.popup_width = popup_width+"";
    bl.popup_scrollbars = "yes";
    bl.popup_resizable = "yes";    
    return bl;
  }

  public static ButtonLink getBackInstance(PageState pageState) {

    return getBackInstance(pageState.getI18n("BACK"), pageState);
  }

  public static ButtonLink getBackInstance(String label, PageState pageState) {
    PageSeed ps = new PageSeed(ApplicationState.contextPath + pageState.href);
    ps.command = Commands.BACK;
    ButtonLink buttonLink = new ButtonLink(label, ps);
    buttonLink.alertOnChange = true;
    return buttonLink;
  }


  public static ButtonLink getAjaxButton(PageSeed ps, String domIdToReload) {
    ButtonLink pl = new ButtonLink(ps);
    pl.ajaxEnabled = true;
    pl.ajaxDomIdToReload = domIdToReload;
    return pl;
  }

  public static ButtonImg getEditInstanceForList(String editPage, LoggableIdentifiableSupport editando, HttpServletRequest request) {
    PageState pageState = PageState.getCurrentPageState(request);
    PageSeed edit = pageState.pageInThisFolder(editPage,request);
    edit.mainObjectId=editando.getId();
    edit.setCommand(Commands.EDIT);
    ButtonLink editLink = ButtonLink.getTextualInstance("", edit);
    Img img = new Img(pageState.getSkin().imgPath + "list/edit.gif", getToolTipForIdentifiable(editando,pageState), "", "", true);
    img.doRollover = false;
    return new ButtonImg(editLink, img);
  }

  public static ButtonImg getDeleteInstanceForList(String editPage, LoggableIdentifiableSupport editando, HttpServletRequest request) {
    PageState pageState = PageState.getCurrentPageState(request);
    PageSeed edit = pageState.pageInThisFolder(editPage,request);
    edit.mainObjectId=editando.getId();
    edit.setCommand(Commands.DELETE_PREVIEW);
    ButtonLink editLink = ButtonLink.getTextualInstance("", edit);
    Img img = new Img(pageState.getSkin().imgPath + "list/del.gif", getToolTipForIdentifiable(editando,pageState), "", "", true);
    img.doRollover = false;
    return new ButtonImg(editLink, img);
  }

  public static String getToolTipForIdentifiable(LoggableIdentifiableSupport it, PageState pageState) {

    String separator="&nbsp;";

    StringBuffer result = new StringBuffer();
    if (it != null) {

      result.append("Database id: " + separator + it.getId()+ separator);
      if (it instanceof SecurableWithArea) {
        Area area = ((SecurableWithArea) it).getArea();
        result.append(separator+separator+pageState.getI18n("AREA") + separator);
        String name = null;
        if (area!=null)
           name = JSP.htmlEncodeApexesAndTags(area.getName());
        result.append(JSP.w(area != null ? name : pageState.getI18n("NO_AREA") ));
      }

      String creator = JSP.htmlEncodeApexesAndTags(it.getCreator());
      String lastModifier = JSP.htmlEncodeApexesAndTags(it.getLastModifier());

      result.append("<br>"+pageState.getI18n("CREATED_BY") + separator + (creator != null ? creator : "-") + separator);
      result.append(pageState.getI18n("ON_DATE") + separator + DateUtilities.dateToTimestampString(it.getCreationDate(), pageState.sessionState));
      result.append("<br>"+pageState.getI18n("LAST_MODIFIED_BY") + separator + (lastModifier != null ? lastModifier+ separator : "-" + separator));
      result.append(pageState.getI18n("ON_DATE") + separator + DateUtilities.dateToTimestampString(it.getLastModified(), pageState.sessionState) + separator);

    }
    return result.toString();
  }

  public static ButtonLink getPDFPrintButton(PageSeed ps) {
    PageSeed seed = new PageSeed(ApplicationState.contextPath + "/commons/tools/printPDF.jsp");
    seed.addClientEntry("PRINTING_PDF", Fields.TRUE);
    seed.addClientEntry("PAGE_TO_PRINT",ps.toLinkToHref());    
    return new ButtonLink(seed);
  }

}

