package org.jblooming.waf.html.button;

import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.utilities.HTMLEncoderOld;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.html.core.HtmlRenderer;
import org.jblooming.waf.html.core.UrlComposer;
import org.jblooming.waf.html.display.Img;
import org.jblooming.waf.view.ClientEntries;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageSeed;

import java.util.Iterator;
import java.util.List;


/**
 * Date: 14-feb-2003
 * Time: 16.53.53
 *
 * @author Pietro Polsinelli dev@open-lab.com
 * @deprecated
 */
public class Link extends UrlComposer {

  public SessionState ses;

  /**
   * this can be the name of the image (no extension, no path) or the string to visualise (after i18n)
   */
  public String label;
  public String target;
  public boolean enabled = true;
  public boolean textual = false;
  public String htmlClass;
  public String script;
  public int tabIndex = 0;
  public String additionalLinkText = null;
  public Img image;
  public String width;
  private boolean popup = false;
  private boolean scrollbars = false;
  private int winWidth;
  private int winHeight;
  public String title;
  public String style;


  public boolean debug;

  public Link(PageSeed url) {
    super(url);
  }

  public Link(Img image, PageSeed wantedView) {
    super(wantedView);
    this.image = image;
  }


  public Link(String label, String imageUrl, PageSeed wantedView, SessionState sess) {
    this(label, imageUrl, false, true, wantedView, sess);
  }

  public Link(String label, String imageUrl, String additionalLinkText, String imgWidth, String imgHeight, boolean isLink, PageSeed url, SessionState sess) {
    this(label, imageUrl, false, true, url, sess);

    this.additionalLinkText = additionalLinkText;
    this.image.width = imgWidth;
    this.image.height = imgHeight;
  }

  public Link(String label, String imageUrl, boolean hasFocus, boolean isEnabled, PageSeed wantedView, SessionState sess) {

    super(wantedView);
    image = new Img(imageUrl, label);
    image.disabled = !isEnabled;

    this.label = label;

    if (imageUrl == null || imageUrl.length() == 0)
      this.textual = true;

    this.enabled = isEnabled;
  }

  public Link(String label, boolean isEnabled, PageSeed wantedView) {
    super(wantedView);
    this.label = label;
    this.enabled = isEnabled;
  }


  /**
   *@deprecated use ButtonLink in TEXT_ONLY modality
   */
  public static Link getTextualInstance(String label, PageSeed url) {
    return getTextualInstance(label, true, url);
  }

  /**
   *@deprecated use ButtonLink in TEXT_ONLY modality
   */
  public static Link getTextualInstance(String label, boolean isEnabled, PageSeed wantedView) {
    Link l = new Link(label, isEnabled, wantedView);
    l.textual = true;
    return l;
  }

  public static Link getJsLocationInstance(String label, boolean hasFocus, boolean isEnabled, String script, PageSeed url, SessionState sess) {
    Link b = new Link(label, null, hasFocus, isEnabled, url, sess);
    b.outputModality = UrlComposer.OUTPUT_AS_JS_WIN_LOCATION;
    b.script = script;
    return b;
  }

  public static Link getIgnoreUrlInstance(String labelCode, boolean hasFocus, boolean isEnabled, String script, SessionState sess) {
    Link b = new Link(labelCode, null, hasFocus, isEnabled, null, sess);
    b.outputModality = UrlComposer.OUTPUT_IGNORING_URL;
    b.script = script;
    return b;
  }

  /**
   * @deprecated use LinkButton
   */
  public static String swapOnMouseOver(String classON, String classOFF) {
    return " class=\"" + classOFF + "\" onMouseOver=\"this.className='" + classON + "';\" onMouseOut=\"this.className='" + classOFF + "';\"";
  }

  public StringBuffer toHtmlStringBuffer() {

    StringBuffer sb = new StringBuffer();

    if (this.enabled) {

      if (!debug) {
        String s = "#";
        if (!popup &&
                !outputModality.equals(UrlComposer.OUTPUT_AS_JS_WIN_LOCATION) &&
                !outputModality.equals(UrlComposer.OUTPUT_IGNORING_URL) &&
                !outputModality.equals(UrlComposer.OUTPUT_AS_JS_LAUNCH_CES)) {
          s = getHref();
        }
        sb.append("<a href =\"").append(s).append('\"');
      }

      if (outputModality.equals(UrlComposer.OUTPUT_AS_JS_WIN_LOCATION))
        sb.append(" onClick=\"window.location='").append(HTMLEncoderOld.removeBadCharsFromJSConstant(getHref())).append("'\"");
      else if (popup)
        sb.append(" onClick=\"centerPopup('" + HTMLEncoderOld.removeBadCharsFromJSConstant(getHref()) + "','" + id + "','" + winWidth + "','" + winHeight + "','" + (scrollbars ? "yes" : "no") + "');\"");
      else if (outputModality.equals(UrlComposer.OUTPUT_AS_JS_LAUNCH_CES)) {
        ClientEntries ces = url.getClientEntries();
        if (ces != null) {
          sb.append(" onClick=\"");
          Iterator i = ces.getEntryKeys().iterator();
          while (i.hasNext()) {
            ClientEntry clientEntry = ces.getEntry((String) i.next());
            sb.append("obj('" + clientEntry.name + "')").append(clientEntry.stringValueNullIfEmpty()).append(';');
          }
          sb.append("\" ");
        }
      }

      if (title != null && title.trim().length() > 0) {
        sb.append(" tooltip=\"").append(title).append('\"');
      }

      if (script != null && script.trim().length() > 0) {
        if ((
                popup ||
                outputModality.equals(UrlComposer.OUTPUT_AS_JS_WIN_LOCATION) ||
                outputModality.equals(UrlComposer.OUTPUT_AS_JS_LAUNCH_CES)
                ) && script.toLowerCase().indexOf("onclick") > -1)
          throw new PlatformRuntimeException("Link::toHtmlStringBuffer Cannot put 'onClick' script in popup || OUTPUT_AS_JS_WIN_LOCATION || OUTPUT_AS_JS_LAUNCH_CES cases");
        sb.append(' ').append(script);
      }

      if (target != null && target.trim().length() > 0)
        sb.append(" target=\"").append(target).append('\"');

      if (tabIndex != 0) {
        sb.append(" tabindex=\"").append(tabIndex).append('\"');
      }

      if (htmlClass != null)
        sb.append(" class=\"" + htmlClass + "\"");

      if (style != null)
        sb.append(" style=\"" + style + "\"");

      if (!debug)
        sb.append(">\n");
    }

    if (image != null && image.toolTip == null)
      image.toolTip = label;

    if (!textual && image != null) {
      sb.append(image.toHtmlStringBuffer(null));
    }

    if (textual || additionalLinkText != null) {
      if (textual)
        sb.append(label);
      else
        sb.append(additionalLinkText);
    }

    if (this.enabled)
      sb.append("</a>\n");

    return sb;
  }

  public static String formButtons(List formButtons) throws ApplicationException {

    StringBuffer sb = new StringBuffer(512);

    sb.append("<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" align=\"right\"><tr>");

    for (int i = 0; i < formButtons.size(); i++) {
      HtmlRenderer button = (HtmlRenderer) formButtons.get(i);
      String width = "80";
      if (button instanceof SubmitButton) {
        SubmitButton subm = (SubmitButton) button;
        width = subm.width;
      }

      boolean isLink = false;
      Link l = null;
      if (button.getClass().equals(Link.class)) {
        isLink = true;
        l = (Link) button;
        if (l.width != null)
          width = l.width;
      }

      sb.append("<td width=\"" + width + "\" height=\"25\" align=\"center\" valign=\"middle\"  class=\"formButton\"\n  ")
              .append("onmouseover=\"this.className='formButtonOver'\" onmouseout=\"this.className='formButton'\">\n");

      if (isLink)
        sb.append(l.toHtmlStringBuffer());
      else
        sb.append(button.toHtmlStringBuffer());
      sb.append("</td>");
    }
    sb.append("</tr></table>");
    return sb.toString();
  }

  /**
   * This writes from <a href=".. until the closing of the opening <; the link content and the closing </a> must be added after it.
   *
   * @param fieldName
   * @param windowWidth
   * @param windowHeight
   * @param url
   * @return
   */
  public static StringBuffer drawPopupInHref(String fieldName, int windowWidth, int windowHeight, PageSeed url) {
    return drawPopupInHref(fieldName, windowWidth, windowHeight, url.toLinkToHref(), false, false);
  }

  /**
   * This writes from <a href=".. until the closing of the opening <; the link content and the closing </a> must be added after it.
   *
   * @param fieldName
   * @param windowWidth
   * @param windowHeight
   * @param url
   * @return
   */
  public static StringBuffer drawPopupInHref(String fieldName, int windowWidth, int windowHeight, String url, boolean urlIsJsFunction, boolean center) {

    StringBuffer sb = new StringBuffer();

    popupPosition(sb, fieldName, center, windowWidth, windowHeight);

    sb.append("<a href=\"javascript:void(0)\" onClick=\"");

    sb.append(popupUrl(urlIsJsFunction, url, fieldName, windowWidth, windowHeight));

    sb.append("\">");

    return sb;
  }

  public static void popupPosition(StringBuffer sb, String fieldName, boolean center, int windowWidth, int windowHeight) {
    sb.append("<script>function " + fieldName + "GetLeft(){\n");

    sb.append("tot = getAbsoluteLeftWin(obj('" + fieldName + "'));\n"); //
    sb.append("if (isNetscape) tot += " + (center ? windowWidth / 2 : windowWidth) + ";\n");
    sb.append("return tot;\n");
    sb.append("}</script>\n");
    sb.append("<script>function " + fieldName + "GetTop(){\n");
    sb.append("tot = getAbsoluteTopWin(obj('" + fieldName + "'));\n");
    sb.append("if (isNetscape) tot += " + (center ? windowHeight / 2 : windowHeight) + ";\n");
    sb.append("return tot;\n");
    sb.append("}</script>\n");
  }


  public static StringBuffer popupUrl(boolean urlIsJsFunction, String url, String fieldName, int windowWidth, int windowHeight) {
    StringBuffer sb = new StringBuffer();
    sb.append("window.open(").append((urlIsJsFunction ? "" : "'")).append(url).append((urlIsJsFunction ? "" : "'")).append(',')
            .append('\'' + fieldName + "','left='+" + fieldName + "GetLeft()+',top='+" + fieldName + "GetTop()+',width=" + windowWidth + ",height=" + windowHeight + ",menubars=no,scrollbars=yes,resizable=yes');");
    return sb;
  }

  public static String normalizeUrl(String baseString) {

    if (baseString != null && baseString.length() > 0 && baseString.indexOf("http://") == -1)
      baseString = "http://" + baseString;

    return baseString;
  }

  public Link setPopup(int width, int height, boolean scrollbars) {
    popup = true;
    this.scrollbars = scrollbars;
    this.winHeight = height;
    this.winWidth = width;
    return this;
  }

}
