package org.jblooming.waf.html.container;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.system.SystemConstants;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.html.button.ButtonJS;
import org.jblooming.waf.html.core.HtmlFinalizer;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.html.core.JspIncluder;
import org.jblooming.waf.html.layout.Skin;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

public class Container extends JspHelper implements HtmlFinalizer {

  public static final String init = Container.class.getName();

  public static final int DEFAULT = 0;
  public static final int COLLAPSED = 1;
  public static final int ICONIZED = 2;
  public static final int MAXIMIZED = 3;
  public static final int HIDDEN = 4;

  public static final String BOX_START = "BOX_START";
  public static final String BOX_END = "BOX_END";
  public static final String BOX_INITIALIZE = "BOX_INITIALIZE";
  public static final String BOX_BOOTSTRAP = "BOX_BOOTSTRAP";

  public boolean draggable = false;
  public boolean collapsable = false;
  public boolean resizable = false;
  public boolean closeable = false;
  public boolean absolutePosition = false;
  public boolean centeredOnScreen = false;  //implies absolutePosition

  /**
   * Level of nestedness in page layout. If > 0, extends the style with postfix _[level]
   */
  public int level = 0;

  public String overflow = null;
  public String top = "0";
  public String left = "0";
  public Skin skin;
  public String width = "100%";
  public String height = null;
  public String tableHeight = null;
  public String contentAlign;
  public String contentVAlign;
  public String title;
  public int status = DEFAULT;
  private boolean closeContainerCalled;
  private boolean openContainerCalled;

  /**
   * used to insert complex content on titlebar
   */
  public JspIncluder embeddedInTitle;

  /**
   * used to insert elements, as buttons, on titlebar' right side
   */
  public List<JspIncluder> titleRightElements = new LinkedList<JspIncluder>();

  /**
   * this attribute must be used only when Explorer doesn't collapse correctly the container.
   * this bug happen only after a combo.
   */
  public boolean removePosition = false;
  public boolean removeVeracchi = true;

  public Container(PageState pageState) {
    this(null, pageState);
  }

  public Container(String id, PageState pageState) {
    this(id, 0, pageState);
  }

  public Container(String id, int level, PageState pageState) {
    this.urlToInclude = "/commons/layout/partContainer.jsp";
    this.skin = pageState.getSkin();
    if (id != null)
      this.id = id;
    this.level = level;
    pageState.htmlBootstrappers.add(this);
  }


  private void init(PageContext pageContext) {
    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    if (!ps.initedElements.contains(init)) {
      pageContext.getRequest().setAttribute(ACTION, BOX_INITIALIZE);
      super.toHtml(pageContext);
      ps.initedElements.add(init);
    }
  }

  public void start(PageContext pageContext) {
    // PageState pageState= PageState.getCurrentPageState((HttpServletRequest)pageContext.getRequest());
    //  pageState.htmlBootstrappers.add(this);
    if (resizable)
      draggable = true;
    openContainerCalled = true;
    init(pageContext);
    pageContext.getRequest().setAttribute(ACTION, BOX_START);
    super.toHtml(pageContext);
  }

  public void end(PageContext pageContext) {
    if (!openContainerCalled)
      throw new PlatformRuntimeException("Call start before end. Container id:" + id);
    closeContainerCalled = true;
    pageContext.getRequest().setAttribute(ACTION, BOX_END);
    super.toHtml(pageContext);
  }

  public String getId() {
    return id;
  }

  public String getDiscriminator() {
    return getId();
  }

  public boolean validate(PageState pageState) throws IOException, ServletException {
    return openContainerCalled && closeContainerCalled;
  }

  /**
   * @deprecated
   */
  public void toHtml(PageContext pageContext) {
    throw new RuntimeException("Call start and end");
  }


  public void finalize(PageContext pageContext) {
    pageContext.getRequest().setAttribute(ACTION, BOX_BOOTSTRAP);
    super.toHtml(pageContext);
  }

  public String getCssLevel() {
    return level != 0 ? "_" + level : "";
  }


  public String getContainerTableId() {
    return "cttbl_" + getId();
  }

  public String getContainerDivId() {
    return "ctdivbdid_" + getId();
  }

  public String getContainerTitleId() {
    return "cttitid_" + getId();
  }

  public String getContainerTDId() {
    return "cttdid_" + getId();
  }

   public ButtonJS getOpenerButton() {
      return getOpenerButton(true);
   }

  /**
   * @return a button JS thats open/close the container
   */
  public ButtonJS getOpenerButton(boolean placeNearButton) {
    ButtonJS bjs = new ButtonJS();
    bjs.id = "bjs_" + this.id;
    bjs.onClickScript = getOpenerScript(placeNearButton ? bjs : null);
    return bjs;
  }

  /**
   * @return a button JS thats open/close the container
   */
  public String getOpenerScript(JspHelper nearWhat) {
    PageSeed rootCommandUrl = new PageSeed(ApplicationState.contextPath + "/commons/tools/command.jsp");
    rootCommandUrl.addClientEntry(SystemConstants.FLD_DOM_ID, getId());

    rootCommandUrl.setCommand(Commands.CMD_RESTORE);
    String open = rootCommandUrl.toLinkToHref();

    rootCommandUrl.setCommand(Commands.CMD_HIDE);
    String close = rootCommandUrl.toLinkToHref();

    return "var vis=showHideSpan('" + getContainerTableId() + "');" +
            "if (vis) " +
            " obj('commander').src='" +open+ "'; " +
            " else " +
            " obj('commander').src='" +close+ "'; " +
            (nearWhat!=null? "nearBestPosition('" + nearWhat.id + "','" + getContainerTableId() + "');" : "")+
            "obj('" + getContainerTableId() + "').style.zIndex='10000';";
  }


}
