package org.jblooming.waf.html.container;

import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.html.button.ButtonJS;
import org.jblooming.waf.html.core.HtmlBootstrap;
import org.jblooming.waf.html.core.HtmlFinalizer;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.html.layout.Skin;
import org.jblooming.waf.html.state.Form;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

public class TabSet extends JspHelper implements HtmlBootstrap, HtmlFinalizer {

  public static final String init = TabSet.class.getName();

  public Skin skin;
  boolean openTabBarCalled;
  boolean closeTabBarCalled;

  public boolean wideTabs;
  
  /**
   * if true first tab is indented
   */
  public boolean indentTab=true;

  boolean jsEmitted = false;

  public String style = null;

  public Form form;

  Collection<Tab> tabset;

  PageState pageState;

  public final static String BAR = "TABSET_BAR";
  public final static String END = "TABSET_END";
  public static final String EMIT_JS = "EMIT_JS";

  public TabSet(String id, PageState pageState) {
    this.id = id;
    this.skin = pageState.getSkin();
    this.urlToInclude = "/commons/layout/partTabSet.jsp";
    this.pageState = pageState;
//    pageState.htmlBootstrappers.add(this);
  }

  public TabSet(Form form, PageState pageState) {
    this("tabSetId", pageState);
    this.form = form;
  }

  private void init(PageContext pageContext) {
    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    if (!ps.initedElements.contains(init)) {
      pageContext.getRequest().setAttribute(Commands.COMMAND, INITIALIZE);
      super.toHtml(pageContext);
      ps.initedElements.add(init);
    }
  }

  public void drawBar(PageContext pageContext) throws IOException, ServletException {
    init(pageContext);
    pageContext.getRequest().setAttribute(Commands.COMMAND, BAR);
    toHtml(pageContext);
    openTabBarCalled = true;
  }

  public void end(PageContext pageContext) throws IOException, ServletException {
    pageContext.getRequest().setAttribute(Commands.COMMAND, END);
    toHtml(pageContext);
    closeTabBarCalled = true;
  }


  public void addTab(Tab tab) {
    if (tabset == null)
      tabset = new LinkedList<Tab>();
    tab.tabSet = this;
    tab.domId = this.id + tab.domId;
    tab.urlToInclude = this.urlToInclude;
    if (!tab.focused)
      tab.focused = (this.id+tab.id).equals(pageState.getEntry(this.id).stringValueNullIfEmpty());
    tabset.add(tab);
  }

  public Iterator iterator() {
    if (tabset != null)
      return tabset.iterator();
    else
      return null;
  }

  public int tabSetSize() {
    if (tabset != null)
      return tabset.size();
    else
      return 0;
  }

  public Tab getTab(String tabId){
    for (Tab tab:tabset){
      if (tab.id.equals(tabId))
        return tab;
    }
    return null;
  }

  public Tab getTabByDomId(String focusedTabDomId) {
    for (Tab tab:tabset){
      if (tab.domId.equals(focusedTabDomId))
        return tab;
    }
    return null;
  }

  public String getDiscriminator() {
    return id;
  }

  public boolean validate(PageState pageState) throws IOException, ServletException {
    boolean result = false;
    for (Iterator iterator = tabset.iterator(); iterator.hasNext();) {
      Tab tab = (Tab) iterator.next();
      result = tab.openTabCalled && tab.closeTabCalled;
      if (!result)
        break;
    }

    return result && openTabBarCalled && closeTabBarCalled;
  }

  public void jsEmit(PageContext pageContext) throws IOException, ServletException {
    pageContext.getRequest().setAttribute(Commands.COMMAND, END);
    toHtml(pageContext);
    jsEmitted = true;
  }

  public Tab setFocusedTab(String focusedId) {
    Tab tab = null;
    Tab result = null;
    for (Iterator iterator = tabset.iterator(); iterator.hasNext();) {
      tab = (Tab) iterator.next();
      if (tab.domId.equals(focusedId)) {
        tab.focused = true;
        result = tab;
      } else {
        tab.focused = false;
      }
    }
    return result;
  }

  public ButtonJS getShowTabButton(Tab tab2) {
    ButtonJS showTabButton = new ButtonJS();
    showTabButton.alertOnChange = false;
    showTabButton.onClickScript = "hideAllTabs_"+this.id+"(); obj('div_tabset_"+tab2.domId+"').style.display='';obj('"+tab2.domId+"').className='tabSelected';obj('"+this.id+"').value='"+tab2.domId+"';";
    showTabButton.label = pageState.getI18n("NEXT");
    return showTabButton;
  }

  public Tab setFocusedTab(PageState pageState) throws ActionException {
    ClientEntry entry = pageState.getEntry(id);
    return setFocusedTab(entry.stringValue());
  }

  public void finalize(PageContext pageContext) throws IOException, ServletException {
    if (!jsEmitted) {
      jsEmit(pageContext);
    }

  }

  public Tab addLazyLoadInstance(String id, String label, PageSeed urlToInclude) {

    ButtonJS loadAiax = new ButtonJS();
    loadAiax.label = label;
    loadAiax.onClickScript = "loadTabSet('" + urlToInclude.toLinkToHref() + "','" + this.id+id + "');";
    Tab tab = new Tab(id,loadAiax);
    this.addTab(tab);
    return tab;
  }

  public static void pointToTab(String tabSetId, String tabId, PageSeed pointer) {
    pointer.addClientEntry(tabSetId,tabSetId+tabId);
  }

}


