package org.jblooming.waf.html.core;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.cursor.exceptions.CursorException;
import org.jblooming.ontology.Identifiable;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.HTMLEncoderOld;
import org.jblooming.utilities.JSP;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.html.state.Form;
import org.jblooming.waf.view.ClientEntries;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageSeed;

import javax.servlet.http.HttpServletRequest;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;


/**
 * Dedicated to Roberto Bicchierai
 * Date: 14-feb-2003
 * Time: 19.54.56
 *
 * @author Pietro Polsinelli dev@open-lab.com
 */
public abstract class UrlComposer extends HtmlRenderer {

  public String outputModality = OUTPUT_AS_LINK;
  public HttpServletRequest request;
  public boolean debug = false;
  protected Form form;

  private static boolean debugActive = false;

  public static final String OUTPUT_AS_LINK = "OUTPUT_AS_LINK";
  public static final String OUTPUT_AS_FORM = "OUTPUT_AS_FORM";
  public static final String OUTPUT_AS_JS_SUBMIT = "OUTPUT_AS_JS_SUBMIT";
  public static final String OUTPUT_AS_JS_WIN_LOCATION = "OUTPUT_AS_JS_WIN_LOCATION";
  public static final String OUTPUT_IGNORING_URL = "OUTPUT_IGNORING_URL";
  public static final String OUTPUT_AS_JS_LAUNCH_CES = "OUTPUT_AS_JS_LAUCH_CES";


  public UrlComposer(PageSeed v) {
    this.url = v;
  }

  public void doDebug(HttpServletRequest request) {
    this.request = request;
    this.debug = true;
  }

  public String getHref() {

    StringBuffer href = new StringBuffer(512);

    if (url != null) {
      if (outputModality.equals(OUTPUT_AS_LINK) || outputModality.equals(OUTPUT_AS_JS_WIN_LOCATION))
        href.append(url.getHref() + '?');

      if (!outputModality.equals(OUTPUT_AS_JS_SUBMIT))
        addPair(href, Fields.VIEW_ID, this.hashCode() + "");

      addPair(href, Commands.COMMAND, url.getCommand());

      if (url.isNoResubmit())
        addPair(href, Fields.NO_RESUBMIT, Fields.TRUE);

      if (url.mainObjectId != null)
        addPair(href, Fields.OBJECT_ID, url.mainObjectId + "");

      if (url.isPopup())
        addPair(href, Fields.POPUP, Fields.TRUE);

      if (outputModality.equals(OUTPUT_AS_FORM)) {

        if (url.getClientEntries().getEntry(Fields.PAGE_NUMBER) == null)
          addPair(href, Fields.PAGE_NUMBER, "");
        if (sessionState != null && (url.getCommand() == null || !url.getCommand().equals(Commands.LINK)))
          href.append(preserveCheckedEntries(url, sessionState));
      }

      for (Iterator iterator = url.getClientEntries().getEntryKeys().iterator(); iterator.hasNext();) {
        String key = (String) iterator.next();
        if (!outputModality.equals(OUTPUT_AS_LINK) || !key.equals(Fields.URL)) {
          String value = url.getClientEntries().getEntry(key).stringValueNullIfEmpty();
          addPair(href, key, value);
        }
      }
    }
    return href.toString();
  }

  private void addPair(StringBuffer href, String key, String value) {

    if (value == null)
      value = "";
    if (outputModality.equals(OUTPUT_AS_FORM)) {
      href.append("<input type =\"hidden\"  id=\"").append(getUniqueName() + key).append("\" name=\"");
      href.append(key).append("\" value=\"").append(JSP.encode(value)).append("\" savedValue=\"\" >\n");
    } else if (outputModality.equals(OUTPUT_AS_JS_SUBMIT)) {
      //ClientEntry formValue = form.url.getClientEntry(key);
      //if (formValue==null || formValue.value!=value)
      href.append("obj('").append(getUniqueName() + key).append("').value='").append(HTMLEncoderOld.removeBadCharsFromJSConstant(JSP.encode(value))).append("';");
    } else if (!value.equals("")) {
      if (href.length() > 0 && !href.toString().endsWith("?"))
        href.append("&");
      href.append(key).append('=').append(HTMLEncoderOld.urlEncode(value));
    }
  }

  public String getUniqueName() {
    //todo: this is completely absurd aaaaaaaaaaaaaaargh !
    if (form == null)
      return id;
    else
      return form.id;
  }

  /**
   * collect client entries for next pages
   *
   */
  private StringBuffer preserveCheckedEntries(PageSeed url, SessionState sess) {
    StringBuffer sb = new StringBuffer(512);

    ClientEntries ces = url.getClientEntries();
    if (ces != null && ces.getEntryKeys() != null) {
      Iterator j = ces.getEntryKeys().iterator();
      while (j.hasNext()) {
        ClientEntry ce = ces.getEntry((String) j.next());

        if (ce.name != null && ce.name.indexOf(Fields.CHECK_PREFIX) > -1) {
          boolean presentInThisPage = false;
          if (sess != null && sess.getCursor(url.getName()) != null) {
            Iterator i = null;
            try {
              i = sess.getCursor(url.getName()).getPage();
            } catch (CursorException e) {
              throw new PlatformRuntimeException(e);
            }
            if (i != null) {

              while (i.hasNext()) {
                String key = ce.name.substring(Fields.FORM_PREFIX.length() + 7);
                Object o = i.next();
                String id;
                if (o.getClass().equals(TreeMap.class)) {
                  Map map = (Map) o;
                  id = (String) map.get("id");
                } else {
                  Identifiable sn = (Identifiable) o;
                  id = sn.getId() + "";
                }
                if (id.equals(key)) presentInThisPage = true;

              }
            }
          }
          if (!presentInThisPage)
            sb.append("<input type = \"hidden\" name = \"" + ce.name + "\" value = \"" + JSP.encode(ce.stringValueNullIfEmpty()) + "\" >");
          if (debugActive) Tracer.addTrace("ViewHlp::preserveCheckedEntries added hidden input: " + ce.name + '-' + ce.stringValueNullIfEmpty());
        }
      }
    }
    return sb;
  }

  public static String stripContext(String url, HttpServletRequest request) {
    return stripContext(url, request.getContextPath());
  }

  public static String stripContext(String url, String contextPath) {
    if (url.startsWith(contextPath)) {
      url = url.substring(contextPath.length());
    }
    return url;
  }

}
