package org.jblooming.waf.html.display;

import org.jblooming.ontology.PersistentFile;
import org.jblooming.system.SystemConstants;
import org.jblooming.utilities.JSP;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.utilities.file.FileUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.html.input.HtmlElement;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 */
public class Img extends HtmlElement {

  public boolean required = false;
  public SessionState sessionState;
  public String script;
  public String imageUrl;
  public String width;
  public String height;
  public boolean doRollover = false;
  public String style;
  public String align = "";
  public String border = "0";
  public boolean inhibitToolTip = false;
  public boolean showLittleHand = false;


  public Img(PersistentFile pf, String title, HttpServletRequest request) {
    this(pf, title, false, request);
  }

  public Img(PersistentFile pf, String title, boolean disableCaching, HttpServletRequest request) {
    if (!PersistentFile.TYPE_WEBAPP_FILESTORAGE.equals(pf.getType())) {
      this.imageUrl = getPersistentFileUrl(pf, request);
    } else {
      this.imageUrl = request.getContextPath() + "/" + StringUtilities.replaceAllNoRegex(pf.getFileLocation(), "\\", "/") + (disableCaching ? "?ts=" + System.currentTimeMillis() : "");
    }
    this.toolTip = title;
    this.translateToolTip = false;
    this.doRollover = false;
  }

  private static String getPersistentFileUrl(PersistentFile pf, HttpServletRequest request) {
    String uplUID = pf.serialize();

    PageSeed imageUrlps = new PageSeed(request.getContextPath() + "/commons/layout/partUploaderView.jsp");
    //myself.setCommand(Commands.FILE_VIEW);
    imageUrlps.addClientEntry(Fields.FILE_TO_UPLOAD, uplUID);
    String imageUrl = imageUrlps.toLinkToHref();
    return imageUrl;
  }

  /**
   * NB:: this method needs that the jsp include showTumb.js
   * @param pf
   * @param underlyingText
   */
  public static String imgOnMouseOver(PersistentFile pf, String underlyingText) {
    String imageUrl="";

    if (PersistentFile.TYPE_FILESTORAGE.equals(pf.getType())) {
      imageUrl = ApplicationState.getApplicationSetting(SystemConstants.FLD_REPOSITORY_URL) + pf.getFileLocation();

    } else if (PersistentFile.TYPE_FILESTORAGE_ABSOLUTE.equals(pf.getType())) {
      imageUrl = pf.getFileLocation();

    } else if (PersistentFile.TYPE_WEBAPP_FILESTORAGE.equals(pf.getType())) {
      imageUrl = ApplicationState.webAppFileSystemRootPath + pf.getFileLocation();
    }
    if (FileUtilities.isImageByFileExt(FileUtilities.getFileExt(pf.getOriginalFileName())))
      return "onMouseOver=\"showPreview('" + StringUtilities.replaceAllNoRegex(imageUrl, "\\", "/") + "','" + JSP.javascriptEncode(underlyingText) + "')\"";
    else
      return "";
  }


  public Img(String imageUrl, String title) {
    this.imageUrl = imageUrl;
    this.toolTip = title;
    this.translateToolTip = false;
    this.doRollover = false;
  }

  public Img(String imageUrl, String title, String width, String heigth) {
    this.imageUrl = imageUrl;
    this.toolTip = title;
    this.width = width;
    this.height = heigth;
  }

  public Img(String imageUrl, String title, String width, String heigth, boolean doRollover) {
    this.imageUrl = imageUrl;
    this.toolTip = title;
    this.width = width;
    this.height = heigth;
    this.doRollover = doRollover;
  }

  public Img(String imageUrl, String title, String width, String heigth, boolean doRollover, SessionState sm, String script, boolean showLittleHand) {
    this.imageUrl = imageUrl;
    this.toolTip = title;
    this.width = width;
    this.height = heigth;
    this.doRollover = doRollover;
    this.script = script;
    this.showLittleHand = showLittleHand;
    this.inhibitToolTip = true;
  }

  public StringBuffer toHtmlStringBuffer(PageContext pageContext) {

    StringBuffer sb = new StringBuffer();

    if (showLittleHand)
      sb.append("<a href=\"javascript:void(0)\"> ");

    sb.append("<img src=\"").append(imageUrl).append('\"');

    imgFill(sb);
    if (align != null && align.length() > 0)

      // issue_ELE_4
      sb.append(" align=\"").append(align).append('\"');


    if (disabled)
      sb.append(" style=\"filter: progid:DXImageTransform.Microsoft.Alpha(opacity=50); -moz-opacity:0.5\" ");

    //MUST be last otherwise ButtonSubmit.getPDFPrintButton gets sputtanated
    if (script != null)
      sb.append(' ').append(script);

    sb.append(">");

    return sb;
  }


  private StringBuffer imgFill(StringBuffer sb) {
    if (width != null && width.trim().length() > 0)
      sb.append(" width=\"").append(width).append("\" ");

    if (height != null && height.trim().length() > 0)
      sb.append(" height=\"").append(height).append("\" ");

    if (border != null && border.trim().length() > 0)
      sb.append(" border=\"").append(border).append("\" ");

    sb.append(" name=\"").append(id).append("\"");

    if (doRollover && imageUrl.indexOf("Over.gif") == -1) {

      sb.append(" onMouseOver=\"").append(id).append(".src='").append(getImageUrlOver()).append("';\" ");

      sb.append(" onMouseOut=\"").append(id).append(".src='").append(imageUrl).append("';\" ");

    }
    if (style != null && style.trim().length() > 0)
      sb.append(" style=\"").append(style).append("\" ");

    if (!inhibitToolTip) {
      /* if (translateTitle)
         try {
           sb.append(" title=\"").append(titleCode).append("\" ");
         } catch (Exception e) { }
       else*/
      sb.append(" tooltip=\"").append(toolTip).append("\" ");
    } else {
      sb.append(" title=\"").append(JSP.w((toolTip!=null && toolTip.trim().length()>0) ? toolTip : "")).append("\"");
    }

    //sb.append(" alt=\"").append(JSP.w(toolTip)!=null ? toolTip : imageUrl).append("\"");

    sb.append(" id=\"").append(id).append("\" ");

    return sb;
  }

  public String getImageUrlOver() {
    int indexOfDot = imageUrl.lastIndexOf(".");
    return (imageUrl.substring(0, indexOfDot)) + "Over." + imageUrl.substring(indexOfDot + 1);
  }

  /**
   * @\deprecated Use with and height as string
   */
  public static void imgSpacer(int width, int height, PageContext pageContext) {
    imgSpacer("" + width, "" + height, pageContext);
  }

  public static void imgSpacer(String width, String height, PageContext pageContext) {
    imgSpacer(width, height, "", pageContext);
  }

  public static void imgSpacer(String width, String height, String tooltip, PageContext pageContext) {

    PageState ps = null;
    ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    Img i = new Img(ps.sessionState.getSkin().imgPath + "blank.gif", "");
    i.width = width;
    i.height = height;
    i.doRollover = false;
    i.inhibitToolTip = true;
    i.toolTip=tooltip;
    i.toHtml(pageContext);

  }

  public Img getNewInstance() {

    Img newImg = new Img(imageUrl, toolTip);
    newImg.width = width;
    newImg.height = height;
    newImg.doRollover = doRollover;
    newImg.script = script;
    newImg.style = style;
    newImg.align = align;
    newImg.border = border;
    newImg.disabled = disabled;
    newImg.translateToolTip = translateToolTip;
    newImg.inhibitToolTip = inhibitToolTip;
    newImg.showLittleHand = showLittleHand;
    return newImg;

  }

  public void toHtml(PageContext pageContext) {

    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());

    //issue_ELE_2
    if (imageUrl.indexOf("/") == -1 && ps.sessionState != null)
      imageUrl = ps.sessionState.getSkin().imgPath + imageUrl;

    if (translateToolTip)
      toolTip = ps.getI18n(toolTip);

    super.toHtml(pageContext);
  }


}
