package org.jblooming.waf.html.display;

import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.html.core.HtmlBootstrap;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;

public class MenuRenderer extends JspHelper implements HtmlBootstrap {

  public static final String init = MenuRenderer.class.getName();
  public String menuDefinitionJS;
  private boolean inited = false;

  /**
   * use only in case menuDefinitionJS defines a <b>single</b> menu.
   * they should be sued as name and label for the menu in the defining js, i.e. menuDefinitionJS value.
   */
  public String menuName;
  public String menuLabel;
  public String width;

  public static final String FLD_MENUNAME = Fields.FORM_PREFIX + "MENUNAME";

  public MenuRenderer(String menuDefinitionJS, PageState pageState) {
    this.menuDefinitionJS = menuDefinitionJS;
    pageState.htmlBootstrappers.add(this);
  }

  public MenuRenderer(PageSeed ps, PageState pageState) {
    this.menuDefinitionJS = ps.toLinkToHref();
    pageState.htmlBootstrappers.add(this);
  }

  /**
   * @param pageContext
   * @throws ApplicationException
   * @throws IOException
   */
  private void init(PageContext pageContext) throws ApplicationException, IOException {
    HttpServletRequest request = ((HttpServletRequest) pageContext.getRequest());
    PageState ps = PageState.getCurrentPageState(request);

    if (!ps.initedElements.contains(init)) {
      pageContext.getOut().print("<script src=\"" + request.getContextPath() + "/commons/js/menu_js.jsp\"></script>");
      pageContext.getOut().print("<link rel=\"stylesheet\" type=\"text/css\" href=\"" + ps.getSessionState().getSkin().css + "menuCss.jsp\" >");
      ps.initedElements.add(init);
    }

    if (!inited) {
      //pageContext.getOut().print("<script src=\"" + menuDefinitionJS + "\"></script>"); //+ "?" + FLD_MENUNAME + "=" + menuName +

      try {
        pageContext.include(menuDefinitionJS);
      } catch (ServletException e) {
        throw new PlatformRuntimeException(e);
      }
      inited = true;
    }



  }

  public String drawMenuBar(String menuName, String menuLabel, PageContext pageContext) throws IOException, ApplicationException {

    init(pageContext);

    StringBuffer sb = new StringBuffer();
    sb.append("<a href=\"javascript:void(0)\" >\n" +
            "      <table width='"+width+"' cellpadding=\"2\"  class=\"menubar\"\n" +
            "          id=\"" + menuName + "Menu\"\n" +
            "          onMouseOver=\"showMenu(" + menuName + ",'" + menuName + "Menu', '" + menuName + "Div', true)\"\n" +
            "          onClick=\"showOnClick();showMenu(" + menuName + ",'" + menuName + "Menu', '" + menuName + "Div', true)\" >\n" +
            "        <tr>\n" +
            "          <td nowrap>" + menuLabel + "</td>\n" +
            "        </tr>\n" +
            "      </table>\n" +
            "    </a>");

    return sb.toString();
  }


  public String getDiscriminator() {
    return init;
  }

  public boolean validate(PageState ps) throws IOException, ServletException {
    return ps.initedElements.contains(init);
  }

}

