package org.jblooming.waf.html.display;

import org.jblooming.ApplicationException;
import org.jblooming.ontology.Node;
import org.jblooming.ontology.PerformantNodeSupport;
import org.jblooming.security.Permission;
import org.jblooming.waf.html.button.ButtonSupport;
import org.jblooming.waf.html.core.HtmlBootstrap;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Comparator;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class PathToObject extends JspHelper implements HtmlBootstrap {

  public PerformantNodeSupport node;
  public Method displayMethod;
  public PageSeed destination;
  public ButtonSupport rootDestination;
  public String separator = " /";
  public boolean lastInBold = true;
  public boolean useParentIfNew = false;
  public Class mainClass;
  public boolean inhibitLinks = false;
  public boolean inhibitLinksToBrothers = false;

  public Comparator comparator;

  public Permission canClick;

  public static final String init = PathToObject.class.getName();
  public static final String DRAW = "DRAW";
  public static final String CLOSE = "CLOSE";
  private boolean drawCalled=false;
  private boolean closeCalled=false;


  public PathToObject(PerformantNodeSupport node) throws NoSuchMethodException {

    displayMethod = Node.class.getMethod("getName");
    this.node = node;
    urlToInclude = "/commons/layout/partPathToObject.jsp";
  }

  public void useParentIfNew(Class mainClass) {
    useParentIfNew = true;
    this.mainClass = mainClass;
  }

  public String getDiscriminator() {
    return init;
  }

  private void init(PageContext pageContext)  {
    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    if (!ps.initedElements.contains(init)) {
      pageContext.getRequest().setAttribute(ACTION, INITIALIZE);
      super.toHtml(pageContext);
      ps.initedElements.add(init);
    }
  }

  public void draw(PageContext pageContext) throws IOException, ServletException, ApplicationException {
    drawCalled = true;
    init(pageContext);
    pageContext.getRequest().setAttribute(ACTION, DRAW);
    super.toHtml(pageContext);
  }

  public boolean validate(PageState pageState) throws IOException, ServletException {
    return drawCalled && closeCalled;
  }

  public void close(PageContext pageContext) throws IOException, ServletException, ApplicationException {
    if (!drawCalled)
      throw new ApplicationException("Call start before end");
    closeCalled = true;
    pageContext.getRequest().setAttribute(ACTION, CLOSE);
    super.toHtml(pageContext);
  }

  public void toHtml(PageContext pageContext) {
    //throw new RuntimeException("Call start and close");
    try {
      this.draw(pageContext);
      this.close(pageContext);
    } catch (IOException e) {
      e.printStackTrace();
    } catch (ServletException e) {
      e.printStackTrace();
    } catch (ApplicationException e) {
      e.printStackTrace();
    }

  }



}
