package org.jblooming.waf.html.display;

import org.jblooming.ApplicationException;
import org.jblooming.waf.html.core.HtmlBootstrap;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;

public class Splitter extends JspHelper implements HtmlBootstrap {

  public static final String init = Splitter.class.getName();

  public static final String SPLIT_START = "SPLIT_START";
  public static final String SPLIT_MIDDLE = "SPLIT_MIDDLE";
  public static final String SPLIT_END = "SPLIT_END";
  public static final String SPLIT_INITIALIZE = "SPLIT_INITIALIZE";

  public static final String SPLIT_HORIZONTAL = "SPLIT_HORIZONTAL";
  public static final String SPLIT_VERTICAL = "SPLIT_VERTICAL";

  public String type = SPLIT_HORIZONTAL;

  public boolean showHandlers = true;
  private boolean openSplitterCalled;
  private boolean closeSplitterCalled;
  private boolean middleSplitterCalled;

  /**
   * startPosition is the starting position of splitter-bar. Is the % width of first cell
   */
  public int startPosition = 50;


  /**
   * splitter size
   */
  public String width = "100%";
  public String height = "100%";


  public Splitter(PageState pageState) {
    this(null, pageState);
  }

  public Splitter(String id, PageState pageState) {
    this(id, "partSplitter.jsp", pageState);
  }


  public Splitter(String id, PageState pageState, String type) {
    this(id, "partSplitter.jsp", pageState);
    this.type = type;
  }


  public Splitter(String id, String urlToInclude, PageState pageState) {
    this.urlToInclude = pageState.getSkin().partsPath + urlToInclude;
    if (id != null)
      this.id = id;
    pageState.htmlBootstrappers.add(this);
  }


  private void init(PageContext pageContext) throws ApplicationException, IOException, ServletException {
    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    if (!ps.initedElements.contains(init)) {
      pageContext.getRequest().setAttribute(ACTION, SPLIT_INITIALIZE);
      super.toHtml(pageContext);
      ps.initedElements.add(init);
    }
  }

  public void start(PageContext pageContext) throws IOException, ServletException, ApplicationException {
    openSplitterCalled = true;
    init(pageContext);
    pageContext.getRequest().setAttribute(ACTION, SPLIT_START);
    super.toHtml(pageContext);
  }

  public void middle(PageContext pageContext) throws IOException, ServletException, ApplicationException {
    if (!openSplitterCalled)
      throw new ApplicationException("Call start before middle");
    middleSplitterCalled = true;
    pageContext.getRequest().setAttribute(ACTION, SPLIT_MIDDLE);
    super.toHtml(pageContext);
  }


  public void end(PageContext pageContext) throws IOException, ServletException, ApplicationException {
    if (!openSplitterCalled || !middleSplitterCalled)
      throw new ApplicationException("Call start and middle before end");
    closeSplitterCalled = true;
    pageContext.getRequest().setAttribute(ACTION, SPLIT_END);
    super.toHtml(pageContext);
  }

  public String getId() {
    return id;
  }

  public String getDiscriminator() {
    return getId();
  }

  public boolean validate(PageState pageState) throws IOException, ServletException {
    return openSplitterCalled && middleSplitterCalled && closeSplitterCalled;
  }

  /**
   * @deprecated
   */
  public void toHtml(PageContext pageContext) {
    throw new RuntimeException("Call start, middle and end");
  }

}
