package org.jblooming.waf.html.display;

import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.ontology.PerformantNodeSupport;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.exceptions.FindException;
import org.jblooming.waf.html.core.JspHelper;

import javax.servlet.ServletException;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

/**
 * TreeDrawer (c) 2005 - Open Lab - www.open-lab.com
 */
public class TreeDrawer extends JspHelper {

  public String applicationUrlToInclude;
  public Class<? extends PerformantNodeSupport> treeClass;
  public String orderingField;
  public PerformantNodeSupport node;
  public Comparator childrenComparator;
  public String additionalOnClickScript;
  public boolean deleteRows = true;
  public boolean multiSelection = false;

  public boolean selectChildren = false;

  /**
   * Query to load "first level" nodes
   */
  public OqlQuery oqlQuery;
  public Set<PerformantNodeSupport> roots;


  /**
   * formatting parameter
   */
  public String tableWidth;
  public String uniqueTableId = "TD_" + Math.random() * 100;
  public String indentUnit;
  public String expandImgPath;

  /**
   * contextual menu parameters
   */
  public boolean showContextualMenu = false;


  public static final String DRAW_INIT = "DRAW_INIT";
  public static final String DRAW_ROOT = "DRAW_ROOT";
  public static final String DRAW_LINE = "DRAW_LINE";
  public static final String DRAW_TREE = "DRAW_TREE";
  public static final String DRAW_HEADER = "DRAW_HEADER";
  public static final String DRAW_FOOTER = "DRAW_FOOTER";
  public static final String DRAW_EXPAND = "DRAW_EXPAND";

  public TreeDrawer() {
    super();
    this.urlToInclude = "/commons/layout/tree/partTreeDrawer.jsp";
  }

  public void drawRootTree(PageContext pageContext) throws IOException, ServletException, ApplicationException {

    String defaultUrlToInclude = this.urlToInclude;
    /**
     * js init
     */
    pageContext.getRequest().setAttribute(ACTION, DRAW_INIT);
    super.toHtml(pageContext);

    /**
     * draw header
     */
    try {
      pageContext.getRequest().setAttribute(ACTION, DRAW_HEADER);
      this.urlToInclude = this.applicationUrlToInclude;
      super.toHtml(pageContext);
    } catch (PlatformRuntimeException a) {
    }

    /**
     * draw tree
     */
    this.urlToInclude = defaultUrlToInclude;
    pageContext.getRequest().setAttribute(ACTION, DRAW_ROOT);
    super.toHtml(pageContext);

    /**
     * draw footer
     */
    try {
      pageContext.getRequest().setAttribute(ACTION, DRAW_FOOTER);
      this.urlToInclude = this.applicationUrlToInclude;
      super.toHtml(pageContext);
    } catch (PlatformRuntimeException a) {
    }

    /**
     * close table
     */
    this.urlToInclude = defaultUrlToInclude;
    super.toHtml(pageContext);
  }

  public void drawLine(PageContext pageContext) {
    pageContext.getRequest().setAttribute(ACTION, DRAW_LINE);
    this.urlToInclude = this.applicationUrlToInclude;
    super.toHtml(pageContext);
  }

  public void drawTree(PageContext pageContext) {
    this.urlToInclude = "/commons/layout/tree/partTreeDrawer.jsp";
    pageContext.getRequest().setAttribute(ACTION, DRAW_TREE);
    super.toHtml(pageContext);
  }

  public void drawExpandCollapse(PageContext pageContext) {
    this.urlToInclude = "/commons/layout/tree/partTreeDrawer.jsp";
    pageContext.getRequest().setAttribute(ACTION, DRAW_EXPAND);
    super.toHtml(pageContext);
  }

  public List<PerformantNodeSupport> firstLevelNodeList() throws FindException {
    List<PerformantNodeSupport> firstLevellers = new ArrayList();

    if (node == null) {
      if (roots!=null) {
        firstLevellers = new ArrayList(roots);
      } else if (oqlQuery == null) {
        boolean ordered = orderingField != null && orderingField.trim().length() > 0;
        String hql = " from " + treeClass.getName() + " as root where root.parent is null " + ((ordered) ? " order by root." + orderingField : "");
        OqlQuery oql = new OqlQuery(hql);
        firstLevellers = oql.list();
      } else {
        firstLevellers = oqlQuery.list();
      }
    } else {
      firstLevellers.add(node);
    }

    return firstLevellers;
  }

}
