package org.jblooming.waf.html.input;

import org.jblooming.utilities.JSP;
import org.jblooming.waf.html.layout.Css;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public abstract class InputElement extends HtmlElement {

  public String label;
  public boolean translateLabel = false;
  public boolean showLabel = true;

  public String separator;

  public String fieldName;
  public int fieldSize;

  protected ClientEntry value;

  public String labelClass;
  public String fieldClass = Css.formElements;

  public boolean fieldValueHasToBeEncoded = true;
  public boolean required = false;
  public boolean excludeFromAlert = false;
  public int tabIndex;
  public boolean readOnly = false;

  public String script;

  public boolean preserveOldValue = true;  

  /**
   * key controls
   */
  public String actionListened;
  public int keyToHandle;
  public boolean checkCtrlKey=false;
  public String launchedJsOnActionListened;

  public void toHtml(PageContext pageContext) {

    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    value = ps.getEntry(fieldName);

    super.toHtml(pageContext);

    if (value != null && value.errorCode != null)
      JSP.feedbackError(value, pageContext);
  }

  /**
   * Handling of DOM events relatively to this control
   *
   * @param keyToHandle    the int code of the key
   * @param launchedJs     on pressing the key, this js will be launched
   * @param actionListened the code is the event listened e.g. keypress
   */
  public void addKeyPressControl(int keyToHandle, String launchedJs, String actionListened) {
    this.keyToHandle = keyToHandle;
    this.launchedJsOnActionListened = launchedJs;
    this.actionListened = actionListened;
  }


   public void toHtmlI18n(PageContext pageContext) {
     PageState pageState = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
     if (label==null)
       label = pageState.getI18n(fieldName);
     else
      label = pageState.getI18n(label);
     toHtml(pageContext);
   }

  public void setValue(ClientEntry value) {
    this.value = value;
  }

}

