package org.jblooming.waf.html.input;

import org.jblooming.ApplicationException;
import org.jblooming.ontology.LookupConfiguration;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.util.List;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 */
public class LookupManager extends JspHelper {

  private List<LookupConfiguration> lookupClasses;

  public static final String MANAGE_A_LOOKUP = "MANAGE_A_LOOKUP";
  public static final String LIST_LOOKUPS = "LIST_LOOKUPS";
  public boolean backToListButton = true;

  public boolean readOnly = false;  

  public LookupManager(List lookupClasses) {
    super();

    this.lookupClasses = lookupClasses;
  }

  public List getLookupClasses() {
    return lookupClasses;
  }

  public void manageALookup(PageContext pageContext) throws IOException, ServletException, ApplicationException, ActionException {
    PageState pageState = PageState.getCurrentPageState((HttpServletRequest)pageContext.getRequest());
    String lkClass = pageState.getEntry("MAIN_CLASS").stringValue();
    LookupConfiguration lc = null;
    if (lkClass!=null)
      lc = getLookupConfigByClassName(lkClass);
    if (lc!=null && lc.listEditable)
      this.urlToInclude = "/commons/layout/partLookupEditableList.jsp";
    else
      this.urlToInclude = "/commons/layout/partLookupList.jsp";

    pageContext.getRequest().setAttribute(ACTION, MANAGE_A_LOOKUP);
    super.toHtml(pageContext);
  }

  public void listLookups(PageContext pageContext) throws IOException, ServletException, ApplicationException {
    this.urlToInclude = "/commons/layout/partLookupsList.jsp";
    pageContext.getRequest().setAttribute(ACTION, LIST_LOOKUPS);
    super.toHtml(pageContext);
  }

  public LookupConfiguration getLookupConfigByClassName(String className) {
    LookupConfiguration lc = null;
    if (lookupClasses != null && lookupClasses.size() > 0) {
      for (LookupConfiguration lookupConfiguration : lookupClasses) {
        if (lookupConfiguration.lookupClass.getName().equals(className)) {
        lc = lookupConfiguration;
          break;
        }
      }
    }
    return lc;
  }
    
}
