package org.jblooming.waf.html.input;

import org.jblooming.utilities.JSP;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;

public class TextArea extends InputElement {

  public int maxlength;
  public int fieldCols;
  public int fieldRows;
  private PageState pageState;
  private PageContext pageContext;


  public TextArea(String fieldName,
                  String separator,
                  int fieldCols,
                  int fieldRows,
                  String fieldClass) {


    this(fieldName, fieldName,
            separator,
            fieldCols,
            fieldRows,
            fieldClass,
            false,
            false,
            "");
  }

  public TextArea(String label, String fieldName,
                  String separator,
                  int fieldCols,
                  int fieldRows,
                  String script) {


    this(label, fieldName,
            separator,
            fieldCols,
            fieldRows,
            "",
            false,
            false,
            script);
  }



  public TextArea(String label, String fieldName,
                  String separator,
                  int fieldCols,
                  int fieldRows,
                  String fieldClass,
                  boolean disabled,
                  boolean readOnly,
                  String script) {

    this.fieldName = fieldName;
    this.id = fieldName;
    this.label = label;
    this.separator = separator;
    this.fieldCols = fieldCols;
    this.fieldRows = fieldRows;
    this.fieldClass = fieldClass;
    this.disabled = disabled;
    this.readOnly = readOnly;
    this.script = script;

  }


  public StringBuffer toHtmlStringBuffer(PageContext pageContext) {

    ClientEntry field = null;
    if (pageState != null && pageState.getClientEntries() != null) field = pageState.getClientEntries().getEntry(fieldName);
    if (field == null) field = new ClientEntry(null, null);

    StringBuffer sb = new StringBuffer(512);

    if (label == null){
      label = pageState.getI18n(fieldName);
    }
    if(showLabel)
      sb.append(label);

    if (required && label.indexOf("*") == -1)
      sb.append('*');

    //if (field.errorCode != null) sb.append(Html.makeTag("font", "color=\"#ff0000\" size=\"1\"", " error"));

    sb.append(separator);

    sb.append("<textarea ").append(" name=\"").append(fieldName).append('\"').append(" id=\"").append(id).append('\"');
    if (fieldCols > 0) sb.append(" COLS=\"").append(fieldCols).append('\"');
    if (fieldRows > 0) sb.append(" ROWS=\"").append(fieldRows).append('\"');

    if (fieldClass == null || fieldClass.trim().length() == 0) {
      fieldClass = "formElements";
    }

    sb.append(" class=\"").append(fieldClass).append('\"');

    if (tabIndex != 0) {
      sb.append(" tabindex=\"").append(tabIndex).append('\"');
    }

    sb.append(disabled ? " disabled" : "");
    sb.append(readOnly ? " readonly" : "");

    sb.append(script != null && script.length() > 0 ? ' ' + script : "");

    if (required)
      sb.append(" required=\"true\" ");

    if (excludeFromAlert)
      sb.append(" excludeFromAlert=\"true\" ");

    String displayedValue = JSP.encode(field.stringValueNullIfEmpty());

    if (preserveOldValue)
       sb. append(" oldValue=\"").append(JSP.w(displayedValue)).append('\"');

    if (maxlength>0) sb.append(" maxlength=" + maxlength);


    sb.append('>');
    if (field.stringValueNullIfEmpty() != null) {
      sb.append(displayedValue);
    }

    sb.append("</TEXTAREA>");

    return sb;
  }

   public void toHtml(PageContext pageContext) {
      PageState ps = PageState.getCurrentPageState((HttpServletRequest)pageContext.getRequest());
      this.pageState = ps;
      this.pageContext = pageContext;
      super.toHtml(pageContext);
  }


}

