package org.jblooming.waf.html.input;

import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.utilities.JSP;
import org.jblooming.waf.html.layout.Css;
import org.jblooming.waf.html.state.Form;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Date: 19-feb-2003
 * Time: 17.25.51
 *
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 */
public class TextField extends InputElement {

  public static final String init = TextField.class.getName();

  public String type;
  public boolean doFeedBackError = true;
  public int maxlength;

  /**
   * @deprecated
   */
  public String mask = null;
  public String allowedSep = null;
  public boolean autoSize = false;

  private TextField() {
  }

  public TextField(String fieldName, String separator) {
    this("text", fieldName, fieldName, separator, 20, false);
  }

  public TextField(String type,
                   String fieldName,
                   String separator,
                   int fieldSize) {

    this(type, null, fieldName, separator, fieldSize, false, false, -1, null);

  }

  public TextField(String label,
                   String fieldName,
                   String separator,
                   int fieldSize,
                   boolean readOnly) {

    this("text", label, fieldName, separator, fieldSize, false, readOnly, -1, null);
  }

  public TextField(String label,
                   String fieldName,
                   String separator,
                   int fieldSize,
                   boolean readOnly,
                   int maxlength) {


    this("text", label, fieldName, separator, fieldSize, false, readOnly, maxlength, null);

  }

  public  TextField(String type,
                   String label,
                   String fieldName,
                   String separator,
                   int fieldSize,
                   boolean readOnly) {

    this(type, label, fieldName, separator, fieldSize, false, readOnly, -1, null);
  }


  /**
   * @param type
   * @param label
   * @param fieldName
   * @param separator
   * @param fieldSize
   * @param disabled
   * @param readOnly
   * @param script
   */
  public TextField(String type,
                   String label,
                   String fieldName,
                   String separator,
                   int fieldSize,
                   boolean disabled,
                   boolean readOnly,
                   String script) {

    this(type, label, fieldName, separator, fieldSize, disabled, readOnly, -1, script);

  }

  public TextField(String type,
                   String label,
                   String fieldName,
                   String separator,
                   int fieldSize,
                   boolean disabled,
                   boolean readOnly,
                   int maxlength,
                   String script) {

    id = fieldName;
    this.fieldName = fieldName;
    this.maxlength = maxlength;
    this.id = fieldName;
    this.type = type;
    this.label = label;
    this.separator = separator;
    this.fieldSize = fieldSize;
    this.disabled = disabled;
    this.readOnly = readOnly;
    this.script = script;

  }

  public void toHtml(PageContext pageContext) {

    try {
      initMask(pageContext);
    } catch (Exception e) {
      throw new PlatformRuntimeException();
    }
    super.toHtml(pageContext);
  }

  private void initMask(PageContext pageContext) throws ApplicationException, IOException {
    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    if (!ps.initedElements.contains(init) && (mask != null && mask.trim().length() > 0)) {
      pageContext.getOut().write("<script src=\"" + ((HttpServletRequest) pageContext.getRequest()).getContextPath() + "/commons/js/maskInput_js.jsp" + "\"></script>");
      ps.initedElements.add(init);
    }
  }

  protected int getDecodedValueLength() {
    if (value == null || value.stringValueNullIfEmpty() == null)
      return 0;
    Matcher counter = Pattern.compile("&[^&;]+;").matcher(value.stringValueNullIfEmpty());
    String tmp = "";
    int start = 0;
    while (counter.find(start)) {
      int groupStart = counter.start();
      tmp += value.stringValueNullIfEmpty().substring(start, groupStart);
      tmp += "?";
      start = counter.end();
    }
    tmp += JSP.encode(value.stringValueNullIfEmpty().substring(start));
    return tmp.length();
  }

  /**
   * To be used in case of calendar field
   */
  public void addDateControl(Form form) {
    this.script = "onblur =\"" +
      "if(!isDate(" + fieldName + ".value)) { " +
      form.getUniqueName() + "." + fieldName + ".value='';" +
      fieldName + ".focus(); " +
      "return false;" +
      "}\"";
  }

  public StringBuffer toHtmlStringBuffer(PageContext pageContext) throws ApplicationException {

    StringBuffer sb = new StringBuffer();

    if (label == null) label = fieldName;

    if (label != null && label.length() > 0 && showLabel) {
      if (labelClass != null && labelClass.length() > 0)
        sb.append("<span class=\"" + labelClass + "\">");
      sb.append(label);
      if (required && label.indexOf("*") == -1)
        sb.append('*');

      if (labelClass != null && labelClass.length() > 0)
        sb.append("</span>");
    }

    sb.append(separator);

    sb.append("<input type=\"").append(type).append("\" name=\"");

    sb.append(fieldName).append('\"').append(" id=\"").append(id).append('\"');

    //compute value
    String displayedValue = "";

    if (value != null && value.stringValueNullIfEmpty() != null) {
      if (fieldValueHasToBeEncoded)
        displayedValue = JSP.encode(value.stringValueNullIfEmpty());
      else
        displayedValue = value.stringValueNullIfEmpty();
    }

    //print size
    sb.append(" size=");
    if (!autoSize)
      sb.append(fieldSize);
    else
      sb.append(displayedValue.length());

    if (fieldClass == null || fieldClass.trim().length() == 0) {
      fieldClass = Css.formElements;
    }

    sb.append(" class=\"").append(fieldClass).append('\"');

    if (tabIndex != 0) {
      sb.append(" tabindex=\"").append(tabIndex).append('\"');
    }

    if (toolTip != null && toolTip.trim().length() > 0) {
      sb.append(" tooltip=\"").append(getToolTip()).append('\"');
    }

    if (disabled)  {
        sb.append(" disabled");
    }

    sb.append(readOnly ? " readonly" : "");

    //this is because explorer does not make it visible
    if (disabled || readOnly)
      sb.append(" style=\"background-color:#f3f3f3\"");

    if (maxlength>0) sb.append(" maxlength=" + maxlength);

    sb.append(script != null && script.length() > 0 ? ' ' + script : "");

    sb. append(" value=\"").append(displayedValue).append('\"');

    if (launchedJsOnActionListened != null) {
      sb.append(' ' + actionListened + "= \"if (event.keyCode==" + keyToHandle +" && "+(checkCtrlKey ? " event.ctrlKey==true " : " event.ctrlKey==false ") + ") { " + launchedJsOnActionListened);
      sb.append("return false;}\"");
    }

    if (mask != null && mask.trim().length() > 0) {
      if (allowedSep != null)
        sb.append(" removedIM=\"" + allowedSep + "\"");
      sb.append(" inputMask=\"" + mask + "\"");
    }

    if (required)
      sb.append(" required=\"true\" ");
    
    if (excludeFromAlert)
      sb.append(" excludeFromAlert=\"true\" ");

    if (preserveOldValue)
       sb. append(" oldValue=\"").append(displayedValue).append('\"');

    sb.append(" >");

    return sb;
  }



  /**
   * @deprecated
   */
  public static TextField getTransparentInstance(String type,
                                                 String label,
                                                 String fieldName,
                                                 String separator,
                                                 int fieldSize,
                                                 boolean readonly,
                                                 boolean focused,
                                                 PageState view) {
    TextField tf = new TextField(type,
      label,
      fieldName,
      separator,
      fieldSize,
      false, readonly,
      " onFocus=\"this.className='formElementsTransparentOver'\" onBlur=\"this.className='formElementsTransparent'\"");

    return tf;
  }


  private static void hiddenInstanceToHtmlPrivate(String id,String fieldName,PageContext pageContext) {
    TextField tf = new TextField("hidden",fieldName,"",1);
    tf.id = id;
    tf.label = "";
    tf.preserveOldValue = false;
    tf.toHtml(pageContext);
  }

   public static void hiddenInstanceToHtml(String fieldName,PageContext pageContext) {
     hiddenInstanceToHtmlPrivate(fieldName,fieldName,pageContext);
  }

  public static void hiddenInstanceToHtml(String fieldName,String value, PageContext pageContext) {
    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    ps.addClientEntry(fieldName,value);
    hiddenInstanceToHtmlPrivate(fieldName,fieldName,pageContext);
  }

  public static void hiddenInstanceOfFormToHtml(String ceName, Form form, PageContext pageContext) {
    hiddenInstanceToHtmlPrivate(form.getUniqueName()+ceName,ceName,pageContext);
  }
}
