package org.jblooming.waf.html.input;

import org.jblooming.ApplicationException;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.html.button.ButtonSubmit;
import org.jblooming.waf.html.core.HtmlBootstrap;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.html.state.Form;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;

public class TinyMCE extends JspHelper implements HtmlBootstrap {

  public static final String init = TinyMCE.class.getName();
  public TextArea textArea;
  public String theme = "simple"; //"advanced"
  public String mode = "textareas"; // exact
  public String fieldName;
  public String width;
  public String height;
  public String label;
  public String separator;
  public int fieldCols;
  public int fieldRows;
  public String external_image_list_url;  // "example_data/example_image_list.js"
  public String external_link_list_url;   // "example_data/example_link_list.js",
  public String content_css;              //"/example_data/example_full.css",
  public boolean resize = false;

  public boolean showHTMLButton = true;

  public static String baseUrl;

  public static final String DRAW = "DRAW";
  public static final String CLOSE = "CLOSE";

  public static final String MODE_EXACT = "exact";  // see also textareas
  public static final String THEME_ADVANCED = "advanced";
  public static final String THEME_SIMPLE = "simple";  

  private boolean closeCalled=false;
  private boolean drawCalled=false;

  /**
   * added for images deletion in imagesManager.jsp (OL TinyMCE plugin)
   */
  public String objectClass;
  public String imageSearchField;
  public boolean readOnly;
  public boolean required;


  public TinyMCE(String label, String fieldName, String separator, int fieldCols, int fieldRows, PageState pageState) {

    if (id != null)
      this.id = fieldName;
    if (this.theme==null)
      this.theme = TinyMCE.THEME_SIMPLE;
    this.fieldName = fieldName;
    this.urlToInclude = "/commons/layout/partTinyMCE.jsp";
    this.textArea=new TextArea(label, fieldName, separator, fieldCols, fieldRows,"");
    this.textArea.preserveOldValue = false;
    this.textArea.id = this.id;
    pageState.htmlBootstrappers.add(this);
  }

  /**
   * @ deprecated
   */
  /*
  public TinyMCE(String id, PageState pageState) {
//    TextArea ta = new TextArea(pageState.getI18n(this.label), this.fieldName, this.separator,this.fieldCols,this.fieldRows,"");
//    if (id != null)
//      this.id = id;
//    this.urlToInclude = "/commons/layout/partTinyMCE.jsp";
//    this.textArea=ta;
//    //this.textArea.id="HTMLTEXT";
//    this.textArea.id = textArea.id;
//    pageState.htmlBootstrappers.add(this);
    this(null, null, null, 0, 0, pageState);
  }
  */
  public String getDiscriminator() {
    return init;
  }

  public boolean validate(PageState pageState) throws IOException, ServletException {
    return drawCalled && closeCalled;
  }

  private void init(PageContext pageContext)  {
    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    if (!ps.initedElements.contains(init)) {
      pageContext.getRequest().setAttribute(ACTION, INITIALIZE);
      super.toHtml(pageContext);
      ps.initedElements.add(init);
    }
  }

  public void draw(PageContext pageContext) throws IOException, ServletException, ApplicationException {
    drawCalled = true;
    init(pageContext);
    pageContext.getRequest().setAttribute(ACTION, DRAW);
    super.toHtml(pageContext);
  }

  public void close(PageContext pageContext) throws IOException, ServletException, ApplicationException {
    if (!drawCalled)
      throw new ApplicationException("Call draw before close");
    closeCalled = true;
    pageContext.getRequest().setAttribute(ACTION, CLOSE);
    super.toHtml(pageContext);
  }

  /**
   *
   * @param f
   * @return a ButtonSubmit with the additionalOnClickScript initialized as needed by HtmlArea to persist content while browsing through tabs
   */
  public static ButtonSubmit getButtonSubmit(Form f) {
    ButtonSubmit submit = new ButtonSubmit(f);
    // todo eliminare
    //submit.additionalOnClickScript = "obj('HTMLTEXT').value = editor.doctype + ' ' +editor.getHTML(editor._doc.documentElement, true, editor);";
    return submit;
  }

  /**
   * @deprecated
   */
  public void toHtml(PageContext pageContext) {
    throw new RuntimeException("Call start and close");
  }

  public TinyMCE() {
  }

  public String tinyCodeCleaner (String text ) {
    return tinyCodeCleaner(text, null);
  }

  public String tinyCodeCleaner (String text, String pathIncludedContext ) {
    if (text!=null) {
      // href code
      text = StringUtilities.replaceAllNoRegex(text, baseUrl.toLowerCase(), "");
      text = StringUtilities.replaceAllNoRegex(text, baseUrl.toUpperCase(), "");
      text = StringUtilities.replaceAllNoRegex(text, "class=\"mceVisualAid\"", "");
      text = StringUtilities.replaceAllNoRegex(text, "<P mce_keep=\"true\">&nbsp;</P>", "");
      text = StringUtilities.replaceAllNoRegex(text, "mce_keep=\"true\"", "");
      text = StringUtilities.replaceAllNoRegex(text, "mceItemAnchor", "");
      // tinyMCE added code
      text = text.replaceAll("mce[\\w]+=\\\"[^> ]+\\\"", "");
      // empty fields
      if ("<br>".equalsIgnoreCase(text))
        text = "";
    }
    return text;
  }

  public static String getApplicationBaseURL(HttpServletRequest request) {
    String protocol = request.getScheme();
    String server = request.getServerName();
    String port = "" + request.getServerPort();
    String contextPath = request.getContextPath();
    String slash = "";
    if( (!("/".equals(contextPath)) || "".equals(contextPath.trim()))) {
      slash = "/";
    }
    if ("80".equals(port))
      port = "";
    else
      port = ":"+port;

    String redirect = protocol + "://" + server + port + request.getContextPath() + slash ;

    return redirect;
  }


}
