package org.jblooming.waf.html.input;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.ontology.PerformantNode;
import org.jblooming.ontology.PerformantNodeSupport;
import org.jblooming.waf.html.core.HtmlBootstrap;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.html.layout.Css;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.IOException;

/**
 * The TreeCombo is a composite object. It is made by the hidden field (called "fieldName") were the selected ID is stored.
 * Usually you should refer this field. In some cases you'll like refer to the value of inserted input text (for instance if you want to
 * create filter businessLogic "on the fly") that is called  "fieldName"+TEXT_FIELD_POSTFIX. The third component is the iframe, shared between
 * all the combo's in the page.
 * TreeCombo uses onBlur, onKeyUp, onFocus javascript event. If you need to add handle you must use the onBlurAdditionalScript property.
 * <p/>
 * toHtml saves in a map on session with key fieldName and value itself.
 * this in order to make iself instantiatible from the part of jsp that produces the list;
 * this is limited in the sense that if two pages in the same session simultaneously instantiate a combo
 * with the same field name, there might be leaked of filtered data among them. it may be improved on need.
 */
public class TreeCombo2 extends JspHelper implements HtmlBootstrap {

  public boolean shooted = false;

  public boolean useQbe;
  public int heigth;
  public int minCharToQuery = 1;

  public String fieldName;
  public String fieldClass = Css.formElements;
  public String separator = "&nbsp;";
  public String label;
  public String initialSelectedCode;
  public int tabIndex;
  public int maxLenght;
  public String script;
  public String tip;
  public String onBlurAdditionalScript;
  public boolean disabled;

  public int columnToCopyInDescription = 1;


  /**
   * The query must select at least two properties (in case you have one, just repeat it) limited to the select part.
   * Then there are two mandatory parameter "whereForFiltering" and "whereForId" to specify how to retrieve data
   * Both MUST uses the parameter named SmartCombo.FILTER_PARAM_NAME. Additional fixed parameter filter must be defined
   * in the query and set on "fixedParams" map
   * <p/>
   * Of the projected properties (columns for the not so priviledged):
   * 0 element is used as value for the hidden field (ID usually)
   * columnToCopyInDescription element is used for filterning and hence to fill the text field
   * from 1 onwards are used for display
   */
  public Class <? extends PerformantNodeSupport> performantNodeClass;

//  public String hql;
//  public String whereForFiltering;
//  public String whereForId;
//
//
//  /**
//   * these usage is left as an exercise
//   */
//  public QueryHelper queryHelperForFiltering;
//
//  /**
//   * additional fixed values for the hql query. Used to set fixed filter (eg.: "hidden" , true)
//   */
//  public Map<String, Object> fixedParams = new HashTable();



  /**
   * key control
   */
  public String actionListened;
  public int keyToHandle;
  public String launchedJsOnActionListened;
  public static final String TEXT_FIELD_POSTFIX = "_txt";
  public static final String INITIALIZE = "IN";
  public static final String DRAW_INPUT = "DI";
  public static final String FILTER_PARAM_NAME = "filter";
  public int iframe_width = 300;
  public int maxRowToFetch = 20;


  /**
   * a valid value must be selected
   */
  public boolean required = false;
  //todo: support it
  public boolean readOnly = false;

  public String idForm;
  /**
   * when true if found a matching row, select it and close the dropdown
   */
  public boolean closeAutomatically = false;

  public boolean preserveOldValue = true;

  /**
   * this script if filled is launched when a value for the smart combo has been selected
   */
  public String onValueSelectedScript;
  public static String SEPARATOR = "/";

  public TreeCombo2() {
    this.urlToInclude = "/commons/layout/partTreeCombo2.jsp";
  }

  public TreeCombo2(String fieldName, Class <? extends PerformantNodeSupport> performantNodeClass) {
    this();
    this.fieldName = fieldName;
    try {
      if (!(performantNodeClass.newInstance() instanceof PerformantNode))
        throw new PlatformRuntimeException("This constructor of TreeCombo accepts PerformantNode extensions only");
    } catch (InstantiationException e) {
      throw new PlatformRuntimeException(e);
    } catch (IllegalAccessException e) {
      throw new PlatformRuntimeException(e);
    }
    this.performantNodeClass = performantNodeClass;

  }



  public String getDiscriminator() {
    return TreeCombo2.class.getName();
  }

  public boolean validate(PageState pageState) throws IOException, ServletException {
    return true;
  }

  private void init(PageContext pageContext) {
    PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    if (!ps.initedElements.contains(getDiscriminator())) {
      pageContext.getRequest().setAttribute(ACTION, INITIALIZE);
      super.toHtml(pageContext);
      ps.initedElements.add(getDiscriminator());
    }
  }

  public void toHtml(PageContext pageContext) {
    if (shooted)
      throw new RuntimeException("You cannot call toHtml twice in case of TreeCombo");
    shooted = true;

    init(pageContext);
    pageContext.getRequest().setAttribute(ACTION, DRAW_INPUT);
    super.toHtml(pageContext);
  }

  public void toHtmlI18n(PageContext pageContext) {
    PageState pageState = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
    if (label == null)
      label = pageState.getI18n(fieldName);
    toHtml(pageContext);
  }


}
