package org.jblooming.waf.html.menu;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.ontology.Node;
import org.jblooming.operator.Operator;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.security.Permission;
import org.jblooming.utilities.JSP;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.html.button.ButtonLink;
import org.jblooming.waf.html.button.ButtonSubmit;
import org.jblooming.waf.html.button.ButtonSupport;
import org.jblooming.waf.html.core.HtmlBootstrap;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.io.Serializable;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;


public class Menu implements Node {

  public String id;
  public String label;

  private Permission permissionRequired;
  public String additionalScript;
  public String image;
  public String toolTip;
  public boolean writeLabel = true;

  public PageSeed pageSeed;
  public ButtonSupport button;
  private String completeUrl = null;

  private boolean requireLogged = false;
  public boolean enabled = true;
  private boolean hideWhenDisabled = false;
  private boolean drawAsSeparator = false;
  private boolean drawAsTitle = false;

  // popup management
  public String popup_height = null;
  public String popup_width = null;
  public String popup_resizable = "yes";
  public boolean popup = false;

  public String target;


  public Menu parent;
  public LinkedList<Menu> children = new LinkedList<Menu>();

  public Menu(String menuID) {
    super();
    this.id = menuID;
    this.label = menuID;
  }

  public Menu addContentLine(String label) {
    return addContentLine(label, null, null);
  }

  public Menu addContentLine(String label, String completeUrl) {
    Menu m = addContentLine(label, null, null);
    m.completeUrl = completeUrl;
    return m;
  }

  public Menu addContentLine(String label, PageSeed pageSeed) {
    return addContentLine(label, pageSeed, null);
  }

  public Menu addContentLine(ButtonLink buttonLink) {
    Menu menu = addContentLine(buttonLink.label);

    //this is only for compatibility with old calls
    menu.pageSeed = buttonLink.pageSeed;
    
    /*
    menu.popup = buttonLink.pageSeed.isPopup();
    if (menu.popup) {
      menu.popup_height = buttonLink.popup_height;
      menu.popup_width = buttonLink.popup_width;
      menu.popup_resizable = buttonLink.popup_resizable;
    }
    */
    menu.button = buttonLink;
    return menu;
  }

  public Menu addContentLine(ButtonSubmit button) {
     Menu menu = addContentLine(button.label);
    menu.button = button;
    return menu;
  }


  public Menu addContentLine(String label, PageSeed pageSeed, String image) {
    String menuId = StringUtilities.normalize(label);
    return addContentLine(menuId, label, pageSeed, image);
  }

  public Menu addContentLine(String menuId, String label, PageSeed pageSeed, String image) {
    return addContentLine(menuId, label, pageSeed, image, false);
  }

  public Menu addContentLine(String menuId, String label, PageSeed pageSeed, String image, boolean onTop) {
    Menu menu = new Menu(menuId);
    menu.label = JSP.htmlEncodeTag(JSP.htmlEncodeApexes(label));
    menu.pageSeed = pageSeed;
    if (pageSeed != null)
      menu.popup = pageSeed.isPopup();
    menu.image = image;
    if (onTop)
      this.children.addFirst(menu);
    else
      this.children.addLast(menu);
    menu.parent = this;
    return menu;
  }


  public Menu addSeparator() {
    return addSeparator(false);
  }

  public Menu addSeparator(boolean onTop) {
    Menu menu = new Menu("sep");
    menu.drawAsSeparator = true;
    if (onTop)
      this.children.addFirst(menu);
    else
      this.children.addLast(menu);
    menu.parent = this;
    return menu;
  }

  public Menu addTitle(String title) {
    Menu menu = new Menu(StringUtilities.normalize(title));
    menu.label = title;
    menu.drawAsTitle = true;
    this.addChild(menu);
    menu.parent = this;
    return menu;
  }

  public Menu setVisibleForAdministratorsOnly() {
    /**
     * this is a fake permission used to identify the administrators. By default administrators owns every permissison.
     */
    setPermissionRequired(new Permission("__adm_only"));
    hideWhenDisabled = true;
    return this;
  }


  public void toHtml(PageContext pageContext) {
    new Drawer(this).toHtml(pageContext);
  }

  // ------------------------------------------------------- Node implementation -----------------------------------------------


  public void setParent(Menu n) {
    parent = n;
  }

  public void setParentNode(Node node) {
    setParent((Menu) node);
  }

  /**
   * @return always null
   * @deprecated
   */
  public List<Node> getAncestors() {
    return null;
  }

  public Node getParentNode() {
    return getParent();
  }

  public Collection getChildrenNode() {
    return children;
  }

  private Menu getParent() {
    return (Menu) parent;
  }

  private void addChild(Menu menu) {
    children.add(menu);
  }

  /**
   * @deprecated
   */
  public int getIntId() {
    return 0;
  }

  public Serializable getId() {
    return id;
  }

  public void setId(Serializable id) {
    this.id = id + "";
  }

  public String getName() {
    return label;
  }

  public void initialize(String defHref, PageContext pageContext) {
    try {
      pageContext.include(defHref);
    } catch (ServletException e) {
      throw new PlatformRuntimeException(e);
    } catch (IOException e) {
      throw new PlatformRuntimeException(e);
    }
  }

  public Menu setRequireLogged() {
    this.requireLogged = true;
    return this;
  }

  public boolean isRequireLogged() {
    return requireLogged;
  }

  public Menu setRequireLogged(boolean requireLogged) {
    this.requireLogged = requireLogged;
    return this;
  }

  public boolean isEnabled() {
    return enabled;
  }

  public Menu setEnabled(boolean enabled) {
    this.enabled = enabled;
    return this;
  }

  public boolean isHideWhenDisabled() {
    return hideWhenDisabled;
  }

  public Menu setHideWhenDisabled(boolean hideWhenDisabled) {
    this.hideWhenDisabled = hideWhenDisabled;
    return this;
  }

  public Permission getPermissionRequired() {
    return permissionRequired;
  }

  public Menu setPermissionRequired(Permission permissionRequired) {
    this.permissionRequired = permissionRequired;
    return this;
  }

  public Menu getNode(String id) {
    Menu result = null;
    for (Menu node : children) {
      if (node.getId().equals(id)) {
        result = node;
        break;
      } else
        result = node.getNode(id);
      if (result != null)
        break;
    }
    return result;
  }

  // ------------------------------------------------------- DRAWER  -----------------------------------------------

  public class Drawer extends JspHelper implements HtmlBootstrap {

    public final String init = Menu.Drawer.class.getName();
    public boolean inited = false;

    public Menu menu;

    public Drawer(Menu menu) {
      this.menu = menu;
      this.urlToInclude = "/commons/layout/partMenuDrawer.jsp";
    }


    private void init(PageContext pageContext) {
      PageState ps = PageState.getCurrentPageState((HttpServletRequest) pageContext.getRequest());
      if (!ps.initedElements.contains(init)) {
        pageContext.getRequest().setAttribute(ACTION, INITIALIZE);
        super.toHtml(pageContext);
        ps.initedElements.add(init);
      }
      if (!inited) {
        pageContext.getRequest().setAttribute(ACTION, "DRAW_MENU_DEFINITION");
        super.toHtml(pageContext);
        inited = true;
      }

    }

    public void toHtml(PageContext pageContext) {
      init(pageContext);
      pageContext.getRequest().setAttribute(ACTION, "DRAW_MENU_BAR");
      super.toHtml(pageContext);
    }


    public String getDiscriminator() {
      return menu.getId() + "";
    }

    public boolean validate(PageState pageState) throws IOException, ServletException {
      return true;
    }


    public void explodeDefinition(Menu menu, PageContext pageContext) throws IOException {

      JspWriter out = pageContext.getOut();

      SessionState ss = SessionState.getSessionState((HttpServletRequest) pageContext.getRequest());
      Operator logged = null;
      try {
        logged = ss.getLoggedOperator();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      }
      int menuItem = 0;
      //define js variable
      out.println("var " + menu.getId() + "= new Array ();");

      // loop on menu children
      for (Menu child : menu.children) {
        // check permission
//        boolean isEnabled = menu.enabled && child.enabled && userCanSee(menu, logged) && userCanSee(child, logged);
        boolean isEnabled = isEnabledForUser(child, logged);


        if (!isEnabled && child.hideWhenDisabled) {
          //skip
        } else {
          if (child.drawAsSeparator) {
            // js: createSeparator ()
            out.println(menu.getId() + "[" + menuItem + "]= createSeparator();");

          } else if (child.drawAsTitle) {
            // js: createSeparator ()
            out.println(menu.getId() + "[" + menuItem + "]= createContentLineTitle('" + child.label + "');");

          } else {

            if (child.children.size() > 0) {
              // create a submenu
              // js: createSubmenuLine (desc, submenu,imgDef)
              if (isEnabled) {
                out.println(menu.getId() + "[" + menuItem + "]= createSubmenuLine('" + child.label + "','" + child.getId() + "'" + (child.image == null ? "" : ",'" + child.image + "'") + ");");
                explodeDefinition(child, pageContext);
              } else {
                String s = menu.getId() + "[" + menuItem + "]= createContentLineDisabled(" +
                        "\"" + child.label + "\"" +
                        ",\"" + JSP.w(child.image) + "\"" +
                        ");";
                out.println(s);
              }
            } else {

              if (child.button==null) {
              // create a menuline
              //js: createContentLine (desc, lineUrl, target, contentTitle, script,imgDef, enabled)

              String url = (child.pageSeed == null ? (child.completeUrl == null ? "#" : child.completeUrl) : child.pageSeed.toLinkToHref());

              if (child.popup) {
                child.additionalScript = "centerPopup('" + url + "', '" + child.id + "', '" + child.popup_width + "', '" + child.popup_height + "', 'yes','" + child.popup_resizable + "');" +
                        (child.additionalScript == null ? "" : child.additionalScript);
                url = "";
              }


              String s = menu.getId() + "[" + menuItem + "]= createContentLine(" +
                      "\"" + child.label + "\"" +
                      ",\"" + url + "\"" +
                      ",\"" + JSP.w(child.target) + "\"" +
                      ",\"" + JSP.w(child.toolTip) + "\"" +
                      ",\"" + (child.additionalScript == null ? "" : child.additionalScript) + "\"" +
                      ",\"" + JSP.w(child.image) + "\"" +
                      ",\"" + (isEnabled ? "true" : "false") + "\"" +
                      ");";
              out.println(s);

              } else {

                String s = menu.getId() + "[" + menuItem + "]= createContentLine(" +
                      "\"" + child.label + "\"" +
                      ",\"#\"" +
                      ",\"" + JSP.w(child.target) + "\"" +
                      ",\"" + JSP.w(child.toolTip) + "\"" +
                      ",\"" + child.button.generateJs() + "\"" +
                      ",\"" + JSP.w(child.image) + "\"" +
                      ",\"" + (isEnabled ? "true" : "false") + "\"" +
                      ");";
              out.println(s);
              }


            }
          }
          menuItem++;
        }
      }

    }

  }

  public static boolean isEnabledForUser(Menu menu, Operator op) {

    boolean enabledForUser = menu.enabled;
    if (enabledForUser) {

      if (menu.getPermissionRequired() != null) {

        if (op != null && op.hasPermissionFor(menu.getPermissionRequired()))
          enabledForUser = true;
        else
          enabledForUser = false;

      } else if (!menu.requireLogged || op != null) {
        enabledForUser = true;

      } else
        enabledForUser = false;
    }
    return enabledForUser;
  }

  public void moveBefore(Menu target) {
    if (parent!=null) {
      parent.children.remove(this);
      parent.children.add(Math.max(parent.children.indexOf(target),0),this);
    }
  }

  public void moveAt(int pos) {
    if (parent!=null) {
      parent.children.remove(this);
      parent.children.add(pos,this);
    }
  }


}
