package org.jblooming.waf.html.state;

import org.jblooming.ApplicationException;
import org.jblooming.operator.Operator;
import org.jblooming.utilities.StringUtilities;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;


public class ScreenElementStatus {

  public String containerId;
  public String x, y;
  public String w, h;
  public boolean collapsed;
  public boolean hidden;
  public boolean iconized;
  public int status;
  public String focusedId;

  public final static String SES_QUALIFIER = "__SESQL__";

  public ScreenElementStatus(String id) throws ApplicationException {
    validate(id);
    containerId = id;
  }

  private void validate(String candidate) throws ApplicationException {
    if (candidate.indexOf("|") > -1 || candidate.indexOf(SES_QUALIFIER) > -1)
      throw new ApplicationException("You should not use | or " + SES_QUALIFIER + "in keys");
  }

  public String toPersistentString(String key) throws ApplicationException {
    validate(key + containerId);
    return key + '|' + containerId + '|' + x + '|' + y + '|' + w + '|' + h + '|' + collapsed + '|' + hidden + '|' + iconized + '|' + status + '|' + focusedId;
  }

  public static Map getInstanceFromOptions(Operator op) throws ApplicationException {

    Map screenElementsStatus = new Hashtable();

    final Iterator optionsKeysetIterator = op.getOptionsKeysetIterator();
    if (optionsKeysetIterator != null) {
      while (optionsKeysetIterator.hasNext()) {

        String opkey = (String) optionsKeysetIterator.next();
        if (opkey.indexOf(SES_QUALIFIER) == 0) {
          String s[] = StringUtilities.splitToArray((String) op.getOption(opkey), "|");
          String key = s[0];
          String containerId = s[1];
          ScreenElementStatus ses = new ScreenElementStatus(containerId);
          ses.x = s[2];
          ses.y = s[3];
          ses.w = s[4];
          ses.h = s[5];
          ses.collapsed = Boolean.valueOf(s[6]).booleanValue();
          ses.hidden = Boolean.valueOf(s[7]).booleanValue();
          ses.iconized = Boolean.valueOf(s[8]).booleanValue();
          ses.status = Integer.valueOf(s[9]).intValue();
          ses.focusedId = s[10];
          screenElementsStatus.put(key, ses);
        }
      }
    }
    return screenElementsStatus;
  }

}
