package org.jblooming.waf.settings;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.DailyRollingFileAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.hibernate.ConnectionReleaseMode;
import org.hibernate.cache.HashtableCacheProvider;
import org.hibernate.cfg.Configuration;
import org.hibernate.cfg.Environment;
import org.hibernate.connection.C3P0ConnectionProvider;
import org.hibernate.connection.ProxoolConnectionProvider;
import org.hibernate.dialect.Dialect;
import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.operator.Operator;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.persistence.hibernate.PlatformAnnotationConfiguration;
import org.jblooming.system.SystemConstants;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.HttpUtilities;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.utilities.file.FileUtilities;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.constants.OperatorConstants;
import org.logicalcobwebs.proxool.ProxoolConstants;
import org.logicalcobwebs.proxool.ProxoolException;
import org.logicalcobwebs.proxool.ProxoolFacade;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;


/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class AdvancedSettingsImpl implements AdvancedSettings {

  public static String POOL_ACQUIRE_INCREMENT = "1";
  public static String POOL_IDLE_TEST_PERIOD = "300";
  public static String POOL_MAX_SIZE = "100";
  public static String POOL_MAX_STATEMENTS = "0";
  public static String POOL_MIN_SIZE = "10";
  public static String POOL_TIMEOUT = "300";

  private String root;
  private String[] activeLocales;
  private Configuration hibConfiguration = new PlatformAnnotationConfiguration();
  public LogConfiguration logConfiguration;

  public String getRoot() {
    return root;
  }

  public void setRoot(String root) {
    this.root = root;
  }

  public LogConfiguration getLogConfiguration() {
    return logConfiguration;
  }

  public void setLogConfiguration(LogConfiguration logConfiguration) {
    this.logConfiguration = logConfiguration;
  }

  public Configuration getHibConfiguration() {
    return hibConfiguration;
  }

  public void setHibConfiguration(Configuration configuration) {
    this.hibConfiguration = configuration;
  }

  public void createDefaultOperator(PersistenceContext pc) throws PersistenceException, ApplicationException {

    OqlQuery q;
    if (pc == null)
      q = new OqlQuery("from " + Operator.class.getName() + " as op where op.administrator = :admin ");
    else
      q = new OqlQuery("from " + Operator.class.getName() + " as op where op.administrator = :admin ", pc);

    //OqlQuery q = new OqlQuery("from " + Operator.class.getName() + " as op where op.administrator = :admin ", pc);
    q.getQuery().setBoolean("admin", true);
    q.getQuery().setMaxResults(1);

    List opL = q.list();

    if (opL == null || opL.size() == 0) {
      Operator administrator = null;
      try {
        administrator = (Operator) PlatformConfiguration.defaultOperatorSubclass.newInstance();
      } catch (Exception e) {
        throw new PlatformRuntimeException(e);
      }

      administrator.setAdministrator(true);
      administrator.setEnabled(true);
      administrator.setLoginName("administrator");
      administrator.changePassword(administrator.getLoginName(), "");
      administrator.setName("System");
      administrator.setSurname("Manager");
      administrator.store();


      Tracer.platformLogger.info("open lab platform - created default operator administrator");
    }
  }

  public void createSystemOperator(PersistenceContext pc) throws PersistenceException, ApplicationException {
    OqlQuery q;
    if (pc == null)
      q = new OqlQuery("from " + Operator.class.getName() + " as op where op.loginName = :system ");
    else
      q = new OqlQuery("from " + Operator.class.getName() + " as op where op.loginName = :system ", pc);
    q.getQuery().setString("system", "system");
    q.getQuery().setMaxResults(1);


    List opL = q.list();


    if (opL == null || opL.size() == 0) {
      Operator system = null;
      try {
        system = (Operator) PlatformConfiguration.defaultOperatorSubclass.newInstance();
      } catch (Exception e) {
        throw new PlatformRuntimeException(e);
      }
      system.setAdministrator(true);
      system.setLoginName("system");

      /*STANISLAO MOULINSKY
      Acerrimo nemico di Carter, finto barone spagnolo di origine russa,
      è l'asso dei travestimenti: quando si traveste il suo corpo cambia totalmente.
      Si è travestito praticamente da tutto: Contessa spagnola, Uomo delle fogne, armadio,
      Ricetrasmittente da guerra, Angelo viola, maggiordomo, cadavere di banchiere...
      Frase celebre:
      "Ebbene sì, maledetto Carter, hai vinto anche stavolta!"*/
      system.changePassword(system.getLoginName(), "stanislaomoulinsky");
      system.setName("System");
      system.setSurname("Operator");
      system.setEnabled(false);

      system.store();
      Tracer.platformLogger.info("open lab platform - created default system operator");
    }
  }

  public void configHibernateProperties(Configuration hibConfiguration, String POOL_ALIAS, Class dialect) {

    try {
      if (!(dialect.newInstance() instanceof Dialect))
        throw new PlatformRuntimeException("Must configure dialect with a class extending org.hibernate.dialect.Dialect");
    } catch (InstantiationException e) {
      throw new RuntimeException(e);
    } catch (IllegalAccessException e) {
      throw new RuntimeException(e);
    }

    //hibernate
    Properties hibInfo = new Properties();
    // hibInfo.setProperty(Environment.QUERY_SUBSTITUTIONS, "true 'Y', false 'N'"); // todo: this is dialect sensitive
    hibInfo.setProperty(Environment.QUERY_SUBSTITUTIONS, "true 1, false 0"); // todo: this is dialect sensitive

    hibInfo.setProperty(Environment.DIALECT, dialect.getName());

    hibInfo.setProperty(Environment.MAX_FETCH_DEPTH, "2");
    hibInfo.setProperty(Environment.USE_STREAMS_FOR_BINARY, "true");
    hibInfo.setProperty(Environment.STATEMENT_BATCH_SIZE, "15");

    /**
     * without proxool
     */
    /*
    hibInfo.setProperty(Environment.DRIVER,"net.sourceforge.jtds.jdbc.Driver");
    hibInfo.setProperty(Environment.URL,"jdbc:jtds:sqlserver://oldb01.open-lab.com:1433/platform_test");
    hibInfo.setProperty(Environment.USER,"sa");
    hibInfo.setProperty(Environment.PASS,"");
    hibInfo.setProperty(Environment.POOL_SIZE,"1");
    */

    if (ApplicationState.platformConfiguration.useProxool) {
      hibInfo.setProperty(Environment.PROXOOL_POOL_ALIAS, POOL_ALIAS);
      hibInfo.setProperty(Environment.PROXOOL_EXISTING_POOL, "true");
      hibInfo.setProperty(Environment.CONNECTION_PROVIDER, ProxoolConnectionProvider.class.getName());
      hibInfo.setProperty(Environment.RELEASE_CONNECTIONS, ConnectionReleaseMode.ON_CLOSE.toString());
    } else if (ApplicationState.platformConfiguration.useC3P0) {
      hibInfo.setProperty(Environment.CONNECTION_PROVIDER, C3P0ConnectionProvider.class.getName());
      hibInfo.setProperty(Environment.RELEASE_CONNECTIONS, ConnectionReleaseMode.ON_CLOSE.toString());
    }

    hibInfo.setProperty(Environment.CACHE_PROVIDER, HashtableCacheProvider.class.getName());
    hibInfo.setProperty(Environment.USE_REFLECTION_OPTIMIZER, (ApplicationState.platformConfiguration.development ? "false" : "true"));
    hibInfo.setProperty(Environment.USE_QUERY_CACHE, "true");

    //no connection isolation with hsqldb
    if (dialect.getName().indexOf("HSQL") == -1)
      hibInfo.setProperty(Environment.ISOLATION, "2");

    hibConfiguration.addProperties(hibInfo);

  }

  public void configNoPool(Properties hibInfo, String db_user_name, String db_user_psw, String driver_class, String driver_url) {
    hibInfo.setProperty(Environment.DRIVER, driver_class);
    hibInfo.setProperty(Environment.URL, driver_url);
    hibInfo.setProperty(Environment.USER, db_user_name);
    hibInfo.setProperty(Environment.PASS, db_user_psw);
    hibInfo.setProperty(Environment.POOL_SIZE, "1");
  }

  public void configPool(Properties hibInfo, String POOL_ALIAS, String db_user_name, String db_user_psw, String driver_class, String driver_url) {

    if (ApplicationState.platformConfiguration.useProxool) {
      String url = "proxool." + POOL_ALIAS + ":" + driver_class + ":" + driver_url;

      Properties proxoolInfo = new Properties();
      proxoolInfo.setProperty(ProxoolConstants.STATISTICS_PROPERTY, "5m,1d");
      proxoolInfo.setProperty(ProxoolConstants.USER_PROPERTY, db_user_name);
      proxoolInfo.setProperty(ProxoolConstants.PASSWORD_PROPERTY, db_user_psw);
      try {
        ProxoolFacade.registerConnectionPool(url, proxoolInfo);
      } catch (ProxoolException e) {
        throw new RuntimeException(e);
      }
    } else {

      hibInfo.setProperty(Environment.DRIVER, driver_class);
      hibInfo.setProperty(Environment.URL, driver_url);
      hibInfo.setProperty(Environment.USER, db_user_name);
      hibInfo.setProperty(Environment.PASS, db_user_psw);
      hibInfo.setProperty(Environment.POOL_SIZE, "1");
      hibInfo.setProperty(Environment.C3P0_ACQUIRE_INCREMENT, POOL_ACQUIRE_INCREMENT);
      hibInfo.setProperty(Environment.C3P0_IDLE_TEST_PERIOD, POOL_IDLE_TEST_PERIOD);
      hibInfo.setProperty(Environment.C3P0_MAX_SIZE, POOL_MAX_SIZE);
      hibInfo.setProperty(Environment.C3P0_MAX_STATEMENTS, POOL_MAX_STATEMENTS);
      hibInfo.setProperty(Environment.C3P0_MIN_SIZE, POOL_MIN_SIZE);
      hibInfo.setProperty(Environment.C3P0_TIMEOUT, POOL_TIMEOUT);

    }

  }

  public void configLog4J() {

    String logFilesRoot = getRoot() + "WEB-INF/log/";
    String pattern = "%d{yyyy MMM dd HH:mm:ss} %5p %c{1}:%L - %m%n";

    createLogger(pattern, logFilesRoot, Tracer.hibernateLogger, "hibConsoleAppender", "hibernate.log", "hibFileAppender");
    createLogger(pattern, logFilesRoot, Tracer.platformLogger, "platformConsoleAppender", "platform.log", "plaFileAppender");
    createLogger(pattern, logFilesRoot, Tracer.i18nLogger, "i18nConsoleAppender", "i18n.log", "i18nFileAppender");
    createLogger(pattern, logFilesRoot, Tracer.jobLogger, "jobLoggerConsoleAppender", "jobLogger.log", "jobLoggerFileAppender");

  }

  public void createLogger(String pattern, String logFilesRoot, Logger logger, String consoleAppenderName, String logFileName, String fileAppenderName) {
    logger.removeAllAppenders();

    PatternLayout pl = new PatternLayout();
    pl.setConversionPattern(pattern);

    if (getLogConfiguration().isLogOnConsole()) {

      ConsoleAppender consoleAppender = new ConsoleAppender(pl);
      consoleAppender.setName(consoleAppenderName);
      logger.addAppender(consoleAppender);
    }

    if (getLogConfiguration().isLogOnFile()) {
      File log = new File(logFilesRoot + logFileName);
      if (!log.exists())
        log.getParentFile().mkdirs();
      DailyRollingFileAppender hibFileAppender = null;
      try {

        hibFileAppender = new DailyRollingFileAppender(pl, log.getPath(), "'.'yyyy-ww");

      } catch (IOException e) {
        throw new RuntimeException(e);
      }
      hibFileAppender.setName(fileAppenderName);
      logger.addAppender(hibFileAppender);
    }
  }


  public void createDefaultOperators() {
    /**
     * recover or create settings
     */
    PersistenceContext pc = null;

    //issue_PLA_2
    try {
      //if u need remove the operators
      //        OqlQuery q = new OqlQuery("from " + Operator.class.getName() + " as op ", pc);
      //        List<Operator> opL = (List<Operator>)q.list();
      //        for(Operator op: opL){
      //          PersistenceHome.remove(op);
      //        }

      //work in our own session: it may have been pulled down
      //ORIGINAL
      //        pc = HibernateFactory.newFreeSession();
      //        createDefaultOperator(pc);
      //        createSystemOperator(pc);
      //        pc.commitAndClose();

      //MUTATED BY RADIATIONS OF VERACCHI
      createDefaultOperator(null);
      createSystemOperator(null);
      Tracer.platformLogger.info("open lab platform - default operator management loaded ok");

    } catch (Exception e) {
      Tracer.platformLogger.error("AdvancedSettingsImpl.setupPersistentSettings failed: platform setup exception", e);
      Tracer.hibernateLogger.fatal("Database access exception", e);
      ApplicationState.hibernateLoaded = false;
    }
  }

  public void configAndBuildHibSessionFactory() {
    if (!ApplicationState.loaded) {
      HibernateFactory.config = getHibConfiguration();
      try {
        HibernateFactory.setSessionFactory(HibernateFactory.config.buildSessionFactory());
      } catch (Throwable e) {
        e.printStackTrace();
        throw new RuntimeException(e);
      }
    }
    ApplicationState.hibernateLoaded = true;
    Tracer.platformLogger.info("open lab platform - config hibernate factory ok");
  }


  public void configGlobalSettings(PageContext pageContext) {
    String globalPath =
            HttpUtilities.getFileSystemRootPathForRequest((HttpServletRequest) pageContext.getRequest()) +
                    File.separator + "commons" + File.separator + "settings" + File.separator + PlatformConfiguration.globalSettingsFileName;
    File global = new File(globalPath);
    if (!global.exists())
      fillApplication(global);
    ApplicationState.applicationSettings.putAll(FileUtilities.getProperties(globalPath));
  }

  public void fillApplication(File global) {

    Properties prop = new Properties();

    /**
     *      layout
     */

    prop.put(OperatorConstants.FLD_CURRENT_SKIN, "navy");
    prop.put(OperatorConstants.RECENT_VIEWS_SIZE, "6");

    /**
     * general
     */

    prop.put(SystemConstants.FLD_COUNTER_PREFIX, "");
    prop.put(SystemConstants.FLD_PASSWORD_EXPIRY, "0");
    prop.put(SystemConstants.FLD_PASSWORD_MIN_LEN, "0");
    prop.put(SystemConstants.FLD_WORKING_DAY_SAT, Fields.FALSE);
    prop.put(SystemConstants.FLD_SCHEDULED_FREQ, "30");
    prop.put(SystemConstants.FLD_SERVER_TIME_ZONE, DateUtilities.DEFAULT_TIME_ZONE.getID());
    prop.put(SystemConstants.FLD_DEBUG_ON_SERVER, Fields.FALSE);
    prop.put(SystemConstants.FLD_PERFORMANCE_TUNING, Fields.FALSE);
    prop.put(SystemConstants.FLD_FRIENDLY_ERRORS, Fields.FALSE);
    prop.put(SystemConstants.FLD_PRETTY_HTML, Fields.FALSE);
    prop.put(SystemConstants.FLD_LOG_LOGIN, Fields.FALSE);
    prop.put(SystemConstants.FLD_LOG_ERROR, Fields.FALSE);
    prop.put(SystemConstants.AUDIT, Fields.FALSE);
    prop.put(SystemConstants.FLD_INDEX_EVERY_HOUR, "0");
    prop.put(SystemConstants.FLD_MAX_FILE_SIZE, "500");
    prop.put(SystemConstants.STORAGE_PATH_ALLOWED, "");
    File rep = new File(getRoot() + "/WEB-INF/data");
    rep.mkdirs();
    String repUrl = rep.getAbsolutePath();
    prop.put(SystemConstants.FLD_REPOSITORY_URL, repUrl);
    prop.put(SystemConstants.DEFAULT_DATE_FORMAT, "yyyy MM dd");
    prop.put(SystemConstants.DEFAULT_DATE_DISPLAY_FORMAT, "EEEE, MMM dd yyyy");
    prop.put(SystemConstants.ACTIVE_LOCALES, StringUtilities.unSplit(Arrays.asList(activeLocales), "|"));

    /**
     *      behaviour
     */

    prop.put(OperatorConstants.FLD_STICKY_BY_MAIL, Fields.TRUE);
    prop.put(OperatorConstants.FLD_ENABLE_VERSION_AUTO, Fields.TRUE);
    prop.put(OperatorConstants.FLD_SELECT_LANG, DateUtilities.DEFAULT_LOCALE.getLanguage());
    prop.put(OperatorConstants.FLD_LOGIN_COOKIES, Fields.FALSE);
    prop.put(OperatorConstants.FLD_LOGOUT_TIME, SystemConstants.DEFAULT_SESSION_EXPIRY + "");
    prop.put(OperatorConstants.FLD_ENABLE_VERSION_AUTO, Fields.TRUE);
    prop.put(OperatorConstants.FLD_WORKING_HOUR_TOTAL, "08:00");
    prop.put(OperatorConstants.FLD_WORKING_HOUR_BEGIN, "09:00");
    prop.put(OperatorConstants.FLD_WORKING_HOUR_END, "18:00");
    prop.put(OperatorConstants.FLD_HOUR_DAY_START, "08:00");
    prop.put(OperatorConstants.FLD_HOUR_DAY_END, "23:00");
    prop.put(OperatorConstants.FLD_REQUIRE_EMAIL, Fields.TRUE);


    try {
      prop.store(new FileOutputStream(global), "global settings");
      Tracer.platformLogger.info("open lab platform - created default global settings file: " + global);
    } catch (IOException e) {
      throw new PlatformRuntimeException(e);
    }
  }
}

