package org.jblooming.waf.settings;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.messaging.MessagingSystem;
import org.jblooming.security.Permission;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.FinalKeyMap;
import org.jblooming.utilities.HashTable;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.utilities.file.FileUtilities;
import org.jblooming.waf.ActionController;
import org.jblooming.waf.DefaultCommandController;

import java.io.*;
import java.util.Map;
import java.util.Properties;
import java.util.TreeSet;

public class ApplicationState {

  public static I18n i18n = new I18n();

  private static Map configuredUrls = new FinalKeyMap();

  public static Map skins = new FinalKeyMap();

  public static PlatformConfiguration platformConfiguration;

  public static boolean loaded;
  public static boolean poolLoaded;
  public static boolean hibernateLoaded;
  private static String version;

  public static final String APPLICATIONSTATE = "APP_ST";
  public static final String SETTINGS = "STS";
  public static final String HIBERNATE_CONFIGURATION = "HIBERNATE_CONFIGURATION";

  public static MessagingSystem ms = new MessagingSystem();

  public static Map applicationSettings = new HashTable();
  public static Map<String, Object> applicationParameters = new HashTable<String, Object>();

  public static Map<String, Permission> permissions = new HashTable();


  /**
   * this is the controller used by the command.jsp to communicate client->server.
   * Can be overriden by other application's specific controller
   */
  public static ActionController commandController = new DefaultCommandController();
  /**
   * http(s)/server name:server port/context path
   */
  public static String serverURL;
  public static String webAppFileSystemRootPath;
  public static String contextPath;


  private ApplicationState() {
  }

  public static String getApplicationSetting(String key) {

    if (ApplicationState.getApplicationSettings() == null)
      return null;

    Object o = ApplicationState.getApplicationSettings().get(key);
    if (o == null)
      return null;

    return (String) o;
  }

  public static String getApplicationSetting(String key, String defaultValue) {
    String value = getApplicationSetting(key);
    if (value == null)
      value = defaultValue;
    return value;
  }

  public static Map getApplicationSettings() {
    return applicationSettings;
  }

  public static Map getConfiguredUrls() {
    return configuredUrls;
  }

  /**
   * @deprecated use the parametric version
   */
  public static String getVersion() throws IOException {
    return getVersion("", false);
  }

  public static String getVersion(String app) throws IOException {
    return getVersion(app, false);
  }

  /**
   * @param app    it's the "appName.number" file in which the version of the particular application is set
   * @param reload
   * @return jblooming.number concatenated to opnlb.number concatenated to app concatenated to build.number
   * @throws IOException
   */
  public static String getVersion(String app, boolean reload) throws IOException {

    if (ApplicationState.version == null || reload) {

      String version = "";

      boolean found = false;

      //for backward compatibility
      if (app.indexOf(".number") > -1)
        app = app.substring(0, app.indexOf(".number"));

      Application application = platformConfiguration.applications.get(app);

      if (application != null) {
        Properties props = null;
        try {
          props = getApplicationBuild(application);
          version += props.getProperty("version") + ".";
        } catch (Exception e) {
          Tracer.platformLogger.error("File " + application.getName() + ".number not found in settings.", e);
        }
        found = true;
      } else
        version += "[" + app + " not found].";

      version += "(jblooming framework version: ";
      version += jbloomingVersion();
      version += ")";
      if (found)
        ApplicationState.version = version;
    }
    return version;
  }

  public static Properties getApplicationBuild(Application application) throws FileNotFoundException, IOException {
    String rootFolder = StringUtilities.replaceAllNoRegex(application.getRootFolder(), "/", File.separator);
    InputStream is = new FileInputStream(
            ApplicationState.webAppFileSystemRootPath +
                    File.separator + rootFolder + File.separator + "settings" + File.separator + application.getName() + ".number");
    Properties props = new Properties();
    props.load(is);
    return props;
  }

  public static String getApplicationVersion() {
    try {
      Properties props = getApplicationBuild(platformConfiguration.defaultApplication);
      return props.getProperty("version");
    } catch (Exception e) {
      Tracer.platformLogger.error("File " + platformConfiguration.defaultApplication + ".number not found in settings.", e);
      return null;
    }
  }


  public static String getApplicationSchemaLog(Application application) throws FileNotFoundException, IOException {
    String rootFolder = StringUtilities.replaceAllNoRegex(application.getRootFolder(), "/", File.separator);
    String pathname = ApplicationState.webAppFileSystemRootPath +
            File.separator + rootFolder + File.separator + "settings" + File.separator + application.getName() + "SchemaLog.number";
    File file = new File(pathname);
    String result = "";
    if (file.exists()) {
      result = FileUtilities.readTextFile(pathname);
    }
    return result;
  }

  public static void setApplicationSchemaLog(Application application, String content) throws FileNotFoundException, IOException {
    String rootFolder = StringUtilities.replaceAllNoRegex(application.getRootFolder(), "/", File.separator);
    String pathname = ApplicationState.webAppFileSystemRootPath +
            File.separator + rootFolder + File.separator + "settings" + File.separator + application.getName() + "SchemaLog.number";
    FileUtilities.writeToFile(pathname, content);
  }

  public static void setApplicationBuild(Application application, Properties p) throws FileNotFoundException, IOException {
    String rootFolder = StringUtilities.replaceAllNoRegex(application.getRootFolder(), "/", File.separator);

    FileOutputStream fos = new FileOutputStream(
            ApplicationState.webAppFileSystemRootPath + File.separator + rootFolder + File.separator + "settings" + File.separator + application.getName() + ".number");

    p.store(fos, "");
  }

  public static String getBuild() throws IOException {
    String build = "";
    Properties props = getBuildProps();
    build = props.getProperty("build");
    return build;
  }


  public static Properties getBuildProps() throws IOException {

    Properties props = new Properties();
    FileInputStream is = is = new FileInputStream(
            ApplicationState.webAppFileSystemRootPath + File.separator + "commons" + File.separator + "settings" + File.separator + "Platform.number");

    if (is != null) {
      props.load(is);
    }

    return props;
  }

  public static String jbloomingVersion() throws IOException {
    String platform = "";
    Properties props = null;
    try {
      props = getBuildProps();
      platform += props.getProperty("version") + ".";
      platform += props.getProperty("build");
    } catch (Exception e) {
      Tracer.platformLogger.error("File Platform.number not found in settings.", e);
    }

    return platform;
  }

  public static void dumpApplicationSettings() {

    String fileName = ApplicationState.webAppFileSystemRootPath +
            File.separator + "commons" + File.separator + "settings" + File.separator + PlatformConfiguration.globalSettingsFileName;
    StringBuffer value = new StringBuffer();

    TreeSet sett = new TreeSet(ApplicationState.applicationSettings.keySet());
    for (Object key : sett) {
      value.append(key).append("=").append(ApplicationState.applicationSettings.get(key)).append("\n");
    }
    try {
      FileUtilities.writeToFile(fileName, value.toString());
    } catch (IOException e) {
      throw new PlatformRuntimeException(e);
    }

  }

  public static Map<String, Permission> getPermissions() {
    synchronized(permissions) {
    return permissions;
    }
  }

  public static void setPermissions(Map<String, Permission> permissions) {
    synchronized(ApplicationState.permissions) {
    ApplicationState.permissions = permissions;
    }
  }
}