package org.jblooming.waf.settings.businessLogic;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.utilities.HttpUtilities;
import org.jblooming.waf.UploadHelper;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.settings.Application;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.settings.I18n;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

public class I18nAction {
  public static final String SEPARATOR = "_-_";


  public void cmdList(PageState pageState) {
  }


  public void cmdEdit(PageState pageState) {
    I18n i18nManager = ApplicationState.i18n;
    String code = pageState.getEntry(Fields.FORM_PREFIX + "code").stringValueNullIfEmpty();
    Set supportedLanguages = i18nManager.supportedLanguages;
    Collection applics = ApplicationState.platformConfiguration.applications.values();

    pageState.addClientEntry(Fields.FORM_PREFIX + "code", code);
    if (code != null) {
      for (Iterator iterator = applics.iterator(); iterator.hasNext();) {
        Application applic = (Application) iterator.next();
        String appname = applic.getName();
        for (Iterator iterator1 = supportedLanguages.iterator(); iterator1.hasNext();) {
          String lang = (String) iterator1.next();
          pageState.addClientEntry(Fields.FORM_PREFIX + SEPARATOR + appname + SEPARATOR + lang, i18nManager.getRawLabel(code, appname, lang));
        }
      }
    }
  }


  public void cmdSave(PageState pageState, HttpServletRequest request) throws ActionException {
    I18n i18nManager = ApplicationState.i18n;
    String code = pageState.getEntry(Fields.FORM_PREFIX + "code").stringValueNullIfEmpty();
    Set supportedLanguages = i18nManager.supportedLanguages;
    Collection applics = ApplicationState.platformConfiguration.applications.values();
    if (code != null) {
      for (Iterator iterator = pageState.getClientEntries().getEntryKeys().iterator(); iterator.hasNext();) {
        String key = (String) iterator.next();
        if (key.startsWith(Fields.FORM_PREFIX + SEPARATOR)) {
          String appname = key.substring((Fields.FORM_PREFIX + SEPARATOR).length());
          String lang = appname.substring(appname.indexOf(SEPARATOR) + 3);
          appname = appname.substring(0, appname.indexOf(SEPARATOR));
          String value = pageState.getEntry(key).stringValueNullIfEmpty();
          if (value != null) {
            I18n.I18nEntry i18ne=i18nManager.addEntry(code, appname, lang, value);
            i18ne.setSuspect(false);
          } else {
            i18nManager.removeEntry(code, appname, lang);
          }
        }
      }

      try {
        cmdDump(pageState, request);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }


    }
  }


  public void cmdDump(PageState pageState, HttpServletRequest request) throws IOException {
    I18n i18nManager = ApplicationState.i18n;
    Collection applics = ApplicationState.platformConfiguration.applications.values();
    for (Iterator iterator = applics.iterator(); iterator.hasNext();) {
      Application application = (Application) iterator.next();
      String root = HttpUtilities.getFileSystemRootPathForRequest(request);
      File file = new File(root + File.separator + application.getRootFolder() + File.separator + "settings" + File.separator + "i18n" + File.separator + application.getName() + ".i18n");
      i18nManager.persistI18nByApplicationAndLanguage(application, file);
    }
  }


  public void cmdReload(PageState pageState, HttpServletRequest request) throws IOException {
    I18n i18nManager = ApplicationState.i18n;
    i18nManager.supportedLanguages.clear();
    i18nManager.codeEntries.clear();
    Collection applics = ApplicationState.platformConfiguration.applications.values();
    for (Iterator iterator = applics.iterator(); iterator.hasNext();) {
      Application application = (Application) iterator.next();
      I18n.loadI18n(application, request);
    }
  }


  public void cmdChangeModality(PageState pageState) throws ActionException, PersistenceException {
    I18n i18nManager = ApplicationState.i18n;
    String selectedEdit = pageState.getEntry(Fields.FORM_PREFIX + "I18N_MODALITY").stringValueNullIfEmpty();
    int selectedLenientLevel = i18nManager.getLenient();
    try {
      selectedLenientLevel = pageState.getEntry(Fields.FORM_PREFIX + "I18N_LENIENT").intValue();
    } catch (ParseException e) {
    }
    i18nManager.setLenient(selectedLenientLevel);

    i18nManager.setI18nEditingOperatorId(0);
    if (I18n.EDIT_STATUS_EDIT.equals(selectedEdit)) {
      i18nManager.setEditStatus(I18n.EDIT_STATUS_EDIT);
      i18nManager.setI18nEditingOperatorId(pageState.sessionState.getLoggedOperator().getId());
    } else if (I18n.EDIT_STATUS_EDIT_POPUP.equals(selectedEdit)) {
        i18nManager.setEditStatus(I18n.EDIT_STATUS_EDIT_POPUP);
        i18nManager.setI18nEditingOperatorId(pageState.sessionState.getLoggedOperator().getIntId());
    } else if (I18n.EDIT_STATUS_APPEND.equals(selectedEdit)) {
      i18nManager.setEditStatus(I18n.EDIT_STATUS_APPEND);
    } else { // default as read
      i18nManager.setEditStatus(I18n.EDIT_STATUS_READ);
    }

  }

  public void cmdNewLanguage(PageState pageState) {
    I18n i18nManager = ApplicationState.i18n;
    String newLanguage = pageState.getEntry(Fields.FORM_PREFIX + "ADD_NEW_LANGUAGE").stringValueNullIfEmpty();
    if (newLanguage != null)
      i18nManager.supportedLanguages.add(newLanguage.toUpperCase());
  }

  public void cmdStoreLabel(PageState pageState) {
    String code = pageState.getEntry(Fields.FORM_PREFIX + "code").stringValueNullIfEmpty();
    String label = pageState.getEntry(Fields.FORM_PREFIX + "label").stringValueNullIfEmpty();
    String application = pageState.getEntry(Fields.FORM_PREFIX + "appl").stringValueNullIfEmpty();
    String language = pageState.getSessionState().getCurrentLocale().getLanguage();
    I18n i18nManager = ApplicationState.i18n;
    i18nManager.addEntry(code, application, language, label);
  }

  public void cmdImportFromFile(PageState pageState) {

    UploadHelper uh = UploadHelper.getInstance("I18N_FILE", pageState);
    File ecchecazz = uh.temporaryFile;
    try {
      ApplicationState.i18n.loadI18n(ecchecazz);
    } catch (IOException e) {
      throw new PlatformRuntimeException(e);
    }
    pageState.removeEntry("I18N_FILE");


  }
}
