package org.jblooming.waf.state;

import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.operator.Operator;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.view.ClientEntries;
import org.jblooming.waf.view.PageState;

import java.util.*;

public class PersistentSearch {

  private String category;
  private String name;
  private Map entries = new Hashtable();
  protected final static String METASEPARATOR = "__x__";
  protected final static String SEPARATOR = "__+__";
  protected final static String SEPARATORPAIR = "__$__";
  public static final String PERSISTENT_SEARCH = "PRSSRC";


  public Map getEntries() {
    return entries;
  }

  public void setEntries(Map entries) {
    this.entries = entries;
  }

  public void putEntry(String key, String value) {
    getEntries().put(key, value);
  }

  public void removeEntryByKey(String key) {
    getEntries().remove(key);
  }

  public void removeEntryByValue(String value) {
    Iterator keySet = new HashSet(getEntries().keySet()).iterator();
    while (keySet.hasNext()) {
      final String key = (String) keySet.next();
      String loopValue = getEntry(key);
      if (loopValue.equals(value)) {
        removeEntryByKey(key);
      }
    }
  }

  public String getEntry(String key) {
    return (String) getEntries().get(key);
  }

  public Iterator getEntriesKeysetIterator() {
    return getEntries().keySet().iterator();
  }

  public int entryKeysetSize() {
    return getEntries().keySet().size();
  }

  public String getCategory() {
    return category;
  }

  public void setCategory(String category) {
    this.category = category;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }


  public String getFlatCopy() {

    StringBuffer value = new StringBuffer();
    if (entries.keySet() != null && entries.keySet().size() > 0) {
      for (Iterator iterator = entries.keySet().iterator(); iterator.hasNext();) {
        String key = (String) iterator.next();
        value.append(key).append(SEPARATOR).append(entries.get(key)).append(SEPARATORPAIR);
      }
    }
    return category + METASEPARATOR + name + METASEPARATOR + value.toString();
  }

  public static PersistentSearch getInstance(String data) {

    String[] codes = StringUtilities.splitToArray(data, METASEPARATOR);
    PersistentSearch pf = new PersistentSearch();

    pf.category = codes[0];
    pf.name = codes[1];
    final String value = codes[2];
    String[] pairs = StringUtilities.splitToArray(value, SEPARATORPAIR);
    for (int j = 0; j < pairs.length; j++) {
      String pair = pairs[j];
      if (pair != null && pair.length() > 0) {
        String[] cv = StringUtilities.splitToArray(pair, SEPARATOR);
        pf.putEntry(cv[0], cv[1]);
      }
    }
    return pf;
  }

  public static void feedPageState(String data, PageState pageState) {

    String[] codes = StringUtilities.splitToArray(data, METASEPARATOR);
    final String value = codes[2];
    String[] pairs = StringUtilities.splitToArray(value, SEPARATORPAIR);
    for (int j = 0; j < pairs.length; j++) {
      String pair = pairs[j];
      if (pair != null && pair.length() > 0) {
        String[] cv = StringUtilities.splitToArray(pair, SEPARATOR);
        pageState.addClientEntry(cv[0], cv[1]);
      }
    }

  }

  public static boolean feedFromSavedSearch(PageState pageState, Operator w) throws FindByPrimaryKeyException, StoreException {
    boolean fed = false;

    String filterName = pageState.getEntry(Fields.FLD_FILTER_NAME).stringValueNullIfEmpty();
    String filterNameInhibited = pageState.getEntry(Fields.FLD_FILTER_NAME_INHIBITED).stringValueNullIfEmpty();

    if (filterName != null) {

      boolean isInhibited = false;

      if (filterNameInhibited != null) {
        if (filterName.equals(filterNameInhibited)) {
          isInhibited = true;
          pageState.deleteEntry(Fields.FLD_FILTER_NAME);
        }
        pageState.deleteEntry(Fields.FLD_FILTER_NAME_INHIBITED);
      }

      final String command = pageState.getCommand();

      if (Commands.CMD_DEL_FILTER.equals(command)) {

        Operator persOp = (Operator) PersistenceHome.findByPrimaryKey(Operator.class, w.getId());
        persOp.removeFilterByKey(filterName);
        persOp.store();
        w.removeFilterByKey(filterName);
        pageState.deleteEntry(Fields.FLD_FILTER_NAME);

      } else if (!isInhibited) {

        String data = w.getFilter(filterName);
        feedPageState(data, pageState);
        pageState.addClientEntry(Fields.FLD_FILTER_NAME_INHIBITED, filterName);

        fed = true;
      }

    }
    return fed;
  }

  public static boolean feedFromDefaultSearch(String category, PageState pageState, Operator w) {
    boolean found = false;
    if (pageState.getEntry(Fields.FLD_FILTER_NAME).stringValueNullIfEmpty() == null) {

      Map<String, String> filters = w.filters;
      for (String fn : filters.keySet()) {
        if (fn.startsWith(category + "d:")) {
          pageState.addClientEntry(Fields.FLD_FILTER_NAME, fn);
          found = true;
          break;
        }
      }
    }
    return found;
  }


  public static void saveSearch(String category, boolean recoveredFromSavedFilter, PageState pageState, Operator w) throws PersistenceException {
    saveSearch(category, recoveredFromSavedFilter, null, null, pageState, w);
  }

  public static void saveSearch(String category, boolean recoveredFromSavedFilter, Set<String> allowedPrefixes, Set<String> allowedCes, PageState pageState, Operator w) throws PersistenceException {

    if (w != null && !recoveredFromSavedFilter) {

      String newFilterName = pageState.getEntry(Fields.FLD_NEW_FILTER_NAME).stringValueNullIfEmpty();
      if (newFilterName != null) {

        PersistentSearch pf = new PersistentSearch();
        pf.setName(category + newFilterName);
        pf.setCategory(category);

        final ClientEntries clientEntries = pageState.getClientEntries();
        for (Iterator iterator = clientEntries.getEntryKeys().iterator(); iterator.hasNext();) {
          String key = (String) iterator.next();

          if (!allowedCes.contains(key) && allowedPrefixes != null) {
            boolean isAllowed = false;
            for (String allPref : allowedPrefixes) {
              if (key.startsWith(allPref))
               isAllowed = true;
            }
            if (!isAllowed)
              continue;
          }

          String value = clientEntries.getEntry(key).stringValueNullIfEmpty();
          if (!Fields.FLD_FILTER_NAME.equals(key) && !Fields.FLD_NEW_FILTER_NAME.equals(key) && value != null) {
            pf.putEntry(key, (String) value);
          }

        }

        Operator persOp = (Operator) PersistenceHome.findByPrimaryKey(Operator.class, w.getId());
        //remove if existing
        for (String currentFilterName :  new HashSet<String>(persOp.getFilters().keySet())) {
          if (pf.getName().toLowerCase().equals(currentFilterName.toLowerCase())) {
            persOp.removeFilterByKey(currentFilterName);
          }
        }

        persOp.putFilter(pf.getName(), pf.getFlatCopy());
        persOp.store();
        try {
          pageState.sessionState.refreshLoggedOperator();
        } catch (ApplicationException e) {
          throw new PlatformRuntimeException(e);
        }

        w.putFilter(pf.getName(), pf.getFlatCopy());
        pageState.deleteEntry(Fields.FLD_NEW_FILTER_NAME);
        pageState.addClientEntry(Fields.FLD_FILTER_NAME, pf.getName());
        pageState.addClientEntry(Fields.FLD_FILTER_NAME_INHIBITED, pf.getName());
      }
    }
  }

}

