/*
 * Created by Roberto Bicchierai and Pietro Polsinelli.
 * User: Pietro Polsinelli
 * Date: Jul 4, 2002
 * Time: 12:11:14 PM
 */
package org.jblooming.waf.view;

import org.jblooming.ontology.Identifiable;
import org.jblooming.utilities.HashTable;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;


public class ClientEntries {

  private Map<String,ClientEntry> clientEntries = new LinkedHashMap<String,ClientEntry>();


  public ClientEntries addEntry(ClientEntry ce) {
    clientEntries.put(ce.name, ce);
    return this;
  }

  public ClientEntries addEntry(String name, String value) {
    ClientEntry ce = new ClientEntry(name, value);
    clientEntries.put(ce.name, ce);
    return this;
  }

  public ClientEntry getEntry(String key) {
    return (ClientEntry) clientEntries.get(key);
  }

  public Set<String> getEntryKeys() {
    return clientEntries.keySet();
  }

  public void addEntries(ClientEntries ce) {
    if (ce != null && ce.clientEntries != null) clientEntries.putAll(ce.clientEntries);
  }

  public void addEntries(Collection<ClientEntry> ces) {
   for (ClientEntry ce : ces) {
     clientEntries.put(ce.name,ce);
   }
  }

  public void setClientEntriesInnerMap(Map<String,ClientEntry> clientEntries) {
    this.clientEntries = clientEntries;
  }

  public Map<String,ClientEntry> getClientEntriesInnerMap() {
    return clientEntries;
  }

  public ClientEntries getNewInstance() {
    ClientEntries newCes = new ClientEntries();
    if (clientEntries != null) {
      for (String s : clientEntries.keySet()) {
        newCes.addEntry(this.getEntry((String) s).getNewInstance());
      }
    }
    return newCes;
  }

  public int size() {
    return clientEntries.size();
  }

  public void deleteEntry(String key) {
    clientEntries.remove(key);
  }

  public void addRequiredEntry(String name, String value) {
    ClientEntry ce = new ClientEntry(name, value);
    ce.required = true;
    clientEntries.put(ce.name, ce);
  }

  public String toString(){
    StringBuffer ret= new StringBuffer();
    for (ClientEntry ce : clientEntries.values()){
      ret.append(ce.name).append(":").append(ce.stringValueNullIfEmpty()).append("\n");
    }
    return ret.toString();
  }

  public Collection<ClientEntry> getClientEntries() {
    return clientEntries.values();
  }

  public Map<String,ClientEntry> getEntriesStartingWithStripped(String prefix) {
    Map<String,ClientEntry> selected = new HashTable<String,ClientEntry>();
    for (ClientEntry ce : getClientEntries()) {
      if (ce.name.startsWith(prefix))
        selected.put(ce.name.substring(prefix.length()),ce);
    }
    return selected;
  }

   public Map<String,ClientEntry> getEntriesStartingWithStripped(String prefix, String requestedValue) {
     Map<String,ClientEntry> selected = new HashTable<String,ClientEntry>();
      for (ClientEntry ce : getClientEntries()) {
        if (ce.name.startsWith(prefix) && requestedValue.equals(ce.stringValueNullIfEmpty()))
          selected.put(ce.name.substring(prefix.length()),ce);
      }
    return selected;

   }

  public boolean validEntries() {

    boolean result  = false;

    for (ClientEntry ce : clientEntries.values()) {
      if (ce.errorCode!=null) {
        result = true;
        break;
      }
    }
    return !result;
  }

  public void addEntry(String fieldName, Identifiable identifiable) {
    if (identifiable!=null)
      addEntry(fieldName,""+identifiable.getId());
  }

}
