package org.jblooming.waf.view;


import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.ontology.Identifiable;
import org.jblooming.operator.Operator;
import org.jblooming.page.Page;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.transaction.hibernate.HibernateTransaction;
import org.jblooming.utilities.HashTable;
import org.jblooming.utilities.HttpUtilities;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.ActionController;
import org.jblooming.waf.ScreenArea;
import org.jblooming.waf.ScreenRoot;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.html.container.ButtonBar;
import org.jblooming.waf.html.core.HtmlBootstrap;
import org.jblooming.waf.html.core.JspIncluder;
import org.jblooming.waf.html.display.DeletePreviewer;
import org.jblooming.waf.html.display.HeaderFooter;
import org.jblooming.waf.html.display.MenuRenderer;
import org.jblooming.waf.html.layout.Skin;
import org.jblooming.waf.html.menu.Menu;
import org.jblooming.waf.html.state.Form;
import org.jblooming.waf.settings.Application;
import org.jblooming.waf.settings.ApplicationState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.util.*;

public class PageState extends PageSeed {

  public Map<String, Page> pages = new HashTable();
  public SessionState sessionState;
  public Identifiable mainObject;
  public Map attributes = new HashMap();

  public Set initedElements = new HashSet();

  /**
   * elements of the screen
   */
  private Form form;
  private HeaderFooter headerFooter;

  /**
   * template composition
   */
  public boolean screenRunning = false;

  public ScreenArea runningControllerScreenArea;

  private Stack screenAreas = new Stack();
  public ScreenRoot rootScreen;

  /**
   * display validation
   */
  public HtmlBootstrap.HtmlBootstrappers htmlBootstrappers = new HtmlBootstrap.HtmlBootstrappers();

  private String focusedObjectDomId = null;

  /**
   * Used for inserting an additonal menu; you may wonder "to which menu",
   * and this we leave to your fantasy :-)
   *
   * @see org.jblooming.waf.html.display.MenuRenderer
   */
  public MenuRenderer additionalMenu;

  private JspIncluder mainJspIncluder;
  public String fileSystemRootPath;
  private ButtonBar mainButtonBar;

  public Menu menu;
  public DeletePreviewer deletePreviewer;
  public String relativeUrlToRoot;

  public PageState(String url, SessionState sm) {
    if (url == null || url.trim().length() == 0)
      throw new PlatformRuntimeException("ViewHlp::buildView invalid parameters.");
    setHref(url);
    setSessionState(sm);
  }

  public void setFocusedObjectDomId(String domId) {
    //if (focusedObjectDomId == null || domId == null)
      focusedObjectDomId = domId;
    //else
    //  throw new PlatformRuntimeException("Only one element can be focused, current is " + focusedObjectDomId);
  }


  public String getI18n(String name) {
    if (name != null && name.trim().length()>0) {
      String locLang = sessionState.getCurrentLocale().getLanguage().toUpperCase();
      String application = getApplication().getName(); 
      String val = ApplicationState.i18n.getLabel(name, getApplication().getName(), locLang, this);
      return val;
    } else
      return "";
  }

  /**
   * all the occurrences of '%%' in the i18nzation of rootToLocalize will be replaced by param
   */
  public String getI18n(String rootToLocalize, String param) {
    String translation = getI18n(rootToLocalize);
    return translation.replaceAll("%%", (param != null ? param : "-"));
  }

  public String getI18n(String rootToLocalize, String param1, String param2) {
    String translation = getI18n(rootToLocalize);

    return translation.replaceFirst("%%", (param1!=null ? param1 : "-")).replaceFirst("%%", (param2!=null ? param2 : "-"));
  }

  public ClientEntries getClientEntries() {
    if (super.getClientEntries() == null)
      super.setClientEntries(new ClientEntries());
    return super.getClientEntries();
  }

  public void setClientEntries(ClientEntries clientEntries) {
    super.setClientEntries(clientEntries);
  }

  public String getDisplayName() throws FindByPrimaryKeyException {
    return getI18n(getName());
  }

  public boolean mainObjIsNew() throws ApplicationException {

    Identifiable mainObject = getMainObject();
    if (mainObject == null && mainObjectId == null)
      throw new ApplicationException("mainObjIsNew cannot be determined as both mainObject and mainObjectId are null");

    if (mainObject != null && PersistenceHome.NEW_EMPTY_ID.equals(mainObject.getId()))
      return true;
    else if (mainObject != null && !PersistenceHome.NEW_EMPTY_ID.equals(mainObject.getId()))
      return false;
    else
      return PersistenceHome.NEW_EMPTY_ID.equals(getMainObjectId());
  }

  public static boolean objIsNew(Identifiable o) throws ApplicationException {
    if (o == null)
      throw new ApplicationException("objIsNew cannot be determined for a null object.");
    return PersistenceHome.NEW_EMPTY_ID.equals(o.getId());
  }

  public SessionState getSessionState() {
    return sessionState;
  }

  public void setSessionState(SessionState sessionState) {
    this.sessionState = (SessionState) sessionState;
  }

  public Identifiable getMainObject() {
    return mainObject;
  }

  public void setMainObject(Identifiable mainObject) {
    this.mainObject = mainObject;
  }

  public Object getAttribute(String key) {
    return attributes.get(key);
  }

  public void setAttribute(String key, Object value) {
    attributes.put(key, value);
  }

  public static PageState getCurrentPageState(HttpServletRequest request) {
    Object o = request.getAttribute(Fields.VIEW);
    if (o == null)
      throw new PlatformRuntimeException("View::getCurrentPageState parameter Fields.VIEW cannot be null.");
    return (PageState) request.getAttribute(Fields.VIEW);
  }

  public PageState perform(HttpServletRequest request, HttpServletResponse response) {
    while (!screenAreas.empty()) {
      try {
        ScreenArea screenArea = (ScreenArea) screenAreas.pop();
        ActionController controller = screenArea.controller;
        if (controller != null) {
          runningControllerScreenArea = screenArea;
          controller.perform(request, response);
        }
      } catch (Exception e) {
        throw new PlatformRuntimeException(e);
      }
    }

    //this is a beatiful hack 30/11/2005 a.d. SPQR
    //this commits the data set on hibernate session before showing the page body
    //it is necessary in order to be able to refresh objects on page w.r.t. database status
    // by using a PersistenceHome.refresh
    HibernateTransaction currentTransaction = HibernateTransaction.getCurrentTransaction();
    if (currentTransaction != null && !currentTransaction.isFailed()) {
      currentTransaction.checkPoint();
    }

    return this;
  }

  public void setForm(Form f) {
    this.form = f;
  }

  public Form getForm() {
    return form;
  }

  public void focusToHtml(PageContext pageContext) throws IOException {
    if (focusedObjectDomId != null)
      pageContext.getOut().print("<script defer=\"yes\">obj('" + focusedObjectDomId + "').focus();</script>");
  }

  public PageState registerPart(ScreenArea screenArea) {
    this.screenAreas.push(screenArea);
    return this;
  }

  public String toString() {
    return "\nScreenBasic = " + rootScreen;
  }

  public void toHtml(PageContext pageContext) {
    if (rootScreen == null)
      throw new PlatformRuntimeException("PageState calling toHtml on rootScreen==null");
    rootScreen.toHtml(pageContext);
  }

  public static void buildPartsAndClientEntries(HttpServletRequest request, PageState pageState) {

    ClientEntries clientEntries = new ClientEntries();

    Enumeration parameters = request.getParameterNames();
    while (parameters.hasMoreElements()) {
      String requestString = (String) parameters.nextElement();
      if (requestString != null) {
        if (requestString.startsWith(Fields.VIEW_PREFIX)) {
          String viewPropertyName = requestString;
          String viewPropertyValue = request.getParameter(requestString).trim();

          if (viewPropertyName.equals(Fields.OBJECT_ID))
            pageState.setMainObjectId(viewPropertyValue);

          else if (viewPropertyName.equals(Fields.NO_RESUBMIT)) {
            if (viewPropertyValue.equals(Fields.TRUE))
              pageState.setNoResubmit(true);

          } else if (viewPropertyName.equals(Fields.VIEW_ID)) {
            pageState.setResubmitCheck(viewPropertyValue);

          } else if (viewPropertyName.equals(Fields.POPUP))
            pageState.setPopup(true);

        } else if (Commands.COMMAND.equals(requestString))
          pageState.setCommand(request.getParameter(requestString));
        else {
          clientEntries.addEntry(new ClientEntry(requestString, StringUtilities.arrayToString(request.getParameterValues(requestString), ",").trim()));
        }
      }
    }
    pageState.setClientEntries(clientEntries);
  }

  public static String getParameterValues(HttpServletRequest request, String paramName) {
    return StringUtilities.arrayToString(request.getParameterValues(paramName), ",");
  }

  public HeaderFooter getHeaderFooter() {
    if (headerFooter == null)
      headerFooter = new HeaderFooter(this);
    return headerFooter;
  }

  public Skin getSkin() {
    return sessionState.getSkin();
  }

  /**
   * @deprecated use getEntry
   */
  public ClientEntry getClientEntry(String clientEntryName) {
    ClientEntry result = null;
    if (getClientEntries() != null && clientEntryName != null)
      result = getClientEntries().getEntry(clientEntryName);
    return result;
  }

  public Page getPage() {
    return (Page) pages.get("DEFAULT_PAGE");
  }

  public void setPage(Page page) {
    this.pages.put("DEFAULT_PAGE", page);
  }

  private void updateSeed(PageSeed newSeed) {
    this.href = newSeed.getHref();
    this.setMainObjectId(newSeed.getMainObjectId());
    setClientEntries(newSeed.getClientEntries());
    this.command = newSeed.getCommand();
    setLoginRequiring(newSeed.isLoginRequiring());
    setPopup(newSeed.isPopup());
    setResubmitCheck(newSeed.getResubmitCheck());
    this.setNoResubmit(newSeed.isNoResubmit());
  }

  /**
   * @see org.jblooming.waf.ScreenBasic changeBody(org.jblooming.waf.view.PageState, org.jblooming.waf.view.PageSeed)
   */
  public void redirect(PageSeed newSeed) {
    updateSeed(newSeed);
    final ScreenArea basicScreen = this.runningControllerScreenArea;
    ScreenRoot lw = basicScreen.parent;
    this.screenRunning = false;
    lw.urlToInclude = newSeed.href;
  }

  public void setApplication(Application application) {
    sessionState.setApplication(application);
  }

  public Application getApplication() {
    return sessionState.getApplication();
  }

  /**
   * @param sessionKey the key with which the page was put in session by setting the value of
   * @see org.jblooming.waf.constants.Fields#FLD_SAVE_HISTORY
   * @deprecated
   */
  public void backTo(String sessionKey) {
    final ClientEntries histCes = sessionState.recentViews.get(sessionKey);
    if (histCes != null)
      this.getClientEntries().addEntries(histCes);
  }

  public PageSeed thisPage(HttpServletRequest request) {
    int i = href.indexOf("?");
    if (i == -1)
      i = href.length();

    String newHref = href.substring(0, i);
    //if there is no path, do not touch it
    if (!(newHref.indexOf("/") == -1))
      newHref = request.getContextPath() + newHref;
    return new PageSeed(newHref);
  }

  public void setMainJspIncluder(JspIncluder jspIncluder) {
    this.mainJspIncluder = jspIncluder;
  }

  public JspIncluder getMainJspIncluder() {
    return mainJspIncluder;
  }

  public PageSeed pageInThisFolder(String page, HttpServletRequest request) {
    String realURI = HttpUtilities.realURI(request);
    String href = request.getContextPath() + realURI.substring(0, realURI.lastIndexOf("/")) + "/" + page;
    return new PageSeed(href);
  }

  /**
   *
   * @param request
   * @return the pagesed representing the current part. It is used tipically in the webParts
   */
  public PageSeed pagePart(HttpServletRequest request) {
     String realURI = HttpUtilities.realURI(request);
     return new PageSeed(request.getContextPath() +realURI);
   }


  /**
   * @param page
   * @return the page from root of current application
   */
  public PageSeed pageFromRoot(String page) {
    if (!page.startsWith("/"))
      page = "/" + page;
    return new PageSeed(ApplicationState.contextPath + "/" + getApplication().getRootFolder() + page);
  }

  /**
   * @param page
   * @return the page from root of current application
   */
  public PageSeed pageFromCommonsRoot(String page) {
    if (!page.startsWith("/"))
      page = "/" + page;
    return new PageSeed(ApplicationState.contextPath + "/commons" + page);
  }

  //new values in pageState override old ones
  public void incrementState() {
    ClientEntries ces = null;

    if (mainObjectId != null)
      ces = sessionState.formsUnion.get(href + mainObjectId);
    else
      ces = sessionState.formsUnion.get(href);

    if (ces == null)
      ces = getClientEntries();
    else {
      //if (preserveTemporaryValues)
      //  getClientEntries().addEntries(ces);
      //else
      ces.addEntries(getClientEntries());
    }

    if (mainObjectId != null)
      sessionState.formsUnion.put(href + mainObjectId, ces);
    else
      sessionState.formsUnion.put(href, ces);

    setClientEntries(ces);
  }

  public void setButtonBar(ButtonBar bb2) {
    this.mainButtonBar = bb2;
  }

  public ButtonBar getButtonBar() {
    return mainButtonBar;
  }

  public boolean validEntries() {
    return getClientEntries().validEntries();
  }

  public ClientEntry getEntryOrDefault(String name) throws PersistenceException {
    ClientEntry ce = getEntry(name);
    ce.persistInOptions = true;
    if (ce.name == null) {
      String option = sessionState.getOperatorOption(name);
      if (option != null) {
        ce.name = name;
        ce.setValue(option);
        addClientEntry(ce);
      }
    }
    return ce;
  }

  public ClientEntry getEntryOrDefault(String name, String defaultValue) throws PersistenceException {
    ClientEntry ce = getEntryOrDefault(name);
    if (ce.stringValueNullIfEmpty() == null) {
      ce.name = name;
      ce.setValue(defaultValue);
      addClientEntry(ce);
    }
    return ce;
  }


  public void saveEntriesInDefaults() throws PersistenceException {
    if (sessionState.getLoggedOperator() != null) {

      Operator op = (Operator) PersistenceHome.findByPrimaryKey(Operator.class, sessionState.getLoggedOperator().getId());

      boolean changedSomeOption = false;
      for (ClientEntry ce : getClientEntriesSet()) {
        if (ce.persistInOptions) {

          String value = ce.stringValueNullIfEmpty();
          String optValue = op.getOption(ce.name);
          if (value != null && !value.equals(optValue)) {
            op.putOption(ce.name, ce.stringValueNullIfEmpty());
            changedSomeOption = true;
          } else if (value == null && optValue != null) {
            op.removeOptionByKey(ce.name);
            changedSomeOption = true;
          }
        }
      }
      if (changedSomeOption) {
        op.store();
        HibernateTransaction.getCurrentTransaction().checkPoint();
        try {
          sessionState.refreshLoggedOperator();
        } catch (ApplicationException e) {
          throw new PlatformRuntimeException(e);
        }
      }
    }
  }

  public void removeEntry(String ceName) {
    getClientEntries().getClientEntriesInnerMap().remove(ceName);
  }

  public CompanyCalendar getCalendar(Date startDate) {
    return getCalendar(startDate.getTime());
  }

  public CompanyCalendar getCalendar(long millis) {
    CompanyCalendar cc = new CompanyCalendar(sessionState.getCurrentLocale());
    cc.setTimeInMillis(millis);
    return cc;
  }

  public void saveInHistory() {
    sessionState.pageHistory.saveInHistory(this);
  }

  public void saveInHistoryAndResetCommand() {
    command = null;
    sessionState.pageHistory.saveInHistory(this);
  }

  public PageSeed getNewInstance() {
    PageSeed ps = super.getNewInstance();
    // pageState differs satanically from pageSeed in that its href by default is not prefixed with context path
    ps.href = ApplicationState.contextPath+ps.href;
    return ps;
  }


  
}
