package org.jblooming.waf.view;

import org.jblooming.ApplicationException;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.Commands;
import org.jblooming.waf.constants.FieldErrorConstants;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.settings.ApplicationState;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.util.*;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Ezio Manetti emanetti@open-lab.com
 */
public abstract class ViewHlp {

  private static boolean debugActive = false;
  private static boolean debugExistActive = false;
  private static String sp = "    ";

  public static String getViewName(HttpServletRequest request) throws ApplicationException {
    if (request.getParameter(Fields.VIEW_NAME) == null) throw new ApplicationException("View::getViewName " + Fields.VIEW_NAME + " cannot be null.");
    return request.getParameter(Fields.VIEW_NAME);
  }

  public static String getCommand(HttpServletRequest request) {
    return request.getParameter(Commands.COMMAND);
  }

  public static String displayLoc(Map locContent, SessionState sm) {
    if (locContent == null || locContent.isEmpty()) return "";
    String loc = sm.getCurrentLocale().getLanguage();
    String result = "";
    if (locContent.get(loc) != null) result = (String) (locContent.get(loc));
    return result.trim();
  }

  /**
   * @deprecated
   */
  public static ClientEntry existsNotNull(String ceFullName, PageState view) throws ActionException {
    return existsNotNull(ceFullName, view, true);
  }

  /**
   * @deprecated
   */
  public static String getItEvenEmpty(String ceFullName, PageState view) {
    ClientEntry ce = view.getClientEntry(ceFullName);
    if (ce == null) return "";
    return ce.stringValueNullIfEmpty();
  }

  /**
   * @deprecated
   */
  public static ClientEntry existsNotNull(String ceFullName, PageState view, boolean insertError) throws ActionException {
    ClientEntries clientEntries = view.getClientEntries();
    if (clientEntries == null || ceFullName == null)
      throw new ActionException(FieldErrorConstants.ERR_FIELD_CANT_BE_NULL, view);

    ClientEntry ce = clientEntries.getEntry(ceFullName);

    if (ce == null) throw new ActionException(FieldErrorConstants.ERR_FIELD_CANT_BE_NULL, view);

    if (ce.stringValue() == null) {
      if (insertError) {
        ce.errorCode = FieldErrorConstants.ERR_FIELD_CANT_BE_NULL;
      }
      throw new ActionException(FieldErrorConstants.ERR_FIELD_CANT_BE_NULL, view);
    }
    return ce;
  }

  /**
   * @deprecated
   */
  public static ClientEntry existsNotEmpty(String ceFullName, PageState view) throws ActionException {
    return existsNotEmpty(ceFullName, view, true);
  }

  /**
   * @deprecated
   */
  public static ClientEntry existsNotEmpty(String ceFullName, PageState view, boolean insertError) throws ActionException {

    ClientEntries clientEntries = view.getClientEntries();
    if (clientEntries == null || ceFullName == null)
      throw new ActionException(FieldErrorConstants.ERR_FIELD_CANT_BE_EMPTY, view);

    ClientEntry ce = clientEntries.getEntry(ceFullName);

    if (ce == null)
      throw new ActionException(FieldErrorConstants.ERR_FIELD_CANT_BE_EMPTY, view);

    if (ce.stringValue() != null && ce.stringValue().trim().equals("")) {
      if (insertError) {
        ce.errorCode = FieldErrorConstants.ERR_FIELD_CANT_BE_EMPTY;
      }
      throw new ActionException(FieldErrorConstants.ERR_FIELD_CANT_BE_EMPTY, view);
    }
    return ce;
  }

  /**
   * @deprecated
   */
  public static ClientEntry existsEmpty(String ceFullName, PageState view) {

    ClientEntries clientEntries = view.getClientEntries();
    ClientEntry ce = clientEntries.getEntry(ceFullName);
    return ce;
  }

  /**
   * @deprecated
   */
  public static long existsLong(String ceFullName, PageState view, boolean insertError) throws ActionException {
    ClientEntry ce = null;
    boolean exists = false;
    long l = 0;

    try {
      ce = existsNotEmpty(ceFullName, view, insertError);
      exists = true;
    } catch (ActionException e) {
      ce = new ClientEntry(ceFullName, 0 + "");
    }

    if (exists) {
      try {
        l = Long.parseLong(ce.stringValue());
      } catch (NumberFormatException e) {
        if (insertError)
          ce.errorCode = FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC;
        throw new ActionException(FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC, view);
      }
    }

    return l;
  }

  /**
   * @deprecated
   */
  public static int existsPercentile(String ceName, PageState view, boolean insertError) throws ActionException {

    ClientEntry ce = existsNotEmpty(ceName, view, insertError);
    int i = 0;
    try {
      i = Integer.parseInt(ce.stringValue());
      if (i < 0 || i > 100) {
        if (insertError)
          ce.errorCode = FieldErrorConstants.ERR_FIELD_MUST_BE_PERCENTILE;
        throw new ActionException(FieldErrorConstants.ERR_FIELD_MUST_BE_PERCENTILE, view);
      }
    } catch (NumberFormatException e) {
      ce.errorCode = FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC;
      throw new ActionException();
    }
    return i;
  }


  /**
   * @deprecated use exist
   */
  public static ClientEntry ifExistsIsInteger(String ceFullName, PageState view) throws ActionException {
    ClientEntry ce = null;
    boolean exists = false;

    try {
      ce = existsNotNull(ceFullName, view, true);
      exists = true;
    } catch (ActionException e) {
      ce = new ClientEntry(ceFullName, 0 + "");
    }

    if (exists) {
      try {
        Integer.parseInt(ce.stringValue());
      } catch (NumberFormatException e) {
        ce.errorCode = FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC;
        throw new ActionException(FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC, view);
      }
    }

    return ce;

    /*
     ViewHlp.getItEvenEmpty()

    ClientEntry ce = view.getClientEntry(ceFullName);
    if (ce == null) return "";
    return ce.value;
    */
  }

  /**
   * @param ceFullName
   * @param view
   * @param insertError
   * @return
   * @throws ActionException
   * @deprecated use exist
   */
  public static ClientEntry ifExistsIsDouble(String ceFullName, PageState view, boolean insertError) throws ActionException {
    ClientEntry ce = null;
    boolean exists = false;

    try {
      ce = existsNotNull(ceFullName, view, insertError);
      exists = true;
    } catch (ActionException e) {
      ce = new ClientEntry(ceFullName, 0 + "");
    }

    if (exists) {
      try {
        Double.parseDouble(ce.stringValue());
      } catch (NumberFormatException e) {
        if (insertError)
          ce.errorCode = FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC;
        throw new ActionException(FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC, view);
      }
    }

    return ce;
  }

  /**
   * @deprecated use exist
   */
  public static long ifExistsIsPercentile(String ceFullName, PageState view, boolean insertError) throws ActionException {

    ClientEntry ce = null;
    boolean exists = false;
    long l = 0;

    try {
      ce = existsNotNull(ceFullName, view, insertError);
      exists = true;
    } catch (ActionException e) {
      ce = new ClientEntry(ceFullName, 0 + "");
    }

    if (exists) {
      try {
        l = Long.parseLong(ce.stringValue());
        if (l < 0 || l > 100) {
          if (insertError)
            ce.errorCode = FieldErrorConstants.ERR_FIELD_MUST_BE_PERCENTILE;
          throw new ActionException(FieldErrorConstants.ERR_FIELD_MUST_BE_PERCENTILE, view);
        }
      } catch (NumberFormatException e) {
        if (insertError)
          ce.errorCode = FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC;
        throw new ActionException(FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC, view);
      }
    }

    return l;
  }

  /**
   * @deprecated
   */
  public static Date existsDate(String ceName, PageState view) throws ActionException {
    return existsDate(ceName, view, true);
  }

  /**
   * @deprecated
   */
  public static Date existsDate(String ceName, PageState view, boolean insertError) throws ActionException {

    ClientEntry ce = existsNotEmpty(ceName, view, insertError);
    Date d = null;
    try {
      d = DateUtilities.dateFromStringJS(ce.stringValue());
    } catch (ParseException e) {
      throw new ActionException(FieldErrorConstants.ERR_FIELD_MUST_BE_DATELIKE, view);
    }

    return d;
  }

  public static Hashtable getLocalizedContents(String fieldIdentifier, HttpServletRequest request) {
    Hashtable des = new Hashtable();
    String activeLocales[] = StringUtilities.splitToArray(ApplicationState.getApplicationSetting("ACTIVE_LOCALES"), "|");
    String lc;
    for (int i = 0; i < activeLocales.length; i++) {

      lc = request.getParameter(Fields.FORM_PREFIX + fieldIdentifier + activeLocales[i]);

      if (lc != null && lc.trim().length() > 0) {
        //html encode
        if ((ApplicationState.getApplicationSetting("HTMLENCODE")).equals("yes"))
        //lc = HTMLEncoder.encode(lc);
          des.put(activeLocales[i], lc.trim());
      }
    }

    return des;
  }

  public static void resetCheckedClientEntries(PageSeed view) {
    ClientEntries ces = view.getClientEntries().getNewInstance();
    if (ces != null && ces.getEntryKeys() != null) {
      Iterator j = ces.getEntryKeys().iterator();
      while (j.hasNext()) {
        ClientEntry ce = ces.getEntry((String) j.next());
        if (ce.name != null && ce.name.indexOf(Fields.CHECK_PREFIX) > -1)
          view.getClientEntries().getClientEntriesInnerMap().remove(ce.name);
      }
    }
  }


  public static StringBuffer viewBeforeDispatchDebug(PageState view, HttpServletRequest request) {

    StringBuffer sb = new StringBuffer(512);

    //header
    sb.append("<hr>");
    sb.append(" VIEW " + view.getHref() + "<br>");
    sb.append("<hr>");
    sb.append("<br>host " + request.getHeader("Host") + " remote addr = " + request.getRemoteAddr() + " url = " + view.getHref());
    sb.append("<br><b>before dispatch</b> debug:<br><br>");

    // main data
    sb.append("<br><br>" + sp + "main obj id = " +
            view.getMainObjectId() +
            (view.getMainObject() != null ? "; obj = " + view.getMainObject() : "   obj is null") +
            "<br>");
    sb.append(sp + "   command = " + view.getCommand() + "<br>");
    clientEntriesDebug(view, sb);
    return sb;
  }

  private static void clientEntriesDebug(PageState view, StringBuffer sb) {

    // ces
    if (view.getClientEntries() != null && view.getClientEntries().getEntryKeys() != null) {
      TreeSet k = new TreeSet(view.getClientEntries().getEntryKeys());
      sb.append("<br>" + sp + "non empty client entries:");
      for (Iterator iterator = k.iterator(); iterator.hasNext();) {
        String key = (String) iterator.next();
        ClientEntry b = view.getEntry(key);
        if (b != null && ((b.stringValueNullIfEmpty() != null && b.stringValueNullIfEmpty().length() > 0) || Tracer.showAllCes)) {
          sb.append("<br>" + sp + "ce: " + b.name + " = " + b.stringValueNullIfEmpty());
          if (b.errorCode!=null)
            sb.append(" error:"+b.errorCode);
        }

      }
      sb.append("<br>");
    }
  }

  public static StringBuffer ceDebug(PageState view) {
    StringBuffer sb = new StringBuffer(512);
    if (view.getClientEntries() != null && view.getClientEntries().getEntryKeys() != null) {
      Collection k = view.getClientEntries().getEntryKeys();
      for (Iterator iterator = k.iterator(); iterator.hasNext();) {
        String key = (String) iterator.next();
        ClientEntry b = view.getEntry(key);
        if (b.errorCode != null)
          sb.append("<br>ce with error: " + b.name + " = " + b.stringValueNullIfEmpty() + " error: " + b.errorCode);
      }
      sb.append("<br>");
    }
    return sb;
  }

  /**
   * @deprecated use PageState instance methods
   */
  public static int existsInteger(String ceName, PageState view, boolean insertError) throws ActionException {

    ClientEntry ce = existsNotEmpty(ceName, view, insertError);
    int i = 0;
    try {
      i = Integer.parseInt(ce.stringValue());
    } catch (NumberFormatException e) {
      ce.errorCode = FieldErrorConstants.ERR_FIELD_MUST_BE_NUMERIC;
      throw new ActionException();
    }
    return i;
  }

  /**
   * @deprecated use PageState instance methods
   */
  public static long existsLong(String ceName, PageState view) throws ActionException {

    ClientEntry ce = existsNotEmpty(ceName, view, true);
    long i = 0;
    try {
      i = Long.parseLong(ce.stringValue());
    } catch (NumberFormatException e) {
      throw new ActionException();
    }
    return i;
  }

  /**
   * @deprecated use PageState instance methods
   */
  public static double existsDouble(String ceName, PageState view) throws ActionException {

    return existsDouble(ceName, view, true);
  }

  /**
   * @deprecated use PageState instance methods
   */
  public static double existsDouble(String ceName, PageState view, boolean insertError) throws ActionException {
    ClientEntry ce = existsNotEmpty(ceName, view, insertError);
    double i = 0;
    try {
      i = Double.parseDouble(ce.stringValue());
    } catch (NumberFormatException e) {
      throw new ActionException();
    }
    return i;
  }

  public static String toStringBoolean(boolean b) {
    if (b)
      return Fields.TRUE;
    else
      return Fields.FALSE;
  }


}