package org.jblooming.designer;

import com.opnlb.website.portlet.Portlet;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.waf.html.container.ButtonBar;
import org.jblooming.waf.html.state.Form;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.io.Serializable;
import java.util.HashSet;
import java.util.LinkedHashMap;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class Designer extends Portlet {

  public boolean fieldsConfig;

  public String designerNamePart;
  public Class referenceClass;
  public Serializable referenceId;

  public Form form;
  public ButtonBar buttonBar = new ButtonBar();

  public boolean readOnly = false;
  public boolean exportable = false;

  public LinkedHashMap<String, DesignerField> designerFields = new LinkedHashMap<String, DesignerField>();

  public static final String FORM_URL = "FORM_URL";
  public static final String FORM_ID = "FORM_ID";
  public static final String FORM_TITLE = "FORM_TITLE";
  public static final String STEP = "STEP";
  public static final String STEP_TITLE = "STEP_TITLE";


  public static final String DRAW_FORM = "DRAW_FORM";
  public static final String DRAW_STEP = "DRAW_STEP";
  public static final String DRAW_BUTTONS = "DRAW_BUTTONS";

  private DesignerData designerData;

  public Designer(String urlToInclude, String designerNamePart, Class referenceClassName, Serializable referenceId) {
    super();
    this.urlToInclude = urlToInclude;
    this.referenceId = referenceId;
    this.referenceClass = referenceClassName;
    this.designerNamePart = designerNamePart;
  }

  public Detail addDetail(String name) {
    Detail d = new Detail();
    d.name = name;
    d.designer = this;
    designerFields.put(name, d);
    return d;
  }

  public String getDiscriminator() {
    return Designer.class.getName();
  }

  public boolean validate(PageState pageState) throws IOException, ServletException {
    return true;
  }

  public DesignerField add(DesignerField designerField) {
    designerField.designer = this;
    designerFields.put(designerField.name, designerField);
    return designerField;
  }

  public void draw(String designerField, PageContext pageContext) {
    designerFields.get(designerField).toHtml(pageContext);
  }

  //innerr class competence

  /**
   * @deprecated call drawForm
   */
  public void toHtml(PageContext pageContext) {
    throw new PlatformRuntimeException("call drawForm");
  }

  public void drawDesigner(Form htmlForm, PageContext pageContext) throws IOException, ServletException {
    this.form = htmlForm;
    drawForm(pageContext);
  }

  private void drawForm(PageContext pageContext) throws IOException, ServletException {
    pageContext.getRequest().setAttribute(Designer.DRAW_STEP, designerNamePart);
    pageContext.getRequest().setAttribute(ACTION, DRAW_FORM);
    super.toHtml(pageContext);
  }


  public void drawButtons(PageContext pageContext) throws IOException, ServletException {
    pageContext.getRequest().setAttribute(ACTION, DRAW_BUTTONS);
    urlToInclude = "/commons/layout/partDesignerButtons.jsp";
    super.toHtml(pageContext);
  }

  public void configFields(PageContext pageContext) throws IOException, ServletException {
    fieldsConfig = true;
    drawForm(pageContext);
    fieldsConfig = false;
  }

  public boolean isRequired() {
    boolean result = false;
    for (DesignerField df : designerFields.values()) {
      if (df.required) {
        result = true;
        break;
      }
    }
    return result;
  }

  public boolean isFulfilled() {

    DesignerData designerData = getDesignerData();
    boolean result = true;
    for (DesignerField df : designerFields.values()) {
      if(df instanceof Detail) {
        Detail detail = (Detail) df;
        if(detail.detailDesignerFields != null && detail.detailDesignerFields.size()>0) {
          for (DesignerField df_D : detail.detailDesignerFields.values()) {
            if(df_D.required) {
              result = checkDetailFulFilled(designerData,detail);
              break;
            }
          }
          if(!result)
            break;
        }
      } else {
        String value = designerData.getValueMap().get(df.name);
        if (df.required && !(value != null && value.trim().length() > 0)) {
          result = false;
          break;
        }
      }
    }
    return result;
  }

  private boolean checkDetailFulFilled(DesignerData designerData,Detail detail) {
    if(designerData.getValueMap() != null && designerData.getValueMap().size()>0){
      HashSet<Integer> ids = new HashSet<Integer>();
      for (String key : designerData.getValueMap().keySet()) {
        if (key.contains(detail.name+"_")) {
          String id = key.substring(key.lastIndexOf("_")+1,key.length());
          try {
            ids.add(Integer.parseInt(id));
          } catch(NumberFormatException e) {
            e.printStackTrace();
          }
        }
      }
      if(ids != null && ids.size()>0) {
        for (int id : ids) {
          for (DesignerField df : detail.detailDesignerFields.values()) {
            String value = designerData.getValueMap().get(detail.name+"_"+df.name+"_"+id);
            if (df.required && !(value != null && value.trim().length() > 0)) {
              return false;
            }
          }
        }
      }
    }
    return true;
  }

  public DesignerData getDesignerData() {
    if (designerData == null)
      designerData = DesignerData.getInstance(designerNamePart, referenceId, referenceClass.getName());
    return designerData;
  }


  public ClientEntry getEntry(String fieldName, PageState pageState) {
    return pageState.getEntry(DesignerField.DESIGNER_FIELD_PREFIX+fieldName);
  }
}


