package org.jblooming.designer;

import org.jblooming.ontology.IdentifiableSupport;
import org.jblooming.oql.OqlQuery;
import org.jblooming.utilities.HashTable;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Map;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class DesignerData extends IdentifiableSupport {

  private Serializable referenceId;
  private String referenceClassName;

  /**
   * the form to which this data refers to, as several forms (here called designers) can have the same reference
   */
  private String designerName;


  private Map<String, String> valueMap = new HashTable();


  public Serializable getReferenceId() {
    return referenceId;
  }

  public void setReferenceId(Serializable referenceId) {
    this.referenceId = referenceId;
  }

  public String getReferenceClassName() {
    return referenceClassName;
  }

  public void setReferenceClassName(String referenceClassName) {
    this.referenceClassName = referenceClassName;
  }

  public Map<String, String> getValueMap() {
    return valueMap;
  }

  public void setValueMap(Map<String, String> valueMap) {
    this.valueMap = valueMap;
  }

  public String getDesignerName() {
    return designerName;
  }

  public void setDesignerName(String designerName) {
    this.designerName = designerName;
  }

  public static DesignerData getInstance(String designerName, Serializable referenceId, String referenceClassName) {

    DesignerData designerData = null;
    String hql = "select dd from " + DesignerData.class.getName() + " as dd where dd.referenceId=:refid and dd.referenceClassName=:rcn and dd.designerName=:dn";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setString("refid", referenceId+"");
    oql.getQuery().setString("rcn", referenceClassName);
    oql.getQuery().setString("dn", designerName);
    designerData = (DesignerData) oql.uniqueResultNullIfEmpty();

    if (designerData == null) {
      designerData = new DesignerData();
      designerData.setIdAsNew();
      designerData.setReferenceClassName(referenceClassName);
      designerData.setReferenceId(referenceId+"");
      designerData.setDesignerName(designerName);
    }
    return designerData;
  }


  public void removeDetail(Detail detail) {
    for (String keyToRemove : new HashSet<String>(getValueMap().keySet())){
      if (keyToRemove.startsWith(detail.name + "_")){
        getValueMap().remove(keyToRemove);
      }
    }
  }
}
