package org.jblooming.designer;

import org.jblooming.PlatformRuntimeException;
import org.jblooming.ontology.Identifiable;
import org.jblooming.utilities.CodeValueList;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.JSP;
import org.jblooming.waf.html.core.HtmlBootstrap;
import org.jblooming.waf.html.core.JspHelper;
import org.jblooming.waf.html.input.SmartCombo;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.ServletException;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.Locale;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class DesignerField {

  public String kind;
  public String name;
  public String label;
  public String initialValue;

  public int cardinality = 1;

  public int maxLength = 0; //valore che in alcuni casi corrisponde alla size su db

  public int fieldSize = 20;
  public int rowsLength = 1;

  public boolean required = false;
  public boolean readOnly = false;

  public Designer designer;

  public CodeValueList cvl;
  public SmartCombo smartCombo;

  public  JspHelper jspHelper;

  public boolean noSortable = false;
  public String mask = null;

  public String separator = null;

   public String format;
  /**
   * method used for callback when displaying field content
   */
  public String toStringCallbackMethod;

  public PageSeed pageSeed;

  public boolean displayAsCombo = false;
  public boolean putLabelFirst = true;
  public boolean classic = false;

  public int minimumFractionDigits;

  /**
   * if false the filed is not used for searching
   */
  public boolean usedForSearch = false;
  public boolean usedComboForSearch = false;
  public boolean useEmptyForAll = false;

  public boolean autoSize = false;

  public static final String DESIGNER_FIELD_PREFIX = "dfp__";
  public static final String DESIGNER_FIELD_SEPARATOR = "_";

  public DesignerField(String kind, String fieldName, String label, boolean required, boolean readOnly, String initialValue) {
    this.kind = kind;
    this.name = fieldName;
    this.label = label;
    this.required = required;
    this.readOnly = readOnly;
    this.initialValue = initialValue;
  }

  public DesignerField(String kind, String fieldName, String label) {
    this(kind, fieldName, label, false,false, null);
  }

  public DesignerField() {

  }

  public void toHtml(PageContext pageContext) {
    new Drawer(this).toHtml(pageContext);
  }


  public class Drawer extends JspHelper implements HtmlBootstrap {

    public DesignerField designerField;

    public Drawer(DesignerField fd) {
      super();
      this.designerField = fd;
      urlToInclude = "/commons/layout/partDesignerField.jsp";
    }

    public String getDiscriminator() {
      return Drawer.class.getName();
    }

    public boolean validate(PageState pageState) throws IOException, ServletException {
      return true;
    }
  }

  public static DesignerField getIdentifiableInstance(String fieldName, String label, Class<? extends Identifiable> identifiableClass, String[] comboDisplayProperties) {

    DesignerField ff = new DesignerField(identifiableClass.getName(),fieldName, label);


    try {
      Object o = identifiableClass.newInstance();

      if (o instanceof Identifiable) {

        String hql = "select p.id";
        for (String cdp : comboDisplayProperties) {
          hql += ", p." + cdp;
        }
        hql += " from " + identifiableClass.getName() + " as p";
        String whereForFiltering = "where p." + comboDisplayProperties[0] + " like :" + SmartCombo.FILTER_PARAM_NAME + " order by p." + comboDisplayProperties[0];
        String whereForId = "where p.id = :" + SmartCombo.FILTER_PARAM_NAME;
        SmartCombo lookup = new SmartCombo(fieldName, hql, whereForFiltering, whereForId);
        ff.smartCombo = lookup;

      } else
        throw new PlatformRuntimeException("DesignerField accepts Identifiable extensions only");

    } catch (InstantiationException e) {
      throw new PlatformRuntimeException(e);
    } catch (IllegalAccessException e) {
      throw new PlatformRuntimeException(e);
    }
    return ff;
  }

  public String toDisplayString(String value){

    String ret = null;
    if (Date.class.getName().equals(this.kind)) {
      if (value != null) {
        Date d;
        try {
          d = DateUtilities.dateFromString(value, "yyyy-MM-dd-HH-mm-ss");
        } catch (ParseException e) {
          throw new PlatformRuntimeException("The serialized date format has been changed.",e);
        }
        ret = DateUtilities.dateToString(d);
      }
    } else if (Double.class.getName().equals(this.kind)) {
      if(value != null && value.trim().length()>0) {
        //intero.setMinimumFractionDigits(df.minimumFractionDigits);
        //pageState.addClientEntry(DesignerField.DESIGNER_FIELD_PREFIX + ceName, intero.format(Double.parseDouble(value))+"");
        NumberFormat nf = NumberFormat.getNumberInstance(Locale.ENGLISH);
        DecimalFormat dFormat = (DecimalFormat) nf;
        dFormat.applyPattern(this.format == null ? "#.000" : this.format);
        dFormat.setMinimumFractionDigits(this.minimumFractionDigits);
        ret = dFormat.format(Double.parseDouble(value));         // -1234.567000
      }

    } else {
      ret = JSP.w(value);
    }
    return ret;

  }


}
