package com.opnlb.rss;

import com.opnlb.website.news.News;
import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.messaging.Message;
import org.jblooming.ontology.PersistentFile;
import org.jblooming.operator.Operator;
import org.jblooming.utilities.HttpUtilities;
import org.jblooming.utilities.JSP;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.utilities.file.FileUtilities;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.settings.ApplicationState;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.OutputStreamWriter;
import java.util.*;

public class RSSUtilities {

  private static Element createItemDomNews(News item, Document dom, String title, String link) {
    Element itemRoot = null;
    try {
      itemRoot = dom.createElement("item");

      Element e;
      String attrib = item.getTitle();
      if (attrib != null && attrib.trim().length() > 0) {
        e = dom.createElement("title");
        e.setTextContent(attrib);
        itemRoot.appendChild(e);
      }

      if (item.getText() != null) {
        e = dom.createElement("description");
        e.setTextContent(JSP.cleanHTML(item.getText()));
        itemRoot.appendChild(e);
      }

      attrib = item.getUrl();
      if (attrib != null && attrib.trim().length() > 0) {
        e = dom.createElement("link");
        e.setTextContent(attrib);
        itemRoot.appendChild(e);
      }

      attrib = item.getCreator();
      if (attrib != null && attrib.trim().length() > 0) {
        e = dom.createElement("author");
        e.setTextContent(attrib);
        itemRoot.appendChild(e);
      }

      if (item.getCategory() != null) {
        e = dom.createElement("category");
        e.setTextContent(item.getCategory().getDescription());
        itemRoot.appendChild(e);
      }


      PersistentFile pf = item.getAttach();
      if (pf == null)
        pf = item.getImage();
      if (pf != null) {
        e = dom.createElement("enclosure");
        String imageUrl = ApplicationState.serverURL + "/" + pf.getFileLocation();
        e.setAttribute("url", StringUtilities.replaceAllNoRegex(imageUrl, "\\", "/"));
        File f = new File(imageUrl);
        e.setAttribute("length", "" + f.length());
        // mime type
        String fileExt = FileUtilities.getFileExt(pf.getOriginalFileName());
        e.setAttribute("type", HttpUtilities.getContentType(fileExt));
        itemRoot.appendChild(e);
      }

      e = dom.createElement("guid");
      e.setTextContent(item.getId().toString());
      e.setAttribute("isPermaLink", "false");
      itemRoot.appendChild(e);

      e = dom.createElement("pubDate");
      e.setTextContent(CompanyCalendar.getInstance().getTime().toString());
      itemRoot.appendChild(e);

      e = dom.createElement("source");
      e.setTextContent(title);
      e.setAttribute("url", link);
      itemRoot.appendChild(e);

    } catch (Exception e) {
      e.printStackTrace();
    }
    return itemRoot;
  }

  private static boolean isValidNow(News item) {
    if (item.getStartingDate() != null && item.getStartingDate().after(CompanyCalendar.getInstance().getTime()))
      return false;
    return !(item.getEndingDate() != null && item.getEndingDate().before(CompanyCalendar.getInstance().getTime()));
  }

  // loads all the news
  public static StreamResult buildDynamicFeedFromNews(RSSFeed feed, Operator op) {
    Document feedDom;
    try {
      String old = System.getProperty("javax.xml.parsers.DocumentBuilderFactory");
      System.setProperty("javax.xml.parsers.DocumentBuilderFactory", "com.sun.org.apache.xerces.internal.jaxp.DocumentBuilderFactoryImpl");
      feedDom = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
      if (old != null)
        System.setProperty("javax.xml.parsers.DocumentBuilderFactory", old);

      // root
      Element root = feedDom.createElement("rss");
      root.setAttribute("version", feed.getVersion());

      // channel
      Element channelRoot = feedDom.createElement("channel");
      try {
        channelBuilder(feedDom, feed, channelRoot);

        List<News> ordItems =  new ArrayList(feed.getItems());
        Collections.sort(ordItems,new Comparator<News>() {
          public int compare(News o1, News o2) {
            return o2.getCreationDate().compareTo(o1.getCreationDate());
          }
        });

        for (News item : ordItems) {
          if (item.isVisible() && isValidNow(item))
            channelRoot.appendChild(createItemDomNews(item, feedDom, feed.getTitle(), feed.getLink()));
        }

      } catch (Exception e) {
        e.printStackTrace();
      }

      root.appendChild(channelRoot);
      feedDom.appendChild(root);
      DOMSource src = new DOMSource(feedDom);
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      StreamResult dst = new StreamResult(new BufferedWriter(new OutputStreamWriter(baos, "UTF-8")));
      dst.setOutputStream(baos);

      old = System.getProperty("javax.xml.transform.TransformerFactory");
      System.setProperty("javax.xml.transform.TransformerFactory", "com.sun.org.apache.xalan.internal.xsltc.trax.TransformerFactoryImpl");
      TransformerFactory.newInstance().newTransformer().transform(src, dst);
      if (old != null)
        System.setProperty("javax.xml.transform.TransformerFactory", old);

      return dst;
    } catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }

  public static void channelBuilder(Document feedDom, RSSFeed feed, Element channelRoot) {
    channelBuilder( feedDom,  feed,  channelRoot,  null);
  }

  public static void channelBuilder(Document feedDom, RSSFeed feed, Element channelRoot, String link) {
    Element e;

    e = feedDom.createElement("title");
    e.setTextContent(feed.getTitle());
    channelRoot.appendChild(e);
    channelRoot.appendChild(feedDom.createTextNode("\n"));

    e = feedDom.createElement("description");
    e.setTextContent(feed.getDescription());
    channelRoot.appendChild(e);
    channelRoot.appendChild(feedDom.createTextNode("\n"));

    e = feedDom.createElement("link");
    if(link!=null)
      e.setTextContent(link);
    else
      e.setTextContent(feed.getLink());
    channelRoot.appendChild(e);
    channelRoot.appendChild(feedDom.createTextNode("\n"));

    String attrib = feed.getLanguage();
    if(feed.getLanguage()!=null && feed.getLanguage().trim().length()>0) {
      e = feedDom.createElement("language");
      e.setTextContent(attrib);
      channelRoot.appendChild(e);
      channelRoot.appendChild(feedDom.createTextNode("\n"));
    }

    attrib = feed.getCopyright();
    if (attrib == null || attrib.trim().length() == 0)
      attrib = "(C) Open-Lab";
    e = feedDom.createElement("copyright");
    e.setTextContent(attrib);
    channelRoot.appendChild(e);
    channelRoot.appendChild(feedDom.createTextNode("\n"));

    attrib = feed.getManagingEditor();
    if (attrib != null && attrib.trim().length() > 0) {
      e = feedDom.createElement("managingEditor");
      e.setTextContent(attrib);
      channelRoot.appendChild(e);
      channelRoot.appendChild(feedDom.createTextNode("\n"));
    }

    attrib = feed.getWebMaster();
    if (attrib != null && attrib.trim().length() > 0) {
      e = feedDom.createElement("webMaster");
      e.setTextContent(attrib);
      channelRoot.appendChild(e);
      channelRoot.appendChild(feedDom.createTextNode("\n"));
    }

    e = feedDom.createElement("pubDate");
    e.setTextContent(CompanyCalendar.getInstance().getTime().toString());
    channelRoot.appendChild(e);
    channelRoot.appendChild(feedDom.createTextNode("\n"));

    e = feedDom.createElement("lastBuildDate");
    e.setTextContent(CompanyCalendar.getInstance().getTime().toString());
    channelRoot.appendChild(e);
    channelRoot.appendChild(feedDom.createTextNode("\n"));

    RSSFeedCategory category = feed.getCategory();
    if(category!=null) {
      attrib = category.getDescription();
      if (attrib != null && attrib.trim().length() > 0) {
        e = feedDom.createElement("category");
        e.setTextContent(attrib);
        attrib = feed.getCategoryDomain();
        if (attrib != null && attrib.trim().length() > 0)
          e.setAttribute("domain", attrib);
        channelRoot.appendChild(e);
        channelRoot.appendChild(feedDom.createTextNode("\n"));
      }
    }

    if (feed.getGenerator() != null) {
      e = feedDom.createElement("generator");
      e.setTextContent(feed.getGenerator());
      channelRoot.appendChild(e);
      channelRoot.appendChild(feedDom.createTextNode("\n"));
    }

    if (feed.getDocs() != null) {
      e = feedDom.createElement("docs");
      e.setTextContent(feed.getDocs());
      channelRoot.appendChild(e);
      channelRoot.appendChild(feedDom.createTextNode("\n"));
    }

    if (feed.getTtl() > 0) {
      e = feedDom.createElement("ttl");
      e.setTextContent(feed.getTtl() + "");
      channelRoot.appendChild(e);
      channelRoot.appendChild(feedDom.createTextNode("\n"));
    }

    if (feed.getImage() != null) {
      e = feedDom.createElement("image");
      Element tmp = feedDom.createElement("url");
      tmp.setTextContent(StringUtilities.replaceAllNoRegex(ApplicationState.serverURL + "/" + feed.getImage().getFileLocation(), "\\", "/"));
      e.appendChild(feedDom.createTextNode("\n"));
      e.appendChild(tmp);
      e.appendChild(feedDom.createTextNode("\n"));

      tmp = feedDom.createElement("title");
      tmp.setTextContent(feed.getImageTitle());
      e.appendChild(tmp);
      e.appendChild(feedDom.createTextNode("\n"));

      tmp = feedDom.createElement("link");
      tmp.setTextContent(feed.getImageLink());
      e.appendChild(tmp);
      e.appendChild(feedDom.createTextNode("\n"));

      tmp = feedDom.createElement("width");
      tmp.setTextContent(feed.getImageWidth() + "");
      e.appendChild(tmp);
      e.appendChild(feedDom.createTextNode("\n"));

      tmp = feedDom.createElement("height");
      tmp.setTextContent(feed.getImageHeight() + "");
      e.appendChild(tmp);
      e.appendChild(feedDom.createTextNode("\n"));

      attrib = feed.getImageDescription();
      if (attrib != null && attrib.trim().length() > 0) {
        tmp = feedDom.createElement("description");
        tmp.setTextContent(attrib);
        e.appendChild(tmp);
        e.appendChild(feedDom.createTextNode("\n"));
      }
      channelRoot.appendChild(e);
    }
  }

  // tw3 subscriptions
  public static StreamResult buildSubscriptionsFeed(List<Message> messages, Operator op) {
    Document feedDom;
    try {
      String old = System.getProperty("javax.xml.parsers.DocumentBuilderFactory");
      System.setProperty("javax.xml.parsers.DocumentBuilderFactory", "com.sun.org.apache.xerces.internal.jaxp.DocumentBuilderFactoryImpl");
      feedDom = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
      if (old != null)
        System.setProperty("javax.xml.parsers.DocumentBuilderFactory", old);

      // root
      Element root = feedDom.createElement("rss");
      root.setAttribute("version", "2.0");

      // channel
      Element channelRoot = feedDom.createElement("channel");
      try {
        Element e;
        e = feedDom.createElement("title");
        e.setTextContent("My subscriptions");
        channelRoot.appendChild(e);

        e = feedDom.createElement("description");
        e.setTextContent("Events I subscribed to");
        channelRoot.appendChild(e);

        e = feedDom.createElement("link");
        e.setTextContent(ApplicationState.serverURL);
        channelRoot.appendChild(e);

        if (op != null) {
          String attrib = op.getOption(OperatorConstants.FLD_SELECT_LANG);
          e = feedDom.createElement("language");
          e.setTextContent(attrib);
          channelRoot.appendChild(e);
        }

        String attrib = "(C) Open-Lab";
        e = feedDom.createElement("copyright");
        e.setTextContent(attrib);
        channelRoot.appendChild(e);

        e = feedDom.createElement("pubDate");
        e.setTextContent(CompanyCalendar.getInstance().getTime().toString());
        channelRoot.appendChild(e);

        e = feedDom.createElement("lastBuildDate");
        e.setTextContent(CompanyCalendar.getInstance().getTime().toString());
        channelRoot.appendChild(e);

        attrib = "subscriptions";
        e = feedDom.createElement("category");
        e.setTextContent(attrib);
        channelRoot.appendChild(e);

        for (Message msg : messages) {
          Element itemRoot;
          try {
            itemRoot = feedDom.createElement("item");

            Element el;
            attrib = msg.getSubject();
            el = feedDom.createElement("title");
            el.setTextContent(attrib);
            itemRoot.appendChild(el);

            el = feedDom.createElement("description");
            el.setTextContent(msg.getMessageBody());
            itemRoot.appendChild(el);

            attrib = msg.getLink();
            if (attrib != null && attrib.trim().length() > 0) {
              e = feedDom.createElement("link");
              e.setTextContent(attrib);
              itemRoot.appendChild(e);
            }

            if (msg.getFromOperator() != null) {
              attrib = msg.getFromOperator().getDisplayName();
              if (attrib != null && attrib.trim().length() > 0) {
                e = feedDom.createElement("author");
                e.setTextContent(attrib);
                itemRoot.appendChild(e);
              }
            }

            e = feedDom.createElement("category");
            e.setTextContent(msg.getSubject());
            itemRoot.appendChild(e);

            e = feedDom.createElement("guid");
            e.setTextContent(msg.getId().toString());
            e.setAttribute("isPermaLink", "false");
            itemRoot.appendChild(e);

            e = feedDom.createElement("pubDate");
            e.setTextContent(CompanyCalendar.getInstance().getTime().toString());
            itemRoot.appendChild(e);
            channelRoot.appendChild(itemRoot);

          } catch (Exception ex) {
            ex.printStackTrace();
          }
        }
      } catch (Exception e) {
        e.printStackTrace();
      }

      root.appendChild(channelRoot);
      feedDom.appendChild(root);

      for (Object o : new ArrayList(messages)) {
        Message msg = (Message) o;
        long days = (new Date().getTime() - msg.getReceived().getTime()) / CompanyCalendar.MILLIS_IN_DAY;
        if (days > 15)
          msg.remove();
      }

      DOMSource src = new DOMSource(feedDom);
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      StreamResult dst = new StreamResult(new BufferedWriter(new OutputStreamWriter(baos, "UTF-8")));
      dst.setOutputStream(baos);

      old = System.getProperty("javax.xml.transform.TransformerFactory");
      System.setProperty("javax.xml.transform.TransformerFactory", "com.sun.org.apache.xalan.internal.xsltc.trax.TransformerFactoryImpl");
      TransformerFactory.newInstance().newTransformer().transform(src, dst);
      if (old != null)
        System.setProperty("javax.xml.transform.TransformerFactory", old);

      return dst;
    } catch (Exception e) {
      e.printStackTrace();
    }

    return null;
  }

}