package com.opnlb.website.news;

import com.opnlb.website.OrderFactorable;
import org.jblooming.ontology.PersistentFile;
import org.jblooming.ontology.SecuredLoggableHideableSupport;
import org.jblooming.operator.Operator;
import org.jblooming.operator.User;
import org.jblooming.security.Permission;
import org.jblooming.security.Securable;

import java.util.Date;

/**
 * News (c) 2005 - Open Lab - www.open-lab.com
 */
public class News extends SecuredLoggableHideableSupport implements Securable, OrderFactorable {

  private String title;
  private String subTitle;
  private String text;
  private String url;
  private String urlDescription;
  private PersistentFile image;
  private String imageAlt;
  private Integer imageWidth = 0;
  private Integer imageHeight = 0;
  private String linkTarget;
  private PersistentFile attach;
  private String attachDescription;

  private Integer orderFactor;

  private NewsCategory category;
  private NewsSuffix suffix;

  private Date startingDate;
  private Date endingDate;
  private boolean visible = true;
  private boolean topNews = false;

  public static final String NEWS = "NEWS";
  public static final String TOP_NEWS = "TOP_NEWS";

  /**
   * CONSTRUCTORS
   */
  public News() {
  }

  /**
   * GETTERS AND SETTERS
   */
  public String getTitle() {
    return title;
  }

  public void setTitle(String title) {
    this.title = title;
  }

  public String getSubTitle() {
    return subTitle;
  }

  public void setSubTitle(String subTitle) {
    this.subTitle = subTitle;
  }

  public String getUrl() {
    return url;
  }

  public void setUrl(String url) {
    this.url = url;
  }

  public Date getStartingDate() {
    return startingDate;
  }

  public void setStartingDate(Date startingDate) {
    this.startingDate = startingDate;
  }

  public Date getEndingDate() {
    return endingDate;
  }

  public void setEndingDate(Date endingDate) {
    this.endingDate = endingDate;
  }

  public NewsCategory getCategory() {
    return category;
  }

  public void setCategory(NewsCategory category) {
    this.category = category;
  }

  public String getText() {
    return text;
  }

  public void setText(String text) {
    this.text = text;
  }

  public PersistentFile getImage() {
    return image;
  }

  public void setImage(PersistentFile image) {
    this.image = image;
  }

  public String getImageAlt() {
    return imageAlt;
  }

  public void setImageAlt(String imageAlt) {
    this.imageAlt = imageAlt;
  }

  public PersistentFile getAttach() {
    return attach;
  }

  public void setAttach(PersistentFile attach) {
    this.attach = attach;
  }

  public boolean isVisible() {
    return visible;
  }

  public void setVisible(boolean visible) {
    this.visible = visible;
  }

  public boolean isTopNews() {
    return topNews;
  }

  public void setTopNews(boolean topNews) {
    this.topNews = topNews;
  }

  public NewsSuffix getSuffix() {
    return suffix;
  }

  public void setSuffix(NewsSuffix suffix) {
    this.suffix = suffix;
  }

  public boolean hasPermissionFor(User u, Permission p) {
    boolean result = false;//u.hasPermissionFor(p);
    if (!result) {
      if (getOwner() != null && getOwner().equals(u)) {
        result = true;
      } else if (getCategory() != null) {
        //DOES u BELONG TO MANAGERS OF CATEGORY?
        result = getCategory().managersContains((Operator)u);
      }
    }
    return result;
  }


  public String getLinkTarget() {
    return linkTarget;
  }

  public void setLinkTarget(String linkTarget) {
    this.linkTarget = linkTarget;
  }

  public Integer getImageWidth() {
    return imageWidth;
  }

  public void setImageWidth(Integer imageWidth) {
    this.imageWidth = imageWidth;
  }

  public Integer getImageHeight() {
    return imageHeight;
  }

  public void setImageHeight(Integer imageHeight) {
    this.imageHeight = imageHeight;
  }

  public String getUrlDescription() {
    return urlDescription;
  }

  public void setUrlDescription(String urlDescription) {
    this.urlDescription = urlDescription;
  }

  public String getAttachDescription() {
    return attachDescription;
  }

  public void setAttachDescription(String attachDescription) {
    this.attachDescription = attachDescription;
  }

  public Integer getOrderFactor() {
    return orderFactor;
  }

  public void setOrderFactor(Integer orderFactor) {
    this.orderFactor = orderFactor;
  }


}
