package com.opnlb.website.page;


import com.opnlb.website.content.Content;
import com.opnlb.website.portlet.Portlet;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.exceptions.FindException;

import java.util.*;

/**
 * WebSitePage (c) 2005 - Open Lab - www.open-lab.com
 */
public class WebSitePage extends WebSitePagePersistent {

  /**
   * CONSTRUCTORS
   */
  public WebSitePage () {
  }

  public WebSitePage (String id) {
    this.id = id;
  }

  public static List createRootPage() throws FindException {
    String hql = "from "+ WebSitePage.class.getName() + " as dt ";
    OqlQuery oql = new OqlQuery(hql);
    return oql.list();
  }

  public static List<Content> addGlobalContents (WebSitePage wsPage, List<Content> choices, boolean addAncestorContentsToo) {
    if (wsPage!=null) {
      List ancestors = wsPage.getAncestors();
      for (int i = 0; i < ancestors.size(); i++) {
        WebSitePage father = (WebSitePage) ancestors.get(i);
        if (!father.equals(wsPage)) {
          Iterator it = father.getContentsIterator();
          while (it.hasNext()) {
            Content content =  (Content) it.next();
            if (content!=null && content.isGlobalAssociation()) {
              choices.add(content);
            }
          }
        }
      }
    }  // page has no contents: thus not only global must be added
    if (addAncestorContentsToo) {
      //List firstAncestorWithContents = getFirstAncestorContents(wsPage, new ArrayList<Content>());
      List firstAncestorWithContents = addAncestorContents(wsPage, new ArrayList<Content>());
      choices.addAll(firstAncestorWithContents);
    }
    Collections.sort(choices, new Content.ContentComp());

    return choices;
  }

  // adding global contents (i.e. contents inherited from ancestors)
  public static List<Content> addAncestorContents (WebSitePage wsPage, List<Content> choices) {
    if (wsPage!=null) {
      /*
      List ancestors = wsPage.getAncestors();
      for (int i = 0; i < ancestors.size(); i++) {
        WebSitePage father = (WebSitePage) ancestors.get(i);
        if (!father.equals(wsPage) && father.getParent()==null) {
          Iterator it = father.getContentsIterator();
          while (it.hasNext()) {
            Content content =  (Content) it.next();
            //if (content!=null && content.isGlobalAssociation()) {
              choices.add(content);
            //}
          }
        }
      }
      */
      WebSitePage father = wsPage.getParent();
      if (father!=null && !father.equals(wsPage)) {
        Iterator it = father.getContentsIterator();
        if (it!=null && it.hasNext()) {
          while (it.hasNext()) {
            Content content =  (Content) it.next();
            if (!content.isGlobalAssociation()) // global has already been added
              choices.add(content);
          }
        } else {
          choices = addAncestorContents(father, choices);
        }
      }
    }
    return choices;
  }

  public static boolean isInheritedPortlet(WebSitePage wsPage, Portlet wp, String templateId) {
    boolean inherited = false;
    List ancestors = wsPage.getAncestors();
    for (int i = 0; i < ancestors.size(); i++) {
      WebSitePage father = (WebSitePage) ancestors.get(i);
      if (!father.equals(wsPage)) {
        Iterator it = father.getContentsIterator();
        while (it.hasNext()) {
          Content content =  (Content) it.next();
          if (content!=null && content.isGlobalAssociation() ) {
            Portlet pt = content.getPortlet();
            if (pt.equals(wp)) {
              //inherited = true;
              //break;
              // todo in case of template null ??
              if(content.getTemplate()!=null && templateId!=null && templateId.equals(content.getTemplate().getId()+""))  {
                inherited = true;
                break;
              }
            }
          }
        }
      }
    }
    return inherited;
  }

  // adding global contents (i.e. contents inherited from ancestors)
  // page, if none setted on itself,  MUST get all the contents from direct father and only those checked as "global association" from ancestors
  public static List<Content> addGlobalContents_ORIGINAL (WebSitePage wsPage, List<Content> choices) {
    List<Content> choicesOriginal = new ArrayList<Content>(choices);
    WebSitePage directFather = wsPage.getParent();
    if (wsPage!=null) {
      List ancestors = wsPage.getAncestors();
        for (int i = 0; i < ancestors.size(); i++) {
        WebSitePage father = (WebSitePage) ancestors.get(i);
        boolean isDirectFatherContent = father.getId().equals(directFather.getId());
        if (!father.equals(wsPage)) {
          Iterator it = father.getContentsIterator();
          while (it.hasNext()) {
            Content content =  (Content) it.next();
            if (content!=null) {             // test (16) ha 4 content
              // duplica le portlet
//              if (choicesOriginal.size()==0) {
//                choices.add(content);
//              } else {
//                if (content.isGlobalAssociation()) {
//                  choices.add(content);
//                }
//              }
            // non carica che quelle global qualora il padre diretto non abbia content propri
//              if (choicesOriginal.size()==0 && isDirectFatherContent) {
//                choices.add(content);
//              } else {
//                if (content.isGlobalAssociation()) {
//                  choices.add(content);
//                }
//              }

            }
          }
        }
      }
    }
    return choices;
  }

  public static class WebSitePageComp implements Comparator {   
    public int compare(Object first, Object second) {
      WebSitePage page1 = (WebSitePage) first;
      WebSitePage page2 = (WebSitePage) second;
      if (page1==null || page2==null ) {
        return 0;
      }
      double of1 = page1.getOrdinal();
      double of2 = page2.getOrdinal();
      int val = (int) (of1-of2);
      return val;
    }

  }

  /*
  private static List getFirstAncestorContents(WebSitePage wsPage, List<Content> finalContents){
    //List<Content> finalContents = new ArrayList<Content>();
    if (wsPage.getParent()!=null) {
      WebSitePage father = wsPage.getParent();
      if (father.getContentsSize()>0) {
        Iterator<Content> contents =  father.getContentsIterator();
        while (contents.hasNext()) {
          Content content = contents.next();
          if (!content.isGlobalAssociation())
            finalContents.add(content);
        }
      } else {
        getFirstAncestorContents(father, finalContents);
      }
    }
    return finalContents;
  }
  */
}
