package com.opnlb.website.util;

import com.opnlb.website.content.Content;
import com.opnlb.website.page.WebSitePage;
import com.opnlb.website.portlet.Portlet;
import com.opnlb.website.template.Template;
import org.jblooming.ontology.PersistentFile;
import org.jblooming.operator.Operator;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindByPrimaryKeyException;
import org.jblooming.persistence.exceptions.FindException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.security.Permission;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.HttpUtilities;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.html.display.MultimediaFile;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.ClientEntry;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * TemplateManager (c) 2005 - Open Lab - www.open-lab.com
 */
public class TemplateManager {

  /**
   * Called in template jsp file to load user contents with this single query
   *
   * @param logged
   * @param pageId
   * @return
   * @throws PersistenceException
   */
  public static List choosenContents (Operator logged, String pageId)  throws PersistenceException {
//    Template defTemplate = null;
//    WebSitePage thisPage = (WebSitePage) PersistenceHome.findByPrimaryKey(WebSitePage.class, pageId);
//    defTemplate = thisPage.getDefaultTemplate();
//    boolean considerTemplate = defTemplate!=null;

    List choices = new ArrayList();
    if (logged!=null) {
      String hql = " from " + Content.class.getName() + " as content where " +
              "content.operator.id=:oid and " +
              (pageId!=null ? "content.page.id=:pid and " : "") +
              //(considerTemplate ? " content.template.id=:tid and " : "") +
              "content.defaultConfiguration=:falsity order by orderx ";
      OqlQuery oql = new OqlQuery(hql);
      oql.getQuery().setString("oid", logged.getId().toString());
      oql.getQuery().setBoolean("falsity", Boolean.FALSE);
      if(pageId!=null)
        oql.getQuery().setString("pid", pageId);
//      if (considerTemplate)
//        oql.getQuery().setString("tid", defTemplate.getId().toString());
      choices = oql.list();
    }

    if (choices==null || choices.size()==0) {
      String hql = " from " + Content.class.getName() + " as content where " +
              (pageId!=null ? "content.page.id=:pid and " : "") +
              //(considerTemplate ? " content.template.id=:tid and " : "") +
              "content.defaultConfiguration=:truth order by orderx ";
      OqlQuery oql = new OqlQuery(hql);
      oql.getQuery().setBoolean("truth", Boolean.TRUE);
      if(pageId!=null)
        oql.getQuery().setString("pid", pageId);
//      if (considerTemplate)
//        oql.getQuery().setString("tid", defTemplate.getId().toString());
      choices = oql.list();
    }
    // adding global contents (i.e. contents inherited from ancestors)
     if (!PersistenceHome.NEW_EMPTY_ID.equals(pageId)) {
       WebSitePage wsPage = (WebSitePage) PersistenceHome.findByPrimaryKey(WebSitePage.class, pageId);
       boolean addContents = choices==null || choices.size()==0;
       choices = WebSitePage.addGlobalContents(wsPage, choices, addContents);
     }

    return choices;
  }

  /**
   * called by displayAdmin to load into an uncostumizable area admin contents
   *
   * @param pageId
   * @param area
   * @return
   * @throws PersistenceException
   */
  public static List<Content> choosenContentsAdmin (String pageId, String area)  throws PersistenceException {
//    Template defTemplate = null;
//    WebSitePage thisPage = (WebSitePage) PersistenceHome.findByPrimaryKey(WebSitePage.class, pageId);
//    defTemplate = thisPage.getDefaultTemplate();
//    boolean considerTemplate = defTemplate!=null;
//    List choices = new ArrayList();
    String hql = " from " + Content.class.getName() + " as content where " +
            "content.area=:aid and " +
            (pageId!=null ? "content.page.id=:pid and " : "") +
            //(considerTemplate ? " content.template.id=:tid and " : "") +
            "content.defaultConfiguration=:truth order by orderx ";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setString("aid", area);
    oql.getQuery().setBoolean("truth", Boolean.TRUE);
    if(pageId!=null)
      oql.getQuery().setString("pid", pageId);
//      if (considerTemplate)
//        oql.getQuery().setString("tid", defTemplate.getId().toString());

    // november06: added for tw3 wizard creates contents in area admin with operator null thus ==>
    //return cleanChoices(oql.list());
    return oql.list();
  }

  /**
   * it's method inserted into an active area tag in the jsp template file
   * admin area
   *
   * @param pageId
   * @param area
   * @param pageContext
   * @param out
   * @throws PersistenceException
   */
  public static void displayAdmin(String pageId, String area, PageContext pageContext, JspWriter out, PageState pageState) throws PersistenceException, IOException{
    boolean preview = "WS_PRW".equals(pageState.getCommand());
    if (preview) {
      try {
        displayPreview (area, pageContext, pageState);
      } catch (FindByPrimaryKeyException t) {
        out.write("WEBSITE_ERROR: " + t);
        Tracer.platformLogger.error("WEBSITE_ERROR",t);
        t.printStackTrace();
      }
    } else {
      List<Content> portlets = choosenContentsAdmin(pageId, area);
      portletLoop(portlets, area, pageContext, out, pageState);
    }
  }

  /**
   * it's method inserted into an active area tag in the jsp template file
   * custom area
   *
   * @param choices
   * @param area
   * @param pageContext
   * @param out
   */
  public static void display(List choices, String area, PageContext pageContext, JspWriter out, PageState pageState) throws IOException {
    boolean preview = "WS_PRW".equals(pageState.getCommand());
    if (preview) {
      try {
        displayPreview (area, pageContext, pageState);
      } catch (FindByPrimaryKeyException t) {
        out.write("WEBSITE_ERROR: " + t);
        Tracer.platformLogger.error("WEBSITE_ERROR",t);
        t.printStackTrace();
      }
    } else {
      /*
      // november06: added for tw3 wizard creates contents in area admin with operator null thus ==>
      try {
        choices = cleanChoices(choices, area);
      } catch (FindByPrimaryKeyException e) { }
      // end
      */
      portletLoop(choices, area, pageContext, out, pageState);
    }
  }

  private static void portletLoop(List choices, String area, PageContext pageContext, JspWriter out, PageState pageState)  {
    Operator logged = null;
    if (pageState!=null) {
      try {
        logged = pageState.sessionState.getLoggedOperator();
      } catch (PersistenceException a) { }
    }

    String searchEntry = pageState.getEntry("WW4_ACTION_SEARCH").stringValueNullIfEmpty();
    String searchResultsInArea = null;
    boolean isActionSearch = searchEntry!=null && Fields.TRUE.equals(searchEntry);
    if (isActionSearch)
      searchResultsInArea = pageState.getEntry("WW4_RESULTS_IN_AREA").stringValueNullIfEmpty();
    boolean searchBlockedThisArea = false;

    HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();
    for (Object choice : choices) {
      Content content = (Content) choice;
      String contentArea = content.getArea();
      if (contentArea.equalsIgnoreCase(area)) {
        Portlet portlet = content.getPortlet();

        if (isActionSearch && searchResultsInArea!=null && contentArea.equals(searchResultsInArea) && !searchBlockedThisArea) {
          HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
          String basePath = HttpUtilities.getFileSystemRootPathForRequest(request);
          basePath = StringUtilities.replaceAllNoRegex(basePath,"\\","/");
          searchBlockedThisArea = true;
          PersistentFile listResults = new PersistentFile(0, "partSearchResults.jsp", PersistentFile.TYPE_WEBAPP_FILESTORAGE);
          String siteRoot = ApplicationState.getApplicationSetting("SITE_FOLDER_LOCATION");
          String rootFolder = null;

          boolean custom = false;
          if (siteRoot!=null && siteRoot.trim().length()>0) {
            File customFile = new File(basePath + siteRoot + "/parts/partSearchResults.jsp");
            if (customFile.exists())
              custom = true;
          }

          if (custom) {
            listResults.setFileLocation(siteRoot + "/parts/partSearchResults.jsp");
            listResults.fileDir = siteRoot + "/parts";
          } else {
            rootFolder = pageState.getApplication().getRootFolder();
            listResults.setFileLocation(rootFolder + "/parts/partSearchResults.jsp");
            listResults.fileDir = rootFolder + "/parts";
          }

          MultimediaFile mf = new MultimediaFile(listResults, req);
          mf.areaName = area;
          mf.width = portlet.getPixelWidth() > 0 ? portlet.getPixelWidth() + "" : "200";
          mf.height = portlet.getPixelHeight() > 0 ? portlet.getPixelHeight() + "" : "200";
          mf.toHtml(pageContext);

        } else if (!searchBlockedThisArea) {
          /**
           * PERMISSIONS SECURITY CHECK
           */
          boolean show = verifyPermissions(pageState, portlet, logged);
          if (show) {
            PersistentFile pf = portlet.getFile();
            boolean installed = portlet.getInstalled();
            if (pf != null && installed) {
              pageState.setAttribute("WW_PORTLET", portlet);
              pageState.setAttribute("WW_PORTLET_PARAMS", content.getPortletParams());
              MultimediaFile mf = new MultimediaFile(pf, req);
              mf.areaName = area;
              mf.width = portlet.getPixelWidth() > 0 ? portlet.getPixelWidth() + "" : "200";
              mf.height = portlet.getPixelHeight() > 0 ? portlet.getPixelHeight() + "" : "200";
              mf.toHtml(pageContext);
              //portlet.toHtml(pageContext);
            }
          }
        }
      }
    }
  }

/*--------------------------------------------------------------------------------------------------------------------------------------------------------------

  DYNAMIC PREVIEW

--------------------------------------------------------------------------------------------------------------------------------------------------------------*/
  public static void displayPreview(String area, PageContext pageContext, PageState pageState)throws FindByPrimaryKeyException {
    Map  cesMap = pageState.getClientEntries().getEntriesStartingWithStripped("input_"+area);
    for (Iterator iterator = cesMap.keySet().iterator(); iterator.hasNext();) {
      Object o =  iterator.next();
      ClientEntry ce = (ClientEntry) cesMap.get(o);
      String value = ce.stringValueNullIfEmpty();
      if ( ("input_"+area).equals(ce.name) ) {
        if (value != null && value.trim().length() > 0) {
          String[] portletList = StringUtilities.splitToArray(value, ",");
          String portletId = null;
          for (int i = 0; i < portletList.length; i++) {
            portletId = portletList[i];
            boolean goOn = false;
            if(portletId.startsWith("wp_")) {
              portletId = portletId.substring(3, portletId.length());
              goOn = true;
            }  else if (portletId.trim().length() > 0) {
              try {
                int id = new Integer(portletId);
                portletId = id+"";
                goOn = true;
              } catch(NumberFormatException d) { }
            }

            if (goOn) {
              Portlet portlet = (Portlet) PersistenceHome.findByPrimaryKey(Portlet.class, portletId);
              PersistentFile pf = portlet.getFile();
              boolean installed = portlet.getInstalled();
              if (pf != null && installed) {
                HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();
                MultimediaFile mf = new MultimediaFile(pf, req);
                mf.areaName = area;
                mf.width = portlet.getPixelWidth() > 0 ? portlet.getPixelWidth() + "" : "200";
                mf.height = portlet.getPixelHeight() > 0 ? portlet.getPixelHeight() + "" : "200";
                mf.toHtml(pageContext);
              }
            }
          }
        }
      }
    }
  }

  private static boolean verifyPermissions(PageState pageState, Portlet portlet, Operator logged) {
    boolean show = true;
    // if called by displayAdmin pagestate is null:: no permissions verify because admin chose such a portlet to be shown
    if (pageState!=null) {
      Set<Permission> portletPermissions = portlet.getPermissions();
      if (portletPermissions!=null && portletPermissions.size()>0) {
        show = false;
        for (Iterator<Permission> iterator = portletPermissions.iterator(); iterator.hasNext();) {
          Permission permission = iterator.next();
          if (logged !=null && logged.hasPermissionFor(permission)) {
            show = true;
            break;
          }
        }
      }
    }
    return show;
  }

  /*
  todo in caso admin modifichi template di default di una pagina dopo che siano stati inseriti contenuti
  con la query seguente viene caricato il template precedente
  */
  public static Template getPersonalTemplateContextualToPage(Operator logged, String pageId) throws PersistenceException {
    //String pageId = getIdFromPageName(pageName);
    String hql;
    OqlQuery oql;
    Template template=null;
    if (logged != null) {
      hql = " select distinct content.template from " + Content.class.getName() + " as content where " +
            "content.operator.id=:oid and " +
            "content.page.id=:pid ";
      oql = new OqlQuery(hql);
      oql.getQuery().setString("oid", logged.getId().toString());
      oql.getQuery().setString("pid", pageId);
      template = (Template) oql.uniqueResultNullIfEmpty();
    }
    if (template==null) {
      WebSitePage wsPage = (WebSitePage) PersistenceHome.findByPrimaryKey(WebSitePage.class, pageId);
      template = wsPage.getDefaultTemplate();
    }
    return template;
  }

  /**
   * Used in ajaxPageUsedPortlets.jsp
   *
   * @param pageId
   * @return
   * @throws PersistenceException
   */
  public static List getAllChoosenContents (String pageId)  throws PersistenceException {
    List choices = new ArrayList();
    String hql = " from " + Content.class.getName() + " as content where content.page.id=:pid order by defaultConfiguration desc, orderx ";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setString("pid", pageId);
    choices = oql.list();

    // adding global contents (i.e. contents inherited from ancestors)
    if (!PersistenceHome.NEW_EMPTY_ID.equals(pageId)) {
      WebSitePage wsPage = (WebSitePage) PersistenceHome.findByPrimaryKey(WebSitePage.class, pageId);
      boolean addAncestorContents = choices==null || choices.size()==0;
      choices = WebSitePage.addGlobalContents(wsPage, choices, addAncestorContents);
    }
    return choices;
  }

  public static String getIdFromPageName(String pageName) throws FindException {
    String hql = " select page.id from " + WebSitePage.class.getName() + " as page where upper(page.name)=:name ";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setString("name", pageName.toUpperCase());
    String id = null;
    try {
      id = (String) oql.uniqueResult();
    } catch (FindException a) {
//      hql = " select page.id from " + WebSitePage.class.getName() + " as page where page.name=:name ";
//      oql = new OqlQuery(hql);
//      oql.getQuery().setString("name", pageName.replaceAll("_", " "));
//      id = (String) oql.uniqueResult();
    }
    // redirect to home
    if (id==null) {
      id = TemplateManager.getRoot();
    }
    return id;
  }

  // todo rifinire per identificare a quale root reindirizzare
  public static String getRoot() throws FindException {
    String hql = " select page.id from " + WebSitePage.class.getName() + " as page where page.parent is null ";
    OqlQuery oql = new OqlQuery(hql);
    List<String> ids = oql.list();
    return ids.get(0);
  }

  // november06: added for tw3 wizard creates contents in area admin with operator null thus ==>
  public static List<Content> cleanChoices(List<Content> choices) throws FindByPrimaryKeyException {
    List<Content> cleaned = new ArrayList<Content>();
    if (choices!=null && choices.size()>0) {
      List<Portlet> pts = new ArrayList<Portlet>();
      for (Content content : choices) {
        if (content.getPortlet()!=null) {
          String id = "" + content.getPortlet().getId();
          Portlet p = (Portlet) PersistenceHome.findByPrimaryKey(Portlet.class, id);
          if (!pts.contains(p)) {
            pts.add(p);
            cleaned.add(content);
          }
        }
      }
    }
    return cleaned;
  }

 // november06: added for tw3 wizard creates contents in area admin with operator null thus ==>
  private static List<Content> cleanChoices(List<Content> choices, String area) throws FindByPrimaryKeyException {
    List<Content> cleaned = new ArrayList<Content>();
    if (choices!=null && choices.size()>0) {
      Map<String, Portlet> pts = new HashMap<String, Portlet>();
      for (Content content : choices) {
        if (content.getPortlet()!=null) {
          String id = "" + content.getPortlet().getId();
          Portlet p = (Portlet) PersistenceHome.findByPrimaryKey(Portlet.class, id);
          if (pts.isEmpty()) {
            pts.put(area, p);
            cleaned.add(content);
          } else {
            if (!p.getId().equals(pts.get(area).getId())) {
              pts.put(area, p);
              cleaned.add(content);
            }
          }
        }
      }
    }
    return cleaned;
  }
}
