package com.opnlb.website.waf;

import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.HashTable;
import org.jblooming.waf.FrontControllerFilter;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.PageSeed;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;

/**
 * WebSiteFilter (c) 2005 - Open Lab - www.open-lab.com
 */
public class WebSiteFilter implements Filter {

  /**
   * keys must be in the form "myOldPage.page"
   * values must be in the form ""applications/webwork/site_mysite/myNewPage.page"
   */
  public static Map<String, String> redirectFromTo = new HashTable();


  public void init(FilterConfig filterConfig) throws ServletException {
  }

  public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws ServletException, IOException {

    HttpServletRequest request = (HttpServletRequest) servletRequest;
    HttpServletResponse response = (HttpServletResponse) servletResponse;
    String uri = request.getRequestURI();
    String bkhome = ApplicationState.getApplicationSetting("NONDEFAULT_WEBWORK_BKHOME", "/applications/webwork/backoffice/bkhome.jsp");
    // redirect to backoffice (thnx to filter in web.xml)
    if ( (request.getContextPath() + "/webwork").equalsIgnoreCase(uri) || uri.toLowerCase().endsWith(request.getContextPath() + "/webwork") ) {
      response.sendRedirect(request.getContextPath() + bkhome);
      return;
    }

    String resource = request.getServletPath(); //.toLowerCase();

    // todo verificare
    int lastSlash = request.getRequestURI().lastIndexOf("/");
    int pageNameEnd = request.getRequestURI().indexOf(FrontControllerFilter.page);
    if (lastSlash > -1 && pageNameEnd > -1 && pageNameEnd>lastSlash) {
      String newTarget = redirectFromTo.get(request.getRequestURI().substring(lastSlash + 1, pageNameEnd + 5));
      if (newTarget != null) {
        resource=newTarget;
      }
    }

    SessionState sessionState = SessionState.getSessionState(request);
    try {
      if (sessionState.getLoggedOperator() == null) {
        // todo security?
        // todo:: verify if resource (i.e. node) has role guest or similiar
      }
    } catch (PersistenceException e) {
      Tracer.platformLogger.error(e);
    }

    if (resource.toLowerCase().endsWith(FrontControllerFilter.page)) {
      PageSeed screenPage = new PageSeed("/" + ApplicationState.platformConfiguration.defaultApplication.getRootFolder() + "/screens/defaultScreen.jsp");
      // get page name from url
      String requestedPageName = resource.substring(resource.lastIndexOf("/") + 1, resource.lastIndexOf(FrontControllerFilter.page));
      request.setAttribute("pageName", requestedPageName);
      request.getRequestDispatcher(screenPage.toLinkToHref()).forward(request, response);

    } else {
      filterChain.doFilter(request, servletResponse);
    }
  }

  public void destroy() {
  }

}
