package com.twproject.document;

import com.twproject.classification.DocumentClassification;
import com.twproject.classification.DocumentClassificationAssociation;
import com.twproject.resource.Person;
import com.twproject.resource.Resource;
import com.twproject.task.Task;
import org.jblooming.ontology.Identifiable;
import org.jblooming.ontology.Node;
import org.jblooming.ontology.VersionComparator;
import org.jblooming.operator.Operator;
import org.jblooming.operator.User;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.exceptions.RemoveException;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.remoteFile.Document;
import org.jblooming.security.Permission;
import org.jblooming.security.Securable;

import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

public class TeamworkDocument extends Document {

  private Task task;
  private Resource resource;
  private Person lockedBy;

  public static final Comparator compareByVersionYoungness = new Comparator() {
    public int compare(Object d1, Object d2) {
      return -((TeamworkDocument) d1).compareVersionTo((TeamworkDocument) d2);
    }
  };


  //read only
  private Set<DocumentClassificationAssociation> documentClassificationAssociations = new HashSet<DocumentClassificationAssociation>();


  public TeamworkDocument() {
    super();
  }

  public TeamworkDocument(String dId) {
    super(dId);
  }

  public void setParent(TeamworkDocument n) {
    parent = n;
  }

  public void setParentNode(Node node) {
    setParent((TeamworkDocument) node);
  }

  public Task getTask() {
    return task;
  }

  public void setTask(Task task) {
    this.task = task;
  }


  public Resource getResource() {
    return resource;
  }

  public void setResource(Resource resource) {
    this.resource = resource;
  }

  public void setLockedBy(Person lockedBy) {
    this.lockedBy = lockedBy;
  }

  public Person getLockedBy() {
    return lockedBy;
  }

  public boolean addClassificationAndPersist(DocumentClassification classification) throws StoreException, RemoveException {

    boolean result = removeClassificationAndDepersist(classification);

    DocumentClassificationAssociation dca = new DocumentClassificationAssociation();
    dca.setClassification(classification);
    dca.setDocument(this);
    this.documentClassificationAssociations.add(dca);
    dca.store();

    return result;
  }

  private Set<DocumentClassificationAssociation> getDocumentClassificationAssociations() {
    return documentClassificationAssociations;
  }

  private void setDocumentClassificationAssociations(Set<DocumentClassificationAssociation> documentClassificationAssociations) {
    this.documentClassificationAssociations = documentClassificationAssociations;
  }

  public boolean removeClassificationAndDepersist(DocumentClassification classification) throws RemoveException {

    boolean result = false;

    //find the tca
    for (DocumentClassificationAssociation dca : new HashSet<DocumentClassificationAssociation>(documentClassificationAssociations)) {
      if (dca.getClassification().equals(classification)) {
        this.documentClassificationAssociations.remove(dca);
        PersistenceHome.remove(dca);
        result = true;
        break;
      }
    }
    return result;
  }

  public Iterator<DocumentClassification> getClassificationIterator() {

    Set<DocumentClassification> dc = new HashSet<DocumentClassification>();
    for (DocumentClassificationAssociation dca : documentClassificationAssociations) {
      dc.add(dca.getClassification());
    }
    return dc.iterator();
  }

  public Identifiable getReferral() {

    Identifiable result = null;

    if (!isNew()) {
      if (getTask() != null)
        result = getTask();
      else
        result = getResource();
      /*String hql = "select task from " + Task.class.getName() + " as task join task.documents as document  where document = :document";
      OqlQuery oql = new OqlQuery(hql);
      oql.getQuery().setEntity("document", this);
      Task task = (Task) oql.uniqueResultNullIfEmpty();
      if (task != null)
        result = task;

      if (result == null) {
        hql = "select resource from " + Resource.class.getName() + " as resource join resource.documents as document  where document = :document";
        oql = new OqlQuery(hql);
        oql.getQuery().setEntity("document", this);
        Resource resource = (Resource) oql.uniqueResultNullIfEmpty();
        if (resource != null)
          result = resource;
      }*/
    }
    return result;
  }

  public void addNewVersionToReferralAndStore(TeamworkDocument newReferred) throws StoreException {

    if (!isNew()) {
      /*String hql = "select task from " + Task.class.getName() + " as task join task.documents as document  where document = :document";
     OqlQuery oql = new OqlQuery(hql);
     oql.getQuery().setEntity("document", this);*/
      Task task = getTask();
      if (task != null) {
        newReferred.setTask(task);
        newReferred.store();
        return;
      }

      /*hql = "select resource from " + Resource.class.getName() + " as resource join resource.documents as document  where document = :document";
     oql = new OqlQuery(hql);
     oql.getQuery().setEntity("document", this);*/
      Resource resource = getResource();
      if (resource != null) {
        //resource.addDocument(newReferred);
        newReferred.setResource(resource);
        newReferred.store();
        return;
      }
    }
  }

  public boolean isEnabled(Operator logged) throws PersistenceException {
    boolean isLocked = getLockedBy() != null;
    return (!isLocked || (isLocked && (getLockedBy().equals(Person.getPersonFromOperator(logged)))));
  }

  public TeamworkDocument getParent() {
    return (TeamworkDocument) parent;
  }

  public Node getParentNode() {
    return getParent();
  }

  public int compareVersionTo(TeamworkDocument doc) {
    VersionComparator vc = new VersionComparator();
    return vc.compare(this.getVersion(), doc.getVersion());
  }

  public boolean hasPermissionFor(User u, Permission p) {

    boolean result = false;

    Securable sec = getTask() != null ? getTask() : getResource();

    if (sec != null)
      result = sec.hasPermissionFor(u, p);
    else
      result = super.hasPermissionFor(u,p);

    return result;
  }


}