package com.twproject.importing;

import com.twproject.agenda.Event;
import com.twproject.document.TeamworkDocument;
import com.twproject.messaging.board.Board;
import com.twproject.messaging.stickyNote.StickyNote;
import com.twproject.operator.TeamworkOperator;
import com.twproject.resource.Company;
import com.twproject.resource.Person;
import com.twproject.resource.Resource;
import com.twproject.resource.Service;
import com.twproject.security.RoleTeamwork;
import com.twproject.security.TeamworkPermissions;
import com.twproject.task.*;
import com.twproject.task.financial.Cost;
import com.twproject.two.agenda.*;
import com.twproject.two.document.DocumentOld;
import com.twproject.two.messaging.board.BoardOld;
import com.twproject.two.messaging.stickyNote.StickyNoteOld;
import com.twproject.two.resource.AnagraphicalDataOld;
import com.twproject.two.resource.OperatorOld;
import com.twproject.two.resource.ResourceOld;
import com.twproject.two.security.AreaOld;
import com.twproject.two.security.PermissionOld;
import com.twproject.two.security.RoleOld;
import com.twproject.two.task.*;
import com.twproject.two.uidgen.CounterOld;
import com.twproject.two.worklog.WorkLogOld;
import com.twproject.worklog.Worklog;
import org.hibernate.Session;
import org.hibernate.proxy.AbstractLazyInitializer;
import org.hibernate.proxy.HibernateProxy;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.agenda.*;
import org.jblooming.anagraphicalData.AnagraphicalData;
import org.jblooming.company.DepartmentType;
import org.jblooming.ontology.Identifiable;
import org.jblooming.ontology.PersistentFile;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.exceptions.FindException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.HibernateUtilities;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.security.*;
import org.jblooming.tracer.Tracer;
import org.jblooming.transaction.hibernate.HibernateTransaction;
import org.jblooming.uidgen.Counter;
import org.jblooming.uidgen.CounterHome;
import org.jblooming.utilities.StringUtilities;
import org.jblooming.utilities.file.FileUtilities;

import javax.servlet.jsp.JspWriter;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.DriverManager;
import java.util.*;


public class ImportHelper {

  private Map<String, Permission> mapPerimssion = null;
  private Map<Serializable, Area> mapAreas = null;
  private Map<Serializable, RoleTeamwork> mapRoles = null;
  private Map<Serializable, Board> mapBoards = null;
  private Map<Serializable, StickyNote> mapStickys = null;
  private Map<Serializable, TeamworkOperator> mapOperators = null;
  private Map<Serializable, Resource> mapResources = null;
  private Map<Serializable, Resource> mapDocIdResource = null;
  private Map<Serializable, Task> mapTasks = null;
  private Map<Serializable, Task> mapDocIdTask = null;
  private Map<Serializable, TeamworkDocument> mapDocuments = null;
  private Map<Serializable, Event> mapEvents = null;
  private Map<Serializable, Issue> mapIssues = null;
  private Map<String, RoleTeamwork> mapAreaRoles = null;
  private Map<String, IssueType> mapIssueTypes = null;
  private Map<String, String> mapCounters = null;
  private JspWriter out;

  private boolean isFirstRun = true;


  private int FLUSH_EVERY = 20;

  public static String TW2_dbUrl = "jdbc:[driver name]:[driver type]://[server ip]:[port]/[db]";
  public static String TW2_user = "[user]";
  public static String TW2_pwd = "[password]";
  public static String TW2_drv = "[driver class name]";
  private Connection conn = null;
  private Session tw2Session = null;

  public ImportHelper(JspWriter out) {
    this.out = out;
  }


  public void resetCounters() throws StoreException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".resetCounters ");
    out.println("<hr>" + this.getClass().getName() + ".resetCounters <br>");
    out.flush();

    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Area.class), getMaxId(getMapAreas().values()) + 1000);
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(RoleTeamwork.class), getMaxId(getMapRoles().values()) + 1000);
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(TeamworkOperator.class), getMaxId(getMapOperators().values()) + 1000);
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Resource.class), getMaxId(getMapResources().values()) + 1000);
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Task.class), getMaxId(getMapTasks().values()) + 1000);
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(TeamworkDocument.class), getMaxId(getMapDocuments().values()) + 1000);
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Event.class), getMaxId(getMapEvents().values()) + 1000);
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Issue.class), getMaxId(getMapIssues().values()) + 1000);


    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }

  private int getMaxId(Collection<? extends Identifiable> identifis) {
    int maxArea = 0;
    for (Identifiable area : identifis) {
      int id = 0;
      try {
        id = Integer.parseInt(area.getId() + "");
      } catch (Exception e) {
      }
      maxArea = (maxArea <= id ? id : maxArea);
    }
    return maxArea;
  }

  public Permission getPermission(String oldPermission) {
    if (oldPermission == null)
      return null;
    return getMapPerimssion().get(oldPermission);
  }

  public Task getTask(TaskOld oldTask) throws PersistenceException {
    Task ret = null;
    if (oldTask != null)
      ret = getMapTasks().get(oldTask.getId());
    return ret;
  }

  public TeamworkDocument getDocument(DocumentOld oldDocument) throws PersistenceException {
    TeamworkDocument ret = null;
    if (oldDocument != null)
      ret = getMapDocuments().get(oldDocument.getId());
    return ret;
  }

  public Resource getResource(ResourceOld oldResource) throws PersistenceException {
    Resource ret = null;
    if (oldResource != null)
      ret = getMapResources().get(oldResource.getId());
    return ret;
  }


  public TeamworkOperator getOperator(OperatorOld oldOperator) throws PersistenceException {
    TeamworkOperator ret = null;
    if (oldOperator != null)
      ret = getMapOperators().get(oldOperator.getLoginName());
    return ret;
  }

  public Task getTaskByDocId(Serializable docId) {
    Task ret = null;
    if (docId != null)
      ret = getMapDocIdTask().get(docId);
    return ret;
  }

  public Resource getResourceByDocId(Serializable docId) {
    Resource ret = null;
    if (docId != null)
      ret = getMapDocIdResource().get(docId);
    return ret;
  }


  public Area getArea(AreaOld oldArea) throws PersistenceException {
    Area ret = null;
    if (oldArea != null)
      ret = getMapAreas().get(oldArea.getId());
    return ret;
  }

  public Role getRole(RoleOld oldRole) throws PersistenceException {
    Role ret = null;
    if (oldRole != null)
      ret = getMapRoles().get(oldRole.getId());
    return ret;
  }

  public IssueType getIssueType(String code) throws PersistenceException {
    IssueType ret = null;
    if (code != null) {
      ret = getMapIssueTypes().get(code);
      if (ret == null) {
        ret = new IssueType();
        ret.setDescription(code);
        getMapIssueTypes().put(code, ret);
        ret.store();
      }
    }
    return ret;
  }


  // todo: standard department should be created at tw startup  -> move on teamworkSettings.jsp
  public int fillDepartmentType() throws StoreException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".fillDepartmentType ");
    out.println("<hr>" + this.getClass().getName() + ".fillDepartmentType <br>");
    out.flush();

    int i = 0;
    for (DepartmentType.Code ctc : DepartmentType.Code.values()) {
      DepartmentType dt = null;
      try {
        dt = DepartmentType.getDefaultDepartment(ctc);
      } catch (FindException f) {
      }
      if (dt == null) {
        dt = new DepartmentType();
        dt.setDescription(ctc.toString());
        dt.setIdAsNew();
        dt.setStringValue(ctc.toString());
        dt.store();
      }
    }
    out.println("<hr>" + this.getClass().getName() + ". <b>Operation completed. See above for details <br></b>");
    return i;
  }

  // todo: standard taskType should be created at tw startup  -> move on teamworkSettings.jsp
  public int fillTaskType() throws StoreException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".fillTaskType ");
    out.println("<hr>" + this.getClass().getName() + ".fillTaskType <br>");
    out.flush();

    int i = 0;
    for (TaskType.Type ctc : TaskType.Type.values()) {
      TaskType dt = null;
      try {
        dt = TaskType.getDefaultTaskType(ctc);
      } catch (FindException f) {
      }
      if (dt == null) {
        dt = new TaskType();
        dt.setDescription(ctc.toString());
        dt.setIdAsNew();
        dt.setStringValue(ctc.toString());
        dt.store();
      }
    }
    out.println("<hr><b>Operation completed. See above for details <br></b>");
    return i;
  }

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------   INITIALIZERS --------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------

  private void inizializeMapPerimssions() throws IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".inizializeMapPerimssions ");
    out.println("<hr>" + this.getClass().getName() + ".inizializeMapPerimssions <br>");
    out.flush();


    mapPerimssion = new HashMap<String, Permission>();
    /* PlatformPermissions */
    getMapPerimssion().put(PermissionOld.perm_role_canRead.name, new Permission(PlatformPermissions.role_canRead.name));
    getMapPerimssion().put(PermissionOld.perm_role_canWrite.name, new Permission(PlatformPermissions.role_canWrite.name));
    //getMapPerimssion().put(PermissionOld.perm_area_canRead.name, new Permission(PlatformPermissions.area_canManage.name));
    getMapPerimssion().put(PermissionOld.perm_area_canWrite.name, new Permission(PlatformPermissions.area_canManage.name));
    getMapPerimssion().put(PermissionOld.perm_operator_canRead.name, new Permission(PlatformPermissions.operator_canRead.name));
    getMapPerimssion().put(PermissionOld.perm_operator_canWrite.name, new Permission(PlatformPermissions.operator_canWrite.name));

    /* TeamworkPermissions */
    getMapPerimssion().put(PermissionOld.perm_resource_canRead.name, new Permission(TeamworkPermissions.resource_canRead.name));
    getMapPerimssion().put(PermissionOld.perm_resource_canWrite.name, new Permission(TeamworkPermissions.resource_canWrite.name));
    //PersistHelper.mapPerimssionString.put(PermissionOld.perm_file_canRead.name, new Permission(TeamworkPermissions.file_canRead.name));
    //PersistHelper.mapPerimssionString.put(PermissionOld.perm_file_canWrite.name, new Permission(TeamworkPermissions.file_canWrite.name));
    getMapPerimssion().put(PermissionOld.perm_issue_canRead.name, new Permission(TeamworkPermissions.issue_canRead.name));
    getMapPerimssion().put(PermissionOld.perm_issue_canWrite.name, new Permission(TeamworkPermissions.issue_canWrite.name));
    getMapPerimssion().put(PermissionOld.perm_cost_canRead.name, new Permission(TeamworkPermissions.task_cost_canRead.name));
    getMapPerimssion().put(PermissionOld.perm_cost_canWrite.name, new Permission(TeamworkPermissions.task_cost_canWrite.name));
    getMapPerimssion().put(PermissionOld.perm_board_can_read.name, new Permission(TeamworkPermissions.board_canRead.name));
    getMapPerimssion().put(PermissionOld.perm_board_can_write.name, new Permission(TeamworkPermissions.board_canWrite.name));

    out.println("<hr><b>Operation completed. See above for details <br></b>");
  }

  /**
   * area use different id
   *
   * @throws PersistenceException
   */
  public void initializeAreaMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeAreaMap ");
    out.println("<hr>" + this.getClass().getName() + ".initializeAreaMap <br>");
    out.flush();

    this.mapAreas = new HashMap<Serializable, Area>();

    // load new areas
    List<Area> lan = new OqlQuery("from " + Area.class.getName()).list();
    Map<Serializable, Area> anm = new HashMap<Serializable, Area>();
    for (Area a : lan) {
      anm.put(a.getName(), a);
    }

    //load old areas
    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<AreaOld> lao = new OqlQuery("from " + AreaOld.class.getName(), pcExt).list();
    Map<Serializable, AreaOld> aom = new HashMap<Serializable, AreaOld>();
    for (AreaOld a : lao) {
      aom.put(a.getName(), a);
    }

    for (Serializable oldName : aom.keySet()) {
      AreaOld areaOld = aom.get(oldName);
      Area areaNew;
      if (anm.containsKey(oldName)) {
        areaNew = anm.get(oldName);
      } else {
        // create the new area
        areaNew = cloneArea(areaOld);
      }
      getMapAreas().put(areaOld.getId(), areaNew);
    }
    setOwnerArea(aom);

    // test for default area
    Area bekked = anm.get(Area.STANDARD_AREA);
    if (bekked == null) {
      // create the default area
      Area area = new Area();
      area.setName(Area.STANDARD_AREA);
      area.setOwner(null);
      area.store();
      bekked = area;
    }
    // add the standard area with Area.STANDARD_AREA as key
    getMapAreas().put(Area.STANDARD_AREA, bekked);

    // set conunters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Area.class), getMaxId(getMapAreas().values()) + 1000);

    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }

  private void setOwnerArea(Map<Serializable, AreaOld> oldAreaMap) throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".setOwnerArea ");
    out.println(this.getClass().getName() + ".setOwnerArea <br>");
    out.flush();


    for (AreaOld ao : oldAreaMap.values()) {
      Area areaNew = getArea(ao);
      if (ao.getOwner() != null && areaNew != null && areaNew.getOwner() == null) {
        TeamworkOperator newOwner = getOperator(ao.getOwner());
        if (newOwner != null) {
          areaNew.setOwner(newOwner);
          areaNew.store();
        }
      }
    }
    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }


  /**
   * roles use the same ID
   *
   * @throws PersistenceException
   */
  public void initializeRoleMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeRoleMap ");
    out.println("<hr>" + this.getClass().getName() + ".initializeRoleMap <br>");
    out.flush();

    // load the new RoleTeamwork list
    List<RoleTeamwork> nrl = new OqlQuery("from " + RoleTeamwork.class.getName()).list();
    mapRoles = new HashMap<Serializable, RoleTeamwork>();
    for (RoleTeamwork r : nrl) {
      getMapRoles().put(r.getId(), r);
    }

    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<RoleOld> ro = new OqlQuery("from " + RoleOld.class.getName(), pcExt).list();
    for (RoleOld oldRole : ro) {
      if (!getMapRoles().containsKey(oldRole.getId())) {
        RoleTeamwork newRole = cloneRole(oldRole);
        getMapRoles().put(oldRole.getId(), newRole);
      }
    }

    // set conunters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(RoleTeamwork.class), getMaxId(getMapRoles().values()) + 1000);

    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }



  /**
   * roles use the same ID
   *
   * @throws PersistenceException
   */
  public void initializeStickyMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeStickyMap ");
    out.println("<hr>" + this.getClass().getName() + ".initializeStickyMap <br>");
    out.flush();

    // load the new RoleTeamwork list
    List<StickyNote> nsl = new OqlQuery("from " + StickyNote.class.getName()).list();
    mapStickys = new HashMap<Serializable, StickyNote>();
    for (StickyNote s : nsl) {
      getMapStickys().put(s.getId(), s);
    }

    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<StickyNoteOld> ro = new OqlQuery("from " + StickyNoteOld.class.getName(), pcExt).list();
    for (StickyNoteOld oldSticky : ro) {
      if (!getMapStickys().containsKey(oldSticky.getId())) {
        StickyNote newSticky = cloneSticky(oldSticky);
        getMapStickys().put(oldSticky.getId(), newSticky);
      }
    }

    // set conunters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(StickyNote.class), getMaxId(getMapStickys().values()) + 1000);

    HibernateTransaction.getCurrentTransaction().checkPoint();


    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }


  public void initializeBoardMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeBoardMap ");
    out.println("<hr>" + this.getClass().getName() + ".initializeBoardMap <br>");
    out.flush();

    // load the new Board list
    List<Board> nbl = new OqlQuery("from " + Board.class.getName()).list();
    mapBoards = new HashMap<Serializable, Board>();
    for (Board r : nbl) {
      getMapBoards().put(r.getId(), r);
    }

    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<BoardOld> ro = new OqlQuery("from " + BoardOld.class.getName(), pcExt).list();
    for (BoardOld oldRole : ro) {
      if (!getMapRoles().containsKey(oldRole.getId())) {
        Board newBoard = cloneBoard(oldRole);
        getMapBoards().put(oldRole.getId(), newBoard);
      }
    }

    // set conunters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Board.class), getMaxId(getMapBoards().values()) + 1000);

    HibernateTransaction.getCurrentTransaction().checkPoint();

    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }



  /**
   * operators mantains the same login used as key in the map
   */
  public void initializeOperatorsMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeOperatorsMap ");
    out.println("<hr>" + this.getClass().getName() + ".initializeOperatorsMap <br>");
    out.flush();

    // load the new operators list
    List<TeamworkOperator> nrl = new OqlQuery("from " + TeamworkOperator.class.getName()).list();
    mapOperators = new HashMap<Serializable, TeamworkOperator>();
    for (TeamworkOperator twop : nrl) {
      getMapOperators().put(twop.getLoginName(), twop);
    }

    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<OperatorOld> opOldList = new OqlQuery("from " + OperatorOld.class.getName(), pcExt).list();
    for (OperatorOld oldOperator : opOldList) {
      if (!getMapOperators().containsKey(oldOperator.getLoginName())) {
        // create the new TeamworkOperator
        TeamworkOperator newOperator = cloneOperator(oldOperator);
        getMapOperators().put(oldOperator.getLoginName(), newOperator);
      }
    }


    HibernateTransaction.getCurrentTransaction().checkPoint();

    // set counters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(TeamworkOperator.class), getMaxId(getMapOperators().values()) + 1000);
    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }

  /**
   * Resources mantains the same ID used as key in the map
   */
  public void initializeResourcesMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeResourcesMap ");
    out.println("<hr>" + this.getClass().getName() + ".initializeResourcesMap <br>");
    out.flush();

    // load the new Resource list
    List<Resource> nrl = new OqlQuery("from " + Resource.class.getName()).list();
    mapResources = new HashMap<Serializable, Resource>(3000);
    mapDocIdResource = new HashMap<Serializable, Resource>(3000);

    for (Resource twre : nrl) {
      getMapResources().put(twre.getId(), twre);
    }

    Map<Serializable, Serializable>mapParents = new HashMap<Serializable, Serializable>(3000);

    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<ResourceOld> opOldList = new OqlQuery("from " + ResourceOld.class.getName(), pcExt).list();
    int i = 0;
    for (ResourceOld oldRes : opOldList) {
      if (!getMapResources().containsKey(oldRes.getId())) {
        // create the new Resource
        Resource newResource = cloneResource(oldRes);

        if (i > FLUSH_EVERY) {
          HibernateTransaction.getCurrentTransaction().checkPoint();
          i = 0;
        }
        i++;
        getMapResources().put(oldRes.getId(), newResource);
      }
      if (oldRes.getParent() != null)
        mapParents.put(oldRes.getId(), oldRes.getParent().getId());

      // get old docs
      for (Object o : oldRes.getDocuments()) {
        DocumentOld od = (DocumentOld) o;
        for (DocumentOld ddo : getAllRelatedDoc(od, new HashSet<DocumentOld>())) {
          if (ddo.getType() != DocumentOld.IS_STRUCTURAL)
            mapDocIdResource.put(ddo.getId(), getMapResources().get(oldRes.getId()));
        }
      }

    }
    HibernateTransaction.getCurrentTransaction().checkPoint();

    // build parents
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeResourcesMap phase 2: build parents");
    out.println(this.getClass().getName() + ".initializeResourcesMap phase 2: build parents <br>");
    out.flush();

    for (Serializable id : mapParents.keySet()) {
      Resource newResource = getMapResources().get(id);
      if (newResource.getParent() == null) {
        Resource newParent = getMapResources().get(mapParents.get(id));
        newResource.setParentAndStore(newParent);
      }
    }
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeResourcesMap ENDED");
    out.println(this.getClass().getName() + ".initializeResourcesMap ENDED <br>");
    out.flush();

    // set counters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Resource.class), getMaxId(getMapResources().values()) + 1000);
    out.println("<hr><b>Operation completed. See above for details <br></b>");

    // this beautiful hack is for rehidrate the inverse in persons
    if (isFirstRun) {
      isFirstRun = false;
      for (Resource toh : getMapResources().values())
        HibernateFactory.getSession().evict(toh);
      initializeResourcesMap();
    }


  }


  public Set<DocumentOld> getAllRelatedDoc(DocumentOld doc, Set<DocumentOld>set) {
    if (set == null)
      set = new HashSet<DocumentOld>();
    if (doc != null && !set.contains(doc)) {
      set.add(doc);
      if (doc.getDocuments() != null) {
        for (Object o : doc.getDocuments()) {
          DocumentOld dol = (DocumentOld) o;
          getAllRelatedDoc(dol, set);
        }
      }
      if (doc.getChildren() != null) {
        for (Object o : doc.getChildren()) {
          DocumentOld dol = (DocumentOld) o;
          getAllRelatedDoc(dol, set);
        }
      }
    }
    return set;
  }


  /**
   * Tasks mantains the same ID used as key in the map
   */
  public void initializeTasksMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeTasksMap phase 1");
    out.println("<hr>" + this.getClass().getName() + ".initializeTasksMap phase 1 <br>");
    out.flush();

    // load the new Task list
    List<Task> nrl = new OqlQuery("from " + Task.class.getName()).list();
    mapTasks = new HashMap<Serializable, Task>(5000);
    mapDocIdTask = new HashMap<Serializable, Task>(5000);
    for (Task twre : nrl) {
      getMapTasks().put(twre.getId(), twre);
    }

    Map<Serializable, Serializable>mapParents = new HashMap<Serializable, Serializable>(5000);

    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<TaskOld> opOldList = new OqlQuery("from " + TaskOld.class.getName(), pcExt).list();
    int i = 0;
    for (TaskOld oldTask : opOldList) {
      if (!getMapTasks().containsKey(oldTask.getId())) {
        // create the new Task
        Task newTask = cloneTask(oldTask);

        if (i > FLUSH_EVERY) {
          HibernateTransaction.getCurrentTransaction().checkPoint();
          i = 0;
        }
        i++;
        if (newTask != null)
          getMapTasks().put(oldTask.getId(), newTask);
      }
      if (oldTask.getParent() != null)
        mapParents.put(oldTask.getId(), oldTask.getParent().getId());

      // get old docs
      for (Object o : oldTask.getDocuments()) {
        DocumentOld od = (DocumentOld) o;
        for (DocumentOld ddo : getAllRelatedDoc(od, new HashSet<DocumentOld>())) {
          if (ddo.getType() != DocumentOld.IS_STRUCTURAL)
            mapDocIdTask.put(ddo.getId(), getMapTasks().get(oldTask.getId()));
        }
      }

    }


    HibernateTransaction.getCurrentTransaction().checkPoint();
    i = 0;
    // build parents
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeTasksMap phase 2: build parents");
    out.println(this.getClass().getName() + ".initializeTasksMap phase 2: build parents <br>");
    out.flush();


    for (Serializable id : mapParents.keySet()) {
      Task newTask = getMapTasks().get(id);
      if (newTask != null && newTask.getParent() == null) {
        Task newParent = getMapTasks().get(mapParents.get(id));
        if (newParent != null) {
          Tracer.platformLogger.debug(this.getClass().getName() + ".initializeTasksMap set parent of " + newTask.getId() + " to " + newParent.getId());
          out.println(this.getClass().getName() + ".initializeTasksMap set parent of " + newTask.getId() + " to " + newParent.getId() + " <br>");
          out.flush();


          newTask.setParentAndStore(newParent);
        }
        if (i > FLUSH_EVERY) {
          HibernateTransaction.getCurrentTransaction().checkPoint();
          i = 0;
        }
        i++;
      }
    }
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeTasksMap ENDED");
    out.println(this.getClass().getName() + ".initializeTasksMap ENDED <br>");
    out.flush();

    // set counters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Task.class), getMaxId(getMapTasks().values()) + 1000);
    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }

  /**
   * Documents mantains the same ID used as key in the map
   */
  public void initializeDocumentsMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeDocumentsMap phase 1");
    out.println("<hr>" + this.getClass().getName() + ".initializeDocumentsMap phase 1 <br>");
    out.flush();

    // load the new TeamworkDocument list
    List<TeamworkDocument> nrl = new OqlQuery("from " + TeamworkDocument.class.getName()).list();
    mapDocuments = new HashMap<Serializable, TeamworkDocument>(5000);
    for (TeamworkDocument twre : nrl) {
      getMapDocuments().put(twre.getId(), twre);
    }

    Map<Serializable, Serializable>mapParents = new HashMap<Serializable, Serializable>(5000);

    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<DocumentOld> opOldList = new OqlQuery("from " + DocumentOld.class.getName(), pcExt).list();
    int i = 0;
    for (DocumentOld oldDocument : opOldList) {
      if (!getMapDocuments().containsKey(oldDocument.getId())) {
        // create the new TeamworkDocument
        TeamworkDocument newDocument = cloneDocument(oldDocument);
        if (i > FLUSH_EVERY) {
          HibernateTransaction.getCurrentTransaction().checkPoint();
          i = 0;
        }
        i++;
        if (newDocument != null)
          getMapDocuments().put(oldDocument.getId(), newDocument);
      }
      if (oldDocument.getParent() != null)
        mapParents.put(oldDocument.getId(), oldDocument.getParent().getId());
    }

    HibernateTransaction.getCurrentTransaction().checkPoint();
    i = 0;
    // build parents
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeDocumentsMap phase 2: build parents");
    out.println(this.getClass().getName() + ".initializeDocumentsMap phase 2: build parents <br>");
    out.flush();


    for (Serializable id : mapParents.keySet()) {
      TeamworkDocument newDocument = getMapDocuments().get(id);
      if (newDocument != null && newDocument.getParent() == null) {
        TeamworkDocument newParent = getMapDocuments().get(mapParents.get(id));
        if (newParent != null) {
          Tracer.platformLogger.debug(this.getClass().getName() + ".initializeDocumentsMap set parent of " + newDocument.getId() + " to " + newParent.getId());
          out.println(this.getClass().getName() + ".initializeDocumentsMap set parent of " + newDocument.getId() + " to " + newParent.getId() + " <br>");
          out.flush();


          try {
            newDocument.setParentAndStore(newParent);
          } catch (Exception e) {
            Tracer.platformLogger.error(this.getClass().getName() + ".initializeDocumentsMap circular reference between " + newDocument.getId() + " and " + newParent.getId());
            out.println("<font color='red'>" + this.getClass().getName() + ".initializeDocumentsMap circular reference between " + newDocument.getId() + " and " + newParent.getId() + "</FONT><br>");
            out.flush();


          }

        }
        if (i > FLUSH_EVERY) {
          HibernateTransaction.getCurrentTransaction().checkPoint();
          i = 0;
        }
        i++;
      }
    }
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeDocumentsMap ENDED");
    out.println(this.getClass().getName() + ".initializeDocumentsMap ENDED <br>");
    out.flush();

    // set counters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(TeamworkDocument.class), getMaxId(getMapDocuments().values()) + 1000);
    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }


  /**
   * Events mantains the same ID used as key in the map
   */
  public void initializeEventsMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeAgendaMap phase 1");
    out.println("<hr>" + this.getClass().getName() + ".initializeAgendaMap phase 1 <br>");
    out.flush();

    // load the new Event list
    List<Event> nrl = new OqlQuery("from " + Event.class.getName()).list();
    mapEvents = new HashMap<Serializable, Event>(5000);
    for (Event twre : nrl) {
      getMapEvents().put(twre.getId(), twre);
    }

    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<EventOld> opOldList = new OqlQuery("from " + EventOld.class.getName(), pcExt).list();
    int i = 0;
    for (EventOld oldEvent : opOldList) {
      if (!getMapEvents().containsKey(oldEvent.getId())) {
        // create the new Event
        Event newEvent = cloneEvent(oldEvent);
        if (i > FLUSH_EVERY) {
          HibernateTransaction.getCurrentTransaction().checkPoint();
          i = 0;
        }
        i++;
        if (newEvent != null)
          getMapEvents().put(oldEvent.getId(), newEvent);
      }
    }

    HibernateTransaction.getCurrentTransaction().checkPoint();
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeEventsMap ENDED");
    out.println("<hr>" + this.getClass().getName() + ".initializeEventsMap ENDED <br>");
    out.flush();

    // set conunters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Event.class), getMaxId(getMapEvents().values()) + 1000);
    out.println("<hr><b>Operation completed. See above for details <br></b>");


  }


  /**
   * Issue mantains the same ID used as key in the map
   */
  public void initializeIssuesMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeIssuesMap phase 1");
    out.println("<hr>" + this.getClass().getName() + ".initializeIssuesMap phase 1 <br>");
    out.flush();

    // load the new Issue list
    List<Issue> nrl = new OqlQuery("from " + Issue.class.getName()).list();
    mapIssues = new HashMap<Serializable, Issue>(5000);
    for (Issue twre : nrl) {
      getMapIssues().put(twre.getId(), twre);
    }

    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<IssueOld> opOldList = new OqlQuery("from " + IssueOld.class.getName(), pcExt).list();
    int i = 0;
    for (IssueOld oldIssue : opOldList) {
      if (!getMapIssues().containsKey(oldIssue.getId())) {
        // create the new Event
        Issue newIssue = cloneIssue(oldIssue);
        if (i > FLUSH_EVERY) {
          HibernateTransaction.getCurrentTransaction().checkPoint();
          i = 0;
        }
        i++;
        if (newIssue != null)
          getMapIssues().put(oldIssue.getId(), newIssue);
      }
    }

    HibernateTransaction.getCurrentTransaction().checkPoint();
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeIssuesMap ENDED");
    out.println(this.getClass().getName() + ".initializeIssuesMap ENDED <br>");
    out.flush();

    // set conunters
    CounterHome.setCounterSeed(HibernateUtilities.counterNameForIdAssigned(Issue.class), getMaxId(getMapIssues().values()) + 1000);
    out.println("<hr><b>Operation completed. See above for details <br></b>");


  }


  private void initializeIssueTypesMap() throws FindException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeIssueTypesMap");
    out.println("<hr>" + this.getClass().getName() + ".initializeIssueTypesMap <br>");
    out.flush();

    // load the new Issue list
    List<IssueType> nrl = new OqlQuery("from " + IssueType.class.getName()).list();
    mapIssueTypes = new HashMap<String, IssueType>(5000);
    for (IssueType it : nrl) {
      getMapIssueTypes().put(it.getDescription(), it);
    }
    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }


  public void initializeCounterMap() throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".initializeCounterMap ");
    out.println("<hr>" + this.getClass().getName() + ".initializeCounterMap <br>");
    out.flush();

    this.mapCounters = new HashMap<String, String>();

    // load new counters
    List<Counter> lan = new OqlQuery("from " + Counter.class.getName()).list();
    Map<String, Counter> anm = new HashMap<String, Counter>();
    for (Counter a : lan) {
      anm.put(a.getName(), a);
    }

    //load old counter
    PersistenceContext pcExt = getPersistenceContextOnOldDB();
    List<CounterOld> lao = new OqlQuery("from " + CounterOld.class.getName() + " as c where not c.name like 'com.tea%'", pcExt).list();
    Map<String, CounterOld> aom = new HashMap<String, CounterOld>();
    for (CounterOld a : lao) {
      aom.put(a.getName(), a);
    }

    for (String oldName : aom.keySet()) {
      CounterOld counterOld = aom.get(oldName);
      String counterNew;
      if (anm.containsKey("COUNT_" + oldName) && anm.get(oldName)!=null) {
        counterNew = anm.get(oldName).getName();
      } else {
        // create the new counter
        counterNew = cloneCounter(counterOld);
      }
      getMapCounters().put(counterOld.getName(), counterNew);
    }

    out.println("<hr><b>Operation completed. See above for details <br></b>");

  }

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------   CLONERS -------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------


  private String cloneCounter(CounterOld counterOld) throws IOException, StoreException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".cloneCounter " + counterOld.getName());
    out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneCounter " + counterOld.getName() + " <br>");
    out.flush();

    String newCounter = null;
    if (!counterOld.getName().startsWith("com.tw")) {
      CounterHome.setCounterSeed("COUNT_" + counterOld.getName(), counterOld.getValue());
      newCounter = "COUNT_" + counterOld.getName();
    }
    return newCounter;

  }


  private Resource cloneResource(ResourceOld oldResource) throws PersistenceException, IOException {
    Resource newResource = null;

    Tracer.platformLogger.debug(this.getClass().getName() + ".cloneResource " + oldResource.getId() + " " + oldResource.getName());
    out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneResource " + oldResource.getId() + " " + oldResource.getName() + " <br>");
    out.flush();


    if (oldResource.getType() == ResourceOld.COMPANY) {// -----------------------------------------------   COMPANY
      Company company = new Company();
      company.setName(oldResource.getName());
      company.setInherit(oldResource.getInherit());
      company.setPropagate(oldResource.getPropagate());
      company.setType(DepartmentType.getDefaultDepartment(DepartmentType.Code.COMPANY));
      newResource = company;

    } else if (oldResource.getType() == ResourceOld.SERVICE) {// -----------------------------------------------   SERVICE
      Service service = new Service();
      service.setName(oldResource.getName());
      newResource = service;

    } else if (oldResource.getType() == ResourceOld.PERSON) {// -----------------------------------------------   PERSON
      Person person = new Person();

      person.setPersonName(oldResource.getPersonName());
      person.setPersonSurname(oldResource.getPersonSurname());

      OperatorOld opOld = oldResource.getOperator();
      if (opOld != null) {
        TeamworkOperator newOp = getOperator(opOld);
        if (newOp != null) {
          if (newOp.getMyPerson() != null) {
            Tracer.platformLogger.warn("&nbsp;&nbsp;&nbsp;" + this.getClass().getName() + "Persist PERSON: new user is already assigned to a resource. New operator id:" + newOp.getId() + ". Old resource Id: " + oldResource.getId() + " PERSON " + oldResource.getPersonName() + " " + oldResource.getPersonSurname());
            out.println("<font color='blue'>" + this.getClass().getName() + "Persist PERSON: new user is already assigned to a resource. New operator id:" + newOp.getId() + ". Old resource Id: " + oldResource.getId() + " PERSON " + oldResource.getPersonName() + " " + oldResource.getPersonSurname() + "</font> <br>");
            out.flush();
          }

          person.setMyself(newOp);
          // resets the operator name
          newOp.setName(oldResource.getPersonName());
          newOp.setSurname(oldResource.getPersonSurname());
          newOp.store();
        }
        List news = oldResource.getOperator().getNews();
        for (Object noticeObj : news) {
          String notice = (String) noticeObj;
          person.addNews(notice);
        }
      }

      //String courtesyTitle is new
      //PersistentFile myPhoto is new
      //Date hiringDate is new

      //String personalInterest is new
      //String jobDescription is new
      //blackBoardNotes is new
      // Map votes NOT IMPORTED!!

      //Set workerEvents is new
      //Set stickAuthored is new
      //Set stickyReceived is new
      //Set eventOwned is new
      //Set eventAuthored is new

      newResource = person;
    }

    newResource.setId(oldResource.getId());

    //parent will be store at second step
    //String code is new
    //String location is new

    if (oldResource.getArea() != null) {
      Area newArea = getArea(oldResource.getArea());
      if (newArea != null)
        newResource.setArea(newArea);
    }

    for (Object ado : oldResource.getAnagraphicalData()) {
      AnagraphicalData adNew = cloneAnagraphicalData((AnagraphicalDataOld) ado);
      if (adNew != null)
        newResource.getAnagraphicalDatas().add(adNew);

    }

    /*//  private List documents;  document link are setted in cloneDocument
    for (Object ob : oldResource.getDocuments()) {
      DocumentOld oldDoc = (DocumentOld) ob;
      TeamworkDocument newDoc = getDocument(oldDoc);

      //checks if the document is already assigned to a resource or another task
      if (newDoc.getTask() != null) {
        Tracer.platformLogger.warn(this.getClass().getName() + ".cloneResource the assigned document id " + newDoc.getId() + " is already assigned to task id " + newDoc.getTask().getId() + ". Will not be assigne to resource id :" + newResource.getId());
        out.println("&nbsp;&nbsp;&nbsp;<font color='blue'>" + this.getClass().getName() + ".cloneResource the assigned document id " + newDoc.getId() + " is already assigned to task id " + newDoc.getTask().getId() + ". Will not be assigne to resource id :" + newResource.getId() + " </font><br>");
        out.flush();
      } else if (newDoc.getResource() != null) {
        Tracer.platformLogger.warn(this.getClass().getName() + ".cloneResource the assigned document id " + newDoc.getId() + " is already assigned to resourse id " + newDoc.getResource().getId() + ". Will not be assigne to resource id :" + newResource.getId());
        out.println("&nbsp;&nbsp;&nbsp;<font color='blue'>" + this.getClass().getName() + ".cloneResource the assigned document id " + newDoc.getId() + " is already assigned to resource id " + newDoc.getResource().getId() + ". Will not be assigne to resource id :" + newResource.getId() + " </font><br>");
        out.flush();
      } else {
        newDoc.setResource(newResource);
        newDoc.store();
        newResource.getDocuments().add(newDoc);
      }
    }*/

    //Person myManager is new;
    //String myCostCenter is new;
    //String shortNotes is new;
    //String myStaff is new;

    //Set<ResourceClassificationAssosiacion> resourceClassificationAssosiacions
    newResource.setInherit(oldResource.getInherit());
    newResource.setPropagate(oldResource.getPropagate());
    newResource.setStaff(false);
    newResource.setNotes(oldResource.getNotes());

    newResource.store();

    return newResource;
  }


  private Area cloneArea(AreaOld areaOld) throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".cloneArea " + areaOld.getId() + " " + areaOld.getName());
    out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneArea " + areaOld.getId() + " " + areaOld.getName() + " <br>");
    out.flush();


    Area areaNew;
    areaNew = new Area();
    areaNew.setName(areaOld.getName());

    //areaNew.setOwner(    // see: setOwnerArea
    areaNew.setPropagate(areaOld.getPropagate());
    areaNew.store();
    return areaNew;
  }


  private TeamworkOperator cloneOperator(OperatorOld oldOperator) throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".cloneOperator " + oldOperator.getId() + " " + oldOperator.getLoginName());
    out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneOperator " + oldOperator.getId() + " " + oldOperator.getLoginName() + " <br>");
    out.flush();


    TeamworkOperator tw3Operator = new TeamworkOperator();

    //Map options do not import
    //Map filters
    //Map favoriteUrls = new HashMap();
    //  fav. urls can't be recovered
    //favoriteUrls.putAll(oldOperator.getFavoriteUrls());
    //tw3Operator.setFavoriteUrls(favoriteUrls);

    //from User
    tw3Operator.setAdministrator(oldOperator.isAdministrator());
    tw3Operator.setLoginName(oldOperator.getLoginName());
    tw3Operator.setPassword(oldOperator.getPassword());
    //String authentification; is new
    for (String lp : (List<String>) oldOperator.getLastPasswordList())
      tw3Operator.addLastPassword(lp);

    tw3Operator.setLastPasswordChangeDate(getHumanDate(oldOperator.getLastPasswordChangeDate()));
    tw3Operator.setPersonalIdentificationQuestion(oldOperator.getPersonalIdentificationQuestion());
    tw3Operator.setPersonalIdentificationAnswer(oldOperator.getPersonalIdentificationAnswer());
    tw3Operator.setName("-"); // the name will be copied when associated to resource
    tw3Operator.setSurname(oldOperator.getFullName());
    tw3Operator.setEnabled(oldOperator.isEnabled());

    AnagraphicalData adNew = cloneAnagraphicalData(oldOperator.getAnagraphicalData());
    tw3Operator.setAnagraphicalData(adNew);


    tw3Operator.store();

    //ScheduleSupport  enableOnlyOn; is new

    for (Object ro : oldOperator.getRoles()) {
      Role newRole = getRole((RoleOld) ro);
      if (newRole != null) {
        OperatorRole opr = new OperatorRole();
        opr.setRole(newRole);
        opr.setOperator(tw3Operator);
        opr.store();
      }
    }

    //Set<OperatorGroup> operatorGroup; is new

    tw3Operator.store();

    return tw3Operator;
  }


  private AnagraphicalData cloneAnagraphicalData(AnagraphicalDataOld ado) throws PersistenceException, IOException {
    if (ado == null) {
      return null;
    }
    Tracer.platformLogger.debug(this.getClass().getName() + ".cloneAnagraphicalData " + ado.getId());
    out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneAnagraphicalData " + ado.getId() + " <br>");
    out.flush();


    AnagraphicalData adNew;
    adNew = new AnagraphicalData();
    adNew.setLocationDescription(ado.getLocationDescription());
    adNew.setAddress(ado.getAddress());
    adNew.setZip(ado.getZip());
    adNew.setTelephone(ado.getTelephone());
    adNew.setMobile(ado.getMobile());
    adNew.setFax(ado.getFax());
    adNew.setEmail(ado.getEmail());
    adNew.setUrl(ado.getUrl());
    adNew.setCity(ado.getCity());
    adNew.setProvince(ado.getProvince());
    adNew.setState(ado.getState());
    int orderFactor = 1;
    try {
      orderFactor = Integer.parseInt(ado.getOrderFactor());
    } catch (NumberFormatException nfe) {
    }
    adNew.setOrderFactor(orderFactor);
    adNew.store();
    return adNew;
  }


  private RoleTeamwork cloneRole(RoleOld oldRole) throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".cloneRole " + oldRole.getId() + " " + oldRole.getName());
    out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneRole " + oldRole.getId() + " " + oldRole.getName() + " <br>");
    out.flush();

// create the new role
    RoleTeamwork newRole = new RoleTeamwork();
    newRole.setId(oldRole.getId());
    newRole.setArea(getArea(oldRole.getArea()));
    newRole.setName(oldRole.getName());

    if (oldRole.getPermissionIds() != null) {
      List<String> perms = StringUtilities.splitToList(oldRole.getPermissionIds(), "|");

      for (String perm : perms) {
        PermissionOld permissionOld = PermissionOld.getPermission(Integer.parseInt(perm));
        if (permissionOld != null) {
          Permission permissionNew = getPermission(permissionOld.name);
          if (permissionNew != null) // non matching parmission are not copied
            newRole.addPermission(permissionNew);
        }
      }
    }

    newRole.store();
    return newRole;
  }


  private Board cloneBoard(BoardOld oldBoard) throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".cloneBoard " + oldBoard.getId() + " " + oldBoard.getName());
    out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneBoard " + oldBoard.getId() + " " + oldBoard.getName() + " <br>");
    out.flush();

    // create the new Board
    Board newBoard = new Board();
    newBoard.setId(oldBoard.getId());
    newBoard.setArea(getArea(oldBoard.getArea()));
    newBoard.setName(oldBoard.getName());
    newBoard.setDescription(oldBoard.getDescription());

    newBoard.store();

    for (Object o: oldBoard.getStickyNotes()){
      StickyNoteOld sn=(StickyNoteOld)o;
      StickyNote snn=getMapStickys().get(sn.getId());
      snn.setBoard(newBoard);
      snn.store();
    }

    return newBoard;
  }


  private StickyNote cloneSticky(StickyNoteOld oldSticky) throws PersistenceException, IOException {
    Tracer.platformLogger.debug(this.getClass().getName() + ".cloneSticky " + oldSticky.getId() + " " + oldSticky.getMessage());
    out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneSticky " + oldSticky.getId() + " " + oldSticky.getMessage() + " <br>");
    out.flush();

    // create the new Sticky
    StickyNote newSticky = new StickyNote();
    newSticky.setId(oldSticky.getId());

    Person newSender=null;
    OperatorOld oldSender = oldSticky.getSender();
    if (oldSender!=null) {
      TeamworkOperator top = getMapOperators().get(oldSender.getLoginName());
      if (top!=null)
        newSender=top.getMyPerson();
    }
    newSticky.setAuthor(newSender);

    Person newReceiver=null;
    OperatorOld oldReceiver = oldSticky.getReceiver();
    if (oldReceiver!=null) {
      TeamworkOperator tor = getMapOperators().get(oldReceiver.getLoginName());
      if (tor!=null)
        newReceiver=tor.getMyPerson();
    }
    newSticky.setReceiver(newReceiver);

    newSticky.setCreated(oldSticky.getCreated());
    newSticky.setH(removePx(oldSticky.getH()));
    newSticky.setW(removePx(oldSticky.getW()));
    newSticky.setX(removePx(oldSticky.getX()));
    newSticky.setY(removePx(oldSticky.getY()));
    newSticky.setMessage(oldSticky.getMessage());
    newSticky.setIconized(oldSticky.isIconized());
    newSticky.setRead(oldSticky.getRead());
    newSticky.setType(oldSticky.getType());
    newSticky.setColor(oldSticky.getColor());

    newSticky.store();
    return newSticky;
  }

  private int removePx(String dimWithPx){
    int ret=0;
    if (dimWithPx!=null){
      String cand=StringUtilities.replaceAllNoRegex(dimWithPx,"px","").trim();
      try {
        ret=Integer.parseInt(cand);
      }catch(NumberFormatException e){}
    }
    return ret;
  }



  private Task cloneTask(TaskOld oldTask) throws PersistenceException, IOException {
    Task newTask = null;

    if (!oldTask.isDeleted()) { // if deleted do not import
      Tracer.platformLogger.debug(this.getClass().getName() + ".cloneTask cloning " + oldTask.getId() + " " + oldTask.getName());
      out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneTask cloning " + oldTask.getId() + " " + oldTask.getName() + " <br>");
      out.flush();


      Area newArea = getArea(oldTask.getArea());
      if (newArea == null) {
        Tracer.platformLogger.error(this.getClass().getName() + ".cloneTask area null for task new " + oldTask.getId() + " " + oldTask.getName() + " set to default area");
        out.println("&nbsp;&nbsp;&nbsp;<font color='red'>" + this.getClass().getName() + ".cloneTask area null for task new " + oldTask.getId() + " " + oldTask.getName() + " set to default area" + "</FONT> <br>");
        out.flush();


        newArea = getMapAreas().get(Area.STANDARD_AREA);
      }

      newTask = new Task();
      newTask.setId(oldTask.getId());
      newTask.setName(oldTask.getName());

      newTask.setArea(newArea);

      //  private Set groups; // not imported

      //  private String code;
      newTask.setCode(oldTask.getCode());

      //   * type can be estimate, production
      //   *
      //   * @see #FEASIBILITY
      //   * @see #PRODUCTION
      //
      //  private int type;
      //  public static final int FEASIBILITY = 0;
      //  public static final int PRODUCTION = 1;
      if (oldTask.getType() == TaskOld.FEASIBILITY)
        newTask.setType(TaskType.getDefaultTaskType(TaskType.Type.FEASIBILITY));
      else
        newTask.setType(TaskType.getDefaultTaskType(TaskType.Type.PRODUCTION));

      //  private String description;
      newTask.setDescription(oldTask.getDescription());

      //   * status of the task
      //   *
      //   * @see #STATUS_ACTIVE
      //   * @see #STATUS_SUSPENDED
      //   * @see #STATUS_DONE
      //   * @see #STATUS_ABORTED

      //  public static final int STATUS_ANY = 0;
      //
      //  public static final int STATUS_ACTIVE = 100;
      //  public static final int STATUS_SUSPENDED = 101;
      //  public static final int STATUS_DONE = 102;
      //  public static final int STATUS_ABORTED = 103;
      //  public static final int STATUS_UNDEFINED = 104;
      //
      //  public static final int STATUS_TO_BE_COMPILED = 200;
      //  public static final int STATUS_TO_BE_SENT_TO_REVIEW = 201;
      //  public static final int STATUS_IN_STAKEHOLDER_REVIEW = 202;
      //  public static final int STATUS_IN_PRODUCTION = 203;
      //  public static final int STATUS_REFUSED = 204;
      //  private int status;
      int oldStatus = oldTask.getStatus();
      if (oldStatus == TaskOld.STATUS_ACTIVE || oldStatus == TaskOld.STATUS_TO_BE_COMPILED || oldStatus == TaskOld.STATUS_TO_BE_SENT_TO_REVIEW || oldStatus == TaskOld.STATUS_IN_STAKEHOLDER_REVIEW || oldStatus == TaskOld.STATUS_IN_PRODUCTION)
        newTask.changeStatusPersistAndPropagate(TaskStatus.STATUS_UNDEFINED, TaskStatus.STATUS_ACTIVE, "", false);

      else if (oldStatus == TaskOld.STATUS_DONE)
        newTask.changeStatusPersistAndPropagate(TaskStatus.STATUS_UNDEFINED, TaskStatus.STATUS_DONE, "", false);

      else if (oldStatus == TaskOld.STATUS_ABORTED || oldStatus == TaskOld.STATUS_REFUSED)
        newTask.changeStatusPersistAndPropagate(TaskStatus.STATUS_UNDEFINED, TaskStatus.STATUS_FAILED, "", false);

      else if (oldStatus == TaskOld.STATUS_SUSPENDED)
        newTask.changeStatusPersistAndPropagate(TaskStatus.STATUS_UNDEFINED, TaskStatus.STATUS_SUSPENDED, "", false);

      else if (oldStatus == TaskOld.STATUS_UNDEFINED || oldStatus == TaskOld.STATUS_ANY)
        newTask.changeStatusPersistAndPropagate(TaskStatus.STATUS_UNDEFINED, TaskStatus.STATUS_UNDEFINED, "", false);


      Date prjStartDate = null;
      if (oldTask.getPeriod() != null) {
        prjStartDate = getHumanDate(oldTask.getPeriod().getStart());
      } else if (oldTask.getEstimatedOpen() != null) {
        prjStartDate = getHumanDate(oldTask.getEstimatedOpen().getStart());
      } else if (oldTask.getMustBeOpen() != null) {
        prjStartDate = getHumanDate(oldTask.getEstimatedOpen().getStart());
      }

      Date prjEndDate = null;
      if (oldTask.getPeriod() != null) {
        prjEndDate = getHumanDate(oldTask.getPeriod().getEnd());
      } else if (oldTask.getEstimatedOpen() != null) {
        prjEndDate = getHumanDate(oldTask.getEstimatedOpen().getEnd());
      } else if (oldTask.getMustBeOpen() != null) {
        prjEndDate = getHumanDate(oldTask.getMustBeOpen().getEnd());
      }

      //  private String relevance;
      int relevance = 0;
      try {
        relevance = Integer.parseInt(oldTask.getRelevance());
      } catch (NumberFormatException nfe) {
      }
      newTask.setRelevance(relevance);

      //  private String likelyhood; // DEAD

      //  private long progress;
      newTask.setProgress(oldTask.getProgress());

      //  private SkillOld skill; // not imported

      //  private int estimatedWorklogRequired; // not imported
      //newTask.setEstimatedWorklogRequired(oldTask.getEstimatedWorklogRequired());

      //  private PeriodOld mustBeOpen;
      if (oldTask.getMustBeOpen() != null) {
        Period period = (Period) cloneSchedule(oldTask.getMustBeOpen());
        period.store();
        TaskScheduleHistory tsh = new TaskScheduleHistory();
        tsh.setSchedule(period);
        tsh.setChangeLog("Must be open");
        tsh.store();
        newTask.addScheduleHistory(tsh);
      }

      //  private PeriodOld estimatedOpen;
      if (oldTask.getEstimatedOpen() != null) {
        Period period = (Period) cloneSchedule(oldTask.getEstimatedOpen());
        period.store();
        TaskScheduleHistory tsh = new TaskScheduleHistory();
        tsh.setSchedule(period);
        tsh.setChangeLog("Estimated open");
        tsh.store();
        newTask.addScheduleHistory(tsh);
      }

      //  private ScheduleSupportOld schedule;
      if (oldTask.getSchedule() != null) {
        Period period = (Period) cloneSchedule(oldTask.getPeriod());
        if (period.getStartDate() == null)
          period.setStartDate(prjStartDate);

        if (period.getEndDate() == null)
          period.setEndDate(prjEndDate);

        period.store();
        newTask.setSchedule(period);
      }

      //  private Set milestones;
      if (oldTask.getMilestones() != null) {
        int i = 1;
        for (MilestoneOld oldMiles : (Set<MilestoneOld>) oldTask.getMilestones()) {
          Task milestone = cloneMilestone(oldMiles, newTask, i);
          newTask.addChild(milestone);
          i++;
        }
      }

      // set default values for milestone flags
      newTask.setStartIsMilestone(false);
      newTask.setEndIsMilestone(false);

      //  private double forecasted;
      newTask.setForecasted(oldTask.getForecasted());

      //  private double effective;
      newTask.setEffective(oldTask.getEffective());

      //  private double invoiced;
      //taskCost.setInvoiced(oldTask.getInvoiced());

      //  private Set costs;
      if (oldTask.getCosts() != null) {
        for (com.twproject.two.task.finantial.CostOld co : (Set<com.twproject.two.task.finantial.CostOld>) oldTask.getCosts()) {
          Cost cn = new Cost();
          cn.setDescription(co.getDescription());
          cn.setRealCost(co.getEffective());
          cn.setEstimatedCost(co.getForecasted());
          cn.store();
        }
      }

      newTask.store();

      //  private Set stakeHolders;
      if (oldTask.getStakeHolders() != null) {
        for (StakeHolderOld oldST : (Set<StakeHolderOld>) oldTask.getStakeHolders()) {
          if (oldST.getHolder() != null) {
            Assignment pmAss = createAssignementForRole(oldST.getHolder(), newTask, "Stakeholder", prjStartDate);
            if (pmAss != null)
              pmAss.setRisk(oldST.getRisk());
            pmAss.setDescription(oldST.getHoldsAs());
            newTask.addAssignmentInMemory(pmAss);
          }
        }
      }

      //  private Set assignements;
      for (Object ob : oldTask.getAssignements()) {
        AssignementOld assOld = (AssignementOld) ob;
        Assignment assNew = cloneAssignment(assOld, newTask);
        newTask.addAssignmentInMemory(assNew);
      }

      //  private ResourceOld oldManager;
      ResourceOld oldManager = oldTask.getManager();
      Assignment pmAss = createAssignementForRole(oldManager, newTask, "Project manager", prjStartDate);
      if (pmAss != null)
        newTask.addAssignmentInMemory(pmAss);

      /* //  private List documents;   Documents link are setted in clone documents
      for (Object ob : oldTask.getDocuments()) {
        DocumentOld oldDoc = (DocumentOld) ob;
        TeamworkDocument newDoc = getDocument(oldDoc);

        //checks if the document is already assigned to a resource or another task
        if (newDoc.getTask() != null) {
          Tracer.platformLogger.warn(this.getClass().getName() + ".cloneTask the assigned document id " + newDoc.getId() + " is already assigned to task id " + newDoc.getTask().getId() + ". Will not be assigne to resource id :" + newTask.getId());
          out.println("&nbsp;&nbsp;&nbsp;<font color='blue'>" + this.getClass().getName() + ".cloneResource the assigned document id " + newDoc.getId() + " is already assigned to task id " + newDoc.getTask().getId() + ". Will not be assigne to task id :" + newTask.getId() + " </font><br>");
          out.flush();
        } else if (newDoc.getResource() != null) {
          Tracer.platformLogger.warn(this.getClass().getName() + ".cloneTask the assigned document id " + newDoc.getId() + " is already assigned to resourse id " + newDoc.getResource().getId() + ". Will not be assigne to resource id :" + newTask.getId());
          out.println("&nbsp;&nbsp;&nbsp;<font color='blue'>" + this.getClass().getName() + ".cloneResource the assigned document id " + newDoc.getId() + " is already assigned to resource id " + newDoc.getResource().getId() + ". Will not be assigne to task id :" + newTask.getId() + " </font><br>");
          out.flush();
        } else {
           newDoc.setTask(newTask);
          newDoc.store();
          newTask.getDocuments().add(newDoc);
        }
      } */

      newTask.store();
    }
    return newTask;
  }


  private TeamworkDocument cloneDocument(DocumentOld oldDocument) throws PersistenceException, IOException {
    TeamworkDocument newDocument = null;

    if (!oldDocument.isDeleted() && oldDocument.getType() != DocumentOld.IS_STRUCTURAL) { // if deleted do not import // if structural not imported
      Tracer.platformLogger.debug(this.getClass().getName() + ".cloneDocument cloning " + oldDocument.getId() + " " + oldDocument.getName());
      out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneDocument cloning " + oldDocument.getId() + " " + oldDocument.getName() + " <br>");
      out.flush();


      Area newArea = getArea(oldDocument.getArea());
      if (newArea == null) {
        Tracer.platformLogger.error(this.getClass().getName() + ".cloneDocument area null for TeamworkDocument new " + oldDocument.getId() + " " + oldDocument.getName() + " set to default area");
        out.println("&nbsp;&nbsp;<font color='red'>" + this.getClass().getName() + ".cloneDocument area null for TeamworkDocument new " + oldDocument.getId() + " " + oldDocument.getName() + " set to default area" + "</FONT> <br>");
        out.flush();


        newArea = getMapAreas().get(Area.STANDARD_AREA);
      }

      newDocument = new TeamworkDocument();
      newDocument.setId(oldDocument.getId());
      newDocument.setName(oldDocument.getName());

      newDocument.setArea(newArea);

      newDocument.setAuthor(oldDocument.getAuthor());
      newDocument.setAuthored(getHumanDate(oldDocument.getAuthored()));
      // oldDocument.getBaseClassification(); // not imported
      newDocument.setCode(oldDocument.getCode());
      newDocument.setConnectionHost(oldDocument.getConnectionHost());
      newDocument.setConnectionNotes(oldDocument.getConnectionNotes());
      newDocument.setConnectionPwd(oldDocument.getConnectionPwd());
      newDocument.setConnectionUser(oldDocument.getConnectionUser());
      newDocument.setContent(oldDocument.getContent());
      if (oldDocument.getCreator() != null)
        newDocument.setCreator(oldDocument.getCreator().getDisplayName(null));
      // oldDocument.getDocuments(); muerti
      //oldDocument.getGroups(); //not imported
      //oldDocument.getInherit();  muerto
      if (oldDocument.getLastModifier() != null)
        newDocument.setLastModifier(oldDocument.getLastModifier().getDisplayName(null));

      if (oldDocument.getKeywords() != null) {
        StringBuffer sb = new StringBuffer();

        for (Object keyW : oldDocument.getKeywords())
          sb.append(keyW + ", ");
        newDocument.setKeywords(sb.toString());
      }
      newDocument.setKind(oldDocument.getKind());

      OperatorOld lockedByOld = oldDocument.getLockedBy();
      if (lockedByOld != null) {
        TeamworkOperator lockedBy = getOperator(lockedByOld);
        if (lockedBy != null && lockedBy.getMyPerson() != null)
          newDocument.setLockedBy(lockedBy.getMyPerson());
      }

      newDocument.setMimeType(oldDocument.getMimeType());
      newDocument.setName(oldDocument.getName());
      //oldDocument.getPropagate(); killed
      //oldDocument.getReadBy(); killed
      newDocument.setSizeInBytes(oldDocument.getSizeInBytes());
      newDocument.setSumma(oldDocument.getSumma());
      //oldDocument.getToBeReadBy();  desaparecido
      newDocument.setType(oldDocument.getType());
      if (oldDocument.isStorage()) {
        newDocument.setType(TeamworkDocument.IS_FILE_STORAGE);
        newDocument.setConnType(TeamworkDocument.ConnectionType.FS);
        newDocument.setContent(oldDocument.getUrl());
      }
      oldDocument.isIndex(); // not imported
      oldDocument.isTemplate(); //not imported

      if (oldDocument.getUrl() != null) {
        String ofn = FileUtilities.getFileNameWithExtension(oldDocument.getUrl().replace('\\', File.separatorChar).replace('/', File.separatorChar));
        PersistentFile pf = new PersistentFile(1, ofn, PersistentFile.TYPE_FILESTORAGE);
        pf.setFileLocation(ofn);
        newDocument.setFile(pf);
      }

      // get document dependencies
      Task task = getMapDocIdTask().get(oldDocument.getId());
      Resource resource = getMapDocIdResource().get(oldDocument.getId());
      newDocument.setTask(task);
      newDocument.setResource(resource);

      newDocument.setVersion(oldDocument.getVersion());
      newDocument.setVersionLabel(oldDocument.getVersionlabel());


      newDocument.store();
    }
    return newDocument;
  }

  private Assignment createAssignementForRole(ResourceOld oldManager, Task newTask, String roleName, Date prjStartDate) throws PersistenceException, IOException {
    Assignment pmAss = null;
    if (oldManager != null) {
      Resource resource = getResource(oldManager);
      if (resource == null) {
        Tracer.platformLogger.warn(this.getClass().getName() + ".createAssignementForRole the new resource is not found for old resource" + oldManager.getId() + " " + oldManager.getName());
        out.println("&nbsp;&nbsp;&nbsp;<font color='blue'>" + this.getClass().getName() + ".createAssignementForRole the new resource is not found for old resource" + oldManager.getId() + " " + oldManager.getName() + " </font><br>");
        out.flush();


      } else {

        if (newTask.getArea() == null) {
          Tracer.platformLogger.warn(this.getClass().getName() + ".createAssignementForRole area is null for task " + newTask.getId() + " " + newTask.getName());
          out.println("&nbsp;&nbsp;&nbsp;<font color='blue'>" + this.getClass().getName() + ".createAssignementForRole area is null for task " + newTask.getId() + " " + newTask.getName() + "</font> <br>");
          out.flush();


        } else {

          RoleTeamwork pmrole = getStandardTeamworkRoleForArea(newTask.getArea(), roleName);
          if (pmrole == null) {
            Tracer.platformLogger.warn(this.getClass().getName() + ".createAssignementForRole the new role \"" + roleName + "\" is not found for area " + newTask.getArea());
            out.println("&nbsp;&nbsp;&nbsp;<font color='blue'>" + this.getClass().getName() + ".createAssignementForRole the new role \"" + roleName + "\" is not found for area " + newTask.getArea() + "</font> <br>");
            out.flush();


          } else {

            pmAss = new Assignment();
            pmAss.setActivity(Assignment.ACTIVITY_ALL_IN_ONE);
            pmAss.setAssignmentDate(prjStartDate);
            pmAss.setEnabled(true);
            pmAss.setResource(resource);
            pmAss.setRole(pmrole);
            pmAss.setTask(newTask);
            pmAss.store();
          }
        }
      }
    }

    return pmAss;
  }

  private Task cloneMilestone(MilestoneOld oldMiles, Task newTask, int milesPosition) throws StoreException, IOException {
    Task miles = null;
    if (oldMiles != null) {
      Tracer.platformLogger.debug(this.getClass().getName() + ".cloneMilestone cloning " + oldMiles.getId() + " " + oldMiles.getDescription());
      out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneMilestone cloning " + oldMiles.getId() + " " + oldMiles.getDescription() + " <br>");
      out.flush();


      miles = new Task();
      // the new task (the milestone) id is set to the parent id + _ + milestone position. In this way no problem with counter.
      miles.setId(newTask.getId() + "_" + milesPosition);
      miles.setDescription(oldMiles.getDescription());
      miles.setRelevance(oldMiles.getCriticity());
      if (oldMiles.getWhen() != null) {
        Period period = new Period(null, getHumanDate(oldMiles.getWhen()));
        period.store();
        miles.setSchedule(period);
        miles.setEndIsMilestone(true);
        miles.setStartIsMilestone(false);
      }
      miles.store();
    }
    return miles;
  }


  private Assignment cloneAssignment(AssignementOld assOld, Task task) throws PersistenceException, IOException {
    Assignment assNew = null;
    if (assOld != null) {
      Tracer.platformLogger.debug(this.getClass().getName() + ".cloneAssignment cloning " + assOld.getId() + " " + assOld.getDescription());
      out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneAssignment cloning " + assOld.getId() + " " + assOld.getDescription() + " <br>");
      out.flush();


      assNew = new Assignment();
      //assNew.setId(assOld.getId()); // uses a new id
      assNew.setEnabled(assOld.isEnabled());
      assNew.setActivity(AssignementOld.ACTIVITY_OCCASIONAL.equals(assOld.getActivity()) ? Assignment.ACTIVITY_REPEATED_IN_TIME : Assignment.ACTIVITY_ALL_IN_ONE);
      assNew.setDescription(assOld.getDescription());
      assNew.setEstimatedWorklog((long)(assOld.getMimimalWorkLog()* CompanyCalendar.MILLIS_IN_HOUR));
      assNew.setTask(task);
      assNew.setRole(getStandardTeamworkRoleForArea(task.getArea(), "Worker"));
      assNew.setAssignmentDate(task.getSchedule() == null ? null : task.getSchedule().getStartDate());

      Resource newResource = getResource(assOld.getResource());
      if (newResource != null)
        assNew.setResource(newResource);

      assNew.setHourlyCost(assOld.getHourlyCost());

      assNew.store();

      Set<WorkLogOld> wlgs = assOld.getWorkLogs();
      if (wlgs != null) {
        for (WorkLogOld wlo : wlgs) {
          Worklog wl = cloneWorkLog(wlo, assNew);
          assNew.addWorklogInMemory(wl);
        }

      }
      assNew.store();
    }
    return assNew;
  }


  private Event cloneEvent(EventOld eventOld) throws PersistenceException, IOException {
    Event eventNew = null;
    if (eventOld != null && !eventOld.isDeleted()) {
      Tracer.platformLogger.debug(this.getClass().getName() + ".cloneEvent cloning " + eventOld.getId() + " " + eventOld.getDescription());
      out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneEvent cloning " + eventOld.getId() + " " + eventOld.getDescription() + " <br>");
      out.flush();


      if (eventOld.getOwner() != null) {
        TeamworkOperator author = null;

        if (eventOld.getAuthor() != null) {
          author = getOperator(eventOld.getAuthor());
        }

        TeamworkOperator operator = getOperator(eventOld.getOwner());
        if (operator != null) {
          Resource target = operator.getMyPersonFromPersistence();
          if (target != null) { // if no target event is ignored

            eventNew = new Event();
            eventNew.setId(eventOld.getId()); // uses the same id

            if (author != null && author.getMyPersonFromPersistence() != null) { // if no author event is ignored
              Person myPerson = author.getMyPerson();
              eventNew.setAuthor(myPerson);
            }

            eventNew.getTargets().add(target);
            //eventOld.getAssignement(); // assignment are ignored
            // eventOld.getContent(); // never used
            eventNew.setDescription(eventOld.getDescription());
            //eventOld.getDocuments(); not imported
            //eventOld.getGroups(); // never used
            eventNew.setLocation(eventOld.getLocation());
            ScheduleSupport schedule = cloneSchedule(eventOld.getSchedule());
            eventNew.setSchedule(schedule);
            eventNew.setStatus(eventOld.getStatus());
            // eventOld.getType(); // never used
            eventNew.setPersonal(eventOld.isPersonal());

            eventNew.store();
          } else {
            Tracer.platformLogger.error(this.getClass().getName() + ".cloneEvent event " + eventOld.getId() + ". Target not found: event not imported");
            out.println("&nbsp;&nbsp;<font color='red'>" + this.getClass().getName() + ".cloneEvent event " + eventOld.getId() + ". Target not found: event not imported" + "</FONT> <br>");
            out.flush();


          }
        } else {
          Tracer.platformLogger.debug(this.getClass().getName() + ".cloneEvent event " + eventOld.getId() + ". Owner not found: event not imported");
          out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneEvent event " + eventOld.getId() + ". Owner not found: event not imported" + " <br>");
          out.flush();


        }
      } else {
        Tracer.platformLogger.error(this.getClass().getName() + ".cloneEvent event " + eventOld.getId() + ". Owner is null: event not imported");
        out.println("&nbsp;&nbsp;<font color='red'>" + this.getClass().getName() + ".cloneEvent event " + eventOld.getId() + ". Owner is null: event not imported" + "</FONT> <br>");
        out.flush();

      }
    }

    return eventNew;
  }

  private ScheduleSupport cloneSchedule(ScheduleSupportOld scheSupOld) throws StoreException {
    // in order to avoid strange date, only date between 1/1/1970 and 31/12/2100 will be preserved

    ScheduleSupport ret = null;

    if (scheSupOld != null) {
      Date start = null;
      Date end = null;
      if (scheSupOld.getStart() != null)
        start = getHumanDate(scheSupOld.getStart());
      if (scheSupOld.getEnd() != null)
        end = getHumanDate(scheSupOld.getEnd());

      Object underlyingObjectTW2 = getUnderlyingObjectTW2(scheSupOld);
      if (underlyingObjectTW2 instanceof PeriodOld) {
        ret = new Period(start, end);

      } else if (underlyingObjectTW2 instanceof ScheduleDailyOld) {
        ScheduleDailyOld scheduleOld = (ScheduleDailyOld) underlyingObjectTW2;
        ScheduleDaily scheduleDaily = null;
        scheduleDaily = new ScheduleDaily();
        scheduleDaily.setStartDate(start);
        scheduleDaily.setDurationInMillis(scheduleOld.getEndMillis() - scheduleOld.getStartMillis());
        scheduleDaily.setStartTimeInMillis(scheduleOld.getStartMillis());
        scheduleDaily.setRepetitions(scheduleOld.getRepetitions());
        scheduleDaily.setFrequency(scheduleOld.getFrequency() <= 0 ? 1 : scheduleOld.getFrequency());
        scheduleDaily.recalculateFields();
        ret = scheduleDaily;

      } else if (underlyingObjectTW2 instanceof ScheduleWeeklyOld) {
        ScheduleWeeklyOld scheduleOld = (ScheduleWeeklyOld) underlyingObjectTW2;
        ScheduleWeekly scheduleWeekly = null;
        scheduleWeekly = new ScheduleWeekly();
        scheduleWeekly.setStartDate(start);
        scheduleWeekly.setDurationInMillis(scheduleOld.getEndMillis() - scheduleOld.getStartMillis());
        scheduleWeekly.setStartTimeInMillis(scheduleOld.getStartMillis());
        scheduleWeekly.setRepetitions(scheduleOld.getRepetitions());
        scheduleWeekly.setFrequency(scheduleOld.getFrequency() <= 0 ? 1 : scheduleOld.getFrequency());
        scheduleWeekly.setDays(scheduleOld.getDays());
        scheduleWeekly.recalculateFields();
        ret = scheduleWeekly;
      }
      ret.store();
    }
    return ret;
  }

  private Worklog cloneWorkLog(WorkLogOld wlo, Assignment ass) throws StoreException {
    Worklog wl = null;
    if (wlo != null) {
      wl = new Worklog();
      wl.setAction(wlo.getAction());
      wl.setDuration(wlo.getDuration());
      wl.setInserted(getHumanDate(wlo.getInserted()));
      wl.setAssig(ass);
      wl.store();
    }
    return wl;
  }

  private RoleTeamwork getStandardTeamworkRoleForArea(Area area, String standard) throws PersistenceException {
    RoleTeamwork rt = null;
    if (area != null) {
      rt = getMapAreaRoles().get(area.getId() + "_" + standard);

      if (rt == null) {
        //create the standard role for this area
        rt = new RoleTeamwork();
        rt.setName(standard.toString());
        rt.setArea(area);
        rt.setLocalToAssignment(true);
        rt.store();
        getMapAreaRoles().put(area.getId() + "_" + standard, rt);
      }
    }
    return rt;
  }


  private Issue cloneIssue(IssueOld oldIssue) throws PersistenceException, IOException {
    Issue newIssue = null;

    if (oldIssue != null) {
      Tracer.platformLogger.debug(this.getClass().getName() + ".cloneIssue cloning " + oldIssue.getId() + " " + oldIssue.getDescription());
      out.println("&nbsp;&nbsp;" + this.getClass().getName() + ".cloneIssue cloning " + oldIssue.getId() + " " + oldIssue.getDescription() + " <br>");
      out.flush();


      newIssue = new Issue();
      newIssue.setId(oldIssue.getId());

      Area newArea = getArea(oldIssue.getArea());
      if (newArea == null) {
        Tracer.platformLogger.error(this.getClass().getName() + ".cloneIssue area null for issue new " + oldIssue.getId() + " " + oldIssue.getDescription() + " set to default area");
        out.println("&nbsp;&nbsp;<font color='red'>" + this.getClass().getName() + ".cloneIssue area null for issue new " + oldIssue.getId() + " " + oldIssue.getDescription() + " set to default area" + "</FONT> <br>");
        out.flush();


        newArea = getMapAreas().get(Area.STANDARD_AREA);
      }
      //newIssue.setArea(newArea);
      if (oldIssue.getAssignedTo() != null) {
        Resource res = getResource(oldIssue.getAssignedTo());
        if (res != null)
          newIssue.setAssignedTo(res);
      }

      if (oldIssue.getCreator() != null)
        newIssue.setCreator(oldIssue.getCreator().getDisplayName(null));

      newIssue.setDescription(oldIssue.getDescription());
      //todo oldIssue.getDocuments();
      newIssue.setGravity(oldIssue.getGravity());

      Date close = null;
      if (oldIssue.getShouldCloseBy() != null)
        close = getHumanDate(oldIssue.getShouldCloseBy());
      newIssue.setShouldCloseBy(close);

      newIssue.setStatus(oldIssue.getStatus());
      if (oldIssue.getTask() != null) {
        Task task = getTask(oldIssue.getTask());
        if (task != null)
          newIssue.setTask(task);
      }

      if (oldIssue.getType() != null) {
        IssueType it = getIssueType(oldIssue.getType());
        if (it != null)
          newIssue.setType(it);
      }
      newIssue.setVotes(oldIssue.getVotes());
      if (oldIssue.getWatchers() != null) {
        for (Object obj : oldIssue.getWatchers()) {
          OperatorOld op = (OperatorOld) obj;
          // todo register for event
        }
      }

      newIssue.store();

    }


    return newIssue;

  }

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------   WRAPPERS  -------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------------

  public Map<String, Permission> getMapPerimssion() {
    if (mapPerimssion == null)
      try {
        inizializeMapPerimssions();
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapPerimssion;
  }

  public Map<String, String> getMapCounters() {
    if (mapCounters == null)
      try {
        initializeCounterMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapCounters;
  }

  public Map<Serializable, Area> getMapAreas() {
    if (mapAreas == null)
      try {
        initializeAreaMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapAreas;
  }

  public Map<Serializable, RoleTeamwork> getMapRoles() {
    if (mapRoles == null)
      try {
        initializeRoleMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapRoles;
  }

  public Map<Serializable, Board> getMapBoards() {
    if (mapBoards == null)
      try {
        initializeBoardMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapBoards;
  }

  public Map<Serializable, StickyNote> getMapStickys() {
    if (mapStickys == null)
      try {
        initializeStickyMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapStickys;
  }


  public Map<Serializable, Task> getMapDocIdTask() {
    if (mapDocIdTask == null) {
      getMapTasks(); // THE MAP IS BUILD WITHIS THE TASK'S ONE
    }
    return mapDocIdTask;
  }

  public Map<Serializable, Resource> getMapDocIdResource() {
    if (mapDocIdResource == null) {
      getMapResources(); // THE MAP IS BUILD WITHIS THE RESOURCE'S ONE
    }
    return mapDocIdResource;
  }


  public Map<Serializable, TeamworkOperator> getMapOperators() {
    if (mapOperators == null)
      try {
        initializeOperatorsMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapOperators;
  }

  public Map<Serializable, Resource> getMapResources() {
    if (mapResources == null)
      try {
        initializeResourcesMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapResources;
  }

  public Map<Serializable, Task> getMapTasks() {
    if (mapTasks == null)
      try {
        initializeTasksMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapTasks;
  }

  public Map<Serializable, TeamworkDocument> getMapDocuments() {
    if (mapDocuments == null)
      try {
        initializeDocumentsMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapDocuments;
  }

  public Map<Serializable, Event> getMapEvents() {
    if (mapEvents == null)
      try {
        initializeEventsMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapEvents;
  }

  public Map<String, RoleTeamwork> getMapAreaRoles() {
    // initializes the map
    if (mapAreaRoles == null) {
      mapAreaRoles = new HashMap<String, RoleTeamwork>();
      for (RoleTeamwork tr : getMapRoles().values()) {
        if (tr.getArea() != null)
          getMapAreaRoles().put(tr.getArea().getId() + "_" + tr.getName(), tr);
      }
    }

    return mapAreaRoles;
  }

  public Map<Serializable, Issue> getMapIssues() {
    // initializes the map
    if (mapIssues == null)
      try {
        initializeIssuesMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapIssues;
  }

  public Map<String, IssueType> getMapIssueTypes() {
    // initializes the map
    if (mapIssueTypes == null)
      try {
        initializeIssueTypesMap();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      } catch (IOException e) {
        throw new PlatformRuntimeException(e);
      }
    return mapIssueTypes;
  }


  public static Object getUnderlyingObjectTW2(Object persistentObject) throws ClassCastException {
    if (persistentObject instanceof HibernateProxy) {
      HibernateProxy proxy = (HibernateProxy) persistentObject;
      AbstractLazyInitializer ali = (AbstractLazyInitializer) proxy.getHibernateLazyInitializer();
      return ali.getImplementation();
    } else
      return persistentObject;
  }


  public PersistenceContext getPersistenceContextOnOldDB() {
    PersistenceContext pc = null;
    try {
      if (conn == null || tw2Session == null || !tw2Session.isOpen()) {
        Class.forName(TW2_drv);
        conn = DriverManager.getConnection(TW2_dbUrl, TW2_user, TW2_pwd);
        tw2Session = HibernateFactory.getSessionFactory().openSession(conn);
      }
      pc = new PersistenceContext();
      pc.session = tw2Session;
      pc.tx = pc.session.beginTransaction();
    } catch (Exception e) {

    }
    return pc;
  }


  private Date getHumanDate(Date date) {
    // Only dates between January 1, 1800 and January 1, 2100  are accepted.
    Date ret = null;
    if (date != null) {
      if (date.after(new Date(-100, 0, 1)) && date.before(new Date(200, 0, 1)))
        ret = date;
    }
    return ret;
  }
}



