package com.twproject.messaging.stickyNote;

import com.twproject.operator.TeamworkOperator;
import org.jblooming.messaging.Message;
import org.jblooming.messaging.MessagingSystem;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.scheduler.ExecutableSupport;
import org.jblooming.scheduler.JobLogData;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.DateUtilities;

import java.util.List;

public class StickyMessageDispatcher extends ExecutableSupport {

  public JobLogData run(JobLogData jobLogData) throws Exception {
    boolean exception = true;
    PersistenceContext pc = null;

    try {

      pc = HibernateFactory.newFreeSession();

      String hql = "from " + Message.class.getName() + " as mess where mess.media = :media";
      OqlQuery query = new OqlQuery(hql, pc);
      query.getQuery().setString("media", MessagingSystem.Media.STICKY.toString());
      List<Message> messages = query.list();

      for (Message message : messages) {
        if (message.getToOperator() != null) {
          String body = message.getMessageBody();
          if (message.getLink() != null)
            body = body + "<hr>link:&nbsp;" + message.getLink();
          StickyNote sn = new StickyNote();
          sn.setIdAsNew();

          if (message.getFromOperator()!=null) {
            TeamworkOperator frop = (TeamworkOperator) PersistenceHome.findByPrimaryKey(TeamworkOperator.class,message.getFromOperator().getId(),pc);
            sn.setAuthor(frop.getMyPerson());
          }

          TeamworkOperator top = (TeamworkOperator) PersistenceHome.findByPrimaryKey(TeamworkOperator.class,message.getToOperator().getId(),pc);
          sn.setReceiver(top.getMyPerson());

          sn.setType(message.getSubject());
          sn.setMessage(body);
          sn.setH(250);
          sn.store(pc);
        }
        PersistenceHome.remove(message, pc);
      }
      pc.commitAndClose();

      jobLogData.notes = jobLogData.notes + "StickyMessageDispatcher executed on " + DateUtilities.dateToTimestampString(DateUtilities.getAbsoluteNow());
      exception = false;
    } catch (Exception e) {
      Tracer.platformLogger.error("StickyMessageDispatcher error",e);
    } finally {
      if (exception) {
        if (pc != null) {
          pc.rollback();
        }
        jobLogData.successfull = false;
      }
    }

    return jobLogData;
  }
}
