package com.twproject.operator;

import com.opnlb.website.operator.ForumOperator;
import com.twproject.resource.Person;
import com.twproject.security.RoleTeamwork;
import com.twproject.security.TeamworkArea;
import com.twproject.task.Issue;
import org.hibernate.HibernateException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.ontology.PersistentFile;
import org.jblooming.operator.Operator;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.FindException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.security.Area;
import org.jblooming.security.Permission;
import org.jblooming.security.PlatformPermissions;
import org.jblooming.security.Role;
import org.jblooming.tracer.Tracer;

import java.util.*;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class TeamworkOperator extends Operator implements ForumOperator {

  private Person myPerson;

  private Set<Issue> issuesOwned;


  public void operatorInitializer() throws HibernateException {
    super.operatorInitializer();
    if (myPerson != null) {
      HibernateFactory.initialize(myPerson);
      HibernateFactory.initialize(myPerson.getVotes());
      if (myPerson.getArea() != null) {
        try {
          TeamworkArea area = (TeamworkArea) PersistenceHome.findByPrimaryKey(TeamworkArea.class, myPerson.getArea().getId());
          area.setLastLoginOnArea(new Date());
          area.store();
        } catch (Exception e) {
          Tracer.platformLogger.error(e);
        }
      }
    }
  }

  public Person getMyPerson() {
    return myPerson;
  }

  public Person getMyPersonFromPersistence() throws PersistenceException {
    String hql = "from " + Person.class.getName() + " as p where p.myself.id=:twopid";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setInteger("twopid", Integer.parseInt(getId() + ""));
    return (Person) oql.uniqueResult();
  }

  // is like inverse
  private void setMyPerson(Person myPerson) {
    this.myPerson = myPerson;
  }


  public Set<Area> getAreasForPermission(Permission aPermission) throws PersistenceException {

    Set<Area> areas = new TreeSet<Area>(new Comparator<Area>() {
      public int compare(Area a1, Area a2) {
        if (a1 != null && a2 != null)
          return a1.getName().compareTo(a2.getName());
        else
          return 0;
      }
    });
    for (Role r : getInheritedRoles()) {
      if (aPermission == null || r.getPermissions().contains(aPermission)) {
        RoleTeamwork tr = (RoleTeamwork) PersistenceHome.findByPrimaryKey(RoleTeamwork.class, r.getId());
        if (tr.getArea() != null)
          areas.add(tr.getArea());
      }
    }

    areas.addAll(getAreasOwned());

    return areas;

  }

  public boolean isAreaManager() {
    try {
      return getAreasOwned().size() > 0 || getAreasForPermission(PlatformPermissions.area_canManage).size() > 0;
    } catch (PersistenceException e) {
      throw new PlatformRuntimeException(e);
    }
  }

  public Set<Area> getAreasOwned() throws FindException {
    //add all those which you own
    String hql = "from " + TeamworkArea.class.getName() + " as area ";

    if (!hasPermissionAsAdmin())
      hql = hql + "where area.owner = :myself";

    OqlQuery oql = new OqlQuery(hql);
    if (!hasPermissionAsAdmin())
      oql.getQuery().setEntity("myself", this);
    return new HashSet<Area>(oql.list());
  }

  public Area getDefaultAreaForPermission(Permission aPermission) throws PersistenceException {
    Set<Area> areas = getAreasForPermission(aPermission);
    if (areas.size() == 0)
      return null;
    if (areas.size() == 1)
      return areas.iterator().next();
    else {
      Person myPersonFromPersistence = getMyPersonFromPersistence();
      if (myPersonFromPersistence != null && myPersonFromPersistence.getArea() != null && areas.contains(myPersonFromPersistence.getArea()))
        return myPersonFromPersistence.getArea();
      else {
        //find one which I own
        for (Area a : areas) {
          if (this.equals(a.getOwner()))
            return a;
        }
        // oth. return randomly
        return areas.iterator().next();
      }
    }
  }


  private Set<Issue> getIssuesOwned() {
    return issuesOwned;
  }

  private void setIssuesOwned(Set<Issue> issuesOwned) {
    this.issuesOwned = issuesOwned;
  }

  public String getDefaultEmail() {
    return getMyPerson().getDefaultEmail();
  }

  /**
   * @deprecated use hasPermissionAsAdmin
   */
  public boolean isAdministrator() {
    return super.isAdministrator();
  }


  public PersistentFile getImage() {
    return getMyPerson().getMyPhoto();
  }
}
