package com.twproject.resource;

import com.twproject.operator.TeamworkOperator;
import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.company.DepartmentType;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.settings.ApplicationState;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class Company extends Resource {

  /**
   * determines the department type, e.g. Company, Department, Office
   */
  private DepartmentType type;


  public DepartmentType getType() {                    
    return type;
  }

  public void setType(DepartmentType type) {
    this.type = type;
  }

  public Set<Person> getPersons() {

    return getPersons(new HashSet<PeopleAggregator>(), new HashSet<Person>());
  }

  /**
   * Method getParentUsers
   *
   * @param visitedNodes a  Collection
   * @param workers      a  Collection
   * @return a Collection of operators
   */
  public Set<Person> getPersons(Set<PeopleAggregator> visitedNodes, Set<Person> workers) {

    if (!visitedNodes.contains(this)) {

      visitedNodes.add(this);

      // visiting parent node

      if (isInherit() && getParent() != null) {
        workers.addAll(((Company) getParent()).getPersons(visitedNodes, workers));
      }

      // visiting children

      if (isPropagate()) {
        Iterator iter = getChildrenIterator();
        Resource t;
        while (iter.hasNext()) {
          t = (Resource) iter.next();
          if (t instanceof Company)
            workers.addAll(((Company) t).getPersons(visitedNodes, workers));
          else if (t instanceof Person)
            workers.addAll(((Person) t).getPersons());
        }
      }
    }
    return workers;
  }

  public boolean isPersonIn(Person o) {
    return getPersons().contains(o);
  }

  
  /*public Iterator getEventsIterator() {
    Iterator i = null;
    if (workerEvents != null) {
      Set events = new HashSet();
      for (Iterator iterator = workerEvents.iterator(); iterator.hasNext();) {
        events.add(((WorkerEvent) iterator.next()).getEvent());
      }
      i = events.iterator();
    }
    return i;
  }*/
  public String getDisplayName() {
    return getName();
  }

  public long getWorkingHourTotal() {

    String applicationSetting = ApplicationState.getApplicationSetting(OperatorConstants.FLD_WORKING_HOUR_TOTAL);
    if (applicationSetting != null && applicationSetting.trim().length() > 0)
      return DateUtilities.millisFromHourMinuteSmart(applicationSetting);
    else
      return 8 * CompanyCalendar.MILLIS_IN_HOUR;
  }

  public TeamworkOperator getMyself() {
    return null;
  }
  

}
