package com.twproject.resource;

import com.twproject.messaging.stickyNote.StickyNote;
import com.twproject.operator.TeamworkOperator;
import com.twproject.security.TeamworkPermissions;
import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.ontology.PerformantNode;
import org.jblooming.operator.Operator;
import org.jblooming.operator.User;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.security.Permission;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.JSP;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.settings.ApplicationState;

import java.io.Serializable;
import java.util.*;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class Person extends Resource {

  private String personName;
  private String personSurname;
  private String courtesyTitle;
  private Date hiringDate;
  private String personalInterest;
  private TeamworkOperator myself;
  private String blackBoardNotes;

  /**
   *
   */
  private Map<String, Integer> votes = new Hashtable();

  private List news = new ArrayList();
  // read only colls
  private Set workerEvents;
  private Set stickyAuthored = new HashSet();
  private Set stickyReceived = new HashSet();
  private Set eventOwned = new HashSet();
  private Set eventAuthored = new HashSet();
  public static final int VOTES = 100;


  public String getPersonName() {
    return personName;
  }

  public void setPersonName(String personName) {
    this.personName = personName;
    setName(getPersonName() + " " + getPersonSurname());
  }

  public String getPersonSurname() {
    return personSurname;
  }

  public void setPersonSurname(String personSurname) {
    this.personSurname = personSurname;
    setName(getPersonName() + " " + getPersonSurname());
  }

  public String getCourtesyTitle() {
    return courtesyTitle;
  }

  public void setCourtesyTitle(String courtesyTitle) {
    this.courtesyTitle = courtesyTitle;
  }

  public Date getHiringDate() {
    return hiringDate;
  }

  public void setHiringDate(Date hiringDate) {
    this.hiringDate = hiringDate;
  }

  public String getPersonalInterest() {
    return personalInterest;
  }

  public void setPersonalInterest(String personalInterest) {
    this.personalInterest = personalInterest;
  }

  public String getName() {
    if (getPersonName() != null || getPersonSurname() != null)
      return JSP.w(getPersonName()) + " " + JSP.w(getPersonSurname());
    else
      return "";
  }

  public Set<Person> getPersons() {
    Set i = new HashSet();
    if (!isHidden())
      i.add(this);
    return i;
  }

  public boolean isPersonIn(Person o) {
    return this.equals(o);
  }

  private Set getStickyAuthored() {
    return stickyAuthored;
  }

  private void setStickyAuthored(Set stickyAuthored) {
    this.stickyAuthored = stickyAuthored;
  }

  public Iterator getStickyAuthoredIterator() {
    return stickyAuthored.iterator();
  }

  public boolean stickyAuthoredContains(StickyNote authored) {
    return stickyAuthored.contains(authored);
  }

  public int stickyAuthoredSize() {
    return stickyAuthored.size();
  }

  private Set getStickyReceived() {
    return stickyReceived;
  }

  private void setStickyReceived(Set stickyReceived) {
    this.stickyReceived = stickyReceived;
  }

  public Iterator getStickyReceivedIterator() {
    return stickyReceived.iterator();
  }

  public boolean stickyReceivedContains(StickyNote received) {
    return stickyReceived.contains(received);
  }

  public int stickyReceivedSize() {
    return stickyReceived.size();
  }

  private Set getEventOwned() {
    return eventOwned;
  }

  private void setEventOwned(Set eventOwned) {
    this.eventOwned = eventOwned;
  }

  public Iterator getEventOwnersIterator() {
    return eventOwned.iterator();
  }

  public boolean eventOwnersContains(Resource eventOwner) {
    return eventOwned.contains(eventOwner);
  }

  public int eventOwnersSize() {
    return eventOwned.size();
  }

  private Set getEventAuthored() {
    return eventAuthored;
  }

  private void setEventAuthored(Set eventAuthored) {
    this.eventAuthored = eventAuthored;
  }

  public Iterator getEventAuthorsIterator() {
    return eventAuthored.iterator();
  }

  public boolean eventAuthorsContains(Resource eventAuthor) {
    return eventAuthored.contains(eventAuthor);
  }

  public int eventAuthorsSize() {
    return eventAuthored.size();
  }

  public Map<String, Integer> getVotes() {
    return votes;
  }

  private void setVotes(Map<String, Integer> votes) {
    this.votes = votes;
  }

  public Iterator getVotesKeysetIterator() {
    return votes.keySet().iterator();
  }

  public int getVotesForIssue(String issueId) {
    Integer vote = votes.get(issueId);
    if (vote == null)
      return 0;
    else
      return vote;
  }

  public int getAvailableVotes() {
    int av = VOTES;
    Set keys = votes.keySet();
    if (keys.size() == 0)
      return VOTES;
    else {
      Iterator i = keys.iterator();
      while (i.hasNext()) {
        av -= votes.get(i.next());
      }
      return av;
    }
  }

  public void setVotesForIssue(Serializable issueId, int vote) {
    votes.put(issueId + "", vote);
  }

  private List getNews() {
    return news;
  }

  private void setNews(List news) {
    this.news = news;
  }

  public void addNews(String notice) {
    news.add(notice);
  }

  public void removeNews(String notice) {
    news.remove(notice);
  }

  public Iterator getNewsIterator() {
    return news.iterator();
  }

  private Set getWorkerEvents() {
    return workerEvents;
  }

  private void setWorkerEvents(Set workerEvents) {
    this.workerEvents = workerEvents;
  }

  public int eventsSize() {
    return workerEvents.size();
  }

  public TeamworkOperator getMyself() {
    return myself;
  }

  public void setMyself(TeamworkOperator myself) {
    this.myself = myself;
  }

  public String getBlackBoardNotes() {
    return blackBoardNotes;
  }

  public void setBlackBoardNotes(String blackBoardNotes) {
    this.blackBoardNotes = blackBoardNotes;
  }

  /*public Iterator getEventsIterator() {
    Iterator i = null;
    if (workerEvents != null) {
      Set events = new HashSet();
      for (Iterator iterator = workerEvents.iterator(); iterator.hasNext();) {
        events.add(((WorkerEvent) iterator.next()).getEvent());
      }
      i = events.iterator();
    }
    return i;
  }*/

  public String getDisplayName() {
    return (getCourtesyTitle() != null ? JSP.w(getCourtesyTitle()) + " " : "") + JSP.w(getPersonName()) + " " + JSP.w(getPersonSurname());
  }


  public List<Resource> getAllMyStaff() throws PersistenceException {

    List<Resource> totalStaff = new ArrayList<Resource>();

    //myself
    totalStaff.add(this);

    //those from chickens
    String hql = "select p from " + Resource.class.getName() + " as p where p.myManagerIds is not null and p.myManagerIds like :myids_stemmed";
    OqlQuery oql = new OqlQuery(hql);
    String param = (this.getMyManagerIds() != null ? this.getMyManagerIds() : "") + this.getId() + PerformantNode.SEPARATOR + "%";
    oql.getQuery().setString("myids_stemmed", param);
    List<Resource> staff = oql.list();
    totalStaff.addAll(staff);

    //those descending from chickens
    for (Resource resource : staff) {
      List descs = resource.getDescendants(Resource.class);
      totalStaff.addAll(descs);
    }

    //those from areas
    /*Set<Area> areas = this.myself.getAreasForPermission(TeamworkPermissions.resource_canAssign);
    if (areas.size() > 0) {
      hql = "select p from " + Resource.class.getName() + " as p where p.area in (:areas)";
      oql = new OqlQuery(hql);
      oql.getQuery().setParameterList("areas", areas);
      totalStaff.addAll(oql.list());
    }*/
    return totalStaff;

  }

  public static Person getLoggedPerson(SessionState sessionState) throws PersistenceException {
    return getPersonFromOperator(sessionState.getLoggedOperator());
  }

  public static Person getPersonFromOperator(Operator logged) throws PersistenceException {
    // get loggedResource
    TeamworkOperator twop = (TeamworkOperator) PersistenceHome.findByPrimaryKey(TeamworkOperator.class, logged.getId());
    Person resource = twop.getMyPerson();
    return resource;
  }

  public static Person getPersonFromOperatorId(String opId) throws PersistenceException {

    String hql = "select person from " + Person.class.getName() + " as person where person.myself.id=:opid";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setString("opid", opId);
    return (Person) oql.uniqueResult();
  }

  public boolean hasPermissionFor(User u, Permission p) {
    if (u.equals(getMyself()) && p.equals(TeamworkPermissions.resource_canRead))
      return true;
    else
      return super.hasPermissionFor(u, p);
  }

  public long getWorkingHourTotal() {

    if (getMyself() != null) {
      String option = getMyself().getOption(OperatorConstants.FLD_WORKING_HOUR_TOTAL);
      if (option != null && option.trim().length() > 0)
        return DateUtilities.millisFromHourMinuteSmart(option);
    }

    String applicationSetting = ApplicationState.getApplicationSetting(OperatorConstants.FLD_WORKING_HOUR_TOTAL);
    if (applicationSetting != null && applicationSetting.trim().length() > 0)
      return DateUtilities.millisFromHourMinuteSmart(applicationSetting);
    else
      return 8 * CompanyCalendar.MILLIS_IN_HOUR;

  }

}
