package com.twproject.setup;

import com.opnlb.website.forum.ForumEntry;
import com.opnlb.website.portlet.Portlet;
import com.twproject.resource.Resource;
import com.twproject.security.TeamworkPermissions;
import com.twproject.task.Issue;
import com.twproject.task.Task;
import org.hibernate.SessionFactory;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.agenda.Period;
import org.jblooming.messaging.Listener;
import org.jblooming.ontology.PerformantNodeSupport;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.HibernatePropertySchemaUpdater;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.scheduler.ExecutableSupport;
import org.jblooming.scheduler.Job;
import org.jblooming.scheduler.JobLogData;
import org.jblooming.security.Role;
import org.jblooming.tracer.Tracer;
import org.jblooming.transaction.hibernate.HibernateTransaction;
import org.jblooming.utilities.StringUtilities;

import java.sql.*;
import java.util.Date;
import java.util.*;

/**
 * (c) Open Lab - www.open-lab.com
 * Date: May 11, 2007
 * Time: 4:26:26 PM
 */
public class SetupSupport {

  public static void tw307and310Update(HibernatePropertySchemaUpdater hpsu) {
    try {

      //3.0.7
      hpsu.addPropertyToCheck(Issue.class, "orderFactor", new Integer(0));
      hpsu.addPropertyToCheck(Job.class, "lastExecutionTime", new Integer(0));
      hpsu.addPropertyToCheck(Portlet.class, "params", "");

      //3.1.0
      HibernatePropertySchemaUpdater.PropertyWithDefault updaterTo310 = hpsu.addPropertyToCheck(Task.class, "duration", 0);
      updaterTo310.preSql.add("alter table twk_task drop column autoSynchPeriods");
      updaterTo310.preSql.add("drop table olpl_joblog");

      //reset all parents due to _ bug
      updaterTo310.execs.add(new ExecutableSupport() {

        public JobLogData run(JobLogData jobLog) throws Exception {

          SessionFactory sf = HibernateFactory.getSessionFactory();
          Map acm = sf.getAllClassMetadata();
          Set keysAcm = acm.keySet();
          for (Iterator iterator = keysAcm.iterator(); iterator.hasNext();) {
            String className = (String) iterator.next();
            Class cln = null;
            try {
              cln = Class.forName(className);
              try {
                List<PerformantNodeSupport> nl = new OqlQuery("from " + cln.getName() + " as pfn where pfn.parent is not null").list();
                for (PerformantNodeSupport n : nl) {
                  n.setAncestorIds(StringUtilities.replaceAllNoRegex(n.getAncestorIds(), "_", "^"));
                  n.store();
                }
                HibernateTransaction.getCurrentTransaction().checkPoint();
              } catch (Exception f) {
                jobLog.notes = f.getMessage();
              }
            } catch (ClassNotFoundException e) {
              jobLog.notes = e.getMessage();
            }
          }
          return jobLog;
        }
      }

      );

      updaterTo310.execs.add(new ExecutableSupport() {

        public JobLogData run(JobLogData jobLog) throws Exception {
          try {
            List<Resource> nl = new OqlQuery("from " + Resource.class.getName() + " as res where res.myManager is not null").list();
            for (Resource n : nl) {
              n.setMyManagerIds(StringUtilities.replaceAllNoRegex(n.getMyManagerIds(), "_", "^"));
              n.store();
            }
            HibernateTransaction.getCurrentTransaction().checkPoint();
          } catch (Exception e) {
            jobLog.notes = e.getMessage();
          }
          return jobLog;
        }
      }

      );

      //add duration
      updaterTo310.execs.add(new ExecutableSupport() {

        public JobLogData run(JobLogData jobLog) throws Exception {

          String hql = "select task from " + Task.class.getName() + " as task where task.schedule is not null";
          OqlQuery oql = new OqlQuery(hql);
          List<Task> all = oql.list();
          for (Task task : all) {
            int durInDays = CompanyCalendar.getWorkingDaysCountInPeriod(task.getSchedule());
            if (durInDays < Integer.MAX_VALUE)
              task.setDuration(durInDays);
            task.store();
          }
          HibernateTransaction.getCurrentTransaction().checkPoint();
          return jobLog;
        }
      }

      );

      //remove SchedulerLogKiller
      updaterTo310.execs.add(new ExecutableSupport() {

        public JobLogData run(JobLogData jobLog) throws Exception {
          String hql = "select job from " + Job.class.getName() + " as job where job.name = :namex";
          OqlQuery oql = new OqlQuery(hql);
          oql.getQuery().setString("namex", "SchedulerLogKiller");
          List<Job> all = oql.list();
          for (Job job : all) {
            job.remove();
          }
          HibernateTransaction.getCurrentTransaction().checkPoint();
          return jobLog;
        }
      }

      );
      //end 3.1.0
    } catch (Exception e) {
      Tracer.platformLogger.fatal(e);
      throw new PlatformRuntimeException(e);
    }
  }


  public static void tw320UpdateForum() throws Exception {

    PersistenceContext pc = HibernateFactory.newFreeSession();
    List<ForumEntry> all = new OqlQuery("from " + ForumEntry.class.getName() + " as fe order by fe.id", pc).list();

    Connection conn = pc.session.connection();
    Statement statement = conn.createStatement();
    ResultSet rs = statement.executeQuery("select id, content from olpl_ws_forum order by id");
    for (ForumEntry fe : all) {
      rs.next();
      int id = rs.getInt("id");
      if (!fe.getId().equals(id))
        throw new PlatformRuntimeException("There are some evil presences here.");
      fe.setContent(rs.getString("content"));
      fe.store(pc);
    }
    rs.close();
    statement.close();

    pc.commitAndClose();
  }

  public static void tw320UpdateSchedules() throws Exception {

    PersistenceContext pc = HibernateFactory.newFreeSession();

    Date now = new Date();
    Date endOfTimes = CompanyCalendar.MAX_DATE;
    List<Period> all = new OqlQuery("from " + Period.class.getName() + " as sc where sc.start is null or sc.end is null", pc).list();
    for (Period scs : all) {
      if (scs.getStartDate() == null && scs.getEndDate() == null) {
        scs.setStartDate(now);
        scs.setEndDate(endOfTimes);        
      } else if (scs.getStartDate() != null && scs.getEndDate() == null) {
        scs.setEndDate(endOfTimes);
      } else if (scs.getStartDate() == null && scs.getEndDate() != null) {
        scs.setStartDate(scs.getEndDate());
      }
      scs.recomputeDuration();
      scs.store(pc);
    }

    CompanyCalendar cc = new CompanyCalendar();
    cc.setTime(now);
    int workingDays = 1;
    cc.addWorkingDays(workingDays);
    Date end = cc.getTime();
    List<Task> allT = new OqlQuery("from " + Task.class.getName() + " as t where t.schedule is null", pc).list();
    for (Task scs : allT) {
      Period p = new Period(now, end);
      p.store(pc);
      scs.setSchedule(p);
      scs.store(pc);
    }

    pc.commitAndClose();
  }

  public static void tw320UpdateListeners() throws Exception {

    PersistenceContext pc = HibernateFactory.newFreeSession();

    OqlQuery query = new OqlQuery("from " + Listener.class.getName() + " as ls where ls.eventType = :iac", pc);
    query.getQuery().setString("iac", "TASK_ISSUE_ADDED_CLOSED");

    List<Listener> ls = query.list();
    for (Listener lo : ls) {
      lo.setEventType("TASK_ISSUE_ADDED");
      lo.store(pc);
      Listener l = new Listener(lo.getOwner());
      l.setIdAsNew();
      l.setTheClass(lo.getClass().getName());
      l.setIdentifiableId(lo.getIdentifiableId());
      l.setMedia(lo.getMedia());
      l.setEventType("TASK_ISSUE_CLOSED");
      l.setListenDescendants(lo.isListenDescendants());
      l.store(pc);
    }
    pc.commitAndClose();
  }


  public static void tw320UpdateRoles() throws Exception {

    PersistenceContext pc = HibernateFactory.newFreeSession();

    List<Role> all = new OqlQuery("from " + Role.class.getName() + " as role", pc).list();
    for (Role role : all) {
      if (role.hasPermissionFor(TeamworkPermissions.fileStorage_canCreate))
        role.addPermission(TeamworkPermissions.fileStorage_explorer_canCreate);
      if (role.hasPermissionFor(TeamworkPermissions.fileStorage_canRead))
        role.addPermission(TeamworkPermissions.fileStorage_explorer_canRead);
      if (role.hasPermissionFor(TeamworkPermissions.fileStorage_canWrite)) {
        role.addPermission(TeamworkPermissions.fileStorage_explorer_canRead);
        role.addPermission(TeamworkPermissions.fileStorage_explorer_canCreate);
        role.addPermission(TeamworkPermissions.fileStorage_explorer_canWrite);
      }
      role.store(pc);
    }

    pc.commitAndClose();
  }

  public static void tw320UpdateIssues() throws Exception {

    PersistenceContext pc = HibernateFactory.newFreeSession();

    List<Issue> all = new OqlQuery("from " + Issue.class.getName() + " as issue", pc).list();
    for (Issue i : all) {
      i.setDateSignalled(i.getCreationDate());
      i.store(pc);
    }
    pc.commitAndClose();
  }

  public static void tw320UpdateScheduleMinute() {
    Connection conn = null;
    try {
      conn = PersistenceContext.getNewConnection();
      conn.setAutoCommit(false);
      String sql = "UPDATE olpl_schedule SET onlyWorkingDays = ?  WHERE discriminator = 'U' AND onlyWorkingDays IS NULL";
      PreparedStatement ps = conn.prepareStatement(sql);
      ps.setObject(1, false);
      ps.execute();
      ps.close();
      conn.commit();
      conn.close();

    } catch (Exception e) {

      try {
        if (conn != null && !conn.isClosed())
          conn.close();
      } catch (SQLException e1) {
      }
      Tracer.platformLogger.error(e);
    }
  }

}
