package com.twproject.setup;

import com.opnlb.rss.RSSFeed;
import com.opnlb.website.content.Content;
import com.opnlb.website.news.NewsCategory;
import com.opnlb.website.page.WebSitePage;
import com.opnlb.website.portlet.Portlet;
import com.opnlb.website.template.Template;
import com.opnlb.website.waf.WebSiteConstants;
import com.twproject.messaging.stickyNote.StickyNote;
import com.twproject.operator.TeamworkOperator;
import com.twproject.resource.Person;
import com.twproject.security.RoleTeamwork;
import com.twproject.security.TeamworkArea;
import com.twproject.security.TeamworkPermissions;
import com.twproject.task.IssueImpact;
import com.twproject.task.IssueType;
import com.twproject.task.TaskType;
import com.twproject.task.financial.CostAggregator;
import com.twproject.task.financial.CostClassification;
import org.jblooming.ApplicationException;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.agenda.EventType;
import org.jblooming.company.DepartmentType;
import org.jblooming.messaging.Message;
import org.jblooming.messaging.MessagingSystem;
import org.jblooming.operator.Operator;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.PersistenceHome;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.security.Area;
import org.jblooming.security.PlatformPermissions;
import org.jblooming.waf.constants.Fields;
import org.jblooming.waf.exceptions.ActionException;
import org.jblooming.waf.html.button.ButtonLink;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class WizardSupport {

  public static void addFeedback(RoleTeamwork pm, StringBuffer feedback) {
    if (pm.isNew())
      feedback.append("Security - created role: " + pm.getName() + "<br>");
    else
      feedback.append("Security - updated role: " + pm.getName() + "<br>");
  }

  public static void getContent(String areaName, Template template, WebSitePage page, int order, Portlet aportlet, StringBuffer feedback, Operator operator) throws PersistenceException {

    // 24/11 do not use operator null for identify default bute use defaultConfiguration
    //String hql = "from " + Content.class.getName() + " as content where content.page=:page and  content.area = :areax and content.operator is null";

    String hql = "from " + Content.class.getName() + " as content where content.page=:page and content.defaultConfiguration =:tr";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setBoolean("tr", true);
    //oql.getQuery().setString("areax", areaName);
    oql.getQuery().setEntity("page", page);

    List<Content> contents = oql.list();
    for (Content c : contents) {
      if (c.getPortlet().getName().equalsIgnoreCase(aportlet.getName())){
        feedback.append("Portal - removing content "+c.getArea()+"<br>");
        c.remove();
      }
    }

    Content content = new Content();
    content.setIdAsNew();
    content.setTemplate(template);
    content.setDefaultConfiguration(true);
    content.setGlobalAssociation(false);
    content.setPage(page);
    content.setArea(areaName);
    content.setOrder(order);
    content.setPortlet(aportlet);
    content.setOperator(operator);
    content.store();
    feedback.append("Portal - created content "+content.getArea()+"<br>");

  }

  public static RoleTeamwork getRole(String roleName, Area area, TeamworkOperator logged) throws PersistenceException {

    String hql = "from " + RoleTeamwork.class.getName() + " as role where role.name=:namex and  role.area = :areax";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setEntity("areax", area);
    oql.getQuery().setString("namex", roleName);

    RoleTeamwork role = (RoleTeamwork) oql.uniqueResultNullIfEmpty();

    if (role == null) {
      role = new RoleTeamwork();
      role.setIdAsNew();
      role.setLocalToAssignment(true);
      role.setName(roleName);
      role.setArea(area);
      role.setOwner(logged);
    }

    // resets permissions
    role.setPermissionIds(null);

    return role;
  }



  public static void setAreaManagerPermissions(RoleTeamwork am) {
    am.setDescription("Has complete control on the area.");
    am.setLocalToAssignment(false);

    // Platform
    am.addPermission(PlatformPermissions.area_canManage);
    am.addPermission(PlatformPermissions.role_canRead);
    am.addPermission(PlatformPermissions.role_canWrite);
    am.addPermission(PlatformPermissions.role_canCreate);

    am.addPermission(TeamworkPermissions.resource_canRead);
    am.addPermission(TeamworkPermissions.resource_canWrite);
    am.addPermission(TeamworkPermissions.resource_canCreate);
    am.addPermission(TeamworkPermissions.resource_cost_canRead);
    am.addPermission(TeamworkPermissions.resource_assignment_manage);
    am.addPermission(TeamworkPermissions.task_canRead);
    am.addPermission(TeamworkPermissions.task_canCreate);
    am.addPermission(TeamworkPermissions.task_canWrite);
    am.addPermission(TeamworkPermissions.task_assignment_manage);
    am.addPermission(TeamworkPermissions.task_cost_canRead);
    am.addPermission(TeamworkPermissions.task_cost_canCreate);
    am.addPermission(TeamworkPermissions.task_cost_canWrite);
    am.addPermission(TeamworkPermissions.issue_canRead);
    am.addPermission(TeamworkPermissions.issue_canWrite);
    //am.addPermission(TeamworkPermissions.issue_canCreate);
    am.addPermission(TeamworkPermissions.document_canRead);
    am.addPermission(TeamworkPermissions.document_canWrite);
    am.addPermission(TeamworkPermissions.document_canCreate);
    am.addPermission(TeamworkPermissions.classificationTree_canManage);
    am.addPermission(TeamworkPermissions.board_canRead);
    am.addPermission(TeamworkPermissions.board_canWrite);
    am.addPermission(TeamworkPermissions.board_canCreate);
    am.addPermission(TeamworkPermissions.fileStorage_canRead);
    //am.addPermission(TeamworkPermissions.fileStorage_canWrite);
    //am.addPermission(TeamworkPermissions.fileStorage_canCreate);
    am.addPermission(TeamworkPermissions.fileStorage_explorer_canRead);
    am.addPermission(TeamworkPermissions.fileStorage_explorer_canWrite);
    am.addPermission(TeamworkPermissions.fileStorage_explorer_canCreate);

  }

  public static void setProjectManagerPermissions(RoleTeamwork pm) {
    pm.setDescription("Can manage the project and its descendants on which she/he has been assigned.");
    pm.setLocalToAssignment(true);
    pm.addPermission(TeamworkPermissions.resource_canRead);
    pm.addPermission(TeamworkPermissions.resource_canWrite);
    pm.addPermission(TeamworkPermissions.resource_canCreate);
    pm.addPermission(TeamworkPermissions.task_canRead);
    pm.addPermission(TeamworkPermissions.task_canCreate);
    pm.addPermission(TeamworkPermissions.task_canWrite);
    pm.addPermission(TeamworkPermissions.task_assignment_manage);
    pm.addPermission(TeamworkPermissions.task_cost_canRead);
    pm.addPermission(TeamworkPermissions.task_cost_canCreate);
    pm.addPermission(TeamworkPermissions.task_cost_canWrite);
    pm.addPermission(TeamworkPermissions.issue_canRead);
    pm.addPermission(TeamworkPermissions.issue_canWrite);
    pm.addPermission(TeamworkPermissions.issue_canCreate);
    pm.addPermission(TeamworkPermissions.document_canRead);
    pm.addPermission(TeamworkPermissions.document_canWrite);
    pm.addPermission(TeamworkPermissions.document_canCreate);
  }

  public static void setStakeholderPermissions(RoleTeamwork so) {
    so.setDescription("Can read and signal issuues on the project and its descendants on which she/he has been assigned.");
    so.setLocalToAssignment(true);
    so.addPermission(TeamworkPermissions.task_canRead);
    so.addPermission(TeamworkPermissions.issue_canRead);
    so.addPermission(TeamworkPermissions.issue_canWrite);
    so.addPermission(TeamworkPermissions.issue_canCreate);
  }

  public static void setOperatorPermissions(RoleTeamwork so) {
    so.setDescription("Minimal standard permission for an operator.");
    so.setLocalToAssignment(true);
    so.addPermission(TeamworkPermissions.resource_canRead);
    so.addPermission(TeamworkPermissions.board_canRead);
  }

  public static void setWorkerPermissions(RoleTeamwork wo) {


    wo.setDescription("Can work (e.g. insert worklog, manage issues and documents) on the project and its descendants on which she/he has been assigned.");
    wo.setLocalToAssignment(true);
    wo.addPermission(TeamworkPermissions.resource_canRead             );
    wo.addPermission(TeamworkPermissions.task_canRead                 );
    wo.addPermission(TeamworkPermissions.issue_canRead                );
    wo.addPermission(TeamworkPermissions.issue_canWrite               );
    wo.addPermission(TeamworkPermissions.issue_canCreate              );
    wo.addPermission(TeamworkPermissions.document_canRead             );
    wo.addPermission(TeamworkPermissions.document_canWrite            );
    wo.addPermission(TeamworkPermissions.document_canCreate           );


  }

  public static void setSupervisorPermissions(RoleTeamwork reader) {

    reader.setDescription("Can read everything and comment on the project and its descendants on which she/he has been assigned.");
    reader.setLocalToAssignment(false);
    reader.addPermission(TeamworkPermissions.resource_canRead             );
    reader.addPermission(TeamworkPermissions.task_canRead                 );
    reader.addPermission(TeamworkPermissions.task_cost_canRead            );
    reader.addPermission(TeamworkPermissions.issue_canRead                );
    reader.addPermission(TeamworkPermissions.document_canRead             );
    reader.addPermission(TeamworkPermissions.board_canRead                );
    reader.addPermission(TeamworkPermissions.board_canWrite               );
    reader.addPermission(TeamworkPermissions.board_canCreate              );
    reader.addPermission(TeamworkPermissions.fileStorage_canRead          );

  }

  public static void setReaderPermissions(RoleTeamwork resReader) {
    resReader.setDescription("Can read resources. Should be set on everyone ");
    resReader.setLocalToAssignment(false);
    resReader.addPermission(TeamworkPermissions.resource_canRead             );
  }

  public static void setScrumProductOwnerPermissions(RoleTeamwork productOwner) {
    productOwner.setDescription("It's the 'pig'. Is the Product's owner. Can manage the product backlog.");
    productOwner.setLocalToAssignment(true);
    productOwner.addPermission(TeamworkPermissions.resource_canRead);
    productOwner.addPermission(TeamworkPermissions.task_canRead);
    productOwner.addPermission(TeamworkPermissions.task_cost_canRead);
    productOwner.addPermission(TeamworkPermissions.task_cost_canCreate);
    productOwner.addPermission(TeamworkPermissions.task_cost_canWrite);
    productOwner.addPermission(TeamworkPermissions.issue_canRead);
    productOwner.addPermission(TeamworkPermissions.issue_canWrite);
    productOwner.addPermission(TeamworkPermissions.issue_canCreate);
    productOwner.addPermission(TeamworkPermissions.document_canRead);
    productOwner.addPermission(TeamworkPermissions.document_canWrite);
    productOwner.addPermission(TeamworkPermissions.document_canCreate);
  }

  public static void setScrumStakeholderPermissions(RoleTeamwork so) {
    so.setDescription("It's a 'chicken'. Can read the project and its descendants on which she/he has been assigned.");
    so.setLocalToAssignment(true);
    so.addPermission(TeamworkPermissions.task_canRead);
    so.addPermission(TeamworkPermissions.issue_canRead);
    so.addPermission(TeamworkPermissions.issue_canWrite);
    so.addPermission(TeamworkPermissions.issue_canCreate);
    so.addPermission(TeamworkPermissions.document_canRead);
  }


  public static void setScrumMasterPermissions(RoleTeamwork pm) {
    pm.setDescription("The ScrumMaster is a facilitator for the team and product owner");
    pm.setLocalToAssignment(true);
    pm.addPermission(TeamworkPermissions.resource_canRead);
    pm.addPermission(TeamworkPermissions.task_canRead);
    pm.addPermission(TeamworkPermissions.task_canCreate);
    pm.addPermission(TeamworkPermissions.task_canWrite);
    pm.addPermission(TeamworkPermissions.task_assignment_manage);
    pm.addPermission(TeamworkPermissions.issue_canRead);
    pm.addPermission(TeamworkPermissions.issue_canWrite);
    pm.addPermission(TeamworkPermissions.issue_canCreate);
    pm.addPermission(TeamworkPermissions.document_canRead);
    pm.addPermission(TeamworkPermissions.document_canWrite);
    pm.addPermission(TeamworkPermissions.document_canCreate);
  }

  public static void setScrumTeamPermissions(RoleTeamwork pm) {
    pm.setDescription("Member of development team.");
    pm.setLocalToAssignment(true);
    pm.addPermission(TeamworkPermissions.resource_canRead);
    pm.addPermission(TeamworkPermissions.task_canRead);
    pm.addPermission(TeamworkPermissions.issue_canRead);
    pm.addPermission(TeamworkPermissions.issue_canWrite);
    pm.addPermission(TeamworkPermissions.issue_canCreate);
    pm.addPermission(TeamworkPermissions.document_canRead);
    pm.addPermission(TeamworkPermissions.document_canWrite);
    pm.addPermission(TeamworkPermissions.document_canCreate);
  }


  public static TeamworkArea createAreaAndStandardRoles(String areaName, TeamworkOperator owner, StringBuffer feedback) throws PersistenceException {

    // check if area already exists
    TeamworkArea area= (TeamworkArea) PersistenceHome.findUnique(Area.class,"name",areaName);

    // create the new area
    if (area ==null){
      area = new TeamworkArea();
      area.setIdAsNew();
      area.setName(areaName);
      area.setOwner(owner);
      area.store();
      feedback.append("Created area: "+areaName+"<br>");
    }


    // ---------------------------------- Area Manager ----------------------------------
    RoleTeamwork am=getRole("Area manager", area, owner);
    setAreaManagerPermissions(am);
    addFeedback(am, feedback);
    am.store();

    //  ---------------------------------- Project manager ----------------------------------
    RoleTeamwork pm=getRole(ApplicationState.getApplicationSetting("DEFAULT_PROJECT_MANAGER_ROLE_NAME","Project manager"), area, owner);
    setProjectManagerPermissions(pm);
    addFeedback(pm, feedback);
    pm.store();


    // ---------------------------------- Stakeholder/Customer ----------------------------------
    RoleTeamwork so=getRole("Stakeholder/Customer", area, owner);
    setStakeholderPermissions(so);
    addFeedback(so, feedback);
    so.store();

    // ---------------------------------- Worker ----------------------------------
    RoleTeamwork wo=getRole(ApplicationState.getApplicationSetting("DEFAULT_WORKER_ROLE_NAME","Worker"), area, owner);
    setWorkerPermissions(wo);
    addFeedback(wo, feedback);
    wo.store();


    // ---------------------------------- Supervisor ----------------------------------
    RoleTeamwork reader=getRole("Supervisor", area, owner);
    setSupervisorPermissions(reader);
    addFeedback(reader, feedback);
    reader.store();

    // ---------------------------------- Resource reader ----------------------------------
    RoleTeamwork resourceReader=getRole("Resource reader", area, owner);
    setReaderPermissions(resourceReader);
    addFeedback(resourceReader, feedback);
    resourceReader.store();

    // ---------------------------------- Operational ----------------------------------
    RoleTeamwork stdOperatorRole = WizardSupport.getRole("Operational", area, owner);
    WizardSupport.setOperatorPermissions(stdOperatorRole);
    stdOperatorRole.store();

    return area;

  }

  public static TeamworkArea createAreaAndScrumRoles(String areaName, TeamworkOperator owner, StringBuffer feedback) throws PersistenceException {
    // check if area already exists
    TeamworkArea area= (TeamworkArea) PersistenceHome.findUnique(Area.class,"name",areaName);

    // create the new area
    if (area ==null){
      area = new TeamworkArea();
      area.setIdAsNew();
      area.setName(areaName);
      area.setOwner(owner);
      area.store();
      feedback.append("Created area: "+areaName+"<br>");
    }


    // ---------------------------------- Area manager ----------------------------------
    RoleTeamwork am=getRole("Area manager", area, owner);
    setAreaManagerPermissions(am);
    addFeedback(am, feedback);
    am.store();

    // ---------------------------------- Product owner ----------------------------------
    RoleTeamwork po=getRole("Product owner", area, owner);
    setScrumProductOwnerPermissions(po);
    addFeedback(po, feedback);
    po.store();


    //  ---------------------------------- Scrum Master ----------------------------------
    RoleTeamwork pm=getRole("Scrum Master", area, owner);
    setScrumMasterPermissions(pm);
    addFeedback(pm, feedback);
    pm.store();


    // ---------------------------------- Stakeholder ----------------------------------
    RoleTeamwork so=getRole("Scrum Stakeholder", area, owner);
    setScrumStakeholderPermissions(so);
    addFeedback(so, feedback);
    so.store();

    // ---------------------------------- Scrum Team ----------------------------------
    RoleTeamwork wo=getRole("Scrum Team", area, owner);
    setScrumTeamPermissions(wo);
    addFeedback(wo, feedback);
    wo.store();

    return area;
  }




  public static void createSampleData() throws PersistenceException {

    OqlQuery oql = new OqlQuery("from " + TeamworkOperator.class.getName() + " as op " +
            "where lower(op.loginName) = lower(:name)");
    oql.getQuery().setString("name", "administrator");
    TeamworkOperator administrator = (TeamworkOperator) oql.uniqueResult();

    Person adminPerson = administrator.getMyPersonFromPersistence();
    if (adminPerson==null)
      throw new PlatformRuntimeException("adminPerson is null");

    //create default area
    TeamworkArea area= WizardSupport.createAreaAndStandardRoles("AREA_RENAME_ME", administrator, new StringBuffer());


    DepartmentType dt = new DepartmentType();
    dt.setDescription("Company");
    dt.setStringValue("COMPANY");
    dt.store();

    createTeamworkTypesForArea(area);

    dt = new DepartmentType();
    dt.setDescription("Department");
    dt.setStringValue("DEPARTMENT");
    dt.store();

    dt = new DepartmentType();
    dt.setDescription("Branch");
    dt.setStringValue("BRANCH");
    dt.store();

    dt = new DepartmentType();
    dt.setDescription("Office");
    dt.setStringValue("OFFICE");
    dt.store();


    NewsCategory nc = new NewsCategory();
    nc.setIdAsNew();
    nc.setDescription("Company news");
    nc.store();
    
    String subject = "Wizard page available";

    Message message = new Message();
    message.setFromOperator(administrator);
    message.setToOperator(administrator);
    message.setMedia(MessagingSystem.Media.NEWS.toString());
    message.setReceived(new Date());
    message.setSubject(subject);

    PageSeed ps  = new PageSeed(ApplicationState.contextPath+"/applications/teamwork/setup/wizard/launchWizards.jsp");
    ButtonLink addBl = new ButtonLink(ps);
    addBl.label = "Go to wizards";
    String messageBody =
            "Click on the link to go to the wizards page. " +
            "There are wizards for security, scrum, import from previous versions, portal management.<br>";
    message.setLink(addBl.toPlainLink());

    message.setMessageBody(messageBody);
    message.store();

    String body = message.getMessageBody();
    body = body + "<hr>link:&nbsp;" + message.getLink();

    StickyNote sn = new StickyNote();
    sn.setIdAsNew();
    sn.setAuthor(adminPerson);
    sn.setReceiver(adminPerson);
    sn.setType(message.getSubject());
    sn.setMessage(body);
    sn.store();

    sn = new StickyNote();
    sn.setIdAsNew();
    sn.setAuthor(adminPerson);
    sn.setReceiver(adminPerson);
    sn.setType("Give a meaningful name to default area");

    ps  = new PageSeed(ApplicationState.contextPath+"/commons/area/area.jsp");
    addBl = new ButtonLink(ps);
    addBl.label = "Go to areas";
    String areaMess =
            "Setup created a default area in which everything is created, "+
            "with temporary name AREA_RENAME_ME.<br>" +
            "Click on the link below to rename it.";
    areaMess = areaMess + "<hr>link:&nbsp;" + addBl.toPlainLink();
    sn.setMessage(areaMess);

    sn.store();

    // RSS company news default feed
    RSSFeed news = new RSSFeed();
    news.setLink(ApplicationState.serverURL);
    news.setTitle("Company News");
    news.setDescription("Company news");
    news.store();
  }

  public static void createTeamworkTypesForArea(TeamworkArea area) throws StoreException {
    //create
    TaskType type = new TaskType();
    type.setDescription("Template");
    type.setStringValue("TPL");
    type.setArea(area);
    type.store();

    type = new TaskType();
    type.setDescription("Production");
    type.setStringValue("PRD");
    type.setArea(area);
    type.store();

    IssueType it = new IssueType();
    it.setIntValue(0);
    it.setDescription("Bug");
    it.setArea(area);
    it.store();

    it = new IssueType();
    it.setIntValue(1);
    it.setDescription("Feature");
    it.setArea(area);
    it.store();

    IssueImpact ii = new IssueImpact();
    ii.setDescription("Light");
    ii.setIntValue(30);
    ii.setArea(area);
    ii.store();

    ii = new IssueImpact();
    ii.setDescription("Medium");
    ii.setIntValue(60);
    ii.setArea(area);
    ii.store();

    ii = new IssueImpact();
    ii.setDescription("Severe");
    ii.setIntValue(90);
    ii.setArea(area);
    ii.store();

    EventType et = new EventType();
    et.setDescription("Meeting");
    et.setArea(area);
    et.store();

    et = new EventType();
    et.setDescription("Travel");
    et.setArea(area);
    et.store();

    CostClassification cc = new CostClassification();
    cc.setDescription("Travel");
    cc.setStringValue("TRL");
    cc.setArea(area);
    cc.store();

    cc = new CostClassification();
    cc.setDescription("Stay");
    cc.setStringValue("STY");
    cc.setArea(area);
    cc.store();

    CostAggregator ca = new CostAggregator();
    ca.setCode("1");
    ca.setDescription("Master sample");
    ca.setArea(area);
    ca.store();
  }


}
