package com.twproject.task;

import com.twproject.resource.Person;
import com.twproject.resource.Resource;
import com.twproject.security.RoleTeamwork;
import com.twproject.task.financial.CostAggregator;
import com.twproject.worklog.Worklog;
import org.hibernate.Query;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.messaging.SomethingHappened;
import org.jblooming.ontology.SecuredLoggableHideableSupport;
import org.jblooming.operator.User;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.exceptions.FindException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.persistence.exceptions.StoreException;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.security.Permission;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.ReflectionUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.html.button.ButtonLink;
import org.jblooming.waf.html.container.TabSet;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.PageSeed;
import org.jblooming.waf.view.PageState;

import java.util.*;


public class Assignment extends SecuredLoggableHideableSupport implements Comparable {

  private String description;

  protected Resource resource;
  private Task task;
  private RoleTeamwork role;

  protected long estimatedWorklog;
  private Date assignmentDate;

  //read only
  private Set worklogs = new HashSet();

  //read only
  private Set<AssignmentPriority> priorities = new TreeSet<AssignmentPriority>(new Comparator() {
    public int compare(Object o1, Object o2) {
      AssignmentPriority rd1 = (AssignmentPriority) o1;
      AssignmentPriority rd2 = (AssignmentPriority) o2;
      //as sql.Timestamp and util.Date may be given
      return (int) (rd2.getCutPoint() - rd1.getCutPoint());
    }
  });


  private boolean counted = false;
  private Date countingStartedAt;
  private String activity = ACTIVITY_ALL_IN_ONE;
  private boolean induceWorklog = true;
  private int risk;

  private boolean enabled = true;

  private double hourlyCost;

  private CostAggregator costCenter;

  public static final String ACTIVITY_ALL_IN_ONE = "ACTIVITY_ALL_IN_ONE";
  public static final String ACTIVITY_REPEATED_IN_TIME = "ACTIVITY_REPEATED_IN_TIME";

  public Assignment() {
  }

  public Assignment(int id) {
    this.setId(id);
  }

  public static List<String> getActivityTypes() {

    List<String> acts = new ArrayList<String>();
    acts.add(ACTIVITY_ALL_IN_ONE);
    acts.add(ACTIVITY_REPEATED_IN_TIME);

    return acts;
  }

  public Resource getResource() {
    return resource;
  }

  public void setResource(Resource resource) {
    this.resource = resource;
  }

  /**
   * @return in HOURS in decimal
   */
  public long getEstimatedWorklog() {
    return estimatedWorklog;
  }

  public void setEstimatedWorklog(long estimatedWorklog) {
    this.estimatedWorklog = estimatedWorklog;
  }

  public Date getAssignmentDate() {
    return assignmentDate;
  }

  public void setAssignmentDate(Date assignmentDate) {
    this.assignmentDate = assignmentDate;
  }

  public Set<Worklog> getWorklogs() {
    return worklogs;
  }

  private void setWorklogs(Set<Worklog> worklogs) {
    this.worklogs = worklogs;
  }

  public Iterator<Worklog> getWorkLogsIterator() {
    return worklogs.iterator();
  }

  public int getWorkLogsSize() {
    return worklogs.size();
  }


  public Task getTask() {
    return task;
  }

  public void setTask(Task task) {
    this.task = task;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public Date getLastModified() {
    return lastModified;
  }

  public void setLastModified(Date lastModified) {
    this.lastModified = lastModified;
  }

  public String getName() {
    return (getDescription() != null ? getDescription() : getTask().getName());
  }

  public boolean isCounted() {
    return counted;
  }

  public void setCounted(boolean counted) {
    this.counted = counted;
    setCountingStartedAt(DateUtilities.getAbsoluteNow());
  }

  public Date getCountingStartedAt() {
    return countingStartedAt;
  }

  public void setCountingStartedAt(Date countingStartedAt) {
    this.countingStartedAt = countingStartedAt;
  }

  public long getCountedIntervalSize() {
    if (!counted)
      return 0;
    return DateUtilities.getAbsoluteNow().getTime() - getCountingStartedAt().getTime();
  }

  public String getActivity() {
    return activity;
  }

  public void setActivity(String activity) {
    this.activity = activity;
  }

  public boolean isInduceWorklog() {
    return induceWorklog;
  }

  public void setInduceWorklog(boolean induceWorklog) {
    this.induceWorklog = induceWorklog;
  }

  public int getRisk() {
    return risk;
  }

  public void setRisk(int risk) {
    this.risk = risk;
  }

  public RoleTeamwork getRole() {
    return role;
  }

  public void setRole(RoleTeamwork role) {
    this.role = role;
  }


  //todo: why ????
  public boolean hasPermissionFor(User u, Permission p) {
    return false;
  }


  public boolean isEnabled() {
    return enabled;
  }

  public void setEnabled(boolean enabled) {
    this.enabled = enabled;
  }

  public List<String> getSubscibableEventNames() {
    List<String> assigEvents = new ArrayList<String>();
    assigEvents.add("worklog");
    return assigEvents;
  }

  /**
   * @return in MILLIS
   */
  public long getWorklogDone() {

    return getWorklogDone(HibernateFactory.getDefaultPersistenceContext());

  }

  /**
   * @return in MILLIS
   */
  public long getWorklogDone(PersistenceContext pc) {
    String hql = "select sum(wklg.duration) from " + Worklog.class.getName() + " as wklg where wklg.assig = :assig";
    OqlQuery oql = null;
    try {
      oql = new OqlQuery(hql, pc);
    } catch (PersistenceException e) {
      throw new PlatformRuntimeException(e);
    }
    oql.getQuery().setEntity("assig", this);
    try {
      return (Long) oql.uniqueResult();
    } catch (FindException e) {
      return 0;
    }
  }


  public String getDisplayName(PageState pageState) {
    return getDisplayName(pageState.getI18n("ASSIG_AS"));
  }

  public String getDisplayName() {
    return getDisplayName(":");
  }

  private String getDisplayName(String as) {
    String result = "";
    if (getTask().getCode() != null)
      result = getTask().getCode() + "/";
    result = result + getTask().getName() + " - " + getResource().getDisplayName() + " " + as + " '" + role.getName() + "'" + (getDescription()!=null ? " "+getDescription() : "");
    return result;
  }


  private Set getPriorities() {
    return priorities;
  }

  private void setPriorities(Set priorities) {
    this.priorities = priorities;
  }

  public void addPriorityInMemory(AssignmentPriority priority) {
    priorities.add(priority);
  }

  public void removePriorityInMemory(AssignmentPriority priority) {
    priorities.remove(priority);
  }

  public Iterator<AssignmentPriority> getPrioritiesIterator() {
    return priorities.iterator();
  }

  public boolean prioritiesContains(AssignmentPriority priority) {
    return priorities.contains(priority);
  }

  public int prioritiesSize() {
    return priorities.size();
  }

  public int getPriorityAtTime(long time) {
    int ret = AssignmentPriority.PRIORITY_LOW;
    AssignmentPriority ap = getAssignmentPriorityAtTime(time);
    if (ap != null)
      ret = ap.getPriority();
    return ret;
  }

  public AssignmentPriority getAssignmentPriorityAtTime(long time) {
    AssignmentPriority ret = null;
    if (priorities != null && priorities.size() > 0) {
      for (AssignmentPriority ap : priorities) {
        if (ap.getCutPoint() <= time) {
          ret = ap;
        } else
          break;
      }
    }
    return ret;
  }

  public void addWorklogInMemory(Worklog wl) {
    worklogs.add(wl);
  }

  public void removeWorklogInMemory(Worklog wl) {
    worklogs.remove(wl);
  }

  public void closeCounter(SessionState sessionState, PersistenceContext persistenceContext) throws StoreException {
    long elapsed = DateUtilities.getAbsoluteNow().getTime() - countingStartedAt.getTime();
    setCounted(false);
    setCountingStartedAt(null);
    store(persistenceContext);
    Worklog w = new Worklog();
    w.setIdAsNew();
    w.setAssig(this);
    w.setDuration(elapsed);
    w.setInserted(DateUtilities.getAbsoluteNow());
    if (sessionState != null) {
      String action = (String) sessionState.getAttribute("ISSUE_WORKLOG_ACTION");
      w.setAction(action);
      sessionState.setAttribute("ISSUE_WORKLOG_ACTION", null);
    }
    w.store(persistenceContext);
  }


  public Assignment getClonedInstance(Task newTask) {
    Assignment a = new Assignment();
    a.setIdAsNew();
    a.setRole(getRole());
    a.setResource(getResource());
    a.setTask(newTask);
    a.setEnabled(isEnabled());
    a.setDescription(getDescription());
    a.setEstimatedWorklog(getEstimatedWorklog());
    a.setAssignmentDate(new Date());
    a.setActivity(getActivity());
    a.setInduceWorklog(isInduceWorklog());
    a.setRisk(getRisk());
    a.setHourlyCost(getHourlyCost());
    return a;
  }

  public void setHourlyCost(double hourlyCost) {
    this.hourlyCost = hourlyCost;
  }

  public double getHourlyCost() {
    return hourlyCost;
  }

  public CostAggregator getCostCenter() {
    return costCenter;
  }

  public void setCostCenter(CostAggregator costCenter) {
    this.costCenter = costCenter;
  }

  public void store(PersistenceContext pc) throws StoreException {
    super.store(pc);
    getTask().addAssignmentInMemory(this);
    getTask().updateProgress(pc);
    checkOverwork(0,pc);
  }

  public void checkOverwork(long additionalWork, PersistenceContext pc) throws StoreException {
    //overwork check
    long worklogDoneAndSaved = getWorklogDone(pc)+additionalWork;
    if (getEstimatedWorklog() < worklogDoneAndSaved) {

      String hql = "select ev from " + SomethingHappened.class.getName() + " as ev where ev.eventType = :eventType and ev.theClass=:theClass and ev.identifiableId=:idId";
      Query query = null;
      try {
        query = new OqlQuery(hql, pc).getQuery();
      } catch (PersistenceException e) {
        throw new PlatformRuntimeException(e);
      }
      query.setString("eventType", "TASK_WORKLOG_OVERFLOW");
      query.setString("theClass", Task.class.getName());
      query.setString("idId", getTask().getId()+"");

      List<SomethingHappened> li = query.list();
      if (li == null || li.size() == 0) {

        SomethingHappened change = new SomethingHappened();
        change.setIdAsNew();
        change.setEventType("TASK_WORKLOG_OVERFLOW");
        change.setMessageTemplate("TASK_WORKLOG_OVERFLOW_MESSAGE_TEMPLATE");
        change.setIdentifiable(getTask());
        change.getMessageParams().put("assig", getDisplayName());
        change.getMessageParams().put("estWklg", getEstimatedWorklogInHoursMinutes());
        change.getMessageParams().put("totWklg", DateUtilities.getMillisInHoursMinutes(worklogDoneAndSaved));
        Resource resource = (Resource) ReflectionUtilities.getUnderlyingObject(getResource());
        if (resource instanceof Person)
          change.setWhoCausedTheEvent(((Person) resource).getMyself());

        PageSeed ps = new PageSeed(ApplicationState.serverURL + "/applications/teamwork/task/taskEditor.jsp");
        ps.setCommand("EDIT_ASSIGNMENTS");
        ps.setMainObjectId(getTask().getId());
        TabSet.pointToTab("taskTabSet", "TASK_ASSIGNMENT_TAB", ps);
        ButtonLink edit = new ButtonLink(ps);
        edit.label = getDisplayName();
        change.setLink(edit.toPlainLink());
        change.store(pc);
      }
    }
  }

  public String  getEstimatedWorklogInHoursMinutes() {
   return DateUtilities.getMillisInHoursMinutes(getEstimatedWorklog());
  }
}
