package com.twproject.task;

import com.twproject.operator.TeamworkOperator;
import com.twproject.resource.Person;
import com.twproject.worklog.Worklog;
import org.jblooming.PlatformRuntimeException;
import org.jblooming.operator.User;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.exceptions.FindException;
import org.jblooming.persistence.exceptions.PersistenceException;
import org.jblooming.security.Permission;
import org.jblooming.security.Securable;
import org.jblooming.utilities.JSP;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

/**
 * @author Pietro Polsinelli ppolsinelli@twproject.com
 */
public class Issue extends IssuePersistent implements Securable {


  public static final String GRAVITY_LOW = "01_GRAVITY_LOW";
  public static final String GRAVITY_MEDIUM = "02_GRAVITY_MEDIUM";
  public static final String GRAVITY_HIGH = "03_GRAVITY_HIGH";
  public static final String GRAVITY_CRITICAL = "04_GRAVITY_CRITICAL";
  public static final String GRAVITY_BLOCK = "05_GRAVITY_BLOCK";

  public static final String STATUS_OPEN = "01_STATUS_OPEN";
  public static final String STATUS_PAUSED = "02_STATUS_PAUSED";
  public static final String STATUS_ABORTED = "03_STATUS_ABORTED";
  public static final String STATUS_CLOSED = "04_STATUS_CLOSED";

  public static final String IMPACT_MINOR = "IMPACT_MINOR";


  public static Comparator compareByTaskName = new Comparator() {

    protected boolean isEmpty(String s) {
      return (s == null || s.trim().length() == 0);
    }

    public int compare(Object o1, Object o2) {
      Issue i1 = (Issue) o1;
      Issue i2 = (Issue) o2;
      String taskName1 = null;
      String taskName2 = null;

      taskName1 = i1.getTask().getName();

      taskName2 = i2.getTask().getName();

      if (isEmpty(taskName1) && isEmpty(taskName2))
        return 0;
      if (isEmpty(taskName1) && !isEmpty(taskName2))
        return 1;
      if (!isEmpty(taskName1) && isEmpty(taskName2))
        return -1;
      return taskName1.compareTo(taskName2);
    }

    ;
  };
  public static Comparator compareByAssignedResource = new Comparator() {
    protected boolean isEmpty(String s) {
      return (s == null || s.trim().length() == 0);
    }

    public int compare(Object o1, Object o2) {
      Issue i1 = (Issue) o1;
      Issue i2 = (Issue) o2;
      String assignedTo1 = null;
      if (i1.getAssignedTo() != null)
        assignedTo1 = i1.getAssignedTo().getDisplayName();
      String assignedTo2 = null;
      if (i2.getAssignedTo() != null)
        assignedTo2 = i2.getAssignedTo().getDisplayName();
      if (isEmpty(assignedTo1) && isEmpty(assignedTo2))
        return 0;
      if (isEmpty(assignedTo1) && !isEmpty(assignedTo2))
        return 1;
      if (!isEmpty(assignedTo1) && isEmpty(assignedTo2))
        return -1;
      return assignedTo1.compareTo(assignedTo2);
    }

    ;
  };

  public Issue() {
  }


  public String getName() {
    return getDescription();
  }

  public Iterator<Worklog> getWorklogsIterator() {
    return worklogs.iterator();
  }
  
  public boolean worklogsContains(Issue issue) {
    return worklogs.contains(issue);
  }

  public int worklogsSize() {
    return worklogs.size();
  }


  public static List<String> getStatusList() {
    List<String> statuses = new ArrayList<String>();

    statuses.add(STATUS_OPEN);
    statuses.add(STATUS_PAUSED);
    statuses.add(STATUS_CLOSED);
    statuses.add(STATUS_ABORTED);
    return statuses;
  }

  public static List<String> getGravityList() {
    List<String> grav = new ArrayList<String>();
    grav.add(GRAVITY_LOW);
    grav.add(GRAVITY_MEDIUM);
    grav.add(GRAVITY_HIGH);
    grav.add(GRAVITY_CRITICAL);
    grav.add(GRAVITY_BLOCK);
    return grav;
  }


  public long getWorklogDone() {

    String hql = "select sum(wklg.duration) from " + Worklog.class.getName() + " as wklg where wklg.issue = :issue";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setEntity("issue", this);
    try {
      return (Long) oql.uniqueResult();
    } catch (FindException e) {
      return 0;
    }

  }

  public String getDisplayName() {
    return JSP.w(getDescription());
  }

  public void addWorklogInMemory(Worklog wl) {
    worklogs.add(wl);
  }

  public void removeWorklogInMemory(Worklog wkl) {
    worklogs.remove(wkl);
  }

  /**
   * @return open,closed,created,score
   */
  public static long[] scoreForPerson(Person p) throws FindException {
    long[] result = new long[4];
    long totOpen = 0;
    long totClosed = 0;
    long totInserted = 0;

    {
      String hql = "select count(issue.id) from " + Issue.class.getName() + " as issue where issue.status = :closed and issue.assignedTo = :resource";
      OqlQuery oql = new OqlQuery(hql);
      oql.getQuery().setEntity("resource", p);
      oql.getQuery().setString("closed", Issue.STATUS_CLOSED);
      totClosed = (Long) oql.uniqueResult();
    }
    {
      String hql = "select count(issue.id) from " + Issue.class.getName() + " as issue where issue.status = :open and issue.assignedTo = :resource";
      OqlQuery oql = new OqlQuery(hql);
      oql.getQuery().setEntity("resource", p);
      oql.getQuery().setString("open", Issue.STATUS_OPEN);
      totOpen = (Long) oql.uniqueResult();
    }
    {
      String hql = "select count(issue.id) from " + Issue.class.getName() + " as issue where issue.owner = :op";
      OqlQuery oql = new OqlQuery(hql);
      oql.getQuery().setEntity("op", p.getMyself());
      totInserted = (Long) oql.uniqueResult();
    }
    result[0] = totClosed;
    result[1] = totOpen;
    result[2] = totInserted;
    result[3] = (totClosed * 3) + totInserted - (totOpen * 2);
    return result;
  }

  public Assignment getUniqueAssignment() {
    Assignment a = null;
    if (getAssignedTo() != null && getTask() != null) {
      //loop on assigs
      Iterator it = getTask().getAssignementsIterator();
      while (it.hasNext()) {
        Assignment assignment = (Assignment) it.next();
        if (assignment.getResource().equals(getAssignedTo())) {
          if (a == null)
            a = assignment;
          else {
            a = null;
            break;
          }
        }
      }
    }
    return a;
  }

  public boolean hasPermissionFor(User u, Permission p) {

    if (getOwner() != null && getOwner().equals(u))
      return true;

    try {
      if (getAssignedTo() != null &&  ((TeamworkOperator)u).getMyPersonFromPersistence().equals(getAssignedTo()))
        return true;
    } catch (PersistenceException e) {
      throw new PlatformRuntimeException(e);
    }


    boolean result = false;

    Securable sec = getTask() != null ? getTask() : getAssignedTo();

    if (sec != null)
      result = sec.hasPermissionFor(u, p);
    else {
      if (u.hasPermissionAsAdmin())
        result = true;
    }
    return result;
  }

}
