/*
 * Created by IntelliJ IDEA.
 * User: ppolsinelli
 * Date: 10-gen-2006
 * Time: 9.53.59
 */
package com.twproject.task;

import com.opnlb.website.forum.ForumEntry;
import com.twproject.classification.TaskClassification;
import com.twproject.classification.TaskClassificationAssociation;
import com.twproject.document.TeamworkDocument;
import com.twproject.task.financial.Cost;
import org.jblooming.agenda.Period;
import org.jblooming.ontology.Documentable;
import org.jblooming.ontology.SecuredNodeWithAreaSupport;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.exceptions.FindException;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public abstract class TaskPersistent extends SecuredNodeWithAreaSupport implements Documentable {

  private String code;
  private String name;
  private TaskType type;
  private String description;
  private String status;
  /**
   * the actual task relevance (for the task manager)
   */
  private int relevance = 0;
  /**
   * degree of completion of this task
   */
  private double progress = 0;
  protected Set<Assignment> assignments = new HashSet<Assignment>();
  private Set<Issue> issues = new HashSet<Issue>();

  private boolean progressByWorklog = false;

  private Period schedule;
  private Set<TaskScheduleHistory> scheduleHistory = new HashSet<TaskScheduleHistory>();

  private int duration;

  private Set<TaskStatusHistory> statusHistory = new HashSet<TaskStatusHistory>();
  private Set<TaskDependency> previouses = new HashSet<TaskDependency>();
  private Set<TaskDependency> nexts = new HashSet<TaskDependency>();
  private Set<TeamworkDocument> documents = new HashSet<TeamworkDocument>();
  /**
   * is stand-alone project (this is usefull to reach the project roots even if are deeply hidden in a tree structure).
   */
  private boolean standAlone = false;
  /**
   * logical task type. This kind of task are structural, for instance they not permits to create assignements or
   * setting duration because this data are inherited from children
   */
  private boolean logical = false;
  private Set<TaskClassificationAssociation> taskClassificationAssociations = new HashSet<TaskClassificationAssociation>();
  private ForumEntry forumEntry = new ForumEntry();
  private String goals;
  private String benefits;
  private String externalCode;
  private String costCenter;
  private String pro;
  private String contra;
  private String notes;
  private String commentOnClose;
  private boolean startIsMilestone = false;
  private boolean endIsMilestone = false;

  /**
   * BEGIN FINANTIAL PROPERTIES
   */
  private double forecasted;
  private double effective;
  private Set<Cost> costs = new HashSet<Cost>();

  private String customField1;
  private String customField2;
  private String customField3;
  private String customField4;
  private String customField5;
  private String customField6;



  public Iterator getIssuesIterator() {
    return issues.iterator();
  }

  public boolean issuesContains(Issue issue) {
    return issues.contains(issue);
  }

  public int issuesSize() {
    return issues.size();
  }

  /**
   * task manager sets it by hand
   */
  public void setTaskProgress(double progress) {
  }

  /**
   *
   */
  public double getTaskProgress() {
    return 0;
  }

  public Period getSchedule() {
    return schedule;
  }

  public void setSchedule(Period schedule) {
    this.schedule = schedule;
  }

  public double getProgress() {
    return progress;
  }

  public void setProgress(double progress) {
    this.progress = progress;
  }

  public Set getDocuments() {
    return documents;
  }

  private void setDocuments(Set documents) {
    this.documents = documents;
  }

  public void addDocument(TeamworkDocument document) {
    documents.add(document);
  }

  public void removeDocument(TeamworkDocument document) {
    documents.remove(document);
  }

  public Iterator<TeamworkDocument> getDocumentsIterator() {
    return documents.iterator();
  }

  public Set<Assignment> getAssignments() {
    return assignments;
  }

  private void setAssignments(Set<Assignment> assignments) {
    this.assignments = assignments;
  }

  public Iterator<Assignment> getAssignementsIterator() {
    return getAssignments().iterator();
  }

  public List<Assignment> getAssignementsSortedByRole() throws FindException {
    String hql = "from " + Assignment.class.getName() + " as assig where assig.task = :myself order by assig.role.name, assig.resource.name";
    OqlQuery oql = new OqlQuery(hql);
    oql.getQuery().setEntity("myself", this);
    return oql.list();

  }


  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public String getCode() {
    return code;
  }

  public void setCode(String code) {
    this.code = code;
  }

  public TaskType getType() {
    return type;
  }

  public void setType(TaskType type) {
    this.type = type;
  }

  public String getStatus() {
    return status;
  }

  public void setStatus(String status) {
    this.status = status;
  }

  public int getRelevance() {
    return relevance;
  }

  public void setRelevance(int relevance) {
    this.relevance = relevance;
  }

  public Set<Issue> getIssues() {
    return issues;
  }

  private void setIssues(Set<Issue> issues) {
    this.issues = issues;
  }

  public boolean isLogical() {
    return logical;
  }

  public void setLogical(boolean logical) {
    this.logical = logical;
  }

  public boolean isStandAlone() {
    return standAlone;
  }

  public void setStandAlone(boolean standAlone) {
    this.standAlone = standAlone;
  }

  public Set<TaskScheduleHistory> getScheduleHistory() {
    return scheduleHistory;
  }

  private void setScheduleHistory(Set<TaskScheduleHistory> scheduleHistory) {
    this.scheduleHistory = scheduleHistory;
  }

  public void addScheduleHistory(TaskScheduleHistory history) {
    scheduleHistory.add(history);
  }

  public void removeScheduleHistory(TaskScheduleHistory history) {
    scheduleHistory.remove(history);
  }

  public Iterator<TaskScheduleHistory> getTaskScheduleHistoryIterator() {
    return scheduleHistory.iterator();
  }

  public boolean scheduleHistoryContains(TaskScheduleHistory history) {
    return scheduleHistory.contains(history);
  }

  public int scheduleHistorySize() {
    return scheduleHistory.size();
  }

  public Set<TaskStatusHistory> getStatusHistory() {
    return statusHistory;
  }

  private void setStatusHistory(Set<TaskStatusHistory> statusHistory) {
    this.statusHistory = statusHistory;
  }

  public void addStatusHistory(TaskStatusHistory history) {
    statusHistory.add(history);
  }

  public void removeStatusHistory(TaskStatusHistory history) {
    statusHistory.remove(history);
  }

  public Iterator<TaskStatusHistory> getStatusHistoryIterator() {

    /*List sortedSH = new ArrayList(statusHistory);
    Collections.sort(sortedSH, new Comparator() {
      public int compare(Object o1, Object o2) {
        TaskStatusHistory rd1 = (TaskStatusHistory) o1;
        TaskStatusHistory rd2 = (TaskStatusHistory) o2;
        //as sql.Timestamp and util.Date may be given
        return new Long(rd1.getLastModified().getTime()).compareTo(rd2.getLastModified().getTime());

      }
    });
    return sortedSH.iterator();*/
    return statusHistory.iterator();
  }

  public boolean statusHistoryContains(TaskStatusHistory history) {
    return statusHistory.contains(history);
  }

  public int statusHistorySize() {
    return statusHistory.size();
  }


  public void setName(String name) {
    this.name = name;
  }

  public String getName() {
    return name;
  }


  public Set<TaskClassificationAssociation> getTaskClassificationAssociations() {
    return taskClassificationAssociations;
  }

  private void setTaskClassificationAssociations(Set<TaskClassificationAssociation> taskClassificationAssociations) {
    this.taskClassificationAssociations = taskClassificationAssociations;
  }

  public Iterator<TaskClassification> getClassificationIterator() {

    Set<TaskClassification> tc = new HashSet<TaskClassification>();
    for (TaskClassificationAssociation tca : getTaskClassificationAssociations()) {
      tc.add(tca.getClassification());
    }
    return tc.iterator();
  }

//  protected Set<TaskDependency> getPreviouses() {

  public Set<TaskDependency> getPreviouses() {
    return previouses;
  }

  private void setPreviouses(Set<TaskDependency> previouses) {
    this.previouses = previouses;
  }

  public Iterator<TaskDependency> getPreviousesModifiableIterator() {
    return new HashSet(previouses).iterator();
  }

  public ForumEntry getForumEntry() {
    if (forumEntry == null)
      forumEntry = new ForumEntry();
    return forumEntry;
  }

  public void setForumEntry(ForumEntry forumEntry) {
    this.forumEntry = forumEntry;
  }

  public String getGoals() {
    return goals;
  }

  public void setGoals(String goals) {
    this.goals = goals;
  }

  public String getBenefits() {
    return benefits;
  }

  public void setBenefits(String benefits) {
    this.benefits = benefits;
  }

  public String getExternalCode() {
    return externalCode;
  }

  public void setExternalCode(String externalCode) {
    this.externalCode = externalCode;
  }

  public String getCostCenter() {
    return costCenter;
  }

  public void setCostCenter(String costCenter) {
    this.costCenter = costCenter;
  }

  public String getPro() {
    return pro;
  }

  public void setPro(String pro) {
    this.pro = pro;
  }

  public String getContra() {
    return contra;
  }

  public void setContra(String contra) {
    this.contra = contra;
  }

  public String getCommentOnClose() {
    return commentOnClose;
  }

  public void setCommentOnClose(String commentOnClose) {
    this.commentOnClose = commentOnClose;
  }

  public boolean isStartIsMilestone() {
    return startIsMilestone;
  }

  public void setStartIsMilestone(boolean startIsMilestone) {
    this.startIsMilestone = startIsMilestone;
  }

  public boolean isEndIsMilestone() {
    return endIsMilestone;
  }

  public void setEndIsMilestone(boolean endIsMilestone) {
    this.endIsMilestone = endIsMilestone;
  }

  public int getAssignementsSize() {
    return getAssignments().size();
  }

  public int getIssuesSize() {
    return getIssues().size();
  }

  public String getNotes() {
    return notes;
  }

  public void setNotes(String notes) {
    this.notes = notes;
  }

  public Set<TaskDependency> getNexts() {
    return nexts;
  }


  public void setNexts(Set<TaskDependency> nexts) {
    this.nexts = nexts;
  }

  public int getPreviousesSize() {
    return previouses.size();
  }

  public int getNextsSize() {
    return nexts.size();
  }

  public double getForecasted() {
    return forecasted;
  }

  public void setForecasted(double forecasted) {
    this.forecasted = forecasted;
  }

  public double getEffective() {
    return effective;
  }

  public void setEffective(double effective) {
    this.effective = effective;
  }


  public Set<Cost> getCosts() {
    return costs;
  }

  public void setCosts(Set<Cost> costs) {
    this.costs = costs;
  }

  public boolean isProgressByWorklog() {
    return progressByWorklog;
  }

  public void setProgressByWorklog(boolean progressByWorklog) {
    this.progressByWorklog = progressByWorklog;
  }



  public int getDuration() {
    return duration;
  }

  public void setDuration(int durationInWorkingDays) {
    this.duration = durationInWorkingDays;
  }

  public String getCustomField1() {
    return customField1;
  }

  public void setCustomField1(String customField1) {
    this.customField1 = customField1;
  }

  public String getCustomField2() {
    return customField2;
  }

  public void setCustomField2(String customField2) {
    this.customField2 = customField2;
  }

  public String getCustomField3() {
    return customField3;
  }

  public void setCustomField3(String customField3) {
    this.customField3 = customField3;
  }

  public String getCustomField4() {
    return customField4;
  }

  public void setCustomField4(String customField4) {
    this.customField4 = customField4;
  }

  public String getCustomField5() {
    return customField5;
  }

  public void setCustomField5(String customField5) {
    this.customField5 = customField5;
  }

  public String getCustomField6() {
    return customField6;
  }

  public void setCustomField6(String customField6) {
    this.customField6 = customField6;
  }
}
