package com.twproject.two.agenda;

import com.twproject.two.document.DocumentOld;
import com.twproject.two.ontology.GroupOld;
import com.twproject.two.resource.OperatorOld;
import com.twproject.two.security.AreaOld;
import com.twproject.two.task.AssignementOld;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * Models the notion of event in time.Events are persistent
 * and collectable by groups. They tipically concern a human operator,
 * but as notion they cover also automation ends.
 */
public class EventOld {

  private String id;
  private OperatorOld owner;
  private OperatorOld author;
  private ScheduleSupportOld schedule;
  private String location;
  private boolean personal;
  private int type;
  private int status;
  private String description;
  private DocumentOld content;
  private Set groups;
  private boolean deleted;
  private Date lastModified;

  private Set documents;

  private AssignementOld assignement;

  public static final String PACKAGE_PREFIX = "com.twproject.agenda";
  public static final String AGENDAEVENT = PACKAGE_PREFIX + ".AGENDAEVENT";

  public EventOld() {
  }

  public EventOld(String aeId) {
    setId(aeId);
  }

  public void setArea(AreaOld area, boolean inherithFromParent, boolean propagate) {
  }

  public String getId() {
    return id;
  }

  public ScheduleSupportOld getSchedule() {
    return schedule;
  }

  public int getType() {
    return type;
  }

  /**
   * Not in use yet.
   */
  public void setType(int type) {
    this.type = type;
  }

  public int getStatus() {
    return status;
  }

  /**
   * Not in use yet.
   */
  public void setStatus(final int status) {
    this.status = status;
  }

  public DocumentOld getContent() {
    return content;
  }

  /**
   * Not in use yet.
   */
  public void setContent(DocumentOld content) {
    this.content = content;
  }

  public void setId(String id) {
    this.id = id;
  }

  /**
   * Permits to calculate the time intersections of it.
   */
  public void setSchedule(ScheduleSupportOld schedule) {
    this.schedule = schedule;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public String getLocation() {
    return location;
  }

  /**
   * Where it will take place.
   */
  public void setLocation(String location) {
    this.location = location;
  }

  public Set getGroups() {
    return groups;
  }

  public void setGroups(Set groups) {
    this.groups = groups;
  }

  public void addGroup(GroupOld groupOld) {
    if (groups == null) {
      groups = new HashSet();
    }
    groups.add(groupOld);
  }

  public void removeGroup(GroupOld groupOld) {
    if (groups != null) {
      groups.remove(groupOld);
    }
  }

  public boolean isPersonal() {
    return personal;
  }

  /**
   * Whether it is visible to non-owners.
   */
  public void setPersonal(boolean personal) {
    this.personal = personal;
  }

  public OperatorOld getOwner() {
    return owner;
  }

  public OperatorOld getAuthor() {
    return author;
  }
  /**
   * Is the creator of the event; typically a logged operator.
   * Notice that events may be produced by the application,
   * not necessarily one by the logged operator.
   */
  public void setOwner(OperatorOld owner) {
    this.owner = owner;
  }

  public boolean isDeleted() {
    return deleted;
  }

  public void setDeleted(boolean deleted) {
    this.deleted = deleted;
  }

  public Date getLastModified() {
    return lastModified;
  }

  public void setLastModified(Date lastModified) {
    this.lastModified = lastModified;
  }

  public OperatorOld OwnergetAuthor() {
    return author;
  }

  public void setAuthor(OperatorOld author) {
    this.author = author;
  }

  public Set getDocuments() {
    return documents;
  }

  public void setDocuments(Set documents) {
    this.documents = documents;
  }

  public static EventOld newInstanceWithScheduleNull(EventOld e) {

    EventOld newEventOld = new EventOld();
    newEventOld.owner = e.owner;
    newEventOld.author = e.author;
    newEventOld.schedule = null;
    newEventOld.location = e.location;
    newEventOld.personal = e.personal;
    newEventOld.type = e.type;
    newEventOld.status = e.status;
    newEventOld.description = e.description;
    newEventOld.content = e.content;
    newEventOld.groups = e.groups;
    newEventOld.deleted = e.deleted;
    newEventOld.lastModified = e.lastModified;
    newEventOld.documents = e.documents;
    return newEventOld;
  }

  public AssignementOld getAssignement() {
    return assignement;
  }

  public void setAssignement(AssignementOld assignement) {
    this.assignement = assignement;
  }

}
