package com.twproject.two.agenda;


import org.jblooming.ApplicationException;
import org.jblooming.agenda.CompanyCalendar;
import org.jblooming.utilities.DateUtilities;

import java.util.*;


public class PeriodOld extends ScheduleSupportOld {

  public static final String DEFAULT_DATE_FORMAT = "DEFAULT_DATE_FORMAT";
  public static final String DEFAULT_DATE_DISPLAY_FORMAT = "DEFAULT_DATE_DISPLAY_FORMAT";

  public PeriodOld() {
  }

  public PeriodOld(String id) {
    super(id);
  }

  public PeriodOld(long start, long end) {
    this(new Date(start), new Date(end));
  }


  public PeriodOld(Date start, Date end) {
    this.setStart(start);
    this.setEnd(end);

    CompanyCalendar cal = new CompanyCalendar();

    if (this.start != null) {
      cal.setTime(this.start);
      setStartMillis(cal.getMillisFromMidnight());
    }

    if (this.end != null) {
      cal.setTime(this.end);
      setEndMillis(cal.getMillisFromMidnight());
    }
  }

  /*
   * @param Date start specifies the initial date (no meaning for time)
   * @param Date end   specifies the final date (no meaning for time)
   * @param long st    specifies the start time in milliseconds from midnight
   * @param long et    specifies the end   time in milliseconds from midnight
   */
  public PeriodOld(Date start, Date end, int st, int et) {
    this(start, end);
    setStartMillis(st);
    setEndMillis(et);
  }

  public static PeriodOld getDayPeriodInstance(Date aDate) {
    CompanyCalendar cc = new CompanyCalendar();
    cc.setTime(aDate);
    return new PeriodOld(cc.setAndGetTimeToDayStart(), cc.setAndGetTimeToDayEnd());
  }

  public long getValidityStart() {
    if (this.start == null)
      return CompanyCalendar.MIN_DATE.getTime();
    return this.getStart().getTime();
  }

  public long getValidityEnd() {
    if (this.end == null)
      return CompanyCalendar.MAX_DATE.getTime();
    return this.getEnd().getTime();
  }

  // specifies the event frequency; 0 = one shoot
  public void setFrequency(int f) {
  }

  public int getFrequency() {
    return 0;
  }

  // specifies the number of cycles before expires
  public void setRepetitions(int vc) {
  }

  public int getRepetitions() {
    // a Period is repeated always once
    return 1;
  }

  /*
  public String getId(){
    return id;
  }
   */
  public boolean contains(Date d) {
    if (getStart() == null || d == null || getEnd() == null)
      return false;
    return (getStart().getTime() <= d.getTime()) && (d.getTime() <= getEnd().getTime());
  }

  /*
  public boolean isIn(Period p){
    return true;
  }
   */

  public long getDuration() {
    return getEnd().getTime() - getStart().getTime();
  }

  /**
   * create a new Period as intersetion with the passed one
   *
   * @param p the intersecting Period
   * @return null if no intersection
   */
  public PeriodOld intersection(PeriodOld p) {
    PeriodOld result = null;
    if (!((p.getValidityEnd() < this.start.getTime()) || (p.getValidityStart() > this.end.getTime()))) {
      result = new PeriodOld(Math.max(this.start.getTime(), p.start.getTime()), Math.min(this.end.getTime(), p.end.getTime()));
    }
    return result;
  }


  public Collection getPeriods(PeriodOld p) throws ApplicationException {
    List returnedPeriods = new ArrayList();
    if (!(p.getValidityEnd() < CompanyCalendar.MAX_DATE.getTime() && p.getValidityStart() > CompanyCalendar.MIN_DATE.getTime()))
      throw new ApplicationException("Invalid period specification");
    PeriodOld ret = this.intersection(p);
    if (ret == null)
      return Collections.EMPTY_LIST;
    returnedPeriods.add(ret);
    return returnedPeriods;
  }


  public Collection getPeriodsold(PeriodOld p) throws ApplicationException {
    if (!((p.getValidityEnd() < this.start.getTime()) || (p.getValidityStart() > this.end.getTime()))) {
      List returnedPeriods = new ArrayList();
      //test for +/- infinity
      if (p.getValidityEnd() < CompanyCalendar.MAX_DATE.getTime() && p.getValidityStart() > CompanyCalendar.MIN_DATE.getTime()) {
        CompanyCalendar st = new CompanyCalendar();
        CompanyCalendar en = new CompanyCalendar();

        st.setTime(new Date(p.getStart().getTime() - p.getStartMillis() + this.getStartMillis()));

        en.setTime(st.getTime());

        en.set(Calendar.HOUR_OF_DAY, 0);
        en.clear(en.MINUTE);
        en.clear(en.SECOND);
        en.clear(en.MILLISECOND);
        en.add(en.MILLISECOND, getEndMillis());

        // loop inside the period
        if (!en.getTime().after(p.getEnd())) {
          PeriodOld p2 = new PeriodOld(st.getTime(), en.getTime());
          returnedPeriods.add(p2);
        }

      } else {
        throw new ApplicationException("Invalid period specification");
      }
      //returnedPeriods.add(this);
      return returnedPeriods;
    } else {
      return Collections.EMPTY_LIST;
    }
  }

  public int lenghtInDays() {
    return (new Long(getDuration() / (1000L * 3600L * 24L))).intValue();
  }

  /**
   * returns the period starting at 00.00 aTime of day aTime, ending at 23.59.59.999
   *
   * @param aTime
   */

  public static PeriodOld getFullDay(Date aTime) {
    Calendar cal = new CompanyCalendar();

    cal.setTime(aTime);
    //cal.clear(cal.HOUR_OF_DAY);
    cal.set(cal.HOUR_OF_DAY, 0);
    cal.clear(cal.MINUTE);
    cal.clear(cal.SECOND);
    cal.clear(cal.MILLISECOND);

    Date start = cal.getTime();

    cal.set(cal.HOUR_OF_DAY, 23);
    cal.set(cal.MINUTE, 59);
    cal.set(cal.SECOND, 59);
    cal.set(cal.MILLISECOND, 999);

    Date end = cal.getTime();

    return new PeriodOld(start, end);
  }

  public String toString() {
    return DateUtilities.dateToString(start, "yyyy MM dd HH:mm:ss") + " - " + DateUtilities.dateToString(end, "yyyy MM dd HH:mm:ss");
  }

}
