package com.twproject.two.agenda;


import org.jblooming.ApplicationException;
import org.jblooming.agenda.CompanyCalendar;

import java.util.*;


public class ScheduleDailyOld extends ScheduleSupportOld {

  /*
  protected Date start;
  protected Date end;

  protected int startTime;                      // Milliseconds elapsed from midnight
  protected int endTime;                        // Milliseconds elapsed from midnight
   */
  private int freq = 1;                 // how many days the event is repeated
  private int repeat = 0;                // number of times the event is repeated
  protected boolean onlyWorkingDays = false;    // whether the event is repeated only on working days

  /*
  public    String     id;
   */

  public ScheduleDailyOld() {
  }


  /*
   * @param Date    start              specifies the initial validity date (no meaning for time)
   * @param Date    end                specifies the final   validity date (no meaning for time)
   * @param int     startTime          specifies the start time in milliseconds from midnight
   * @param int     endTime            specifies the end   time in milliseconds from midnight
   * @param int     frequency          specifies the frequency (how many days) the event is repeated
   * @param int     validityForCycles  specifies how many times the event is repeated
   * @param boolean onlyWorkingDays    specifies if repeates only on the workin days
   */
  public ScheduleDailyOld(String i, PeriodOld p) {
    this(i,
        new Date(p.getValidityStart()),
        new Date(p.getValidityEnd()),
        p.getStartMillis(),
        p.getEndMillis());
  }

  public ScheduleDailyOld(String i) {
    this(i, CompanyCalendar.MIN_DATE, CompanyCalendar.MAX_DATE, CompanyCalendar.MIN_TIME, CompanyCalendar.MAX_TIME);
  }

  public ScheduleDailyOld(String i, Date s) {
    this(i, s, CompanyCalendar.MAX_DATE, CompanyCalendar.MIN_TIME, CompanyCalendar.MAX_TIME);
  }

  public ScheduleDailyOld(String i, Date s, Date e) {
    this(i, s, e, CompanyCalendar.MIN_TIME, CompanyCalendar.MAX_TIME);
  }

  public ScheduleDailyOld(String i, Date s, Date e, int st, int et) {
    this(i, s, e, st, et, 1, 1, false);
  }

  public ScheduleDailyOld(String pId, Date start, Date end, int startTime, int endTime, int freq, int rep, boolean onlyWorkingDays) {
    this.setStart(start);
    this.setEnd(end);
    this.setId(pId);

    setFrequency(freq);
    setStartMillis(startTime);
    setEndMillis(endTime);
    setRepetitions(rep);
    setOnlyWorkingDays(onlyWorkingDays);
  }

  /*
   * This method return the validity period.
   * The period is trimmed to the starting time and the ending time.
   * There is no test for working days validity (eg. for example if  the validity period starts on sat at 8.00
   * and you choos working days only, the validity period start on sat at 8.00 NOT on mon 8.00)
   */
  public PeriodOld getValidityPeriod() {
    return new PeriodOld(getStart(), getEnd());
  }


  /*
   *  Sets the event frequency in days; 0 = one shoot
   */
  public void setFrequency(int f) {
    freq = f > 0 ? f : 1;
    setRepetitions(repeat); // in this way the endpoints are synchronized
  }

  /*
   *  Return the event frequency in days; 0 = one shoot
   */
  public int getFrequency() {
    return freq;
  }


  public void setOnlyWorkingDays(boolean f) {
    onlyWorkingDays = f;
  }

  public boolean getOnlyWorkingDays() {
    return onlyWorkingDays;
  }


  /**
   * specifies the number of cycles before expires
   * If rep is >=1 the end date is recalculated using start+frequency*(repetition-1)
   * repetitions=0 means forever (the end date is unchanged)
   */
  public void setRepetitions(int rep) {

    int originalEndTime = getEndMillis();
    Calendar cal;
    if (rep > 0) {
      repeat = rep;
      cal = new CompanyCalendar();
      cal.setTime(getStart());
      cal.add(cal.DATE, (repeat - 1) * freq);

    } else {
      repeat = 0;
      cal = new CompanyCalendar();
      cal.setTime(CompanyCalendar.MAX_DATE);
      cal.set(Calendar.HOUR_OF_DAY, 0);
      cal.clear(cal.MINUTE);
      cal.clear(cal.SECOND);
      cal.clear(cal.MILLISECOND);
      cal.add(cal.MILLISECOND, getEndMillis());
    }
    setEnd(cal.getTime());
    setEndMillis(originalEndTime);

  }


  public int getRepetitions() {
    return repeat;
  }


  /*
   * Return true if the specified date is in ANY of schedule; false oterwhise
   *
   */
  public boolean contains(Date d) {
    long distInMillisec;
    int distInDays;

    // tests if the date is in the validity period
    if (!getValidityPeriod().contains(d))
      return false;

    // is is not cyclic
    if (freq == 0)
      return true;

    // test for working days
    CompanyCalendar cal = new CompanyCalendar();
    cal.setTime(d);
    if (onlyWorkingDays && (cal.isWorkingDay()))
      return false;

    //normalize dist
    distInMillisec = d.getTime() - getStart().getTime();
    distInDays = (int) (distInMillisec + CompanyCalendar.MAX_TIME - 1) / CompanyCalendar.MAX_TIME;

    // test if the day is candidate using module computation
    if (distInDays % freq != 0)
      return false;

    Date s, e;
    cal.setTime(getStart());
    cal.roll(cal.DATE, distInDays);
    s = cal.getTime();

    cal.roll(cal.MILLISECOND, getEndMillis() - getStartMillis());
    e = cal.getTime();

    return (s.getTime() <= d.getTime()) && (d.getTime() <= e.getTime());
  }

  /**
   * This method return true when the passed period is
   * overlapped (partially o complete) on the validity period, even if actually, it is not, looking on days.
   */
  public boolean isIn(PeriodOld p) {
    return !((getEnd().getTime() < p.getValidityEnd()) || (getStart().getTime() > p.getValidityEnd()));
  }

  public long getDuration() {
    if (getStart().getTime() == CompanyCalendar.MIN_DATE.getTime() || getEnd().getTime() == CompanyCalendar.MAX_DATE.getTime())
      return Long.MAX_VALUE;
    return getEnd().getTime() - getStart().getTime();
  }

  public Collection getPeriods(PeriodOld p) throws ApplicationException {
    if (!((p.getValidityEnd() < this.start.getTime()) || (p.getValidityStart() > this.end.getTime()))) {
      int increment;
      increment = freq > 0 ? freq : 1;
      List returnedPeriods = new ArrayList();
      //test for +/- infinity
      if (p.getValidityEnd() < CompanyCalendar.MAX_DATE.getTime() && p.getValidityStart() > CompanyCalendar.MIN_DATE.getTime()) {
        CompanyCalendar st = new CompanyCalendar();
        CompanyCalendar en = new CompanyCalendar();

        st.setTime(new Date(// sets start to the first day of required period, with millis of this period
            //p.getStart().getTime()-p.getStartTimeInMillis()+this.getStartTimeInMillis()
            this.getStart().getTime()));

        en.setTime(st.getTime());
        en.setMillisFromMidnight(getEndMillis());

        // loop inside the period
        for (int i = 0; !en.getTime().after(p.getEnd()) && (i < repeat || repeat == 0); i++) {
          if (p.contains(en.getTime()) || p.contains(st.getTime())) {
            PeriodOld p2 = new PeriodOld(st.getTime(), en.getTime());
            // test for working days
            if (onlyWorkingDays) {
              if (st.isWorkingDay() && en.isWorkingDay())
                returnedPeriods.add(p2);
            } else {
              returnedPeriods.add(p2);
            }
          }
          st.add(CompanyCalendar.DAY_OF_YEAR, increment);
          en.add(CompanyCalendar.DAY_OF_YEAR, increment);
        }

      } else {
        throw new ApplicationException("Invalid period specification");
      }
      return returnedPeriods;
    } else {
      return Collections.EMPTY_LIST;
    }


  }

  protected int getFreq() {
    return freq;
  }

  protected void setFreq(int freq) {
    this.freq = freq;
  }

  protected int getRepeat() {
    return repeat;
  }

  protected void setRepeat(int repeat) {
    this.repeat = repeat;
  }
}

