package com.twproject.two.agenda;


import org.jblooming.ApplicationException;
import org.jblooming.agenda.CompanyCalendar;

import java.util.Collection;
import java.util.Date;


/**
 * 1) usato in Task (cyclePeriod) per indicare il periodo su cui questo task ricorre.
 * Si intende che non debba essere solo una data di inizio e fine, ma qualcosa
 * di pi� articolato (pi� inizio/fine). Usato anche in TaskFilter per le selezioni
 * <p/>
 * 2) in ResourceConstants (unavailability) per indicare quando la risorsa non � disponibile
 * (nome di giorni, giorni specifici, inizio/fine, etc. di non disponibilit�).
 * Usato anche in ResourceFilter per le selezioni.
 * <p/>
 * 3) usato in Calendar (holidays) per indicare i giorni non lavorativi. L'attributo "holidays"
 * � usato dal metodo getWorkingDaysCountInPeriod(Period period) per sapere quanti giorni
 * lavorativi ci sono nel periodo in argomento.
 * <p/>
 * 4) WorkLog (attributi di tipo Date start/end) e WorkLogFilter
 * <p/>
 * 5) In ResourceTimeCounter (metodo getTiming) viene usato per costruire lo start/end di un WorkLog
 * di una risorsa
 * <p/>
 * <p/>
 * <p/>
 * forse servirebbero dei metodi per intersecare, unire e complementare periodi
 * <p/>
 * <p/>
 * <p/>
 * L'idea � di avere 5 tipologie di ricorrenza:
 * 1) non ricorrente
 * 2) giornaliero (ogni quanti giorni)
 * 3) settimanale (LMMGVSD e ogni quante settimane)
 * 4) mensile (giorno x del mese o ogni x LMMGVSDFerFest) ogni quanti mesi
 * 5) annuale (ogni giorno x del mese y o ogni x LMMGVSDFerFest del  mese y)
 * <p/>
 * Tutte dovrebbero implementare almeno isIn(Date)
 * altro metodo utile Collection explodeRecurrency(Date start, Date end) che restituisce l'insieme di coppie (inizio,fine)
 * di cui � composto il Period tra le date specificate (metodo utile per disegnare di stonfo il calendario).
 */

public class ScheduleSupportOld  {

  private String id;

  protected Date start;
  protected Date end;

  private int startTime; // Milliseconds elapsed from midnight
  private int endTime;   // Milliseconds elapsed from midnight

  private boolean deleted;
  private Date lastModified;

  public ScheduleSupportOld() {
  }

  public ScheduleSupportOld(String id) {
    this.id = id;
  }

  public String getId() {
    return id;
  }

  public int getStartMillis() {
    return getStartTime();
  }

  public void setStartMillis(int startTime) {
    this.setStartTime(startTime);
    CompanyCalendar cal = new CompanyCalendar();
    cal.setTime(this.getStart());
    cal.setMillisFromMidnight(startTime);
    this.setStart(cal.getTime());
  }

  public int getEndMillis() {
    return getEndTime();
  }

  public void setEndMillis(int endTime) {
    this.setEndTime(endTime);
    CompanyCalendar cal = new CompanyCalendar();
    cal.setTime(this.getEnd());
    cal.setMillisFromMidnight(endTime);
    this.setEnd(cal.getTime());
  }

  //
  /**
   * specifies the event frequency;
   *
   * @deprecated use one of the extending classes
   */
  public void setFrequency(int f) {

  }

  /**
   * @deprecated use one of the extending classes
   */
  public int getFrequency() {
    return 0;
  }


  /**
   * Specifies the number of cycles before expires
   * If cycle is >=1 the end date is recalculated using start+frequency*(repetition-1)
   * repetitions=0 means forever (the end date is moved to MAXDATE)
   */
  public void setRepetitions(int cycle) {

  }


  /**
   * @return true if there is an overlapping betwheen periods
   */
  public boolean overlap(PeriodOld p) {
    return !(this.getStart().after(p.getEnd()) || this.getEnd().before(p.getStart()));
  }


  public Date getStart() {
    return start;
  }

  /**
   * Sets the start date. If greater of end date swap.
   */
  public void setStart(Date start) {
    if (start == null || (this.end == null || this.end.getTime() >= start.getTime()))
      this.start = start;
    else {
      this.start = this.end;
      this.end = start;
    }
  }

  public Date getEnd() {
    return end;
  }

  /**
   * Sets the end date. If lower of start date swap.
   */
  public void setEnd(Date end) {

    if (end == null || (this.start == null || this.start.getTime() <= end.getTime()))
      this.end = end;
    else {
      this.end = this.start;
      this.start = end;
    }
  }

  public void setId(String id) {
    this.id = id;
  }

  //to be used only by persistence
  protected int getStartTime() {
    return startTime;
  }

  protected void setStartTime(int startTime) {
    this.startTime = startTime;
  }

  protected int getEndTime() {
    return endTime;
  }

  protected void setEndTime(int endTime) {
    this.endTime = endTime;
  }

  /**
   * Used to get the smaller period that wraps the whole recurrent set.
   * In case of not recurrent a new period with same ends is returned.
   */
  public PeriodOld getPeriod() {
    return new PeriodOld(this.getStart(), this.getEnd());
  }

  public boolean isDeleted() {
    return deleted;
  }

  public void setDeleted(boolean deleted) {
    this.deleted = deleted;
  }

  public Date getLastModified() {
    return lastModified;
  }

  public void setLastModified(Date lastModified) {
    this.lastModified = lastModified;
  }

  /**
   * fake implementation in order to get compatibility with CGLIB
   *
   * @param date
   */
  public boolean contains(Date date) {
    return false;
  };

  /**
   * fake implementation in order to get compatibility with CGLIB
   *
   */
  public long getDuration() {
    return 0;
  }

  /**
   * fake implementation in order to get compatibility with CGLIB
   *
   * @param p
   */
  public Collection getPeriods(PeriodOld p) throws ApplicationException {
    return null;
  };

  /**
   * fake implementation in order to get compatibility with CGLIB
   *
   */
  public int getRepetitions() {
    return 0;
  }


}
