package com.twproject.two.document;


import com.twproject.two.ontology.SecuredSupportOld;
import com.twproject.two.resource.OperatorOld;

import java.util.*;


/**
 * <code>TeamworkDocument</code> formalises an extended document concept
 * e.g. papers, images, videos, .. .
 * <code>author</code> the author of the original document
 * <code>creator</code> the operator that creates the doc in teamwork
 * <code>lockedBy</code> if null the document is unlocked
 * <code>keywords</code> list of Strings
 * <code>authored</code> date of original creation
 * <code>created</code> date of insertion in teamwork
 * <code>lastModified</code> date last update in teamwork
 * the implementation of interface OperatorAggregator is related to the users that
 * are contained in the CredentialWallet
 *
 * @author Pietro Polsinelli
 * @author Roberto Bicchierai
 * @version 2 alpha
 * @since JDK 1.3
 */
public class DocumentOld extends SecuredSupportOld {
  private String code;
  protected String url;
  protected String summa;
  private String kind;
  private String content;
  private String mimeType;
  private String versionlabel;
  protected String version;
  private String author;
  private Set toBeReadBy;
  private Set readBy;
  private OperatorOld creator;
  protected OperatorOld lockedBy;
  private DocumentOld parent;
  private Set children;
  protected Set keywords;
  private Date authored;
  private Date created;
  private int type;
  private OperatorOld lastModifier;
  private long sizeInBytes;
  private boolean template = false;
  private String baseClassification;
  private boolean storage;
  private boolean index;
  private boolean inherit;
  private boolean propagate = true;

  private String connectionHost;
  private String connectionUser;
  private String connectionPwd;
  private String connectionNotes;
  private String connectionType;

  public static final String package_prefix = "com.twproject.document";
  public static final String DOCUMENT = package_prefix + ".DOCUMENT";
  public static final String DOCUMENT_CHILDREN = package_prefix + ".DOCUMENT_CHILDREN";
  public static final String DOCUMENT_NAME_EXISTING = package_prefix + ".document_name_existing";

  public static final int IS_STRUCTURAL = 0;
  public static final int IS_UPLOAD = 1;
  public static final int IS_CONTENT = 2;
  public static final int IS_URL = 3;

  public static final String CONNECTION_FILE_SYSTEM = DOCUMENT + "CONNECTION_FILE_SYSTEM";
  public static final String CONNECTION_FTP = DOCUMENT + "CONNECTION_FTP";

  public DocumentOld() {
    super();
  }

  public DocumentOld(String dId) {
    super(dId);
  }

  /**
   * Method setUrl
   *
   * @param url a  String
   */
  public void setUrl(String url) {
    this.url = url;
  }

  /**
   * Method getUrl
   *
   * @return a String
   */
  public String getUrl() {
    return url;
  }

  /**
   * Method setVersion
   *
   * @param version a  String
   */
  public void setVersion(String version) {
    this.version = version;
  }

  /**
   * Method getVersion
   *
   * @return a String
   */
  public String getVersion() {
    return version;
  }

  /**
   * Method setKind
   *
   * @param kind a  String
   */
  public void setKind(String kind) {
    this.kind = kind;
  }

  /**
   * Method getKind
   *
   * @return a String
   */
  public String getKind() {
    return kind;
  }

  /**
   * Method setSumma
   *
   * @param summa a  String
   */
  public void setSumma(String summa) {
    this.summa = summa;
  }

  /**
   * Method getSumma
   *
   * @return a String
   */
  public String getSumma() {
    return summa;
  }

  /**
   * Method setKeywords
   *
   * @param keywords a  Collection of string
   */
  public void setKeywords(Set keywords) {
    this.keywords = keywords;
  }

  /**
   * Method getKeywords
   *
   * @return a Collection of string
   */
  public Set getKeywords() {
    return keywords;
  }

  /**
   * Method setLockedBy
   *
   * @param lockedBy an Operator
   */
  public void setLockedBy(OperatorOld lockedBy) {
    this.lockedBy = lockedBy;
  }

  /**
   * Method getLockedBy
   *
   * @return an Operator
   */
  public OperatorOld getLockedBy() {
    return lockedBy;
  }

  public Collection getOperators() {
    return getOperators(new HashSet(), new HashSet());
  };

  private Collection getOperators(Collection visitedNodes, Collection operators) {

    if (!visitedNodes.contains(this)) {
      if (getCreator() != null)
        operators.add(getCreator());
      if (getLockedBy() != null)
        operators.add(getLockedBy());
      if (getLastModifier() != null)
        operators.add(getLastModifier());

      visitedNodes.add(this);

      // visiting parent node

      if (getInherit() && getParent() != null) {
        operators.addAll(((DocumentOld) getParent()).getOperators(visitedNodes, operators));
      }

      // visiting children
      if (getPropagate()) {
        Iterator iter = getChildrenIterator();
        DocumentOld r;
        while (iter.hasNext()) {
          r = (DocumentOld) iter.next();
          operators.addAll(r.getOperators(visitedNodes, operators));
        }
      }
    }
    return operators;
  }

  public Iterator getChildrenIterator() {
   if (getChildren() != null) return getChildren().iterator();
   return null;
 }

  public void setPropagate(boolean propagate) {
    this.propagate = propagate;
  }

  public boolean getPropagate() {
    return propagate;
  }

  public void setInherit(boolean inherit) {
    this.inherit = inherit;
  }

  public boolean getInherit() {
    return inherit;
  }


  public boolean isOperatorIn(OperatorOld o) {
    return getOperators().contains(o);
  }

  public String getAuthor() {
    return author;
  }

  public void setAuthor(String author) {
    this.author = author;
  }

  public Set getToBeReadBy() {
    return toBeReadBy;
  }

  public void setToBeReadBy(Set toBeReadBy) {
    this.toBeReadBy = toBeReadBy;
  }

  public Set getReadBy() {
    return readBy;
  }

  public void setReadBy(Set readBy) {
    this.readBy = readBy;
  }

  public String getVersionlabel() {
    return versionlabel;
  }

  public void setVersionlabel(String versionlabel) {
    this.versionlabel = versionlabel;
  }

  public String getContent() {
    return content;
  }

  public void setContent(String content) {
    this.content = content;
  }

  public String getMimeType() {
    return mimeType;
  }

  public void setMimeType(String mimeType) {
    this.mimeType = mimeType;
  }

  public OperatorOld getCreator() {
    return creator;
  }

  public void setCreator(OperatorOld creator) {
    this.creator = creator;
  }

  public Date getAuthored() {
    return authored;
  }

  public void setAuthored(Date authored) {
    this.authored = authored;
  }

  public Date getCreated() {
    return created;
  }

  public void setCreated(Date created) {
    this.created = created;
  }

  public int getType() {
    return type;
  }

  public void setType(int type) {
    this.type = type;
  }

  public OperatorOld getLastModifier() {
    return lastModifier;
  }

  public void setLastModifier(OperatorOld lastModifier) {
    this.lastModifier = lastModifier;
  }

  public long getSizeInBytes() {
    return sizeInBytes;
  }

  public void setSizeInBytes(long sizeInBytes) {
    this.sizeInBytes = sizeInBytes;
  }

  public boolean isTemplate() {
    return template;
  }

  public void setTemplate(boolean template) {
    this.template = template;
  }

  public String getBaseClassification() {
    return baseClassification;
  }

  public void setBaseClassification(String baseClassification) {
    this.baseClassification = baseClassification;
  }

  public String getCode() {
    return code;
  }

  public void setCode(String code) {
    this.code = code;
  }

  public boolean isStorage() {
    return storage;
  }

  public void setStorage(boolean storage) {
    this.storage = storage;
  }

  public boolean isIndex() {
    return index;
  }

  public void setIndex(boolean index) {
    this.index = index;
  }

  public String getConnectionHost() {
    return connectionHost;
  }

  public void setConnectionHost(String connectionHost) {
    this.connectionHost = connectionHost;
  }

  public String getConnectionUser() {
    return connectionUser;
  }

  public void setConnectionUser(String connectionUser) {
    this.connectionUser = connectionUser;
  }

  public String getConnectionPwd() {
    return connectionPwd;
  }

  public void setConnectionPwd(String connectionPwd) {
    this.connectionPwd = connectionPwd;
  }

  public String getConnectionNotes() {
    return connectionNotes;
  }

  public void setConnectionNotes(String connectionNotes) {
    this.connectionNotes = connectionNotes;
  }

  public String getConnectionType() {
    return connectionType;
  }

  public void setConnectionType(String connectionType) {
    this.connectionType = connectionType;
  }

  public String getAreaId() {
    return getArea().getId();
  }

  public List getAncestors() {

    if (this.getParent() == null) {
      List recursor = new ArrayList();
      recursor.add(this);
      return recursor;
    } else {
      List recursor = this.getParent().getAncestors();
      recursor.add(this);
      return recursor;
    }
  }

  public SecuredSupportOld getAncestor() {
    if (this.getParent() == null) {
      return this;
    } else {
      return this.getParent().getAncestor();
    }
  }

  public void setChildren(Set children) {
   this.children = children;
 }

 public Set getChildren() {
   return children;
 }

  public DocumentOld getParent() {
    return parent;
  }

  public void setParent(DocumentOld n) {
    this.parent=n;
  }

}
