package com.twproject.two.ontology;


import com.twproject.two.security.AreaOld;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * This class represents the Tree structure for many TW'objects<br>
 * Each nod contains two different list: the children and the associated documents.<br>
 * Both are list of SecuredNodeConstants.<br>
 * The children list is used to create the "tree".<br>
 * The document list is used to contains document related to the node<br>
 * The security is implemented using CredentialWallet<br>
 * <br>
 *
 * @author Roberto Bicchierai & Pietro Polsinelli
 * @version 2 alpha<br>
 * @since JDK 1.4
 */
public abstract class SecuredSupportOld {

  private String id;
  private String name;

  private AreaOld area;

  private Set groups;

  private boolean deleted;
  private Date lastModified;

  /**
   * a list of DocumentConstants instances representing this node "attachment"
   */
  private List documents;

  private String ancestorIds;


  /**
   * ******************************************************************************************************
   * <p/>
   * constructors
   * <p/>
   * ******************************************************************************************************
   */

  public SecuredSupportOld() {
  }

  protected SecuredSupportOld(String snId) {
    setId(snId);
  }

  /**
   * ******************************************************************************************************
   * <p/>
   * generic field setters and getters
   * <p/>
   * ******************************************************************************************************
   */

  public String getId() {
    return id;
  }

  public void setId(String id) {
    this.id = id;
  }

  public AreaOld getArea() {
    return area;
  }


  public void setArea(AreaOld area) {
    this.area = area;
  }


  public void setName(String s) {
    name = s;
  }

  public String getName() {
    return name;
  }
  public void setDocuments(List documents) {
    this.documents = documents;
  }


  public List getDocuments() throws SecurityException {
    return documents;
  }

  public void removeDocument(SecuredSupportOld d) {
    documents.remove(d);
  }

  public Set getGroups() {
    return groups;
  }

  public void setGroups(Set groups) {
    this.groups = groups;
  }

  public void addGroup(GroupOld groupOld) {
    if (groups == null) groups = new HashSet();
    groups.add(groupOld);
  }

  public void removeGroup(GroupOld groupOld) {
    if (groups != null)
      groups.remove(groupOld);
  }

  public boolean isDeleted() {
    return deleted;
  }

  public void setDeleted(boolean deleted) {
    this.deleted = deleted;
  }

  public Date getLastModified() {
    return lastModified;
  }

  public void setLastModified(Date lastModified) {
    this.lastModified = lastModified;
  }

   public String getAncestorIds() {
    return ancestorIds;
  }

  public void setAncestorIds(String ancestorIds) {
    this.ancestorIds = ancestorIds;
  }

}

