package com.twproject.two.resource;


import com.twproject.two.security.AreaOld;
import com.twproject.two.security.RoleOld;
import com.twproject.two.task.IssueOld;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.waf.SessionState;

import java.util.*;


/**
 * This class represents the TW user. Each operator is relate with a system
 * user.
 * <code>options</code> this collection holds personalized option (skin, startpage,  etc.)
 * keys hold:
 * mainColor = main color of application
 * <p/>
 * <p/>
 * <code>recentViews</code> this collection holds the last objects used. Can be: task, document
 * <code>openTimeCounters</code> list of AssignementTimeCounter
 *
 * @author Pietro Polsinelli, Roberto Bicchierai
 * @version 2 alpha
 * @since JDK 1.3.01 02
 */
public class OperatorOld implements Comparable {

  protected String id;

  /**
   * begin user implementation
   */
  private Set roles;
  private boolean administrator;
  private AreaOld area;
  private String loginName;
  private String password;
  private List lastPasswordList; //Collection of Strings
  private Date lastPasswordChangeDate;
  private String personalIdentificationQuestion;
  private String personalIdentificationAnswer;
  private Map votes = new Hashtable();

  /**
   * end user implementation
   */

  protected String fullName;

  protected AnagraphicalDataOld anagraphicalData;
  private Map options;

  private Map favoriteUrls;

  private boolean enabled = true;

  private boolean deleted;
  private Date lastModified;

  private List news;

  private Set resources;

  public static final String package_prefix = "com.twproject.resource";

  public static final String OPERATOR = package_prefix + ".OPERATOR";

  public static final int VOTES = 100;

  public OperatorOld() {
  }

  public OperatorOld(String rId) {
    this(rId, null, null);
  }

  protected OperatorOld(String oId, String name) {
    this(oId, name, null);
  }

  protected OperatorOld(String oId, String name, AnagraphicalDataOld anagraphicalData) {
    this.id = oId;
    this.fullName = name;
    this.anagraphicalData = anagraphicalData;
  }

  /**
   * Method setFullName
   *
   * @param fullName a  String
   */
  public void setFullName(String fullName) {
    this.fullName = fullName;
  }

  /**
   * Method getFullName
   *
   * @return a String
   */
  public String getFullName() {
    return fullName;
  }

  /**
   * Method setAnagraphicalData
   *
   * @param anagraphicalData an AnagraphicalData
   */
  public void setAnagraphicalData(AnagraphicalDataOld anagraphicalData) {
    this.anagraphicalData = anagraphicalData;
  }

  /**
   * Method getAnagraphicalDatas
   *
   * @return an AnagraphicalData
   */
  public AnagraphicalDataOld getAnagraphicalData() {
    return anagraphicalData;
  }

  /**
   * Method setId
   *
   * @param id a  String
   */
  public void setId(String id) {
    this.id = id;
  }

  /**
   * Method getId
   *
   * @return a String
   */
  public String getId() {
    return id;
  }

  public Map getOptions() {
    return options;
  }

  public void setOptions(Map options) {
    this.options = options;
  }

  public void setInherit(boolean inherit) {
  }

  public boolean getInherit() {
    return false;
  }

  public void setPropagate(boolean propagate) {
  }

  public boolean getPropagate() {
    return false;
  }

  public Collection getOperators() {
    Collection coll = new HashSet();
    coll.add(this);
    return coll;
  }

  public boolean isOperatorIn(OperatorOld o) {
    return (this.equals(o));
  }





  public boolean isEnabled() {
    return enabled;
  }

  public void setEnabled(boolean enabled) {
    this.enabled = enabled;
  }

  public boolean isDeleted() {
    return deleted;
  }

  public void setDeleted(boolean deleted) {
    this.deleted = deleted;
  }

  public Date getLastModified() {
    return lastModified;
  }

  public void setLastModified(Date lastModified) {
    this.lastModified = lastModified;
  }

  public boolean isAdministrator() {
    return administrator;
  }

  public void setAdministrator(boolean administrator) {
    this.administrator = administrator;
  }

  public AreaOld getArea() {
    return area;
  }

  public void setArea(AreaOld area) {
    this.area = area;
  }

  public String getLoginName() {
    return loginName;
  }

  public void setLoginName(String loginName) {
    this.loginName = loginName;
  }

  public String getPassword() {
    return password;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public List getLastPasswordList() {
    if (lastPasswordList == null) {
      lastPasswordList = new ArrayList();
    }
    return lastPasswordList;
  }

  public void setLastPasswordList(List lastPasswordList) {
    this.lastPasswordList = lastPasswordList;
  }

  public Date getLastPasswordChangeDate() {
    return lastPasswordChangeDate;
  }

  public void setLastPasswordChangeDate(Date lastPasswordChangeDate) {
    this.lastPasswordChangeDate = lastPasswordChangeDate;
  }

  public String getPersonalIdentificationQuestion() {
    return personalIdentificationQuestion;
  }

  public void setPersonalIdentificationQuestion(String personalIdentificationQuestion) {
    this.personalIdentificationQuestion = personalIdentificationQuestion;
  }

  public String getPersonalIdentificationAnswer() {
    return personalIdentificationAnswer;
  }

  public void setPersonalIdentificationAnswer(String personalIdentificationAnswer) {
    this.personalIdentificationAnswer = personalIdentificationAnswer;
  }




  public String getDisplayName(SessionState sm) {
    return getFullName();
  }

  /**
   * ******************************************************************************************************
   * <p/>
   * begin Role
   * <p/>
   * ******************************************************************************************************
   */

  public void addRole(RoleOld roleOld) {
    if (getRoles() == null) {
      setRoles(new HashSet());
    }
    getRoles().add(roleOld);
  }

  public Set getRoles() {
    if (roles == null) {
      roles = new HashSet();
    }
    return roles;
  }

  public Iterator getRoleIterator() {
    if (roles == null) {
      roles = new HashSet();
    }
    return roles.iterator();
  }

  public void setRoles(Set roles) {
    this.roles = roles;
  }

  public void removeRole(RoleOld roleOld) {
    if (roles != null) roles.remove(roleOld);
  }

  public Map getVotes() {
    return votes;
  }

  public void setVotes(Map votes) {
    this.votes = votes;
  }

  public int getVotesForIssue(IssueOld i) {
    Object vote = votes.get(i.getId());
    if (vote == null)
      return 0;
    else
      return Integer.parseInt((String) vote);
  }

  public int getAvailableVotes() {
    int av = VOTES;
    Set keys = votes.keySet();
    if (keys.size() == 0)
      return VOTES;
    else {
      Iterator i = keys.iterator();
      while (i.hasNext()) {
        av -= Integer.parseInt((String) votes.get(i.next()));
      }
      return av;
    }

  }


  public void setVotesForIssue(IssueOld i, int vote) {
    votes.put(i.getId(), vote + "");
  }

  public List getNews() {
    return news;
  }

  public void setNews(List news) {
    this.news = news;
  }

  public void addNews(String news) {
    if (getNews() == null) {
      setNews(new ArrayList());
    }
    getNews().add(DateUtilities.dateToString(new Date(),"dd MMM yy") +" - "+ news);
  }

  public Map getFavoriteUrls() {
    return favoriteUrls;
  }

  public void setFavoriteUrls(Map favoriteUrls) {
    this.favoriteUrls = favoriteUrls;
  }

  public void addFavoriteUrl(String key, String value) {
    if (getFavoriteUrls() == null)
      setFavoriteUrls(new Hashtable());
    getFavoriteUrls().put(key, value);

  }

  public Set getResources() {
    return resources;
  }

  public void setResources(Set resources) {
    this.resources = resources;
  }

  public void addResource(ResourceOld resource) {
    if (getResources() == null)
      setResources(new HashSet());
    getResources().add(resource);

  }

  public int compare(Object o1, Object o2) {
    if (o1 == null || o2 == null || ((OperatorOld) o1).getFullName() == null || ((OperatorOld) o2).getFullName() == null)
      return 0;
    return ((OperatorOld) o1).getFullName().compareTo(((OperatorOld) o2).getFullName());
  }

  public boolean equals(Object o) {
    return this.compareTo(o) == 0;
  }

  public int hashCode() {

    int result = 0;
    if (id != null)
      result = id.hashCode();
    return result;
  }

  public int compareTo(Object o) {
    if (o == null)
      return 0;
    else
      return this.getId().compareTo(((OperatorOld) o).getId());
  }

}