package com.twproject.two.resource;

import com.twproject.two.agenda.PeriodOld;
import com.twproject.two.ontology.SecuredSupportOld;
import org.jblooming.ApplicationException;

import java.util.*;


/**
 * <code>Resource</code> formalises an extended resource concept
 * e.g. companies, individuals.
 *
 * @author Pietro Polsinelli, Roberto Bicchierai
 * @since JDK 1.4
 */

public class ResourceOld extends SecuredSupportOld  {

  private String personName;
  private String personSurname;

  protected OperatorOld operator;

  private Set skills;
  private Set anagraphicalData;

  private ResourceOld parent;

  private Set children;

  private int type;

  //                          interface OperatorAggregator implementation
  //                          start
  private boolean inherit;
  private boolean propagate = true;
  //                          end

  private String notes;

  private String title;

  private long hourlyCost = 0;

  public static final String package_prefix = "com.twproject.resource";
  public static final String RESOURCE = package_prefix + ".RESOURCE";
  public static final String RESOURCE_CHILDREN = package_prefix + ".RESOURCE_CHILDREN";

  // type
  public static final int COMPANY = 0;
  public static final int PERSON = 1;
  public static final int SERVICE = 2;


  /**
   * ******************************************************************************************************
   * <p/>
   * constructors
   * <p/>
   * ******************************************************************************************************
   */

  public ResourceOld() {
    super();
  }


  public ResourceOld(String rId) {
    super(rId);
  }


  /**
   * ******************************************************************************************************
   * <p/>
   * AnagraphicalData
   * <p/>
   * ******************************************************************************************************
   */

  public Set getAnagraphicalData() {
    return anagraphicalData;
  }

  public void setAnagraphicalData(Set anagraphicalData) {
    this.anagraphicalData = anagraphicalData;
  }

  public void addAnagraphicalData(AnagraphicalDataOld ad) {

    if (ad == null) return;

    if (getAnagraphicalData() == null) this.anagraphicalData = new HashSet();
    anagraphicalData.add(ad);

  }

  public void removeAnagraphicalData(AnagraphicalDataOld ad) {
    if (anagraphicalData != null) anagraphicalData.remove(ad);
  }

  public Iterator getAnagraphicalDatas() {
    if (getAnagraphicalData() != null) return getAnagraphicalData().iterator();
    return null;
  }

  public boolean isInAnagraphicalData(AnagraphicalDataOld ad) {
    if (getAnagraphicalData() == null) return false;
    return getAnagraphicalData().contains(ad) ? true : false;
  }

  public Set getSkills() {
    return skills;
  }

  public void setSkills(Set skills) {
    this.skills = skills;
  }

  public void addSkill(SkillOld sk) {

    if (sk == null) return;

    if (getSkills() == null) this.skills = new HashSet();
    skills.add(sk);
  }

  public void setPropagate(boolean propagate) {
    this.propagate = propagate;
  }

  public boolean getPropagate() {
    return propagate;
  }

  public void setInherit(boolean inherit) {
    this.inherit = inherit;
  }

  public boolean getInherit() {
    return inherit;
  }

  public void setOperator(OperatorOld operator) {
    if (operator != null)
      operator.addResource(this);
    this.operator = operator;
  }

  public OperatorOld getOperator() {
    return operator;
  }

  public PeriodOld getAvailability() {
    return null;
  }

  public Collection getTasks() {
    return null;
  }

  public String toXML() {
    return "";
  }

  public void removeUnavailability(PeriodOld p) {
  }

  /**
   * Method getUsers
   * interface OperatorAggregator implementation
   *
   * @return a Collection of User.
   */
  public Collection getOperators() {
    return getOperators(new HashSet(), new HashSet());
  };

  /**
   * Method isUserIn
   * This method navigate the tree Task structure (respecting Inherit and Propagate) to find
   * the user u
   *
   * @param o an User
   * @return a boolean
   */
  public boolean isOperatorIn(OperatorOld o) {
    return getOperators().contains(o);
  }

  // end


  /**
   * Method getParentUsers
   *
   * @param visitedNodes a  Collection
   * @param operators    a  Collection
   * @return a Collection of operators
   */
  private Collection getOperators(Collection visitedNodes, Collection operators) {

    if (!visitedNodes.contains(this)) {
      if (getOperator() != null)
        operators.add(getOperator());
      visitedNodes.add(this);

      // visiting parent node

      if (getInherit() && getParent() != null) {
        operators.addAll(((ResourceOld) getParent()).getOperators(visitedNodes, operators));
      }

      // visiting children
      if (getPropagate()) {
        Iterator iter = getChildrenIterator();
        ResourceOld r;
        while (iter.hasNext()) {
          r = (ResourceOld) iter.next();
          operators.addAll(r.getOperators(visitedNodes, operators));
        }
      }
    }
    return operators;
  }


  public boolean isPerson() {
    return type == ResourceOld.PERSON;
  }

  /**
   * @param person
   * @deprecated
   */
  public void setPerson(boolean person) {
    type = ResourceOld.PERSON;
  }


  public String getNotes() {
    return notes;
  }

  public void setNotes(String notes) {
    this.notes = notes;
  }

  public String getPersonName() {
    return personName;
  }

  public void setPersonName(String personName) {
    this.personName = personName;
  }

  public String getPersonSurname() {
    return personSurname;
  }

  public void setPersonSurname(String personSurname) {
    this.personSurname = personSurname;
  }

  public String getResourceClearName() {
    if (isPerson()) {
      return getResourceClearName(getPersonName(), getPersonSurname());
    } else
      return getName() != null ? getName() : "";
  }

  public static String getResourceClearName(String firstName, String lastName) {
    String name = (firstName != null ? firstName : "");
    String surname = (lastName != null ? lastName + " " : "");
    return surname + name;
  }


  public String getTitle() {
    return title;
  }

  public void setTitle(String title) {
    this.title = title;
  }

  public long getHourlyCost() {
    return hourlyCost;
  }

  public void setHourlyCost(long hourlyCost) {
    this.hourlyCost = hourlyCost;
  }

  public int getType() {
    return type;
  }

  public void setType(int type) {
    this.type = type;
  }

  public List getAncestors() {

    if (this.getParent() == null) {
      List recursor = new ArrayList();
      recursor.add(this);
      return recursor;
    } else {
      List recursor = this.getParent().getAncestors();
      recursor.add(this);
      return recursor;
    }
  }

  public SecuredSupportOld getAncestor() {
    if (this.getParent() == null) {
      return this;
    } else {
      return this.getParent().getAncestor();
    }
  }

  public void setChildren(Set children) {
   this.children = children;
 }

 public Set getChildren() {
   return children;
 }

  public Iterator getChildrenIterator() {
   if (getChildren() != null) return getChildren().iterator();
   return null;
 }

  public ResourceOld getParent() {
    return parent;
  }

  public void setParent(ResourceOld n) throws ApplicationException {
    this.parent=n;
  }

}



