package com.twproject.two.task;


import com.twproject.two.resource.OperatorOld;
import com.twproject.two.resource.ResourceOld;
import com.twproject.two.security.AreaOld;

import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Pietro Polsinelli ppolsinelli@twproject.com
 */
public class IssueOld {

  private String id;
  private boolean deleted;
  private Date lastModified;

  private String type;
  private String status;
  private String description;
  private String gravity;
  private int votes;
  private Date shouldCloseBy;
  private String notes;
  private Date created;
  private Set documents;
  private TaskOld task;
  private OperatorOld creator;
  private ResourceOld assignedTo;
  private Set watchers = new HashSet();

  public static final String package_prefix = "com.twproject.task";
  public static final String ISSUE = package_prefix + ".ISSUE";

  public static final String GRAVITY_LOW = "01_GRAVITY_LOW";
  public static final String GRAVITY_MEDIUM = "02_GRAVITY_MEDIUM";
  public static final String GRAVITY_HIGH = "03_GRAVITY_HIGH";
  public static final String GRAVITY_CRITICAL = "04_GRAVITY_CRITICAL";
  public static final String GRAVITY_BLOCK = "05_GRAVITY_BLOCK";

  public static final String STATUS_OPEN = "01_STATUS_OPEN";
  public static final String STATUS_PAUSED = "02_STATUS_ABORTED";
  public static final String STATUS_ABORTED = "03_STATUS_ABORTED";
  public static final String STATUS_CLOSED = "04_STATUS_CLOSED";

  public static Comparator compareByTaskName = new Comparator() {

    protected boolean isEmpty(String s) {
      return (s==null || s.trim().length()==0);
    }
    public int compare(Object o1, Object o2) {

      return 0;
    };
  };
  public static Comparator compareByAssignedResource = new Comparator() {
    protected boolean isEmpty(String s) {
      return (s==null || s.trim().length()==0);
    }
    public int compare(Object o1, Object o2) {
      IssueOld i1 = (IssueOld)o1;
      IssueOld i2 = (IssueOld)o2;
      String assignedTo1 = null;
      if (i1.getAssignedTo()!=null)
        assignedTo1 = i1.getAssignedTo().getResourceClearName();
      String assignedTo2 = null;
      if (i2.getAssignedTo()!=null)
        assignedTo2 = i2.getAssignedTo().getResourceClearName();
      if (isEmpty(assignedTo1) && isEmpty(assignedTo2))
        return 0;
      if (isEmpty(assignedTo1) && !isEmpty(assignedTo2))
        return 1;
      if (!isEmpty(assignedTo1) && isEmpty(assignedTo2))
        return -1;
      return assignedTo1.compareTo(assignedTo2);
    };
  };

  public IssueOld() {
  }

  public IssueOld(String id) {
    this.id = id;
  }

  public String getId() {
    return id;
  }

  public void setId(String id) {
    this.id = id;
  }

  public boolean isDeleted() {
    return deleted;
  }

  public void setDeleted(boolean deleted) {
    this.deleted = deleted;
  }

  public Date getLastModified() {
    return lastModified;
  }

  public void setLastModified(Date lastModified) {
    this.lastModified = lastModified;
  }

  public String getType() {
    return type;
  }

  public void setType(String type) {
    this.type = type;
  }

  public String getStatus() {
    return status;
  }

  public void setStatus(String status) {
    this.status = status;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public String getGravity() {
    return gravity;
  }

  public void setGravity(String gravity) {
    this.gravity = gravity;
  }

  public int getVotes() {
    return votes;
  }

  public void setVotes(int votes) {
    this.votes = votes;
  }

  public Date getShouldCloseBy() {
    return shouldCloseBy;
  }

  public void setShouldCloseBy(Date shouldCloseBy) {
    this.shouldCloseBy = shouldCloseBy;
  }

  public String getNotes() {
    return notes;
  }

  public void setNotes(String notes) {
    this.notes = notes;
  }

  public Date getCreated() {
    return created;
  }

  public void setCreated(Date created) {
    this.created = created;
  }

  public Set getDocuments() {
    return documents;
  }

  public void setDocuments(Set documents) {
    this.documents = documents;
  }

  public OperatorOld getCreator() {
    return creator;
  }

  public void setCreator(OperatorOld creator) {
    this.creator = creator;
  }

  public Set getWatchers() {
    return watchers;
  }

  public void setWatchers(Set watchers) {
    this.watchers = watchers;
  }

  public void addWatcher(OperatorOld op) {

    if (op == null) return;

    if (getWatchers() == null) watchers = new HashSet();
    watchers.add(op);
  }

  public TaskOld getTask() {
    return task;
  }

  public void setTask(TaskOld task) {
    this.task = task;
  }

  public ResourceOld getAssignedTo() {
    return assignedTo;
  }

  public void setAssignedTo(ResourceOld assignedTo) {
    this.assignedTo = assignedTo;
  }



  public AreaOld getArea() {
    if (task != null)
      return task.getArea();
    return null;
  }
}
