package com.twproject.two.task;

import com.twproject.two.agenda.PeriodOld;
import com.twproject.two.agenda.ScheduleSupportOld;
import com.twproject.two.ontology.SecuredSupportOld;
import com.twproject.two.resource.OperatorOld;
import com.twproject.two.resource.ResourceOld;
import com.twproject.two.resource.SkillOld;
import com.twproject.two.task.finantial.CostOld;

import java.util.*;

public class TaskOld extends SecuredSupportOld  {
  protected TaskOld parent;

  private Set children;


  /**
   * BEGIN DESCRIPTION PROPERTIES
   */
  private String code;

  /**
   * type can be estimate, production
   *
   * @see #FEASIBILITY
   * @see #PRODUCTION
   */
  private int type;
  public static final int FEASIBILITY = 0;
  public static final int PRODUCTION = 1;

  private String description;

  /**
   * Collection of Requirement that condition progress and state of task
   */
  private Set requirements;

  /**
   * status of the task
   *
   * @see #STATUS_ACTIVE
   * @see #STATUS_SUSPENDED
   * @see #STATUS_DONE
   * @see #STATUS_ABORTED
   */
  private int status;

  public static final int STATUS_ANY = 0;

  public static final int STATUS_ACTIVE = 100;
  public static final int STATUS_SUSPENDED = 101;
  public static final int STATUS_DONE = 102;
  public static final int STATUS_ABORTED = 103;
  public static final int STATUS_UNDEFINED = 104;

  public static final int STATUS_TO_BE_COMPILED = 200;
  public static final int STATUS_TO_BE_SENT_TO_REVIEW = 201;
  public static final int STATUS_IN_STAKEHOLDER_REVIEW = 202;
  public static final int STATUS_IN_PRODUCTION = 203;
  public static final int STATUS_REFUSED = 204;


  /**
   * BEGIN MANAGEMENT PROPERTIES
   */

  private ResourceOld manager;
  /**
   * the actual task relevance (for the task manager)
   */
  private String relevance;

  /**
   * An evaluation of the likelihood of going into production
   */
  private String likelyhood;


  /**
   *      BEGIN WORK PROPERTIES
   */

  /**
   * degree of completion of this task
   */
  private long progress;
  private SkillOld skill;
  private Set assignements;
  /**
   * this is in terms of worklog distribution.
   */
  private int estimatedWorklogRequired;


  /**
   * BEGIN TIMING PROPERTIES
   */

  private ScheduleSupportOld schedule;
  private PeriodOld mustBeOpen;
  private PeriodOld estimatedOpen;
  private Set milestones;

  /**
   * BEGIN FINANTIAL PROPERTIES
   */

  private double forecasted;
  private double effective;
  private double invoiced;


  private Set costs;

  private Set stakeHolders;

  /**
   *
   */


  /**
   * END PROPERTIES
   */

  public static final String package_prefix = "com.twproject.task";
  public static final String TASK = package_prefix + ".TASK";
  public static final String TASK_CHILDREN = package_prefix + ".TASK_CHILDREN";
  public static final String TASK_CUSTOMERS = package_prefix + ".TASK_CUSTOMERS";


  /**
   * Task constructor
   */

  public TaskOld() {
  }

  public TaskOld(String tId) {

    super(tId);
    setAssignements(new HashSet());
    setStakeHolders(new HashSet());
    setStatus(STATUS_UNDEFINED);
  }

  public void addChild(TaskOld c) {
    c.setParent(this);
    if (children == null) children = new HashSet();
    children.add(c);
  }

  public void setParent(TaskOld n) {
    parent = n;
  }

  /**
   * Method getParent.
   *
   * @return a Node
   */
  public TaskOld getParent() {
    return parent;
  }

  public List getAncestors() {

    if (this.getParent() == null) {
      List recursor = new ArrayList();
      recursor.add(this);
      return recursor;
    } else {
      List recursor = this.getParent().getAncestors();
      recursor.add(this);
      return recursor;
    }
  }

  public TaskOld getAncestor() {
    if (this.getParent() == null) {
      return this;
    } else {
      return this.getParent().getAncestor();
    }
  }

  public Set getChildren() {
    return children;
  }

  public void setChildren(Set children) {
    this.children = children;
  }


  public boolean areRequirementsFulfilled() {
    Iterator i = requirements.iterator();
    while (i.hasNext())

      if (!((RequirementOld) i.next()).isFulfilled())
        return false;

    return true;
  }

  public int getStatus() {
    return status;
  }

  public void setPeriod(ScheduleSupportOld s) {
    setSchedule(s);
  }

  public PeriodOld getPeriod() {
    if (getSchedule() != null)
      return getSchedule().getPeriod();
    return null;
  }

  public void addRequiredSkill(SkillOld s) {
  }

  public void removeRequiredSkill(SkillOld s) {
  }

  public Collection getRequiredSkills() {
    return null;
  }

  /**
   * task manager sets it by hand
   */
  public void setTaskProgress(double progress) {
  }

  /**
   *
   */
  public double getTaskProgress() {
    return 0;
  }

  public void setManager(ResourceOld tm) {
    manager = tm;
  }

  public ResourceOld getManager() {
    return manager;
  }

// interface OperatorAggregator implementation
// start

  private boolean inherit = false;
  private boolean propagate = true;

  /**
   * Method getUsers
   * interface OperatorAggregator implementation
   *
   * @return a Collection of Operator
   *         #PENDING#
   */
  public Collection getOperators() {
    Collection res = getResources();
    Collection ops = new HashSet();
    if (res != null) {
      for (Iterator iterator = res.iterator(); iterator.hasNext();) {
        ResourceOld resource = (ResourceOld) iterator.next();
        OperatorOld op = resource.getOperator();
        if (op != null)
          ops.add(op);
      }
    }
    return ops;
  }


  public Collection getResources() {
    return getResources(new HashSet(), new HashSet());
  }


  public Iterator getChildrenIterator() {
    if (getChildren() != null) return getChildren().iterator();
    return null;
  }

  /**
   * Method getParentUsers
   *
   * @param visitedNodes a  Collection
   * @param resources    a  Collection
   * @return a Collection of operators
   */
  private Collection getResources(Collection visitedNodes, Collection resources) {
    ResourceOld res;
    if (!visitedNodes.contains(this)) {
      if (assignements != null)
        for (Iterator iterator = assignements.iterator(); iterator.hasNext();) {
          AssignementOld assignement = (AssignementOld) iterator.next();
          res = assignement.getResource();
          if (res != null)
            resources.add(res);
        }
      visitedNodes.add(this);

      // visiting parent node

      if ((getInherit()) && getParent() != null) {
        resources.addAll(((TaskOld) getParent()).getResources(visitedNodes, resources));
      }

      // visiting children

      if (getPropagate()) {
        Iterator iter = getChildrenIterator();
        TaskOld t;
        while (iter.hasNext()) {
          t = (TaskOld) iter.next();
          resources.addAll(t.getResources(visitedNodes, resources));
        }
      }
    }
    return resources;
  }


  public void setPropagate(boolean propagate) {
    this.propagate = propagate;
  }

  public boolean getPropagate() {
    return propagate;
  }

  public void setInherit(boolean inherit) {
    this.inherit = inherit;
  }

  public boolean getInherit() {
    return inherit;
  }

  /**
   * Method isUserIn
   * This method navigates the tree Task structure (respecting Inherit and Propagate) to find the user u
   *
   * @param o a User
   * @return a boolean
   */
  public boolean isOperatorIn(OperatorOld o) {
    return getOperators().contains(o);
  }


  public SkillOld getSkill() {
    return skill;
  }

  public void setSkill(SkillOld skill) {
    this.skill = skill;
  }

  public ScheduleSupportOld getSchedule() {
    return schedule;
  }

  public void setSchedule(ScheduleSupportOld schedule) {
    this.schedule = schedule;
  }

  public Set getStakeHolders() {
    if (stakeHolders == null) stakeHolders = new HashSet();
    return stakeHolders;
  }

  public void setStakeHolders(Set stakeHolders) {
    this.stakeHolders = stakeHolders;
  }

  public void addStakeHolder(StakeHolderOld stakeHolderOld) {
    getStakeHolders().add(stakeHolderOld);
  }

  public void removeStakeHolder(StakeHolderOld stakeHolderOld) {
    if (stakeHolders != null)
      stakeHolders.remove(stakeHolderOld);
  }

  public long getProgress() {
    return progress;
  }

  public void setProgress(long progress) {
    this.progress = progress;
  }

  public void setRelevance(String relevance) {
    this.relevance = relevance;
  }

  public String getRelevance() {
    return relevance;
  }

  public void setStatus(int status) {
    this.status = status;
  }

// end


  public int getType() {
    return type;
  }

  public void setType(int type) {
    this.type = type;
  }

  public Set getAssignements() {
    return assignements;
  }

  public void setAssignements(Set assignements) {
    this.assignements = assignements;
  }

  public void addAssignement(AssignementOld as) {
    if (assignements == null)
      assignements = new HashSet();

    assignements.add(as);
    as.setTask(this);
  }

  public void removeAssignements(AssignementOld as) {
    getAssignements().remove(as);
  }

  public void removeAssignement(AssignementOld as) {
    if (assignements != null)
      assignements.remove(as);
  }

  public Set getRequirements() {
    return requirements;
  }

  public void setRequirements(Set requirements) {
    this.requirements = requirements;
  }

  public void addRequirement(RequirementOld req) {
    if (requirements == null)
      requirements = new HashSet();

    requirements.add(req);
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public String getCode() {
    return code;
  }

  public void setCode(String code) {
    this.code = code;
  }

  public String getLikelyhood() {
    return likelyhood;
  }

  public void setLikelyhood(String likelyhood) {
    this.likelyhood = likelyhood;
  }

  public Set getDescendantIds() {
    Set desc = new HashSet();
    desc.add(getId());
    if (getChildren() != null && getChildren().size() > 0) {
      Iterator i = getChildren().iterator();
      while (i.hasNext()) {
        desc.addAll(((TaskOld) i.next()).getDescendantIds());
      }
    }
    return desc;
  }

  public PeriodOld getMustBeOpen() {
    return mustBeOpen;
  }

  public void setMustBeOpen(PeriodOld mustBeOpen) {
    this.mustBeOpen = mustBeOpen;
  }

  public PeriodOld getEstimatedOpen() {
    return estimatedOpen;
  }

  public void setEstimatedOpen(PeriodOld estimatedOpen) {
    this.estimatedOpen = estimatedOpen;
  }


  public StakeHolderOld getStakeHolder(String id) {
    for (Iterator iterator = getStakeHolders().iterator(); iterator.hasNext();) {
      StakeHolderOld sh = (StakeHolderOld) iterator.next();
      if (sh.getId().equals(id))
        return sh;
    }
    return null;
  }

  public void removeStakeHolder(String id) {
    for (Iterator iterator = new HashSet(getStakeHolders()).iterator(); iterator.hasNext();) {
      StakeHolderOld sh = (StakeHolderOld) iterator.next();
      if (sh.getId().equals(id))
        getStakeHolders().remove(sh);
    }
  }

  public Set getMilestones() {
    if (milestones == null) milestones = new HashSet();
    return milestones;
  }

  public void setMilestones(Set milestones) {
    this.milestones = milestones;
  }

  public MilestoneOld getMilestone(String id) {
    for (Iterator iterator = getMilestones().iterator(); iterator.hasNext();) {
      MilestoneOld sh = (MilestoneOld) iterator.next();
      if (sh.getId().equals(id))
        return sh;
    }
    return null;
  }

  public void removeMilestone(String id) {
    for (Iterator iterator = new HashSet(getMilestones()).iterator(); iterator.hasNext();) {
      MilestoneOld sh = (MilestoneOld) iterator.next();
      if (sh.getId().equals(id))
        getMilestones().remove(sh);
    }
  }

  public Set getCosts() {
    return costs;
  }

  public void setCosts(Set costs) {
    this.costs = costs;
  }

  public CostOld getCost(String id) {
    for (Iterator iterator = getCosts().iterator(); iterator.hasNext();) {
      CostOld sh = (CostOld) iterator.next();
      if (sh.getId().equals(id))
        return sh;
    }
    return null;
  }

  public void removeCost(String id) {
    for (Iterator iterator = new HashSet(getCosts()).iterator(); iterator.hasNext();) {
      CostOld sh = (CostOld) iterator.next();
      if (sh.getId().equals(id))
        getCosts().remove(sh);
    }
  }


  public int getEstimatedWorklogRequired() {
    return estimatedWorklogRequired;
  }

  public void setEstimatedWorklogRequired(int estimatedWorklogRequired) {
    this.estimatedWorklogRequired = estimatedWorklogRequired;
  }



  public int compare(Object o1, Object o2) {
    if (o1 == null || o2 == null || ((TaskOld) o1).getId() == null || ((TaskOld) o2).getId() == null)
      return 0;
    return ((TaskOld) o1).getId().compareTo(((TaskOld) o2).getId());
  }

  public boolean equals(Object o) {
    return this.compareTo(o) == 0;
  }

  public int hashCode() {

    int result = 0;
    if (getId() != null)
      result = getId().hashCode();
    return result;
  }

  public int compareTo(Object o) {
    if (o == null)
      return 0;
    else
      return this.getId().compareTo(((TaskOld) o).getId());
  }

  public double getForecasted() {
    return forecasted;
  }

  public void setForecasted(double forecasted) {
    this.forecasted = forecasted;
  }

  public double getEffective() {
    return effective;
  }

  public void setEffective(double effective) {
    this.effective = effective;
  }

  public double getInvoiced() {
    return invoiced;
  }

  public void setInvoiced(double invoiced) {
    this.invoiced = invoiced;
  }


  public Set getAllDescendants() {
    Set set = new HashSet();
    getAllDescendants(this,set);
    return set;
  }

  private void getAllDescendants(TaskOld tsk,Set set) {
    set.addAll(tsk.children);
    Iterator i = tsk.getChildrenIterator();
    while (i.hasNext()) {
      TaskOld task = (TaskOld) i.next();
      if (!set.contains(task))
        getAllDescendants(task,set);
    }
  }
}
