package com.twproject.worklog;

import com.twproject.operator.TeamworkOperator;
import com.twproject.resource.Person;
import com.twproject.resource.Resource;
import com.twproject.task.Assignment;
import org.jblooming.messaging.Message;
import org.jblooming.messaging.MessagingSystem;
import org.jblooming.operator.Operator;
import org.jblooming.oql.OqlQuery;
import org.jblooming.persistence.hibernate.HibernateFactory;
import org.jblooming.persistence.hibernate.PersistenceContext;
import org.jblooming.scheduler.ExecutableSupport;
import org.jblooming.scheduler.JobLogData;
import org.jblooming.tracer.Tracer;
import org.jblooming.utilities.DateUtilities;
import org.jblooming.utilities.ReflectionUtilities;
import org.jblooming.waf.SessionState;
import org.jblooming.waf.constants.OperatorConstants;
import org.jblooming.waf.html.button.ButtonLink;
import org.jblooming.waf.settings.Application;
import org.jblooming.waf.settings.ApplicationState;
import org.jblooming.waf.view.PageSeed;

import java.util.Date;
import java.util.List;

/**
 * @author Pietro Polsinelli ppolsinelli@open-lab.com
 * @author Roberto Bicchierai rbicchierai@open-lab.com
 */
public class ExecuteTimeCounterChecks extends ExecutableSupport {

  public JobLogData run(JobLogData jobLogData) {

    boolean exception = true;
    PersistenceContext pc = null;
    try {
      
      pc = HibernateFactory.newFreeSession();

      String hql = "from " + Assignment.class.getName() + " assig where assig.counted=:truth";
      OqlQuery oql = new OqlQuery(hql, pc);
      oql.getQuery().setBoolean("truth", Boolean.TRUE);
      List countedAssig = null;

      countedAssig = oql.list();

      Application app = ApplicationState.platformConfiguration.defaultApplication;

      for (Object assigO : countedAssig) {
        Assignment assig = (Assignment) assigO;
        Resource r = (Resource) ReflectionUtilities.getUnderlyingObject(assig.getResource());

        if (r instanceof Person) {
          TeamworkOperator op = ((Person) r).getMyself();
          String language = SessionState.getOperatorOption(op, OperatorConstants.FLD_SELECT_LANG);

          String max = op.getOption(OperatorConstants.FLD_WORKING_HOUR_TOTAL);
          if (max != null) {
            long millis = (int)DateUtilities.millisFromHourMinuteSmart(max);
            if ((System.currentTimeMillis() - assig.getCountingStartedAt().getTime()) > millis) {

              assig.closeCounter(null, pc);

              Message message = new Message();
              message.setFromOperator(Operator.getSystemOperator());
              message.setToOperator(op);

              message.setMedia(MessagingSystem.Media.STICKY.toString());
              String subject = ApplicationState.i18n.getLabelNoPageState("COUNTER_EXPIRED", app.getName(), language);
              message.setSubject(subject);

              PageSeed ps = new PageSeed(ApplicationState.serverURL + "/applications/teamwork/task/worklog/worklogAssignmentList.jsp");
              ps.mainObjectId = assig.getId();
              ButtonLink editLink = ButtonLink.getTextualInstance(subject, ps);
              message.setLink(editLink.toPlainLink());
              message.setMessageBody(assig.getDisplayName());
              message.setReceived(new Date());
              message.store(pc);

              assig.store(pc);
              Tracer.platformLogger.debug("ExecuteTimeCounterChecks counter closed for assig " + assig.getId());              

            }
          }
        }
      }

      pc.commitAndClose();

      jobLogData.notes = jobLogData.notes + "ExecuteTimeCounterChecks executed on " + DateUtilities.dateToTimestampString(DateUtilities.getAbsoluteNow());
      exception = false;

    } catch (Exception e) {
      Tracer.platformLogger.error("ExecuteTimeCounterChecks error", e);

    } finally {
      if (exception) {
        if (pc != null) {
          pc.rollback();
        }
        jobLogData.successfull = false;
      }
    }
    return jobLogData;
  }


}
